@@ -33,6 +33,7 @@
 #include "rcfile.h"
 #include "dnd.h"
 #include "ui_utildlg.h"
+#include "layout.h"
 
 
 #include <math.h>
@@ -97,7 +98,7 @@ static void bar_pane_exif_entry_update_title(ExifEntry *ee);
 static void bar_pane_exif_update(PaneExifData *ped);
 static gboolean bar_pane_exif_menu_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data);
 static void bar_pane_exif_notify_cb(FileData *fd, NotifyType type, gpointer data);
-
+static gboolean bar_pane_exif_copy_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data);
 
 static void bar_pane_exif_entry_changed(GtkEntry *text_entry, gpointer data)
 {
@@ -185,6 +186,7 @@ static GtkWidget *bar_pane_exif_add_entry(PaneExifData *ped, const gchar *key, c
 
 	bar_pane_exif_entry_dnd_init(ee->ebox);
 	g_signal_connect(ee->ebox, "button_release_event", G_CALLBACK(bar_pane_exif_menu_cb), ped);
+	g_signal_connect(ee->ebox, "button_press_event", G_CALLBACK(bar_pane_exif_copy_cb), ped);
 
 	bar_pane_exif_setup_entry_box(ped, ee);
 
@@ -556,7 +558,7 @@ static void bar_pane_exif_conf_dialog(GtkWidget *widget)
 	g_signal_connect(G_OBJECT(widget), "destroy",
 			 G_CALLBACK(bar_pane_exif_edit_close_cb), gd);
 
-	generic_dialog_add_message(gd, NULL, ee ? _("Configure entry") : _("Add entry"), NULL);
+	generic_dialog_add_message(gd, NULL, ee ? _("Configure entry") : _("Add entry"), NULL, FALSE);
 
 	generic_dialog_add_button(gd, GTK_STOCK_OK, NULL,
 				  bar_pane_exif_edit_ok_cb, TRUE);
@@ -598,6 +600,19 @@ static void bar_pane_exif_delete_entry_cb(GtkWidget *menu_widget, gpointer data)
 	gtk_widget_destroy(entry);
 }
 
+static void bar_pane_exif_copy_entry_cb(GtkWidget *menu_widget, gpointer data)
+{
+	GtkWidget *widget = data;
+	GtkClipboard *clipboard;
+	const gchar *value;
+	ExifEntry *ee;
+
+	ee = g_object_get_data(G_OBJECT(widget), "entry_data");
+	value = gtk_label_get_text(GTK_LABEL(ee->value_widget));
+	clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_set_text(clipboard, value, -1);
+}
+
 static void bar_pane_exif_toggle_show_all_cb(GtkWidget *menu_widget, gpointer data)
 {
 	PaneExifData *ped = data;
@@ -619,9 +634,11 @@ static void bar_pane_exif_menu_popup(GtkWidget *widget, PaneExifData *ped)
 		/* for the entry */
 		gchar *conf = g_strdup_printf(_("Configure \"%s\""), ee->title);
 		gchar *del = g_strdup_printf(_("Remove \"%s\""), ee->title);
+		gchar *copy = g_strdup_printf(_("Copy \"%s\""), ee->title);
 
 		menu_item_add_stock(menu, conf, GTK_STOCK_EDIT, G_CALLBACK(bar_pane_exif_conf_dialog_cb), widget);
 		menu_item_add_stock(menu, del, GTK_STOCK_DELETE, G_CALLBACK(bar_pane_exif_delete_entry_cb), widget);
+		menu_item_add_stock(menu, copy, GTK_STOCK_COPY, G_CALLBACK(bar_pane_exif_copy_entry_cb), widget);
 		menu_item_add_divider(menu);
 
 		g_free(conf);
@@ -646,6 +663,25 @@ static gboolean bar_pane_exif_menu_cb(GtkWidget *widget, GdkEventButton *bevent,
 	return FALSE;
 }
 
+static gboolean bar_pane_exif_copy_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+{
+	const gchar *value;
+	GtkClipboard *clipboard;
+	ExifEntry *ee;
+
+	if (bevent->button == MOUSE_BUTTON_LEFT)
+		{
+		ee = g_object_get_data(G_OBJECT(widget), "entry_data");
+		value = gtk_label_get_text(GTK_LABEL(ee->value_widget));
+		clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+		gtk_clipboard_set_text(clipboard, value, -1);
+
+		return TRUE;
+		}
+
+	return FALSE;
+}
+
 
 
 static void bar_pane_exif_entry_write_config(GtkWidget *entry, GString *outstr, gint indent)
@@ -673,6 +709,7 @@ static void bar_pane_exif_write_config(GtkWidget *pane, GString *outstr, gint in
 	write_char_option(outstr, indent, "id", ped->pane.id);
 	write_char_option(outstr, indent, "title", gtk_label_get_text(GTK_LABEL(ped->pane.title)));
 	WRITE_BOOL(ped->pane, expanded);
+	WRITE_BOOL(*ped, show_all);
 	WRITE_STRING(">");
 	indent++;
 
@@ -690,6 +727,40 @@ static void bar_pane_exif_write_config(GtkWidget *pane, GString *outstr, gint in
 	WRITE_NL(); WRITE_STRING("</pane_exif>");
 }
 
+GList * bar_pane_exif_list()
+{
+	PaneExifData *ped;
+	GList *list;
+	GList *work_windows;
+	GList *exif_list = NULL;
+	LayoutWindow *lw;
+	GtkWidget *bar;
+	GtkWidget *pane;
+	GtkWidget *entry;
+	ExifEntry *ee;
+
+	work_windows = layout_window_list;
+	lw = work_windows->data;
+	bar = lw->bar;
+	pane = bar_find_pane_by_id(bar, PANE_EXIF, "exif");
+	if (pane)
+		{
+		ped = g_object_get_data(G_OBJECT(pane), "pane_data");
+
+		list = gtk_container_get_children(GTK_CONTAINER(ped->vbox));
+		while (list)
+			{
+			entry = list->data;
+			list = list->next;
+			ee = g_object_get_data(G_OBJECT(entry), "entry_data");
+			exif_list = g_list_append(exif_list, g_strdup(ee->title));
+			exif_list = g_list_append(exif_list, g_strdup(ee->key));
+			}
+
+		g_list_free(list);
+		}
+	return exif_list;
+}
 
 void bar_pane_exif_close(GtkWidget *widget)
 {
@@ -732,7 +803,7 @@ static void bar_pane_exif_size_allocate(GtkWidget *pane, GtkAllocation *alloc, g
 #endif
 }
 
-static GtkWidget *bar_pane_exif_new(const gchar *id, const gchar *title, gboolean expanded)
+static GtkWidget *bar_pane_exif_new(const gchar *id, const gchar *title, gboolean expanded, gboolean show_all)
 {
 	PaneExifData *ped;
 
@@ -745,6 +816,7 @@ static GtkWidget *bar_pane_exif_new(const gchar *id, const gchar *title, gboolea
 	ped->pane.id = g_strdup(id);
 	ped->pane.expanded = expanded;
 	ped->pane.type = PANE_EXIF;
+	ped->show_all = show_all;
 
 	ped->size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
 	ped->widget = gtk_event_box_new();
@@ -780,6 +852,7 @@ GtkWidget *bar_pane_exif_new_from_config(const gchar **attribute_names, const gc
 	gchar *title = NULL;
 	gchar *id = g_strdup("exif");
 	gboolean expanded = TRUE;
+	gboolean show_all = FALSE;
 	GtkWidget *ret;
 
 	while (*attribute_names)
@@ -790,12 +863,13 @@ GtkWidget *bar_pane_exif_new_from_config(const gchar **attribute_names, const gc
 		if (READ_CHAR_FULL("id", id)) continue;
 		if (READ_CHAR_FULL("title", title)) continue;
 		if (READ_BOOL_FULL("expanded", expanded)) continue;
+		if (READ_BOOL_FULL("show_all", show_all)) continue;
 
 		log_printf("unknown attribute %s = %s\n", option, value);
 		}
 
 	bar_pane_translate_title(PANE_EXIF, id, &title);
-	ret = bar_pane_exif_new(id, title, expanded);
+	ret = bar_pane_exif_new(id, title, expanded, show_all);
 	g_free(title);
 	g_free(id);
 	return ret;
@@ -816,6 +890,7 @@ void bar_pane_exif_update_from_config(GtkWidget *pane, const gchar **attribute_n
 
 		if (READ_CHAR_FULL("title", title)) continue;
 		if (READ_BOOL_FULL("expanded", ped->pane.expanded)) continue;
+		if (READ_BOOL_FULL("show_all", ped->show_all)) continue;
 		if (READ_CHAR_FULL("id", ped->pane.id)) continue;
 
 
