@@ -37,11 +37,17 @@
 #include "rcfile.h"
 #include "layout.h"
 #include "dnd.h"
+#include "secure_save.h"
 
 
 //static void bar_pane_keywords_keyword_update_all(void);
 static void bar_pane_keywords_changed(GtkTextBuffer *buffer, gpointer data);
 
+static void autocomplete_keywords_list_load(const gchar *path);
+static GtkListStore *keyword_store = NULL;
+static gboolean autocomplete_keywords_list_save(gchar *path);
+static gboolean autocomplete_activate_cb(GtkWidget *widget, gpointer data);
+
 /*
  *-------------------------------------------------------------------
  * keyword / comment utils
@@ -62,6 +68,19 @@ GList *keyword_list_pull(GtkWidget *text_widget)
 	return list;
 }
 
+static GList *keyword_list_pull_selected(GtkWidget *text_widget)
+{
+	GList *list;
+	gchar *text;
+
+	text = text_widget_text_pull_selected(text_widget);
+	list = string_to_keywords_list(text);
+
+	g_free(text);
+
+	return list;
+}
+
 /* the "changed" signal should be blocked before calling this */
 static void keyword_list_push(GtkWidget *textview, GList *list)
 {
@@ -123,6 +142,10 @@ struct _PaneKeywordsData
 	FileData *fd;
 	gchar *key;
 	gint height;
+
+	GList *expanded_rows;
+
+	GtkWidget *autocomplete;
 };
 
 typedef struct _ConfDialogData ConfDialogData;
@@ -247,20 +270,69 @@ void bar_pane_keywords_set_fd(GtkWidget *pane, FileData *fd)
 	bar_pane_keywords_update(pkd);
 }
 
+void bar_keyword_tree_get_expanded_cb(GtkTreeView *keyword_treeview, GtkTreePath *path,  gpointer data)
+{
+	GList **expanded = data;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gchar *path_string;
+
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(keyword_treeview));
+	gtk_tree_model_get_iter(model, &iter, path);
+
+	path_string = gtk_tree_model_get_string_from_iter(model, &iter);
+
+	*expanded = g_list_append(*expanded, g_strdup(path_string));
+	g_free(path_string);
+}
+
+static void bar_pane_keywords_entry_write_config(gchar *entry, GString *outstr, gint indent)
+{
+	struct {
+		gchar *path;
+	} expand;
+
+	expand.path = entry;
+
+	WRITE_NL(); WRITE_STRING("<expanded ");
+	WRITE_CHAR(expand, path);
+	WRITE_STRING("/>");
+}
+
 static void bar_pane_keywords_write_config(GtkWidget *pane, GString *outstr, gint indent)
 {
 	PaneKeywordsData *pkd;
+	GList *path_expanded = NULL;
 
 	pkd = g_object_get_data(G_OBJECT(pane), "pane_data");
 	if (!pkd) return;
 
+	pkd->height = options->info_keywords.height;
+
 	WRITE_NL(); WRITE_STRING("<pane_keywords ");
 	write_char_option(outstr, indent, "id", pkd->pane.id);
 	write_char_option(outstr, indent, "title", gtk_label_get_text(GTK_LABEL(pkd->pane.title)));
 	WRITE_BOOL(pkd->pane, expanded);
 	WRITE_CHAR(*pkd, key);
 	WRITE_INT(*pkd, height);
-	WRITE_STRING("/>");
+	WRITE_STRING(">");
+	indent++;
+
+	gtk_tree_view_map_expanded_rows(GTK_TREE_VIEW(pkd->keyword_treeview),
+								(bar_keyword_tree_get_expanded_cb), &path_expanded);
+
+	g_list_first(path_expanded);
+	while (path_expanded)
+		{
+		bar_pane_keywords_entry_write_config(path_expanded->data, outstr, indent);
+		g_free(path_expanded->data);
+		path_expanded = path_expanded->next;
+		}
+	g_list_free(path_expanded);
+
+	indent--;
+	WRITE_NL();
+	WRITE_STRING("</pane_keywords>");
 }
 
 gint bar_pane_keywords_event(GtkWidget *bar, GdkEvent *event)
@@ -272,6 +344,10 @@ gint bar_pane_keywords_event(GtkWidget *bar, GdkEvent *event)
 
 	if (gtk_widget_has_focus(pkd->keyword_view)) return gtk_widget_event(pkd->keyword_view, event);
 
+	if (gtk_widget_has_focus(pkd->autocomplete))
+		{
+		return gtk_widget_event(pkd->autocomplete, event);
+		}
 	return FALSE;
 }
 
@@ -365,7 +441,7 @@ static void bar_pane_keywords_set_selection(PaneKeywordsData *pkd, gboolean appe
 	GList *list = NULL;
 	GList *work;
 
-	keywords = keyword_list_pull(pkd->keyword_view);
+	keywords = keyword_list_pull_selected(pkd->keyword_view);
 
 	list = layout_selection_list(pkd->pane.lw);
 	list = file_data_process_groups_in_selection(list, FALSE, NULL);
@@ -409,8 +485,8 @@ static void bar_pane_keywords_populate_popup_cb(GtkTextView *textview, GtkMenu *
 	PaneKeywordsData *pkd = data;
 
 	menu_item_add_divider(GTK_WIDGET(menu));
-	menu_item_add_stock(GTK_WIDGET(menu), _("Add keywords to selected files"), GTK_STOCK_ADD, G_CALLBACK(bar_pane_keywords_sel_add_cb), pkd);
-	menu_item_add_stock(GTK_WIDGET(menu), _("Replace existing keywords in selected files"), GTK_STOCK_CONVERT, G_CALLBACK(bar_pane_keywords_sel_replace_cb), pkd);
+	menu_item_add_stock(GTK_WIDGET(menu), _("Add selected keywords to selected files"), GTK_STOCK_ADD, G_CALLBACK(bar_pane_keywords_sel_add_cb), pkd);
+	menu_item_add_stock(GTK_WIDGET(menu), _("Replace existing keywords in selected files with selected keywords"), GTK_STOCK_CONVERT, G_CALLBACK(bar_pane_keywords_sel_replace_cb), pkd);
 }
 
 
@@ -881,14 +957,14 @@ static void bar_pane_keywords_edit_dialog(PaneKeywordsData *pkd, gboolean edit_e
 	pkd->click_tpath = NULL;
 	cdd->edit_existing = edit_existing;
 
-	cdd->gd = gd = generic_dialog_new(name ? _("Edit keyword") : _("Add keywords"), "keyword_edit",
+	cdd->gd = gd = generic_dialog_new(name ? _("Edit keyword") : _("New keyword"), "keyword_edit",
 				pkd->widget, TRUE,
 				bar_pane_keywords_edit_cancel_cb, cdd);
 	g_signal_connect(G_OBJECT(gd->dialog), "destroy",
 			 G_CALLBACK(bar_pane_keywords_edit_destroy_cb), cdd);
 
 
-	generic_dialog_add_message(gd, NULL, name ? _("Configure keyword") : _("Add keyword"), NULL);
+	generic_dialog_add_message(gd, NULL, name ? _("Configure keyword") : _("New keyword"), NULL, FALSE);
 
 	generic_dialog_add_button(gd, GTK_STOCK_OK, NULL,
 				  bar_pane_keywords_edit_ok_cb, TRUE);
@@ -965,6 +1041,35 @@ static void bar_pane_keywords_connect_mark_cb(GtkWidget *menu_widget, gpointer d
 	meta_data_connect_mark_with_keyword(keyword_tree, &kw_iter, mark);
 }
 
+static void bar_pane_keywords_disconnect_marks_ok_cb(GenericDialog *gd, gpointer data)
+{
+	keyword_tree_disconnect_marks();
+}
+
+static void dummy_cancel_cb(GenericDialog *gd, gpointer data)
+{
+	/* no op, only so cancel button appears */
+}
+
+static void bar_pane_keywords_disconnect_marks_cb(GtkWidget *menu_widget, gpointer data)
+{
+	PaneKeywordsData *pkd = data;
+
+	GenericDialog *gd;
+	GString *message = g_string_new("");
+
+	message = g_string_append(message, _("This will disconnect all Marks Keywords connections"));
+
+	gd = generic_dialog_new(_("Marks Keywords"),
+				"marks_keywords", menu_widget, TRUE, dummy_cancel_cb, pkd);
+	generic_dialog_add_message(gd, GTK_STOCK_DIALOG_WARNING,
+				"Disconnect all Marks Keywords connections?", message->str, TRUE);
+	generic_dialog_add_button(gd, GTK_STOCK_OK, NULL, bar_pane_keywords_disconnect_marks_ok_cb, TRUE);
+
+	gtk_widget_show(gd->dialog);
+
+	g_string_free(message, TRUE);
+}
 
 static void bar_pane_keywords_delete_cb(GtkWidget *menu_widget, gpointer data)
 {
@@ -1013,6 +1118,11 @@ static void bar_pane_keywords_show_all_cb(GtkWidget *menu_widget, gpointer data)
 
 	GtkTreeModel *keyword_tree;
 
+	string_list_free(pkd->expanded_rows);
+	pkd->expanded_rows = NULL;
+	gtk_tree_view_map_expanded_rows(GTK_TREE_VIEW(pkd->keyword_treeview),
+								(bar_keyword_tree_get_expanded_cb), &pkd->expanded_rows);
+
 	pkd->hide_unchecked = FALSE;
 
 	model = gtk_tree_view_get_model(GTK_TREE_VIEW(pkd->keyword_treeview));
@@ -1024,6 +1134,28 @@ static void bar_pane_keywords_show_all_cb(GtkWidget *menu_widget, gpointer data)
 	bar_keyword_tree_sync(pkd);
 }
 
+static void bar_pane_keywords_revert_cb(GtkWidget *menu_widget, gpointer data)
+{
+	PaneKeywordsData *pkd = data;
+	GList *work;
+	GtkTreePath *tree_path;
+	gchar *path;
+
+	gtk_tree_view_collapse_all(GTK_TREE_VIEW(pkd->keyword_treeview));
+
+	work = pkd->expanded_rows;
+	while (work)
+		{
+		path = work->data;
+		tree_path = gtk_tree_path_new_from_string(path);
+		gtk_tree_view_expand_to_path(GTK_TREE_VIEW(pkd->keyword_treeview), tree_path);
+		work = work->next;
+		gtk_tree_path_free(tree_path);
+		}
+
+	bar_keyword_tree_sync(pkd);
+}
+
 static void bar_pane_keywords_expand_checked_cb(GtkWidget *menu_widget, gpointer data)
 {
 	PaneKeywordsData *pkd = data;
@@ -1033,6 +1165,34 @@ static void bar_pane_keywords_expand_checked_cb(GtkWidget *menu_widget, gpointer
 	gtk_tree_model_foreach(model, bar_keyword_tree_expand_if_set_cb, pkd);
 }
 
+static void bar_pane_keywords_collapse_all_cb(GtkWidget *menu_widget, gpointer data)
+{
+	PaneKeywordsData *pkd = data;
+
+	string_list_free(pkd->expanded_rows);
+	pkd->expanded_rows = NULL;
+	gtk_tree_view_map_expanded_rows(GTK_TREE_VIEW(pkd->keyword_treeview),
+								(bar_keyword_tree_get_expanded_cb), &pkd->expanded_rows);
+
+	gtk_tree_view_collapse_all(GTK_TREE_VIEW(pkd->keyword_treeview));
+
+	bar_keyword_tree_sync(pkd);
+}
+
+static void bar_pane_keywords_revert_hidden_cb(GtkWidget *menu_widget, gpointer data)
+{
+	PaneKeywordsData *pkd = data;
+	GtkTreeModel *model;
+	GtkTreeModel *keyword_tree;
+
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(pkd->keyword_treeview));
+	keyword_tree = gtk_tree_model_filter_get_model(GTK_TREE_MODEL_FILTER(model));
+
+	keyword_revert_hidden_in(GTK_TREE_STORE(keyword_tree), model);
+
+	bar_keyword_tree_sync(pkd);
+}
+
 static void bar_pane_keywords_collapse_unchecked_cb(GtkWidget *menu_widget, gpointer data)
 {
 	PaneKeywordsData *pkd = data;
@@ -1149,11 +1309,7 @@ static void bar_pane_keywords_menu_popup(GtkWidget *widget, PaneKeywordsData *pk
 
 	menu = popup_menu_short_lived();
 
-	menu_item_add_stock(menu, _("Add keyword"), GTK_STOCK_EDIT, G_CALLBACK(bar_pane_keywords_add_dialog_cb), pkd);
-
-	menu_item_add_divider(menu);
-
-	menu_item_add(menu, _("Add keyword to all selected images"), G_CALLBACK(bar_pane_keywords_add_to_selected_cb), pkd);
+	menu_item_add_stock(menu, _("New keyword"), GTK_STOCK_NEW, G_CALLBACK(bar_pane_keywords_add_dialog_cb), pkd);
 
 	menu_item_add_divider(menu);
 
@@ -1163,6 +1319,7 @@ static void bar_pane_keywords_menu_popup(GtkWidget *widget, PaneKeywordsData *pk
 		gchar *text;
 		gchar *mark;
 		gint i;
+		gboolean keyword;
 
 		GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(pkd->keyword_treeview));
 
@@ -1171,25 +1328,37 @@ static void bar_pane_keywords_menu_popup(GtkWidget *widget, PaneKeywordsData *pk
 		gchar *name;
 
 		gtk_tree_model_get(model, &iter, FILTER_KEYWORD_COLUMN_NAME, &name,
-						 FILTER_KEYWORD_COLUMN_MARK, &mark, -1);
+						FILTER_KEYWORD_COLUMN_MARK, &mark,
+						FILTER_KEYWORD_COLUMN_IS_KEYWORD, &keyword, -1);
+
+		if (keyword)
+			{
+			text = g_strdup_printf(_("Add \"%s\" to all selected images"), name);
+			menu_item_add_stock(menu, text, GTK_STOCK_ADD, G_CALLBACK(bar_pane_keywords_add_to_selected_cb), pkd);
+			g_free(text);
+			}
+		menu_item_add_divider(menu);
 
 		text = g_strdup_printf(_("Hide \"%s\""), name);
-		menu_item_add_stock(menu, text, GTK_STOCK_EDIT, G_CALLBACK(bar_pane_keywords_hide_cb), pkd);
+		menu_item_add(menu, text, G_CALLBACK(bar_pane_keywords_hide_cb), pkd);
 		g_free(text);
 
 		submenu = gtk_menu_new();
 		for (i = 0; i < FILEDATA_MARKS_SIZE; i++)
 			{
-			text = g_strdup_printf(_("Mark %d"), i + 1);
+			text = g_strdup_printf(_("Mark %d"), 1 + (i < 9 ? i : -1));
 			item = menu_item_add(submenu, text, G_CALLBACK(bar_pane_keywords_connect_mark_cb), pkd);
 			g_object_set_data(G_OBJECT(item), "mark", GINT_TO_POINTER(i + 1));
 			g_free(text);
 			}
-		text = g_strdup_printf(_("Connect \"%s\" to mark"), name);
-		item = menu_item_add(menu, text, NULL, NULL);
-		gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
-		g_free(text);
 
+		if (keyword)
+			{
+			text = g_strdup_printf(_("Connect \"%s\" to mark"), name);
+			item = menu_item_add(menu, text, NULL, NULL);
+			gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
+			g_free(text);
+			}
 		menu_item_add_divider(menu);
 
 		text = g_strdup_printf(_("Edit \"%s\""), name);
@@ -1207,6 +1376,12 @@ static void bar_pane_keywords_menu_popup(GtkWidget *widget, PaneKeywordsData *pk
 			g_free(text);
 			}
 
+		if (keyword)
+			{
+			text = g_strdup_printf(_("Disconnect all Mark Keyword connections"));
+			menu_item_add_stock(menu, text, GTK_STOCK_DELETE, G_CALLBACK(bar_pane_keywords_disconnect_marks_cb), pkd);
+			g_free(text);
+			}
 		menu_item_add_divider(menu);
 		g_free(mark);
 		g_free(name);
@@ -1217,7 +1392,12 @@ static void bar_pane_keywords_menu_popup(GtkWidget *widget, PaneKeywordsData *pk
 	menu_item_add(menu, _("Expand checked"), G_CALLBACK(bar_pane_keywords_expand_checked_cb), pkd);
 	menu_item_add(menu, _("Collapse unchecked"), G_CALLBACK(bar_pane_keywords_collapse_unchecked_cb), pkd);
 	menu_item_add(menu, _("Hide unchecked"), G_CALLBACK(bar_pane_keywords_hide_unchecked_cb), pkd);
+	menu_item_add(menu, _("Revert all hidden"), G_CALLBACK(bar_pane_keywords_revert_hidden_cb), pkd);
+	menu_item_add_divider(menu);
 	menu_item_add(menu, _("Show all"), G_CALLBACK(bar_pane_keywords_show_all_cb), pkd);
+	menu_item_add(menu, _("Collapse all"), G_CALLBACK(bar_pane_keywords_collapse_all_cb), pkd);
+	menu_item_add(menu, _("Revert"), G_CALLBACK(bar_pane_keywords_revert_cb), pkd);
+	menu_item_add_divider(menu);
 
 	submenu = gtk_menu_new();
 	item = menu_item_add(menu, _("On any change"), NULL, NULL);
@@ -1262,8 +1442,13 @@ void bar_pane_keywords_close(GtkWidget *bar)
 static void bar_pane_keywords_destroy(GtkWidget *widget, gpointer data)
 {
 	PaneKeywordsData *pkd = data;
+	gchar *path;
 
-        if (pkd->click_tpath) gtk_tree_path_free(pkd->click_tpath);
+	path = g_build_filename(get_rc_dir(), "keywords", NULL);
+	autocomplete_keywords_list_save(path);
+
+	string_list_free(pkd->expanded_rows);
+	if (pkd->click_tpath) gtk_tree_path_free(pkd->click_tpath);
 	if (pkd->idle_id) g_source_remove(pkd->idle_id);
 	file_data_unregister_notify_func(bar_pane_keywords_notify_cb, pkd);
 
@@ -1277,13 +1462,15 @@ static void bar_pane_keywords_destroy(GtkWidget *widget, gpointer data)
 static GtkWidget *bar_pane_keywords_new(const gchar *id, const gchar *title, const gchar *key, gboolean expanded, gint height)
 {
 	PaneKeywordsData *pkd;
-	GtkWidget *hbox;
+	GtkWidget *hbox, *vbox;
 	GtkWidget *scrolled;
 	GtkTextBuffer *buffer;
 	GtkTreeModel *store;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
 	GtkTreeIter iter;
+	GtkEntryCompletion *completion;
+	gchar *path;
 
 	pkd = g_new0(PaneKeywordsData, 1);
 
@@ -1300,10 +1487,13 @@ static GtkWidget *bar_pane_keywords_new(const gchar *id, const gchar *title, con
 	pkd->key = g_strdup(key);
 
 	pkd->expand_checked = TRUE;
+	pkd->expanded_rows = NULL;
 
+	vbox = gtk_vbox_new(FALSE, PREF_PAD_GAP);
 	hbox = gtk_hbox_new(FALSE, PREF_PAD_GAP);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 
-	pkd->widget = hbox;
+	pkd->widget = vbox;
 	g_object_set_data(G_OBJECT(pkd->widget), "pane_data", pkd);
 	g_signal_connect(G_OBJECT(pkd->widget), "destroy",
 			 G_CALLBACK(bar_pane_keywords_destroy), pkd);
@@ -1327,13 +1517,36 @@ static GtkWidget *bar_pane_keywords_new(const gchar *id, const gchar *title, con
 	g_signal_connect(G_OBJECT(buffer), "changed",
 			 G_CALLBACK(bar_pane_keywords_changed), pkd);
 
-	scrolled = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled), GTK_SHADOW_IN);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
-				       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_box_pack_start(GTK_BOX(hbox), scrolled, TRUE, TRUE, 0);
-	gtk_widget_show(scrolled);
+	if (options->show_predefined_keyword_tree)
+		{
+		scrolled = gtk_scrolled_window_new(NULL, NULL);
+		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled), GTK_SHADOW_IN);
+		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
+						GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+		gtk_box_pack_start(GTK_BOX(hbox), scrolled, TRUE, TRUE, 0);
+		gtk_widget_show(scrolled);
+		}
 
+	pkd->autocomplete = gtk_entry_new();
+	gtk_box_pack_end(GTK_BOX(vbox), pkd->autocomplete, FALSE, FALSE, 0);
+	gtk_widget_show(pkd->autocomplete);
+	gtk_widget_show(vbox);
+	gtk_widget_set_tooltip_text(pkd->autocomplete, "Keyword autocomplete");
+
+	path = g_build_filename(get_rc_dir(), "keywords", NULL);
+	autocomplete_keywords_list_load(path);
+
+	completion = gtk_entry_completion_new();
+	gtk_entry_set_completion(GTK_ENTRY(pkd->autocomplete), completion);
+	gtk_entry_completion_set_inline_completion(completion, TRUE);
+	gtk_entry_completion_set_inline_selection(completion, TRUE);
+	g_object_unref(completion);
+
+	gtk_entry_completion_set_model(completion, GTK_TREE_MODEL(keyword_store));
+	gtk_entry_completion_set_text_column(completion, 0);
+
+	g_signal_connect(G_OBJECT(pkd->autocomplete), "activate",
+			 G_CALLBACK(autocomplete_activate_cb), pkd);
 
 	if (!keyword_tree || !gtk_tree_model_get_iter_first(GTK_TREE_MODEL(keyword_tree), &iter))
 		{
@@ -1416,8 +1629,11 @@ static GtkWidget *bar_pane_keywords_new(const gchar *id, const gchar *title, con
 	g_signal_connect(G_OBJECT(pkd->keyword_treeview), "button_release_event",
 			 G_CALLBACK(bar_pane_keywords_menu_cb), pkd);
 
-	gtk_container_add(GTK_CONTAINER(scrolled), pkd->keyword_treeview);
-	gtk_widget_show(pkd->keyword_treeview);
+	if (options->show_predefined_keyword_tree)
+		{
+		gtk_container_add(GTK_CONTAINER(scrolled), pkd->keyword_treeview);
+		gtk_widget_show(pkd->keyword_treeview);
+		}
 
 	file_data_register_notify_func(bar_pane_keywords_notify_cb, pkd, NOTIFY_PRIORITY_LOW);
 
@@ -1448,6 +1664,7 @@ GtkWidget *bar_pane_keywords_new_from_config(const gchar **attribute_names, cons
 		log_printf("unknown attribute %s = %s\n", option, value);
 		}
 
+	options->info_keywords.height = height;
 	bar_pane_translate_title(PANE_KEYWORDS, id, &title);
 	ret = bar_pane_keywords_new(id, title, key, expanded, height);
 	g_free(id);
@@ -1491,4 +1708,283 @@ void bar_pane_keywords_update_from_config(GtkWidget *pane, const gchar **attribu
 }
 
 
+void bar_pane_keywords_entry_add_from_config(GtkWidget *pane, const gchar **attribute_names, const gchar **attribute_values)
+{
+	PaneKeywordsData *pkd;
+	gchar *path = NULL;
+	GtkTreePath *tree_path;
+
+	pkd = g_object_get_data(G_OBJECT(pane), "pane_data");
+	if (!pkd) return;
+
+	while (*attribute_names)
+		{
+		const gchar *option = *attribute_names++;
+		const gchar *value = *attribute_values++;
+
+		if (READ_CHAR_FULL("path", path))
+			{
+			tree_path = gtk_tree_path_new_from_string(path);
+			gtk_tree_view_expand_to_path(GTK_TREE_VIEW(pkd->keyword_treeview), tree_path);
+			gtk_tree_path_free(tree_path);
+			pkd->expanded_rows = g_list_append(pkd->expanded_rows, g_strdup(path));
+			continue;
+			}
+		log_printf("unknown attribute %s = %s\n", option, value);
+		}
+}
+
+/*
+ *-----------------------------------------------------------------------------
+ * Autocomplete keywords
+ *-----------------------------------------------------------------------------
+ */
+
+static gboolean autocomplete_activate_cb(GtkWidget *widget, gpointer data)
+{
+	PaneKeywordsData *pkd = data;
+	gchar *entry_text;
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
+	GtkTreeIter iter_t;
+	gchar *kw_cr;
+	gchar *kw_split;
+	gboolean valid;
+	gboolean found = FALSE;
+	gchar *string;
+
+	entry_text = g_strdup(gtk_entry_get_text(GTK_ENTRY(pkd->autocomplete)));
+	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(pkd->keyword_view));
+
+	kw_split = strtok(entry_text, ",");
+	while (kw_split != NULL)
+		{
+		kw_cr = g_strconcat(kw_split, "\n", NULL);
+		g_strchug(kw_cr);
+		gtk_text_buffer_get_end_iter(buffer, &iter);
+		gtk_text_buffer_insert(buffer, &iter, kw_cr, -1);
+
+		kw_split = strtok(NULL, ",");
+		g_free(kw_cr);
+		}
+
+	g_free(entry_text);
+	entry_text = g_strdup(gtk_entry_get_text(GTK_ENTRY(pkd->autocomplete)));
+
+	gtk_entry_set_text(GTK_ENTRY(pkd->autocomplete), "");
+
+	valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(keyword_store), &iter_t);
+	while (valid)
+		{
+		gtk_tree_model_get (GTK_TREE_MODEL(keyword_store), &iter_t, 0, &string, -1);
+		if (g_strcmp0(entry_text, string) == 0)
+			{
+			found = TRUE;
+			break;
+			}
+		valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(keyword_store), &iter_t);
+		}
+
+	if (!found)
+		{
+		gtk_list_store_append (keyword_store, &iter_t);
+		gtk_list_store_set(keyword_store, &iter_t, 0, entry_text, -1);
+		}
+
+	g_free(entry_text);
+	return FALSE;
+}
+
+gint autocomplete_sort_iter_compare_func (GtkTreeModel *model,
+									GtkTreeIter *a,
+									GtkTreeIter *b,
+									gpointer userdata)
+{
+	gint ret = 0;
+	gchar *name1, *name2;
+
+	gtk_tree_model_get(model, a, 0, &name1, -1);
+	gtk_tree_model_get(model, b, 0, &name2, -1);
+
+	if (name1 == NULL || name2 == NULL)
+		{
+		if (name1 == NULL && name2 == NULL)
+			{
+			ret = 0;
+			}
+		else
+			{
+			ret = (name1 == NULL) ? -1 : 1;
+			}
+		}
+	else
+		{
+		ret = g_utf8_collate(name1,name2);
+		}
+
+	g_free(name1);
+	g_free(name2);
+
+	return ret;
+}
+
+static void autocomplete_keywords_list_load(const gchar *path)
+{
+	FILE *f;
+	gchar s_buf[1024];
+	gchar *pathl;
+	gint len;
+	GtkTreeIter iter;
+	GtkTreeSortable *sortable;
+
+	if (keyword_store) return;
+	keyword_store = gtk_list_store_new(1, G_TYPE_STRING);
+
+	sortable = GTK_TREE_SORTABLE(keyword_store);
+	gtk_tree_sortable_set_sort_func(sortable, 0, autocomplete_sort_iter_compare_func,
+												GINT_TO_POINTER(0), NULL);
+
+	gtk_tree_sortable_set_sort_column_id(sortable, 0, GTK_SORT_ASCENDING);
+
+	pathl = path_from_utf8(path);
+	f = fopen(pathl, "r");
+	g_free(pathl);
+
+	if (!f)
+		{
+		log_printf("Warning: keywords file %s not loaded", pathl);
+		return;
+		}
+
+	/* first line must start with Keywords comment */
+	if (!fgets(s_buf, sizeof(s_buf), f) ||
+					strncmp(s_buf, "#Keywords", 9) != 0)
+		{
+		fclose(f);
+		log_printf("Warning: keywords file %s not loaded", pathl);
+		return;
+		}
+
+	while (fgets(s_buf, sizeof(s_buf), f))
+		{
+		if (s_buf[0]=='#') continue;
+
+		len = strlen(s_buf);
+		if( s_buf[len-1] == '\n' )
+			{
+			s_buf[len-1] = 0;
+			}
+		gtk_list_store_append (keyword_store, &iter);
+		gtk_list_store_set(keyword_store, &iter, 0, g_strdup(s_buf), -1);
+		}
+
+	fclose(f);
+}
+
+static gboolean autocomplete_keywords_list_save(gchar *path)
+{
+	SecureSaveInfo *ssi;
+	gchar *pathl;
+	gchar *string;
+	gchar *string_nl;
+	GtkTreeIter  iter;
+	gboolean     valid;
+
+	pathl = path_from_utf8(path);
+	ssi = secure_open(pathl);
+	g_free(pathl);
+
+	if (!ssi)
+		{
+		log_printf(_("Error: Unable to write keywords list to: %s\n"), path);
+		return FALSE;
+		}
+
+	secure_fprintf(ssi, "#Keywords list\n");
+
+	valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(keyword_store), &iter);
+
+	while (valid)
+		{
+		gtk_tree_model_get (GTK_TREE_MODEL(keyword_store), &iter, 0, &string, -1);
+		string_nl = g_strconcat(string, "\n", NULL);
+		secure_fprintf(ssi, "%s", string_nl);
+
+		valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(keyword_store), &iter);
+
+		g_free(string_nl);
+		}
+
+	secure_fprintf(ssi, "#end\n");
+	return (secure_close(ssi) == 0);
+}
+
+GList *keyword_list_get()
+{
+	GList *ret_list = NULL;
+	gchar *string;
+	gchar *string_nl;
+	GtkTreeIter iter;
+	gboolean valid;
+
+	valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(keyword_store), &iter);
+
+	while (valid)
+		{
+		gtk_tree_model_get (GTK_TREE_MODEL(keyword_store), &iter, 0, &string, -1);
+		string_nl = g_strconcat(string, "\n", NULL);
+		ret_list = g_list_append(ret_list, string);
+		valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(keyword_store), &iter);
+
+		g_free(string_nl);
+		}
+
+	return ret_list;
+}
+
+void keyword_list_set(GList *keyword_list)
+{
+	GList *ret = NULL;
+	GtkTreeIter  iter;
+
+	if (!keyword_list) return;
+
+	gtk_list_store_clear(keyword_store);
+
+	while (keyword_list)
+		{
+		gtk_list_store_append (keyword_store, &iter);
+		gtk_list_store_set(keyword_store, &iter, 0, keyword_list->data, -1);
+
+		keyword_list = keyword_list->next;
+		}
+}
+
+gboolean bar_keywords_autocomplete_focus(LayoutWindow *lw)
+{
+	GtkWidget *pane;
+	GtkWidget *current_focus;
+	GList *children;
+	gboolean ret;
+
+	current_focus = gtk_window_get_focus(GTK_WINDOW(lw->window));
+	pane = bar_find_pane_by_id(lw->bar, PANE_KEYWORDS, "keywords");
+
+	children = gtk_container_get_children(GTK_CONTAINER(pane));
+	children = g_list_last(children);
+
+	if (current_focus == children->data)
+		{
+		ret = TRUE;
+		}
+	else
+		{
+		gtk_widget_grab_focus(children->data);
+		ret = FALSE;
+		}
+
+	g_list_free(children);
+
+	return ret;
+}
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
