@@ -32,6 +32,7 @@
 #include "ui_spinner.h"
 #include "ui_tabcomp.h"
 #include "ui_utildlg.h"
+#include "window.h"
 
 
 typedef struct _CMData CMData;
@@ -47,6 +48,7 @@ struct _CMData
 	GtkWidget *button_close;
 	gboolean clear;
 	gboolean metadata;
+	gboolean remote;
 };
 
 #define PURGE_DIALOG_WIDTH 400
@@ -116,11 +118,14 @@ static void cache_maintain_home_stop(CMData *cm)
 		cm->idle_id = 0;
 		}
 
-	gtk_entry_set_text(GTK_ENTRY(cm->entry), _("done"));
-	spinner_set_interval(cm->spinner, -1);
+	if (!cm->remote)
+		{
+		gtk_entry_set_text(GTK_ENTRY(cm->entry), _("done"));
+		spinner_set_interval(cm->spinner, -1);
 
-	gtk_widget_set_sensitive(cm->button_stop, FALSE);
-	gtk_widget_set_sensitive(cm->button_close, TRUE);
+		gtk_widget_set_sensitive(cm->button_stop, FALSE);
+		gtk_widget_set_sensitive(cm->button_close, TRUE);
+		}
 }
 
 static gboolean cache_maintain_home_cb(gpointer data)
@@ -133,6 +138,7 @@ static gboolean cache_maintain_home_cb(gpointer data)
 	gboolean still_have_a_file = TRUE;
 	gsize base_length;
 	const gchar *cache_folder;
+	gboolean filter_disable;
 
 	if (cm->metadata)
 		{
@@ -157,6 +163,13 @@ static gboolean cache_maintain_home_cb(gpointer data)
 
 	DEBUG_1("purge chk (%d) \"%s\"", (cm->clear && !cm->metadata), fd->path);
 
+/**
+ * It is necessary to disable the file filter when clearing the cache,
+ * otherwise the .sim (file similarity) files are not deleted.
+ */
+	filter_disable = options->file_filter.disable;
+	options->file_filter.disable = TRUE;
+
 	if (g_list_find(cm->done_list, fd) == NULL)
 		{
 		cm->done_list = g_list_prepend(cm->done_list, fd);
@@ -193,6 +206,8 @@ static gboolean cache_maintain_home_cb(gpointer data)
 				}
 			}
 		}
+	options->file_filter.disable = filter_disable;
+
 	filelist_free(list);
 
 	cm->list = g_list_concat(dlist, cm->list);
@@ -223,7 +238,7 @@ static gboolean cache_maintain_home_cb(gpointer data)
 		file_data_unref(fd);
 		}
 
-	if (cm->list)
+	if (cm->list && !cm->remote)
 		{
 		const gchar *buf;
 
@@ -291,6 +306,7 @@ void cache_maintain_home(gboolean metadata, gboolean clear, GtkWidget *parent)
 	cm->done_list = NULL;
 	cm->clear = clear;
 	cm->metadata = metadata;
+	cm->remote = FALSE;
 
 	if (metadata)
 		{
@@ -316,7 +332,7 @@ void cache_maintain_home(gboolean metadata, gboolean clear, GtkWidget *parent)
 	cm->button_stop = generic_dialog_add_button(cm->gd, GTK_STOCK_STOP, NULL,
 						    cache_maintain_home_stop_cb, FALSE);
 
-	generic_dialog_add_message(cm->gd, NULL, msg, NULL);
+	generic_dialog_add_message(cm->gd, NULL, msg, NULL, FALSE);
 	gtk_window_set_default_size(GTK_WINDOW(cm->gd->dialog), PURGE_DIALOG_WIDTH, -1);
 
 	hbox = gtk_hbox_new(FALSE, 0);
@@ -338,6 +354,41 @@ void cache_maintain_home(gboolean metadata, gboolean clear, GtkWidget *parent)
 	cm->idle_id = g_idle_add(cache_maintain_home_cb, cm);
 }
 
+void cache_maintain_home_remote(gboolean metadata, gboolean clear)
+{
+	CMData *cm;
+	GList *dlist;
+	FileData *dir_fd;
+	const gchar *cache_folder;
+
+	if (metadata)
+		{
+		cache_folder = get_metadata_cache_dir();
+		}
+	else
+		{
+		cache_folder = get_thumbnails_cache_dir();
+		}
+
+	dir_fd = file_data_new_dir(cache_folder);
+	if (!filelist_read(dir_fd, NULL, &dlist))
+		{
+		file_data_unref(dir_fd);
+		return;
+		}
+
+	dlist = g_list_append(dlist, dir_fd);
+
+	cm = g_new0(CMData, 1);
+	cm->list = dlist;
+	cm->done_list = NULL;
+	cm->clear = clear;
+	cm->metadata = metadata;
+	cm->remote = TRUE;
+
+	cm->idle_id = g_idle_add(cache_maintain_home_cb, cm);
+}
+
 static void cache_file_move(const gchar *src, const gchar *dest)
 {
 	if (!dest || !src || !isfile(src)) return;
@@ -525,6 +576,8 @@ struct _CleanData
 	gboolean local;
 	gboolean recurse;
 
+	gboolean remote;
+
 	guint idle_id; /* event source id */
 };
 
@@ -554,14 +607,16 @@ static void cache_manager_render_close_cb(GenericDialog *fd, gpointer data)
 static void cache_manager_render_finish(CleanData *cd)
 {
 	cache_manager_render_reset(cd);
+	if (!cd->remote)
+		{
+		gtk_entry_set_text(GTK_ENTRY(cd->progress), _("done"));
+		spinner_set_interval(cd->spinner, -1);
 
-	gtk_entry_set_text(GTK_ENTRY(cd->progress), _("done"));
-	spinner_set_interval(cd->spinner, -1);
-
-	gtk_widget_set_sensitive(cd->group, TRUE);
-	gtk_widget_set_sensitive(cd->button_start, TRUE);
-	gtk_widget_set_sensitive(cd->button_stop, FALSE);
-	gtk_widget_set_sensitive(cd->button_close, TRUE);
+		gtk_widget_set_sensitive(cd->group, TRUE);
+		gtk_widget_set_sensitive(cd->button_start, TRUE);
+		gtk_widget_set_sensitive(cd->button_stop, FALSE);
+		gtk_widget_set_sensitive(cd->button_close, TRUE);
+		}
 }
 
 static void cache_manager_render_stop_cb(GenericDialog *fd, gpointer data)
@@ -623,7 +678,10 @@ static gboolean cache_manager_render_file(CleanData *cd)
 		success = thumb_loader_start((ThumbLoader *)cd->tl, fd);
 		if (success)
 			{
-			gtk_entry_set_text(GTK_ENTRY(cd->progress), fd->path);
+			if (!cd->remote)
+				{
+				gtk_entry_set_text(GTK_ENTRY(cd->progress), fd->path);
+				}
 			}
 		else
 			{
@@ -659,26 +717,62 @@ static void cache_manager_render_start_cb(GenericDialog *fd, gpointer data)
 	CleanData *cd = data;
 	gchar *path;
 
-	if (cd->list || !gtk_widget_get_sensitive(cd->button_start)) return;
+	if(!cd->remote)
+		{
+		if (cd->list || !gtk_widget_get_sensitive(cd->button_start)) return;
+		}
 
 	path = remove_trailing_slash((gtk_entry_get_text(GTK_ENTRY(cd->entry))));
 	parse_out_relatives(path);
 
 	if (!isdir(path))
 		{
-		warning_dialog(_("Invalid folder"),
-				_("The specified folder can not be found."),
-			       GTK_STOCK_DIALOG_WARNING, cd->gd->dialog);
+		if (!cd->remote)
+			{
+			warning_dialog(_("Invalid folder"),
+			_("The specified folder can not be found."),
+			GTK_STOCK_DIALOG_WARNING, cd->gd->dialog);
+			}
+		else
+			{
+			log_printf("The specified folder can not be found: %s\n", path);
+			}
 		}
 	else
 		{
 		FileData *dir_fd;
-		gtk_widget_set_sensitive(cd->group, FALSE);
-		gtk_widget_set_sensitive(cd->button_start, FALSE);
-		gtk_widget_set_sensitive(cd->button_stop, TRUE);
-		gtk_widget_set_sensitive(cd->button_close, FALSE);
+		if(!cd->remote)
+			{
+			gtk_widget_set_sensitive(cd->group, FALSE);
+			gtk_widget_set_sensitive(cd->button_start, FALSE);
+			gtk_widget_set_sensitive(cd->button_stop, TRUE);
+			gtk_widget_set_sensitive(cd->button_close, FALSE);
 
-		spinner_set_interval(cd->spinner, SPINNER_SPEED);
+			spinner_set_interval(cd->spinner, SPINNER_SPEED);
+			}
+		dir_fd = file_data_new_dir(path);
+		cache_manager_render_folder(cd, dir_fd);
+		file_data_unref(dir_fd);
+		while (cache_manager_render_file(cd));
+		}
+
+	g_free(path);
+}
+
+static void cache_manager_render_start_render_remote(CleanData *cd, const gchar *user_path)
+{
+	gchar *path;
+
+	path = remove_trailing_slash(user_path);
+	parse_out_relatives(path);
+
+	if (!isdir(path))
+		{
+		log_printf("The specified folder can not be found: %s\n", path);
+		}
+	else
+		{
+		FileData *dir_fd;
 
 		dir_fd = file_data_new_dir(path);
 		cache_manager_render_folder(cd, dir_fd);
@@ -697,6 +791,7 @@ static void cache_manager_render_dialog(GtkWidget *widget, const gchar *path)
 	GtkWidget *button;
 
 	cd = g_new0(CleanData, 1);
+	cd->remote = FALSE;
 
 	cd->gd = generic_dialog_new(_("Create thumbnails"),
 				    "create_thumbnails",
@@ -712,7 +807,7 @@ static void cache_manager_render_dialog(GtkWidget *widget, const gchar *path)
 						    cache_manager_render_stop_cb, FALSE);
 	gtk_widget_set_sensitive(cd->button_stop, FALSE);
 
-	generic_dialog_add_message(cd->gd, NULL, _("Create thumbnails"), NULL);
+	generic_dialog_add_message(cd->gd, NULL, _("Create thumbnails"), NULL, FALSE);
 
 	hbox = pref_box_new(cd->gd->vbox, FALSE, GTK_ORIENTATION_HORIZONTAL, 0);
 	pref_spacer(hbox, PREF_PAD_INDENT);
@@ -721,7 +816,7 @@ static void cache_manager_render_dialog(GtkWidget *widget, const gchar *path)
 	hbox = pref_box_new(cd->group, FALSE, GTK_ORIENTATION_HORIZONTAL, PREF_PAD_SPACE);
 	pref_label_new(hbox, _("Folder:"));
 
-	label = tab_completion_new(&cd->entry, path, NULL, NULL);
+	label = tab_completion_new(&cd->entry, path, NULL, NULL, NULL, NULL);
 	tab_completion_add_select_button(cd->entry,_("Select folder") , TRUE);
 	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
 	gtk_widget_show(label);
@@ -749,8 +844,17 @@ static void cache_manager_render_dialog(GtkWidget *widget, const gchar *path)
 	gtk_widget_show(cd->gd->dialog);
 }
 
+void cache_manager_render_remote(const gchar *path, gboolean recurse, gboolean local)
+{
+	CleanData *cd;
 
+	cd = g_new0(CleanData, 1);
+	cd->recurse = recurse;
+	cd->local = local;
+	cd->remote = TRUE;
 
+	cache_manager_render_start_render_remote(cd, path);
+}
 
 static void cache_manager_standard_clean_close_cb(GenericDialog *gd, gpointer data)
 {
@@ -767,12 +871,14 @@ static void cache_manager_standard_clean_close_cb(GenericDialog *gd, gpointer da
 
 static void cache_manager_standard_clean_done(CleanData *cd)
 {
-	gtk_widget_set_sensitive(cd->button_stop, FALSE);
-	gtk_widget_set_sensitive(cd->button_close, TRUE);
-
-	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(cd->progress), 1.0);
-	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(cd->progress), _("done"));
+	if (!cd->remote)
+		{
+		gtk_widget_set_sensitive(cd->button_stop, FALSE);
+		gtk_widget_set_sensitive(cd->button_close, TRUE);
 
+		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(cd->progress), 1.0);
+		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(cd->progress), _("done"));
+		}
 	if (cd->idle_id)
 		{
 		g_source_remove(cd->idle_id);
@@ -810,10 +916,13 @@ static gint cache_manager_standard_clean_clear_cb(gpointer data)
 		file_data_unref(next_fd);
 
 		cd->count_done++;
-		if (cd->count_total != 0)
+		if (!cd->remote)
 			{
-			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(cd->progress),
-						      (gdouble)cd->count_done / cd->count_total);
+			if (cd->count_total != 0)
+				{
+				gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(cd->progress),
+							      (gdouble)cd->count_done / cd->count_total);
+				}
 			}
 
 		return TRUE;
@@ -837,10 +946,13 @@ static void cache_manager_standard_clean_valid_cb(const gchar *path, gboolean va
 			}
 
 		cd->count_done++;
-		if (cd->count_total != 0)
+		if (!cd->remote)
 			{
-			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(cd->progress),
-						      (gdouble)cd->count_done / cd->count_total);
+			if (cd->count_total != 0)
+				{
+				gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(cd->progress),
+							      (gdouble)cd->count_done / cd->count_total);
+				}
 			}
 		}
 
@@ -862,36 +974,39 @@ static void cache_manager_standard_clean_valid_cb(const gchar *path, gboolean va
 		}
 }
 
-static void cache_manager_standard_clean_start_cb(GenericDialog *gd, gpointer data)
+static void cache_manager_standard_clean_start(GenericDialog *gd, gpointer data)
 {
 	CleanData *cd = data;
 	GList *list;
 	gchar *path;
 	FileData *dir_fd;
 
-	if (cd->list || !gtk_widget_get_sensitive(cd->button_start)) return;
+	if (!cd->remote)
+	{
+		if (cd->list || !gtk_widget_get_sensitive(cd->button_start)) return;
 
-	gtk_widget_set_sensitive(cd->button_start, FALSE);
-	gtk_widget_set_sensitive(cd->button_stop, TRUE);
-	gtk_widget_set_sensitive(cd->button_close, FALSE);
+		gtk_widget_set_sensitive(cd->button_start, FALSE);
+		gtk_widget_set_sensitive(cd->button_stop, TRUE);
+		gtk_widget_set_sensitive(cd->button_close, FALSE);
 
-	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(cd->progress), _("running..."));
+		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(cd->progress), _("running..."));
+	}
 
-	path = g_build_filename(homedir(), THUMB_FOLDER_GLOBAL, THUMB_FOLDER_NORMAL, NULL);
+	path = g_build_filename(get_thumbnails_standard_cache_dir(), THUMB_FOLDER_NORMAL, NULL);
 	dir_fd = file_data_new_dir(path);
 	filelist_read(dir_fd, &list, NULL);
 	cd->list = list;
 	file_data_unref(dir_fd);
 	g_free(path);
 
-	path = g_build_filename(homedir(), THUMB_FOLDER_GLOBAL, THUMB_FOLDER_LARGE, NULL);
+	path = g_build_filename(get_thumbnails_standard_cache_dir(), THUMB_FOLDER_LARGE, NULL);
 	dir_fd = file_data_new_dir(path);
 	filelist_read(dir_fd, &list, NULL);
 	cd->list = g_list_concat(cd->list, list);
 	file_data_unref(dir_fd);
 	g_free(path);
 
-	path = g_build_filename(homedir(), THUMB_FOLDER_GLOBAL, THUMB_FOLDER_FAIL, NULL);
+	path = g_build_filename(get_thumbnails_standard_cache_dir(), THUMB_FOLDER_FAIL, NULL);
 	dir_fd = file_data_new_dir(path);
 	filelist_read(dir_fd, &list, NULL);
 	cd->list = g_list_concat(cd->list, list);
@@ -912,6 +1027,11 @@ static void cache_manager_standard_clean_start_cb(GenericDialog *gd, gpointer da
 		}
 }
 
+static void cache_manager_standard_clean_start_cb(GenericDialog *gd, gpointer data)
+{
+	cache_manager_standard_clean_start(gd, data);
+}
+
 static void cache_manager_standard_process(GtkWidget *widget, gboolean clear)
 {
 	CleanData *cd;
@@ -920,6 +1040,7 @@ static void cache_manager_standard_process(GtkWidget *widget, gboolean clear)
 
 	cd = g_new0(CleanData, 1);
 	cd->clear = clear;
+	cd->remote = FALSE;
 
 	if (clear)
 		{
@@ -945,10 +1066,13 @@ static void cache_manager_standard_process(GtkWidget *widget, gboolean clear)
 						    cache_manager_standard_clean_stop_cb, FALSE);
 	gtk_widget_set_sensitive(cd->button_stop, FALSE);
 
-	generic_dialog_add_message(cd->gd, stock_id, msg, NULL);
+	generic_dialog_add_message(cd->gd, stock_id, msg, NULL, FALSE);
 
 	cd->progress = gtk_progress_bar_new();
 	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(cd->progress), _("click start to begin"));
+#if GTK_CHECK_VERSION(3,0,0)
+	gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(cd->progress), TRUE);
+#endif
 	gtk_box_pack_start(GTK_BOX(cd->gd->vbox), cd->progress, FALSE, FALSE, 0);
 	gtk_widget_show(cd->progress);
 
@@ -959,6 +1083,20 @@ static void cache_manager_standard_process(GtkWidget *widget, gboolean clear)
 	gtk_widget_show(cd->gd->dialog);
 }
 
+void cache_manager_standard_process_remote(gboolean clear)
+{
+	CleanData *cd;
+
+	cd = g_new0(CleanData, 1);
+	cd->clear = clear;
+	cd->days = 30;
+	cd->tl = NULL;
+	cd->idle_id = 0;
+	cd->remote = TRUE;
+
+	cache_manager_standard_clean_start(NULL, cd);
+}
+
 static void cache_manager_standard_clean_cb(GtkWidget *widget, gpointer data)
 {
 	cache_manager_standard_process(widget, FALSE);
@@ -994,7 +1132,7 @@ void cache_manager_main_clear_confirm(GtkWidget *parent)
 				"clear_cache", parent, TRUE,
 				dummy_cancel_cb, NULL);
 	generic_dialog_add_message(gd, GTK_STOCK_DIALOG_QUESTION, _("Clear cache"),
-				   _("This will remove all thumbnails that have\nbeen saved to disk, continue?"));
+				   _("This will remove all thumbnails that have\nbeen saved to disk, continue?"), TRUE);
 	generic_dialog_add_button(gd, GTK_STOCK_OK, NULL, cache_manager_main_clear_ok_cb, TRUE);
 
 	gtk_widget_show(gd->dialog);
@@ -1029,15 +1167,17 @@ static void cache_manager_close_cb(GenericDialog *gd, gpointer data)
 	cache_manager = NULL;
 }
 
+static void cache_manager_help_cb(GenericDialog *gd, gpointer data)
+{
+	help_window_show("GuideReferenceManagement.html");
+}
+
 static GtkWidget *cache_manager_location_label(GtkWidget *group, const gchar *subdir)
 {
 	GtkWidget *label;
 	gchar *buf;
-	gchar *path;
 
-	path = g_build_filename(homedir(), subdir, NULL);
-	buf = g_strdup_printf(_("Location: %s"), path);
-	g_free(path);
+	buf = g_strdup_printf(_("Location: %s"), subdir);
 	label = pref_label_new(group, buf);
 	g_free(buf);
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
@@ -1052,6 +1192,7 @@ void cache_manager_show(void)
 	GtkWidget *button;
 	GtkWidget *table;
 	GtkSizeGroup *sizegroup;
+	gchar *path;
 
 	if (cache_manager)
 		{
@@ -1070,12 +1211,14 @@ void cache_manager_show(void)
 	gd->cancel_cb = cache_manager_close_cb;
 	generic_dialog_add_button(gd, GTK_STOCK_CLOSE, NULL,
 				  cache_manager_close_cb, FALSE);
+	generic_dialog_add_button(gd, GTK_STOCK_HELP, NULL,
+				  cache_manager_help_cb, FALSE);
 
-	generic_dialog_add_message(gd, NULL, _("Cache and Data Maintenance"), NULL);
+	generic_dialog_add_message(gd, NULL, _("Cache and Data Maintenance"), NULL, FALSE);
 
 	sizegroup = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
 
-	group = pref_group_new(gd->vbox, FALSE, _("Thumbnail cache"), GTK_ORIENTATION_VERTICAL);
+	group = pref_group_new(gd->vbox, FALSE, _("Geeqie thumbnail cache"), GTK_ORIENTATION_VERTICAL);
 
 	cache_manager_location_label(group, get_thumbnails_cache_dir());
 
@@ -1089,12 +1232,14 @@ void cache_manager_show(void)
 	button = pref_table_button(table, 0, 1, GTK_STOCK_DELETE, _("Clear cache"), FALSE,
 				   G_CALLBACK(cache_manager_main_clear_cb), cache_manager);
 	gtk_size_group_add_widget(sizegroup, button);
-	pref_table_label(table, 1, 1, _("Delete all cached thumbnails."), 0.0);
+	pref_table_label(table, 1, 1, _("Delete all cached data."), 0.0);
 
 
 	group = pref_group_new(gd->vbox, FALSE, _("Shared thumbnail cache"), GTK_ORIENTATION_VERTICAL);
 
-	cache_manager_location_label(group, THUMB_FOLDER_GLOBAL);
+	path = g_build_filename(get_thumbnails_standard_cache_dir(), NULL);
+	cache_manager_location_label(group, path);
+	g_free(path);
 
 	table = pref_table_new(group, 2, 2, FALSE, FALSE);
 
@@ -1116,6 +1261,7 @@ void cache_manager_show(void)
 				   G_CALLBACK(cache_manager_render_cb), cache_manager);
 	gtk_size_group_add_widget(sizegroup, button);
 	pref_table_label(table, 1, 1, _("Render thumbnails for a specific folder."), 0.0);
+	gtk_widget_set_sensitive(group, options->thumbnails.enable_caching);
 
 	group = pref_group_new(gd->vbox, FALSE, _("Metadata"), GTK_ORIENTATION_VERTICAL);
 
