@@ -33,12 +33,15 @@
 #include "layout.h"
 #include "layout_image.h"
 #include "menu.h"
+#include "metadata.h"
+#include "pixbuf_util.h"
 #include "print.h"
 #include "utilops.h"
 #include "ui_fileops.h"
 #include "ui_menu.h"
 #include "ui_tree_edit.h"
 #include "uri_utils.h"
+#include "view_file.h"
 
 #include "icons/marker.xpm"
 #define MARKER_WIDTH 26
@@ -62,14 +65,6 @@ enum {
 	CTABLE_COLUMN_COUNT
 };
 
-typedef enum {
-	SELECTION_NONE		= 0,
-	SELECTION_SELECTED	= 1 << 0,
-	SELECTION_PRELIGHT	= 1 << 1,
-	SELECTION_FOCUS		= 1 << 2
-} SelectionType;
-
-
 #define INFO_SELECTED(x) (x->flag_mask & SELECTION_SELECTED)
 
 
@@ -260,6 +255,16 @@ static void collection_table_toggle_filenames(CollectTable *ct)
 	collection_table_populate_at_new_size(ct, allocation.width, allocation.height, TRUE);
 }
 
+static void collection_table_toggle_stars(CollectTable *ct)
+{
+	GtkAllocation allocation;
+	ct->show_stars = !ct->show_stars;
+	options->show_star_rating = ct->show_stars;
+
+	gtk_widget_get_allocation(ct->listview, &allocation);
+	collection_table_populate_at_new_size(ct, allocation.width, allocation.height, TRUE);
+}
+
 static gint collection_table_get_icon_width(CollectTable *ct)
 {
 	gint width;
@@ -408,7 +413,7 @@ static void collection_table_select_invert_all(CollectTable *ct)
 	collection_table_update_status(ct);
 }
 
-static void collection_table_select(CollectTable *ct, CollectInfo *info)
+void collection_table_select(CollectTable *ct, CollectInfo *info)
 {
 	ct->prev_selection = info;
 
@@ -527,11 +532,23 @@ static void tip_show(CollectTable *ct)
 {
 	GtkWidget *label;
 	gint x, y;
-	gint x_win = 0, y_win = 0;
+#if GTK_CHECK_VERSION(3,0,0)
+	GdkDisplay *display;
+	GdkDeviceManager *device_manager;
+	GdkDevice *device;
+#endif
 
 	if (ct->tip_window) return;
 
+#if GTK_CHECK_VERSION(3,0,0)
+	device_manager = gdk_display_get_device_manager(gdk_window_get_display(
+								gtk_widget_get_window(ct->listview)));
+	device = gdk_device_manager_get_client_pointer(device_manager);
+	gdk_window_get_device_position(gtk_widget_get_window(ct->listview),
+								device, &x, &y, NULL);
+#else
 	gdk_window_get_pointer(gtk_widget_get_window(ct->listview), &x, &y, NULL);
+#endif
 
 	ct->tip_info = collection_table_find_data_by_coord(ct, x, y, NULL);
 	if (!ct->tip_info) return;
@@ -547,13 +564,16 @@ static void tip_show(CollectTable *ct)
 	gtk_widget_show(label);
 
 #if GTK_CHECK_VERSION(3,0,0)
-	gdk_window_get_origin(gtk_widget_get_window(ct->listview), &x_win, &y_win);
+	display = gdk_display_get_default();
+	device_manager = gdk_display_get_device_manager(display);
+	device = gdk_device_manager_get_client_pointer(device_manager);
+	gdk_device_get_position(device, NULL, &x, &y);
 #else
 	gdk_window_get_pointer(NULL, &x, &y, NULL);
 #endif
 
 	if (!gtk_widget_get_realized(ct->tip_window)) gtk_widget_realize(ct->tip_window);
-	gtk_window_move(GTK_WINDOW(ct->tip_window), x_win + x + 16, y_win + y + 16);
+	gtk_window_move(GTK_WINDOW(ct->tip_window), x + 16, y + 16);
 	gtk_widget_show(ct->tip_window);
 }
 
@@ -601,13 +621,22 @@ static void tip_unschedule(CollectTable *ct)
 
 static void tip_update(CollectTable *ct, CollectInfo *info)
 {
+#if GTK_CHECK_VERSION(3,0,0)
+	GdkDisplay *display = gdk_display_get_default();
+	GdkDeviceManager *device_manager = gdk_display_get_device_manager(display);
+	GdkDevice *device = gdk_device_manager_get_client_pointer(device_manager);
+#endif
+
 	tip_schedule(ct);
 
 	if (ct->tip_window)
 		{
 		gint x, y;
-
+#if GTK_CHECK_VERSION(3,0,0)
+		gdk_device_get_position(device, NULL, &x, &y);
+#else
 		gdk_window_get_pointer(NULL, &x, &y, NULL);
+#endif
 		gtk_window_move(GTK_WINDOW(ct->tip_window), x + 16, y + 16);
 
 		if (info != ct->tip_info)
@@ -705,6 +734,15 @@ static void collection_table_popup_delete_cb(GtkWidget *widget, gpointer data)
 {
 	CollectTable *ct = data;
 
+	options->file_ops.safe_delete_enable = FALSE;
+	file_util_delete(NULL, collection_table_popup_file_list(ct), ct->listview);
+}
+
+static void collection_table_popup_move_to_trash_cb(GtkWidget *widget, gpointer data)
+{
+	CollectTable *ct = data;
+
+	options->file_ops.safe_delete_enable = TRUE;
 	file_util_delete(NULL, collection_table_popup_file_list(ct), ct->listview);
 }
 
@@ -712,7 +750,14 @@ static void collection_table_popup_copy_path_cb(GtkWidget *widget, gpointer data
 {
 	CollectTable *ct = data;
 
-	file_util_copy_path_list_to_clipboard(collection_table_popup_file_list(ct));
+	file_util_copy_path_list_to_clipboard(collection_table_popup_file_list(ct), TRUE);
+}
+
+static void collection_table_popup_copy_path_unquoted_cb(GtkWidget *widget, gpointer data)
+{
+	CollectTable *ct = data;
+
+	file_util_copy_path_list_to_clipboard(collection_table_popup_file_list(ct), FALSE);
 }
 
 static void collection_table_popup_sort_cb(GtkWidget *widget, gpointer data)
@@ -784,6 +829,11 @@ static void collection_table_popup_select_invert_cb(GtkWidget *widget, gpointer
 	ct->prev_selection= ct->click_info;
 }
 
+static void collection_table_popup_rectangular_selection_cb(GtkWidget *widget, gpointer data)
+{
+	options->collections.rectangular_selection = !(options->collections.rectangular_selection);
+}
+
 static void collection_table_popup_remove_cb(GtkWidget *widget, gpointer data)
 {
 	CollectTable *ct = data;
@@ -804,12 +854,15 @@ static void collection_table_popup_remove_cb(GtkWidget *widget, gpointer data)
 	g_list_free(list);
 }
 
-static void collection_table_popup_add_filelist_cb(GtkWidget *widget, gpointer data)
+static void collection_table_popup_add_file_selection_cb(GtkWidget *widget, gpointer data)
 {
 	CollectTable *ct = data;
 	GList *list;
+	LayoutWindow *lw = NULL;
 
-	list = layout_list(NULL);
+	if (!layout_valid(&lw)) return;
+
+	list = vf_selection_get_list(lw->vf);
 
 	if (list)
 		{
@@ -825,12 +878,32 @@ static void collection_table_popup_add_collection_cb(GtkWidget *widget, gpointer
 	collection_dialog_append(NULL, ct->cd);
 }
 
+static void collection_table_popup_goto_original_cb(GtkWidget *widget, gpointer data)
+{
+	CollectTable *ct = data;
+	GList *list;
+	LayoutWindow *lw = NULL;
+	FileData *fd;
+
+	if (!layout_valid(&lw)) return;
+	list = collection_table_selection_get_list(ct);
+	if (list)
+		{
+		fd = list->data;
+		if (fd)
+			{
+			layout_set_fd(lw, fd);
+			}
+		}
+	g_list_free(list);
+}
+
 static void collection_table_popup_find_dupes_cb(GtkWidget *widget, gpointer data)
 {
 	CollectTable *ct = data;
 	DupeWindow *dw;
 
-	dw = dupe_window_new(DUPE_MATCH_NAME);
+	dw = dupe_window_new();
 	dupe_window_add_collection(dw, ct->cd);
 }
 
@@ -841,7 +914,7 @@ static void collection_table_popup_print_cb(GtkWidget *widget, gpointer data)
 
 	fd = (ct->click_info) ? ct->click_info->fd : NULL;
 
-	print_window_new(fd, collection_table_selection_get_list(ct), collection_table_get_list(ct), ct->listview);
+	print_window_new(fd, collection_table_selection_get_list(ct), collection_table_get_list(ct), gtk_widget_get_toplevel(ct->listview));
 }
 
 static void collection_table_popup_show_names_cb(GtkWidget *widget, gpointer data)
@@ -851,6 +924,13 @@ static void collection_table_popup_show_names_cb(GtkWidget *widget, gpointer dat
 	collection_table_toggle_filenames(ct);
 }
 
+static void collection_table_popup_show_stars_cb(GtkWidget *widget, gpointer data)
+{
+	CollectTable *ct = data;
+
+	collection_table_toggle_stars(ct);
+}
+
 static void collection_table_popup_destroy_cb(GtkWidget *widget, gpointer data)
 {
 	CollectTable *ct = data;
@@ -882,12 +962,14 @@ static GtkWidget *collection_table_popup_menu(CollectTable *ct, gboolean over_ic
 			G_CALLBACK(collection_table_popup_view_cb), ct);
 	menu_item_add_stock_sensitive(menu, _("View in _new window"), GTK_STOCK_NEW, over_icon,
 			G_CALLBACK(collection_table_popup_view_new_cb), ct);
+	menu_item_add_stock(menu, _("Go to original"), GTK_STOCK_FIND,
+			G_CALLBACK(collection_table_popup_goto_original_cb), ct);
 	menu_item_add_divider(menu);
 	menu_item_add_stock_sensitive(menu, _("Rem_ove"), GTK_STOCK_REMOVE, over_icon,
 			G_CALLBACK(collection_table_popup_remove_cb), ct);
 
-	menu_item_add_stock(menu, _("Append from file list"), GTK_STOCK_ADD,
-			G_CALLBACK(collection_table_popup_add_filelist_cb), ct);
+	menu_item_add_stock(menu, _("Append from file selection"), GTK_STOCK_ADD,
+			G_CALLBACK(collection_table_popup_add_file_selection_cb), ct);
 	menu_item_add_stock(menu, _("Append from collection..."), GTK_STOCK_OPEN,
 			G_CALLBACK(collection_table_popup_add_collection_cb), ct);
 	menu_item_add_divider(menu);
@@ -900,6 +982,8 @@ static GtkWidget *collection_table_popup_menu(CollectTable *ct, gboolean over_ic
 			G_CALLBACK(collection_table_popup_unselectall_cb), ct);
 	menu_item_add(submenu, _("Invert selection"),
 			G_CALLBACK(collection_table_popup_select_invert_cb), ct);
+	menu_item_add_check(submenu, _("Rectangular selection"), (options->collections.rectangular_selection != FALSE),
+			G_CALLBACK(collection_table_popup_rectangular_selection_cb), ct);
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
 	menu_item_add_divider(menu);
 
@@ -916,12 +1000,22 @@ static GtkWidget *collection_table_popup_menu(CollectTable *ct, gboolean over_ic
 			G_CALLBACK(collection_table_popup_move_cb), ct);
 	menu_item_add_sensitive(menu, _("_Rename..."), over_icon,
 			G_CALLBACK(collection_table_popup_rename_cb), ct);
-	menu_item_add_stock_sensitive(menu, _("_Delete..."), GTK_STOCK_DELETE, over_icon,
-			G_CALLBACK(collection_table_popup_delete_cb), ct);
 	menu_item_add_sensitive(menu, _("_Copy path"), over_icon,
 				G_CALLBACK(collection_table_popup_copy_path_cb), ct);
-	menu_item_add_divider(menu);
+	menu_item_add_sensitive(menu, _("_Copy path unquoted"), over_icon,
+				G_CALLBACK(collection_table_popup_copy_path_unquoted_cb), ct);
 
+	menu_item_add_divider(menu);
+	menu_item_add_stock_sensitive(menu,
+				options->file_ops.confirm_move_to_trash ? _("Move to Trash...") :
+					_("Move to Trash"), PIXBUF_INLINE_ICON_TRASH, over_icon,
+				G_CALLBACK(collection_table_popup_move_to_trash_cb), ct);
+	menu_item_add_stock_sensitive(menu,
+				options->file_ops.confirm_delete ? _("_Delete...") :
+					_("_Delete"), GTK_STOCK_DELETE, over_icon,
+				G_CALLBACK(collection_table_popup_delete_cb), ct);
+
+	menu_item_add_divider(menu);
 	submenu = submenu_add_sort(NULL, G_CALLBACK(collection_table_popup_sort_cb), ct, FALSE, TRUE, FALSE, 0);
 	menu_item_add_divider(submenu);
 	menu_item_add(submenu, _("Randomize"),
@@ -931,6 +1025,8 @@ static GtkWidget *collection_table_popup_menu(CollectTable *ct, gboolean over_ic
 
 	menu_item_add_check(menu, _("Show filename _text"), ct->show_text,
 			G_CALLBACK(collection_table_popup_show_names_cb), ct);
+	menu_item_add_check(menu, _("Show star rating"), ct->show_stars,
+				G_CALLBACK(collection_table_popup_show_stars_cb), ct);
 	menu_item_add_divider(menu);
 	menu_item_add_stock(menu, _("_Save collection"), GTK_STOCK_SAVE,
 			G_CALLBACK(collection_table_popup_save_cb), ct);
@@ -950,7 +1046,7 @@ static GtkWidget *collection_table_popup_menu(CollectTable *ct, gboolean over_ic
  *-------------------------------------------------------------------
  */
 
-static void collection_table_set_focus(CollectTable *ct, CollectInfo *info)
+void collection_table_set_focus(CollectTable *ct, CollectInfo *info)
 {
 	GtkTreeIter iter;
 	gint row, col;
@@ -1272,11 +1368,25 @@ static CollectInfo *collection_table_insert_find(CollectTable *ct, CollectInfo *
 	GtkTreeIter iter;
 	GtkTreePath *tpath;
 	GtkTreeViewColumn *column;
+#if GTK_CHECK_VERSION(3,0,0)
+	GdkDeviceManager *device_manager;
+	GdkDevice *device;
+#endif
 
 	store = gtk_tree_view_get_model(GTK_TREE_VIEW(ct->listview));
 
+#if GTK_CHECK_VERSION(3,0,0)
+	if (!use_coord)
+		{
+		device_manager = gdk_display_get_device_manager(gdk_window_get_display(
+									gtk_widget_get_window(ct->listview)));
+		device = gdk_device_manager_get_client_pointer(device_manager);
+		gdk_window_get_device_position(gtk_widget_get_window(ct->listview),
+									device, &x, &y, NULL);
+		}
+#else
 	if (!use_coord) gdk_window_get_pointer(gtk_widget_get_window(ct->listview), &x, &y, NULL);
-
+#endif
 	if (source)
 		{
 		gint col;
@@ -1490,11 +1600,21 @@ static gboolean collection_table_auto_scroll_idle_cb(gpointer data)
 	GdkWindow *window;
 	gint x, y;
 	gint w, h;
+#if GTK_CHECK_VERSION(3,0,0)
+	GdkDeviceManager *device_manager;
+	GdkDevice *device;
+#endif
 
 	if (!ct->drop_idle_id) return FALSE;
 
 	window = gtk_widget_get_window(ct->listview);
+#if GTK_CHECK_VERSION(3,0,0)
+	device_manager = gdk_display_get_device_manager(gdk_window_get_display(window));
+	device = gdk_device_manager_get_client_pointer(device_manager);
+	gdk_window_get_device_position(window, device, &x, &y, NULL);
+#else
 	gdk_window_get_pointer(window, &x, &y, NULL);
+#endif
 	w = gdk_window_get_width(window);
 	h = gdk_window_get_height(window);
 	if (x >= 0 && x < w && y >= 0 && y < h)
@@ -1544,11 +1664,11 @@ static void collection_table_scroll(CollectTable *ct, gboolean scroll)
  *-------------------------------------------------------------------
  */
 
-static gboolean collection_table_motion_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gboolean collection_table_motion_cb(GtkWidget *widget, GdkEventMotion *event, gpointer data)
 {
 	CollectTable *ct = data;
 
-	collection_table_motion_update(ct, (gint)bevent->x, (gint)bevent->y, FALSE);
+	collection_table_motion_update(ct, (gint)event->x, (gint)event->y, FALSE);
 
 	return FALSE;
 }
@@ -1735,7 +1855,7 @@ static void collection_table_populate(CollectTable *ct, gboolean resize)
 				{
 				g_object_set(G_OBJECT(cell), "fixed_width", thumb_width,
 							     "fixed_height", options->thumbnails.max_height,
-							     "show_text", ct->show_text, NULL);
+							     "show_text", ct->show_text || ct->show_stars, NULL);
 				}
 			}
 		if (gtk_widget_get_realized(ct->listview)) gtk_tree_view_columns_autosize(GTK_TREE_VIEW(ct->listview));
@@ -2365,10 +2485,21 @@ static void collection_table_cell_data_cb(GtkTreeViewColumn *tree_column, GtkCel
 	CollectInfo *info;
 	GdkColor color_fg;
 	GdkColor color_bg;
+	gchar *star_rating = NULL;
+	gchar *display_text = NULL;
 
 	ct = cd->ct;
 
 	gtk_tree_model_get(tree_model, iter, CTABLE_COLUMN_POINTER, &list, -1);
+
+#if GTK_CHECK_VERSION(3,0,0)
+	/* FIXME this is a primitive hack to stop a crash.
+	 * When compiled with GTK3, if a Collection window containing
+	 * say, 50 or so, images has its width changed, there is a segfault
+	 * https://github.com/BestImageViewer/geeqie/issues/531
+	 */
+	if (cd->number == COLLECT_TABLE_MAX_COLUMNS) return;
+#endif
 	info = g_list_nth_data(list, cd->number);
 
 	style = gtk_widget_get_style(ct->listview);
@@ -2388,12 +2519,45 @@ static void collection_table_cell_data_cb(GtkTreeViewColumn *tree_column, GtkCel
 		shift_color(&color_bg, -1, 0);
 		}
 
+	if (ct->show_stars && info && info->fd)
+		{
+		star_rating = metadata_read_rating_stars(info->fd);
+		}
+	else
+		{
+		star_rating = g_strdup("");
+		}
+
+	if (info && info->fd)
+		{
+		if (ct->show_text && ct->show_stars)
+			{
+			display_text = g_strconcat(info->fd->name, "\n", star_rating, NULL);
+			}
+		else if (ct->show_text)
+			{
+			display_text = g_strdup(info->fd->name);
+			}
+		else if (ct->show_stars)
+			{
+			display_text = g_strdup(star_rating);
+			}
+		else
+			{
+			display_text = g_strdup("");
+			}
+		}
+	else
+		{
+		display_text = g_strdup("");
+		}
+
 	if (GQV_IS_CELL_RENDERER_ICON(cell))
 		{
 		if (info)
 			{
 			g_object_set(cell,	"pixbuf", info->pixbuf,
-						"text", info->fd->name,
+						"text",  display_text,
 						"cell-background-gdk", &color_bg,
 						"cell-background-set", TRUE,
 						"foreground-gdk", &color_fg,
@@ -2409,6 +2573,9 @@ static void collection_table_cell_data_cb(GtkTreeViewColumn *tree_column, GtkCel
 						"has-focus", FALSE,  NULL);
 			}
 		}
+
+	g_free(display_text);
+	g_free(star_rating);
 }
 
 static void collection_table_append_column(CollectTable *ct, gint n)
@@ -2449,6 +2616,16 @@ static void collection_table_destroy(GtkWidget *widget, gpointer data)
 {
 	CollectTable *ct = data;
 
+	/* If there is no unsaved data, save the window geometry
+	 */
+	if (!ct->cd->changed)
+		{
+		if (!collection_save(ct->cd, ct->cd->path))
+			{
+			log_printf("failed saving to collection path: %s\n", ct->cd->path);
+			}
+		}
+
 	if (ct->popup)
 		{
 		g_signal_handlers_disconnect_matched(G_OBJECT(ct->popup), G_SIGNAL_MATCH_DATA,
@@ -2482,6 +2659,7 @@ CollectTable *collection_table_new(CollectionData *cd)
 
 	ct->cd = cd;
 	ct->show_text = options->show_icon_names;
+	ct->show_stars = options->show_star_rating;
 
 	ct->scrolled = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(ct->scrolled), GTK_SHADOW_IN);
