@@ -188,7 +188,7 @@ static ColorManCache *color_man_cache_new(ColorManProfileType in_type, const gch
 					   (has_alpha) ? TYPE_RGBA_8 : TYPE_RGB_8,
 					   cc->profile_out,
 					   (has_alpha) ? TYPE_RGBA_8 : TYPE_RGB_8,
-					   INTENT_PERCEPTUAL, 0);
+					   options->color_profile.render_intent, 0);
 
 	if (!cc->transform)
 		{
@@ -443,7 +443,7 @@ static gchar *color_man_get_profile_name(ColorManProfileType type, cmsHPROFILE p
 				cmsUInt32Number r;
 				char buffer[20];
 				buffer[0] = '\0';
-				r = cmsGetProfileInfoASCII(profile, cmsInfoDescription, "en", "US", buffer, 20);
+				cmsGetProfileInfoASCII(profile, cmsInfoDescription, "en", "US", buffer, 20);
 				buffer[19] = '\0'; /* Just to be sure */
 				return g_strdup(buffer);
 #else
