@@ -189,6 +189,7 @@ static void editor_window_new(const gchar *src_path, const gchar *desktop_name)
 
 
 	ew->window = window_new(GTK_WINDOW_TOPLEVEL, "Desktop", PIXBUF_INLINE_ICON_CONFIG, NULL, _("Desktop file"));
+	DEBUG_NAME(ew->window);
 	gtk_window_set_type_hint(GTK_WINDOW(ew->window), GDK_WINDOW_TYPE_HINT_DIALOG);
 
 	g_signal_connect(G_OBJECT(ew->window), "delete_event",
@@ -350,7 +351,7 @@ static void editor_list_window_delete_cb(GtkWidget *widget, gpointer data)
 
 		text = g_strdup_printf(_("About to delete the file:\n %s"), path);
 		generic_dialog_add_message(ewl->gd, GTK_STOCK_DIALOG_QUESTION,
-					   _("Delete file"), text);
+					   _("Delete file"), text, TRUE);
 		g_free(text);
 
 		gtk_widget_show(ewl->gd->dialog);
@@ -383,6 +384,11 @@ static void editor_list_window_new_cb(GtkWidget *widget, gpointer data)
 	editor_window_new(DESKTOP_FILE_TEMPLATE, _("new.desktop"));
 }
 
+static void editor_list_window_help_cb(GtkWidget *widget, gpointer data)
+{
+	help_window_show("GuidePluginsConfig.html");
+}
+
 static void editor_list_window_selection_changed_cb(GtkWidget *widget, gpointer data)
 {
 	EditorListWindow *ewl = data;
@@ -409,6 +415,7 @@ static gint editor_list_window_sort_cb(GtkTreeModel *model, GtkTreeIter *a, GtkT
 {
 	gint n = GPOINTER_TO_INT(data);
 	gint ret = 0;
+	gboolean bool1, bool2;
 
 	switch (n)
 		{
@@ -436,6 +443,25 @@ static gint editor_list_window_sort_cb(GtkTreeModel *model, GtkTreeIter *a, GtkT
 			g_free(s2);
 			}
 			break;
+		case DESKTOP_FILE_COLUMN_DISABLED:
+			{
+			gtk_tree_model_get(model, a, n, &bool1, -1);
+			gtk_tree_model_get(model, b, n, &bool2, -1);
+
+			if (bool1 == bool2)
+				{
+				ret = 0;
+				}
+			else if (bool1 > bool2)
+				{
+				ret = -1;
+				}
+			else
+				{
+				ret = 1;
+				}
+			break;
+			}
 
     		default:
        			g_return_val_if_reached(0);
@@ -444,6 +470,67 @@ static gint editor_list_window_sort_cb(GtkTreeModel *model, GtkTreeIter *a, GtkT
 	return ret;
 }
 
+static void plugin_disable_cb(GtkCellRendererToggle *renderer, gchar *path_str, gpointer data)
+{
+	EditorListWindow *ewl = data;
+	GtkTreePath *tpath;
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	gboolean disabled;
+	gchar *path;
+	GList *list;
+	gchar *haystack;
+
+	tpath = gtk_tree_path_new_from_string(path_str);
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(ewl->view));
+	gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter, tpath);
+	gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, DESKTOP_FILE_COLUMN_DISABLED, &disabled, -1);
+	gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, DESKTOP_FILE_COLUMN_PATH, &path, -1);
+
+	gtk_list_store_set(GTK_LIST_STORE(desktop_file_list), &iter, DESKTOP_FILE_COLUMN_DISABLED, !disabled, -1);
+
+	if (!disabled)
+		{
+		options->disabled_plugins = g_list_append((options->disabled_plugins), g_strdup(path));
+		}
+	else
+		{
+		list = options->disabled_plugins;
+		while (list)
+			{
+			haystack = list->data;
+
+			if (haystack && strcmp(haystack, path) == 0)
+				{
+				g_free(haystack);
+				options->disabled_plugins = g_list_remove(options->disabled_plugins, haystack);
+				}
+
+			list = list->next;
+			}
+		}
+
+	layout_editors_reload_start();
+	layout_editors_reload_finish();
+}
+
+static void plugin_disable_set_func(GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
+							GtkTreeModel *tree_model, GtkTreeIter *iter, gpointer data)
+{
+	gboolean disabled;
+
+	gtk_tree_model_get(tree_model, iter, DESKTOP_FILE_COLUMN_DISABLED, &disabled, -1);
+
+	if (disabled)
+		{
+		g_object_set(GTK_CELL_RENDERER(cell), "active", TRUE, NULL);
+		}
+	else
+		{
+		g_object_set(GTK_CELL_RENDERER(cell), "active", FALSE, NULL);
+		}
+}
+
 static void editor_list_window_create(void)
 {
 	GtkWidget *win_vbox;
@@ -459,7 +546,8 @@ static void editor_list_window_create(void)
 
 	editor_list_window = ewl = g_new0(EditorListWindow, 1);
 
-	ewl->window = window_new(GTK_WINDOW_TOPLEVEL, "editors", PIXBUF_INLINE_ICON_CONFIG, NULL, _("Editors"));
+	ewl->window = window_new(GTK_WINDOW_TOPLEVEL, "editors", PIXBUF_INLINE_ICON_CONFIG, NULL, _("Plugins"));
+	DEBUG_NAME(ewl->window);
 	gtk_window_set_type_hint(GTK_WINDOW(ewl->window), GDK_WINDOW_TYPE_HINT_DIALOG);
 	g_signal_connect(G_OBJECT(ewl->window), "delete_event",
 			 G_CALLBACK(editor_list_window_delete), NULL);
@@ -478,6 +566,12 @@ static void editor_list_window_create(void)
 	gtk_widget_show(hbox);
 
 
+	button = pref_button_new(NULL, GTK_STOCK_HELP, NULL, FALSE,
+				 G_CALLBACK(editor_list_window_help_cb), ewl);
+	gtk_container_add(GTK_CONTAINER(hbox), button);
+	gtk_widget_set_can_default(button, TRUE);
+	gtk_widget_show(button);
+
 	button = pref_button_new(NULL, GTK_STOCK_NEW, NULL, FALSE,
 				 G_CALLBACK(editor_list_window_new_cb), ewl);
 	gtk_container_add(GTK_CONTAINER(hbox), button);
@@ -520,6 +614,19 @@ static void editor_list_window_create(void)
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(ewl->view), FALSE);
 
+	column = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title(column, _("Disabled"));
+	gtk_tree_view_column_set_resizable(column, TRUE);
+
+	renderer = gtk_cell_renderer_toggle_new();
+	g_signal_connect(G_OBJECT(renderer), "toggled",
+			 G_CALLBACK(plugin_disable_cb), ewl);
+	gtk_tree_view_column_pack_start(column, renderer, FALSE);
+	gtk_tree_view_column_set_cell_data_func(column, renderer, plugin_disable_set_func,
+						NULL, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(ewl->view), column);
+	gtk_tree_view_column_set_sort_column_id(column, DESKTOP_FILE_COLUMN_DISABLED);
+
 	column = gtk_tree_view_column_new();
 	gtk_tree_view_column_set_title(column, _("Name"));
 	gtk_tree_view_column_set_resizable(column, TRUE);
@@ -568,9 +675,11 @@ static void editor_list_window_create(void)
 					GINT_TO_POINTER(DESKTOP_FILE_COLUMN_NAME), NULL);
 	gtk_tree_sortable_set_sort_func(sortable, DESKTOP_FILE_COLUMN_PATH, editor_list_window_sort_cb,
 					GINT_TO_POINTER(DESKTOP_FILE_COLUMN_PATH), NULL);
+	gtk_tree_sortable_set_sort_func(sortable, DESKTOP_FILE_COLUMN_DISABLED, editor_list_window_sort_cb,
+					GINT_TO_POINTER(DESKTOP_FILE_COLUMN_DISABLED), NULL);
 
 	/* set initial sort order */
-    	//gtk_tree_sortable_set_sort_column_id(sortable, DESKTOP_FILE_COLUMN_KEY, GTK_SORT_ASCENDING);
+    gtk_tree_sortable_set_sort_column_id(sortable, DESKTOP_FILE_COLUMN_NAME, GTK_SORT_ASCENDING);
 
 	gtk_container_add(GTK_CONTAINER(scrolled), ewl->view);
 	gtk_widget_show(ewl->view);
