@@ -32,9 +32,11 @@
 #include "img-view.h"
 #include "layout.h"
 #include "layout_image.h"
+#include "layout_util.h"
 #include "md5-util.h"
 #include "menu.h"
 #include "misc.h"
+#include "pixbuf_util.h"
 #include "print.h"
 #include "thumb.h"
 #include "ui_fileops.h"
@@ -574,6 +576,8 @@ static void dupe_listview_add(DupeWindow *dw, DupeItem *parent, DupeItem *child)
 	g_free(text[DUPE_COLUMN_DIMENSIONS]);
 }
 
+static void dupe_listview_select_dupes(DupeWindow *dw, DupeSelectType parents);
+
 static void dupe_listview_populate(DupeWindow *dw)
 {
 	GtkListStore *store;
@@ -607,6 +611,16 @@ static void dupe_listview_populate(DupeWindow *dw)
 		}
 
 	gtk_tree_view_columns_autosize(GTK_TREE_VIEW(dw->listview));
+
+	if (options->duplicates_select_type == DUPE_SELECT_GROUP1)
+		{
+		dupe_listview_select_dupes(dw, DUPE_SELECT_GROUP1);
+		}
+	else if (options->duplicates_select_type == DUPE_SELECT_GROUP2)
+		{
+		dupe_listview_select_dupes(dw, DUPE_SELECT_GROUP2);
+		}
+
 }
 
 static void dupe_listview_remove(DupeWindow *dw, DupeItem *di)
@@ -712,7 +726,7 @@ static gboolean dupe_listview_item_is_selected(DupeWindow *dw, DupeItem *di, Gtk
 	return found;
 }
 
-static void dupe_listview_select_dupes(DupeWindow *dw, gint parents)
+static void dupe_listview_select_dupes(DupeWindow *dw, DupeSelectType parents)
 {
 	GtkTreeModel *store;
 	GtkTreeSelection *selection;
@@ -729,7 +743,7 @@ static void dupe_listview_select_dupes(DupeWindow *dw, gint parents)
 		DupeItem *di;
 
 		gtk_tree_model_get(store, &iter, DUPE_COLUMN_POINTER, &di, -1);
-		if ( (dupe_match_find_parent(dw, di) == di) == (parents) )
+		if ((dupe_match_find_parent(dw, di) == di) == (parents == DUPE_SELECT_GROUP1))
 			{
 			gtk_tree_selection_select_iter(selection, &iter);
 			}
@@ -1054,6 +1068,20 @@ static void dupe_match_sort_groups(GList *list)
 		}
 }
 
+static gint dupe_match_totals_sort_cb(gconstpointer a, gconstpointer b)
+{
+	DupeItem *da = (DupeItem *)a;
+	DupeItem *db = (DupeItem *)b;
+
+	if (g_list_length(da->group) > g_list_length(db->group)) return -1;
+	if (g_list_length(da->group) < g_list_length(db->group)) return 1;
+
+	if (da->group_rank < db->group_rank) return -1;
+	if (da->group_rank > db->group_rank) return 1;
+
+	return 0;
+}
+
 static gint dupe_match_rank_sort_cb(gconstpointer a, gconstpointer b)
 {
 	DupeItem *da = (DupeItem *)a;
@@ -1087,6 +1115,15 @@ static GList *dupe_match_rank_sort(GList *source_list)
 	return g_list_sort(list, dupe_match_rank_sort_cb);
 }
 
+/* returns allocated GList of dupes sorted by totals */
+static GList *dupe_match_totals_sort(GList *source_list)
+{
+	source_list = g_list_sort(source_list, dupe_match_totals_sort_cb);
+
+	source_list = g_list_first(source_list);
+	return g_list_reverse(source_list);
+}
+
 static void dupe_match_rank(DupeWindow *dw)
 {
 	GList *list;
@@ -1104,6 +1141,11 @@ static void dupe_match_rank(DupeWindow *dw)
 	if (required_debug_level(2)) dupe_match_print_list(list);
 
 	list = dupe_match_rank_sort(list);
+	if (options->sort_totals)
+		{
+		list = dupe_match_totals_sort(list);
+		}
+	if (required_debug_level(2)) dupe_match_print_list(list);
 
 	g_list_free(dw->dupes);
 	dw->dupes = list;
@@ -1756,6 +1798,40 @@ static void dupe_files_add(DupeWindow *dw, CollectionData *collection, CollectIn
 
 	if (!di) return;
 
+	/* Ensure images in the lists have unique FileDatas */
+	GList *work;
+	DupeItem *di_list;
+	work = g_list_first(dw->list);
+	while (work)
+		{
+		di_list = work->data;
+		if (di_list->fd == di->fd)
+			{
+			return;
+			}
+		else
+			{
+			work = work->next;
+			}
+		}
+
+	if (dw->second_list)
+		{
+		work = g_list_first(dw->second_list);
+		while (work)
+			{
+			di_list = work->data;
+			if (di_list->fd == di->fd)
+				{
+				return;
+				}
+			else
+				{
+				work = work->next;
+				}
+			}
+		}
+
 	if (dw->second_drop)
 		{
 		dupe_second_add(dw, di);
@@ -2095,6 +2171,7 @@ static void dupe_menu_select_all_cb(GtkWidget *widget, gpointer data)
 	DupeWindow *dw = data;
 	GtkTreeSelection *selection;
 
+	options->duplicates_select_type = DUPE_SELECT_NONE;
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(dw->listview));
 	gtk_tree_selection_select_all(selection);
 }
@@ -2104,6 +2181,7 @@ static void dupe_menu_select_none_cb(GtkWidget *widget, gpointer data)
 	DupeWindow *dw = data;
 	GtkTreeSelection *selection;
 
+	options->duplicates_select_type = DUPE_SELECT_NONE;
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(dw->listview));
 	gtk_tree_selection_unselect_all(selection);
 }
@@ -2112,14 +2190,16 @@ static void dupe_menu_select_dupes_set1_cb(GtkWidget *widget, gpointer data)
 {
 	DupeWindow *dw = data;
 
-	dupe_listview_select_dupes(dw, TRUE);
+	options->duplicates_select_type = DUPE_SELECT_GROUP1;
+	dupe_listview_select_dupes(dw, DUPE_SELECT_GROUP1);
 }
 
 static void dupe_menu_select_dupes_set2_cb(GtkWidget *widget, gpointer data)
 {
 	DupeWindow *dw = data;
 
-	dupe_listview_select_dupes(dw, FALSE);
+	options->duplicates_select_type = DUPE_SELECT_GROUP2;
+	dupe_listview_select_dupes(dw, DUPE_SELECT_GROUP2);
 }
 
 static void dupe_menu_edit_cb(GtkWidget *widget, gpointer data)
@@ -2133,13 +2213,6 @@ static void dupe_menu_edit_cb(GtkWidget *widget, gpointer data)
 	dupe_window_edit_selected(dw, key);
 }
 
-static void dupe_menu_collection_cb(GtkWidget *widget, gpointer data)
-{
-	DupeWindow *dw = data;
-
-	dupe_window_collection_from_selection(dw);
-}
-
 static void dupe_menu_print_cb(GtkWidget *widget, gpointer data)
 {
 	DupeWindow *dw = data;
@@ -2177,6 +2250,15 @@ static void dupe_menu_delete_cb(GtkWidget *widget, gpointer data)
 {
 	DupeWindow *dw = data;
 
+	options->file_ops.safe_delete_enable = FALSE;
+	file_util_delete(NULL, dupe_listview_get_selection(dw, dw->listview), dw->window);
+}
+
+static void dupe_menu_move_to_trash_cb(GtkWidget *widget, gpointer data)
+{
+	DupeWindow *dw = data;
+
+	options->file_ops.safe_delete_enable = TRUE;
 	file_util_delete(NULL, dupe_listview_get_selection(dw, dw->listview), dw->window);
 }
 
@@ -2184,7 +2266,14 @@ static void dupe_menu_copy_path_cb(GtkWidget *widget, gpointer data)
 {
 	DupeWindow *dw = data;
 
-	file_util_copy_path_list_to_clipboard(dupe_listview_get_selection(dw, dw->listview));
+	file_util_copy_path_list_to_clipboard(dupe_listview_get_selection(dw, dw->listview), TRUE);
+}
+
+static void dupe_menu_copy_path_unquoted_cb(GtkWidget *widget, gpointer data)
+{
+	DupeWindow *dw = data;
+
+	file_util_copy_path_list_to_clipboard(dupe_listview_get_selection(dw, dw->listview), FALSE);
 }
 
 static void dupe_menu_remove_cb(GtkWidget *widget, gpointer data)
@@ -2231,12 +2320,32 @@ static GList *dupe_window_get_fd_list(DupeWindow *dw)
 	return list;
 }
 
+/**
+ * @brief Add file selection list to a collection
+ * @param[in] widget 
+ * @param[in] data Index to the collection list menu item selected, or -1 for new collection
+ * 
+ * 
+ */
+static void dupe_pop_menu_collections_cb(GtkWidget *widget, gpointer data)
+{
+	DupeWindow *dw;
+	GList *selection_list;
+
+	dw = submenu_item_get_data(widget);
+	selection_list = dupe_listview_get_selection(dw, dw->listview);
+	pop_menu_collections(selection_list, data);
+
+	filelist_free(selection_list);
+}
+
 static GtkWidget *dupe_menu_popup_main(DupeWindow *dw, DupeItem *di)
 {
 	GtkWidget *menu;
 	GtkWidget *item;
 	gint on_row;
 	GList *editmenu_fd_list;
+	GtkWidget *submenu;
 
 	on_row = (di != NULL);
 
@@ -2262,8 +2371,11 @@ static GtkWidget *dupe_menu_popup_main(DupeWindow *dw, DupeItem *di)
 			 G_CALLBACK(dupe_menu_popup_destroy_cb), editmenu_fd_list);
 	submenu_add_edit(menu, &item, G_CALLBACK(dupe_menu_edit_cb), dw, editmenu_fd_list);
 	if (!on_row) gtk_widget_set_sensitive(item, FALSE);
-	menu_item_add_stock_sensitive(menu, _("Add to new collection"), GTK_STOCK_INDEX, on_row,
-				G_CALLBACK(dupe_menu_collection_cb), dw);
+
+	submenu_add_collections(menu, &item,
+								G_CALLBACK(dupe_pop_menu_collections_cb), dw);
+	gtk_widget_set_sensitive(item, on_row);
+
 	menu_item_add_stock_sensitive(menu, _("Print..."), GTK_STOCK_PRINT, on_row,
 				G_CALLBACK(dupe_menu_print_cb), dw);
 	menu_item_add_divider(menu);
@@ -2273,10 +2385,21 @@ static GtkWidget *dupe_menu_popup_main(DupeWindow *dw, DupeItem *di)
 				G_CALLBACK(dupe_menu_move_cb), dw);
 	menu_item_add_sensitive(menu, _("_Rename..."), on_row,
 				G_CALLBACK(dupe_menu_rename_cb), dw);
-	menu_item_add_stock_sensitive(menu, _("_Delete..."), GTK_STOCK_DELETE, on_row,
-				G_CALLBACK(dupe_menu_delete_cb), dw);
 	menu_item_add_sensitive(menu, _("_Copy path"), on_row,
 				G_CALLBACK(dupe_menu_copy_path_cb), dw);
+	menu_item_add_sensitive(menu, _("_Copy path unquoted"), on_row,
+				G_CALLBACK(dupe_menu_copy_path_unquoted_cb), dw);
+
+	menu_item_add_divider(menu);
+	menu_item_add_stock_sensitive(menu,
+				options->file_ops.confirm_move_to_trash ? _("Move to Trash...") :
+					_("Move to Trash"), PIXBUF_INLINE_ICON_TRASH, on_row,
+				G_CALLBACK(dupe_menu_move_to_trash_cb), dw);
+	menu_item_add_stock_sensitive(menu,
+				options->file_ops.confirm_delete ? _("_Delete...") :
+					_("_Delete"), GTK_STOCK_DELETE, on_row,
+				G_CALLBACK(dupe_menu_delete_cb), dw);
+
 	menu_item_add_divider(menu);
 	menu_item_add_stock_sensitive(menu, _("Rem_ove"), GTK_STOCK_REMOVE, on_row,
 				G_CALLBACK(dupe_menu_remove_cb), dw);
@@ -2598,6 +2721,15 @@ static void dupe_second_set_toggle_cb(GtkWidget *widget, gpointer data)
 	dupe_window_recompare(dw);
 }
 
+static void dupe_sort_totals_toggle_cb(GtkWidget *widget, gpointer data)
+{
+	DupeWindow *dw = data;
+
+	options->sort_totals = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+	dupe_window_recompare(dw);
+
+}
+
 /*
  *-------------------------------------------------------------------
  * match type menu
@@ -2619,6 +2751,8 @@ static void dupe_menu_type_cb(GtkWidget *combo, gpointer data)
 	if (!gtk_combo_box_get_active_iter(GTK_COMBO_BOX(combo), &iter)) return;
 	gtk_tree_model_get(store, &iter, DUPE_MENU_COLUMN_MASK, &dw->match_mask, -1);
 
+	options->duplicates_match = dw->match_mask;
+
 	dupe_window_recompare(dw);
 }
 
@@ -2793,6 +2927,7 @@ static void dupe_window_show_thumb_cb(GtkWidget *widget, gpointer data)
 	DupeWindow *dw = data;
 
 	dw->show_thumbs = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+	options->duplicates_thumbnails = dw->show_thumbs;
 
 	if (dw->show_thumbs)
 		{
@@ -2821,6 +2956,41 @@ static void dupe_window_show_thumb_cb(GtkWidget *widget, gpointer data)
 	dupe_listview_set_height(dw->listview, dw->show_thumbs);
 }
 
+static void dupe_window_rotation_invariant_cb(GtkWidget *widget, gpointer data)
+{
+	DupeWindow *dw = data;
+
+	options->rot_invariant_sim = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+	dupe_window_recompare(dw);
+}
+
+static void dupe_window_custom_threshold_cb(GtkWidget *widget, gpointer data)
+{
+	DupeWindow *dw = data;
+	DupeMatchType match_type;
+	GtkTreeModel *store;
+	gboolean valid;
+	GtkTreeIter iter;
+
+	options->duplicates_similarity_threshold = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
+	dw->match_mask = DUPE_MATCH_SIM_CUSTOM;
+
+	store = gtk_combo_box_get_model(GTK_COMBO_BOX(dw->combo));
+	valid = gtk_tree_model_get_iter_first(store, &iter);
+	while (valid)
+		{
+		gtk_tree_model_get(store, &iter, DUPE_MENU_COLUMN_MASK, &match_type, -1);
+		if (match_type == DUPE_MATCH_SIM_CUSTOM)
+			{
+			break;
+			}
+		valid = gtk_tree_model_iter_next(store, &iter);
+		}
+
+	gtk_combo_box_set_active_iter(GTK_COMBO_BOX(dw->combo), &iter);
+	dupe_window_recompare(dw);
+}
+
 static void dupe_popup_menu_pos_cb(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer data)
 {
 	GtkWidget *view = data;
@@ -2918,6 +3088,7 @@ static gboolean dupe_window_keypress_cb(GtkWidget *widget, GdkEventKey *event, g
 					file_util_rename(NULL, dupe_listview_get_selection(dw, listview), dw->window);
 					break;
 				case 'D': case 'd':
+					options->file_ops.safe_delete_enable = TRUE;
 					file_util_delete(NULL, dupe_listview_get_selection(dw, listview), dw->window);
 					break;
 				default:
@@ -2989,10 +3160,12 @@ static gboolean dupe_window_keypress_cb(GtkWidget *widget, GdkEventKey *event, g
 					}
 				break;
 			case '1':
-				dupe_listview_select_dupes(dw, TRUE);
+				options->duplicates_select_type = DUPE_SELECT_GROUP1;
+				dupe_listview_select_dupes(dw, DUPE_SELECT_GROUP1);
 				break;
 			case '2':
-				dupe_listview_select_dupes(dw, FALSE);
+				options->duplicates_select_type = DUPE_SELECT_GROUP2;
+				dupe_listview_select_dupes(dw, DUPE_SELECT_GROUP2);
 				break;
 			case GDK_KEY_Menu:
 			case GDK_KEY_F10:
@@ -3018,6 +3191,11 @@ static gboolean dupe_window_keypress_cb(GtkWidget *widget, GdkEventKey *event, g
 				break;
 			}
 		}
+	if (!stop_signal && is_help_key(event))
+		{
+		help_window_show("GuideImageSearchFindingDuplicates.html");
+		stop_signal = TRUE;
+		}
 
 	return stop_signal;
 }
@@ -3071,7 +3249,7 @@ static gint dupe_window_delete(GtkWidget *widget, GdkEvent *event, gpointer data
 }
 
 /* collection and files can be NULL */
-DupeWindow *dupe_window_new(DupeMatchType match_mask)
+DupeWindow *dupe_window_new()
 {
 	DupeWindow *dw;
 	GtkWidget *vbox;
@@ -3086,9 +3264,21 @@ DupeWindow *dupe_window_new(DupeMatchType match_mask)
 
 	dw = g_new0(DupeWindow, 1);
 
-	dw->match_mask = match_mask;
+	dw->match_mask = DUPE_MATCH_NAME;
+	if (options->duplicates_match == DUPE_MATCH_NAME) dw->match_mask = DUPE_MATCH_NAME;
+	if (options->duplicates_match == DUPE_MATCH_SIZE) dw->match_mask = DUPE_MATCH_SIZE;
+	if (options->duplicates_match == DUPE_MATCH_DATE) dw->match_mask = DUPE_MATCH_DATE;
+	if (options->duplicates_match == DUPE_MATCH_DIM) dw->match_mask = DUPE_MATCH_DIM;
+	if (options->duplicates_match == DUPE_MATCH_SUM) dw->match_mask = DUPE_MATCH_SUM;
+	if (options->duplicates_match == DUPE_MATCH_PATH) dw->match_mask = DUPE_MATCH_PATH;
+	if (options->duplicates_match == DUPE_MATCH_SIM_HIGH) dw->match_mask = DUPE_MATCH_SIM_HIGH;
+	if (options->duplicates_match == DUPE_MATCH_SIM_MED) dw->match_mask = DUPE_MATCH_SIM_MED;
+	if (options->duplicates_match == DUPE_MATCH_SIM_LOW) dw->match_mask = DUPE_MATCH_SIM_LOW;
+	if (options->duplicates_match == DUPE_MATCH_SIM_CUSTOM) dw->match_mask = DUPE_MATCH_SIM_CUSTOM;
+	if (options->duplicates_match == DUPE_MATCH_NAME_CI) dw->match_mask = DUPE_MATCH_NAME_CI;
 
 	dw->window = window_new(GTK_WINDOW_TOPLEVEL, "dupe", NULL, NULL, _("Find duplicates"));
+	DEBUG_NAME(dw->window);
 
 	geometry.min_width = DEFAULT_MINIMAL_WINDOW_SIZE;
 	geometry.min_height = DEFAULT_MINIMAL_WINDOW_SIZE;
@@ -3192,12 +3382,21 @@ DupeWindow *dupe_window_new(DupeMatchType match_mask)
 	gtk_widget_show(dw->combo);
 
 	dw->button_thumbs = gtk_check_button_new_with_label(_("Thumbnails"));
+	dw->show_thumbs = options->duplicates_thumbnails;
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dw->button_thumbs), dw->show_thumbs);
 	g_signal_connect(G_OBJECT(dw->button_thumbs), "toggled",
 			 G_CALLBACK(dupe_window_show_thumb_cb), dw);
 	gtk_box_pack_start(GTK_BOX(status_box), dw->button_thumbs, FALSE, FALSE, PREF_PAD_SPACE);
 	gtk_widget_show(dw->button_thumbs);
 
+	dw->button_rotation_invariant = gtk_check_button_new_with_label(_("Ignore Rotation"));
+	gtk_widget_set_tooltip_text(GTK_WIDGET(dw->button_rotation_invariant), "Ignore image orientation");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dw->button_rotation_invariant), options->rot_invariant_sim);
+	g_signal_connect(G_OBJECT(dw->button_rotation_invariant), "toggled",
+			 G_CALLBACK(dupe_window_rotation_invariant_cb), dw);
+	gtk_box_pack_start(GTK_BOX(status_box), dw->button_rotation_invariant, FALSE, FALSE, PREF_PAD_SPACE);
+	gtk_widget_show(dw->button_rotation_invariant);
+
 	button = gtk_check_button_new_with_label(_("Compare two file sets"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), dw->second_set);
 	g_signal_connect(G_OBJECT(button), "toggled",
@@ -3210,6 +3409,7 @@ DupeWindow *dupe_window_new(DupeMatchType match_mask)
 	gtk_widget_show(status_box);
 
 	frame = gtk_frame_new(NULL);
+	DEBUG_NAME(frame);
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
 	gtk_box_pack_start(GTK_BOX(status_box), frame, TRUE, TRUE, 0);
 	gtk_widget_show(frame);
@@ -3218,8 +3418,31 @@ DupeWindow *dupe_window_new(DupeMatchType match_mask)
 	gtk_container_add(GTK_CONTAINER(frame), dw->status_label);
 	gtk_widget_show(dw->status_label);
 
+	button = gtk_check_button_new_with_label(_("Sort"));
+	gtk_widget_set_tooltip_text(GTK_WIDGET(button), "Sort by group totals");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), options->sort_totals);
+	g_signal_connect(G_OBJECT(button), "toggled",
+			 G_CALLBACK(dupe_sort_totals_toggle_cb), dw);
+	gtk_box_pack_start(GTK_BOX(status_box), button, FALSE, FALSE, PREF_PAD_SPACE);
+	gtk_widget_show(button);
+
+	label = gtk_label_new(_("Custom Threshold"));
+	gtk_box_pack_start(GTK_BOX(status_box), label, FALSE, FALSE, PREF_PAD_SPACE);
+	gtk_widget_show(label);
+	dw->custom_threshold = gtk_spin_button_new_with_range(1, 100, 1);
+	gtk_widget_set_tooltip_text(GTK_WIDGET(dw->custom_threshold), "Custom similarity threshold");
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(dw->custom_threshold), options->duplicates_similarity_threshold);
+	g_signal_connect(G_OBJECT(dw->custom_threshold), "value_changed",
+													G_CALLBACK(dupe_window_custom_threshold_cb), dw);
+	gtk_box_pack_start(GTK_BOX(status_box), dw->custom_threshold, FALSE, FALSE, PREF_PAD_SPACE);
+	gtk_widget_show(dw->custom_threshold);
+
 	dw->extra_label = gtk_progress_bar_new();
 	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(dw->extra_label), 0.0);
+#if GTK_CHECK_VERSION(3,0,0)
+	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(dw->extra_label), "");
+	gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(dw->extra_label), TRUE);
+#endif
 	gtk_box_pack_end(GTK_BOX(status_box), dw->extra_label, FALSE, FALSE, 0);
 	gtk_widget_show(dw->extra_label);
 
