@@ -111,23 +111,34 @@ static GList *editor_mime_types_to_extensions(gchar **mime_types)
 		{"image/*",		"*"},
 		{"image/bmp",		".bmp"},
 		{"image/gif",		".gif"},
-		{"image/jpeg",		".jpeg;.jpg"},
+		{"image/jpeg",		".jpeg;.jpg;.mpo"},
 		{"image/jpg",		".jpg;.jpeg"},
 		{"image/pcx",		".pcx"},
 		{"image/png",		".png"},
 		{"image/svg",		".svg"},
 		{"image/svg+xml",	".svg"},
 		{"image/svg+xml-compressed", 	".svg"},
-		{"image/tiff",		".tiff;.tif"},
+		{"image/tiff",		".tiff;.tif;.mef"},
+		{"image/vnd-ms.dds",	".dds"},
+		{"image/x-adobe-dng",	".dng"},
 		{"image/x-bmp",		".bmp"},
 		{"image/x-canon-crw",	".crw"},
+		{"image/x-canon-cr2",	".cr2"},
 		{"image/x-cr2",		".cr2"},
-		{"image/x-dcraw",	"%raw"},
+		{"image/x-dcraw",	"%raw;.mos"},
+		{"image/x-epson-erf",	"%erf"},
 		{"image/x-ico",		".ico"},
+		{"image/x-kodak-kdc",	".kdc"},
 		{"image/x-mrw",		".mrw"},
+		{"image/x-minolta-mrw",	".mrw"},
 		{"image/x-MS-bmp",	".bmp"},
 		{"image/x-nef",		".nef"},
+		{"image/x-nikon-nef",	".nef"},
+		{"image/x-panasonic-raw",	".raw"},
+		{"image/x-panasonic-rw2",	".rw2"},
+		{"image/x-pentax-pef",	".pef"},
 		{"image/x-orf",		".orf"},
+		{"image/x-olympus-orf",	".orf"},
 		{"image/x-pcx",		".pcx"},
 		{"image/xpm",		".xpm"},
 		{"image/x-png",		".png"},
@@ -137,12 +148,17 @@ static GList *editor_mime_types_to_extensions(gchar **mime_types)
 		{"image/x-portable-pixmap",	".ppm"},
 		{"image/x-psd",		".psd"},
 		{"image/x-raf",		".raf"},
+		{"image/x-fuji-raf",	".raf"},
 		{"image/x-sgi",		".sgi"},
+		{"image/x-sony-arw",	".arw"},
+		{"image/x-sony-sr2",	".sr2"},
+		{"image/x-sony-srf",	".srf"},
 		{"image/x-tga",		".tga"},
 		{"image/x-xbitmap",	".xbm"},
 		{"image/x-xcf",		".xcf"},
 		{"image/x-xpixmap",	".xpm"},
 		{"image/x-x3f",		".x3f"},
+		{"application/x-navi-animation",		".ani"},
 		{"application/x-ptoptimizer-script",	".pto"},
 		{NULL, NULL}};
 
@@ -168,6 +184,8 @@ gboolean editor_read_desktop_file(const gchar *path)
 	gchar *try_exec;
 	GtkTreeIter iter;
 	gboolean category_geeqie = FALSE;
+	GList *work;
+	gboolean disabled;
 
 	if (g_hash_table_lookup(editors, key)) return FALSE; /* the file found earlier wins */
 
@@ -302,7 +320,7 @@ gboolean editor_read_desktop_file(const gchar *path)
 	editor->exec = g_key_file_get_string(key_file, DESKTOP_GROUP, "Exec", NULL);
 
 	editor->menu_path = g_key_file_get_string(key_file, DESKTOP_GROUP, "X-Geeqie-Menu-Path", NULL);
-	if (!editor->menu_path) editor->menu_path = g_strdup("EditMenu/ExternalMenu");
+	if (!editor->menu_path) editor->menu_path = g_strdup("PluginsMenu");
 
 	editor->hotkey = g_key_file_get_string(key_file, DESKTOP_GROUP, "X-Geeqie-Hotkey", NULL);
 
@@ -336,9 +354,25 @@ gboolean editor_read_desktop_file(const gchar *path)
 
 	if (editor->ignored) return TRUE;
 
+	work = options->disabled_plugins;
+
+	disabled = FALSE;
+	while (work)
+		{
+		if (g_strcmp0(path, work->data) == 0)
+			{
+			disabled = TRUE;
+			break;
+			}
+		work = work->next;
+		}
+
+	editor->disabled = disabled;
+
 	gtk_list_store_append(desktop_file_list, &iter);
 	gtk_list_store_set(desktop_file_list, &iter,
 			   DESKTOP_FILE_COLUMN_KEY, key,
+			   DESKTOP_FILE_COLUMN_DISABLED, editor->disabled,
 			   DESKTOP_FILE_COLUMN_NAME, editor->name,
 			   DESKTOP_FILE_COLUMN_HIDDEN, editor->hidden ? _("yes") : _("no"),
 			   DESKTOP_FILE_COLUMN_WRITABLE, access_file(path, W_OK),
@@ -367,7 +401,7 @@ void editor_table_clear(void)
 		}
 	else
 		{
-		desktop_file_list = gtk_list_store_new(DESKTOP_FILE_COLUMN_COUNT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_STRING);
+		desktop_file_list = gtk_list_store_new(DESKTOP_FILE_COLUMN_COUNT, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_STRING);
 		}
 	if (editors)
 		{
@@ -454,6 +488,11 @@ static void editor_list_add_cb(gpointer key, gpointer value, gpointer data)
 	    strcmp(editor->key, CMD_DELETE) == 0 ||
 	    strcmp(editor->key, CMD_FOLDER) == 0) return;
 
+	if (editor->disabled)
+		{
+		return;
+		}
+
 	*listp = g_list_prepend(*listp, editor);
 }
 
@@ -537,7 +576,7 @@ static EditorVerboseData *editor_verbose_window(EditorData *ed, const gchar *tex
 				   NULL, FALSE,
 				   NULL, ed);
 	buf = g_strdup_printf(_("Output of %s"), text);
-	generic_dialog_add_message(vd->gd, NULL, buf, NULL);
+	generic_dialog_add_message(vd->gd, NULL, buf, NULL, FALSE);
 	g_free(buf);
 	vd->button_stop = generic_dialog_add_button(vd->gd, GTK_STOCK_STOP, NULL,
 						   editor_verbose_window_stop, FALSE);
@@ -566,6 +605,10 @@ static EditorVerboseData *editor_verbose_window(EditorData *ed, const gchar *tex
 	vd->progress = gtk_progress_bar_new();
 	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(vd->progress), 0.0);
 	gtk_box_pack_start(GTK_BOX(hbox), vd->progress, TRUE, TRUE, 0);
+#if GTK_CHECK_VERSION(3,0,0)
+	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(vd->progress), "");
+	gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(vd->progress), TRUE);
+#endif
 	gtk_widget_show(vd->progress);
 
 	vd->spinner = spinner_new(NULL, SPINNER_SPEED);
@@ -1024,7 +1067,7 @@ static EditorFlags editor_command_one(const EditorDescription *editor, GList *li
 		args[n++] = command;
 		args[n] = NULL;
 
-		if ((ed->flags & EDITOR_DEST) && fd->change && fd->change->dest) /* FIXME: error handling */
+		if ((ed->flags & EDITOR_DEST) && fd && fd->change && fd->change->dest) /* FIXME: error handling */
 			{
 			g_setenv("GEEQIE_DESTINATION", fd->change->dest, TRUE);
 			}
