@@ -1,16 +1,24 @@
 /*
- * Geeqie
- * (C) 2004 John Ellis
- * Copyright (C) 2008 - 2012 The Geeqie Team
+ * Copyright (C) 2004 John Ellis
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
  * Author: John Ellis
  *
- * This software is released under the GNU General Public License (GNU GPL).
- * Please read the included file COPYING for more information.
- * This software comes with no warranty of any kind, use at your own risk!
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-
 #ifndef EDITORS_H
 #define EDITORS_H
 
@@ -50,6 +58,7 @@ struct _EditorDescription {
 	EditorFlags flags;
 	gboolean hidden;	/* explicitly hidden, shown in configuration dialog */
 	gboolean ignored;	/* not interesting, do not show at all */
+	gboolean disabled;	/* display disabled by user */
 };
 
 #define EDITOR_ERRORS(flags) ((flags) & EDITOR_ERROR_MASK)
@@ -66,6 +75,7 @@ enum {
 
 enum {
 	DESKTOP_FILE_COLUMN_KEY,
+	DESKTOP_FILE_COLUMN_DISABLED,
 	DESKTOP_FILE_COLUMN_NAME,
 	DESKTOP_FILE_COLUMN_HIDDEN,
 	DESKTOP_FILE_COLUMN_WRITABLE,
