@@ -1,9 +1,21 @@
 /*
- * Geeqie
- * (C) 2006 John Ellis
- * Copyright (C) 2008 - 2012 The Geeqie Team
+ * Copyright (C) 2006 John Ellis
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
-*/
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -41,9 +53,12 @@
 #include "filefilter.h"
 #include "filecache.h"
 #include "format_raw.h"
+#include "glua.h"
 #include "ui_fileops.h"
 #include "cache.h"
 #include "jpeg_parser.h"
+#include "misc.h"
+#include "zonedetect.h"
 
 
 static gdouble exif_rational_to_double(ExifRational *r, gint sign)
@@ -238,6 +253,57 @@ static gchar *exif_build_formatted_DateTime(ExifData *exif)
 	return text;
 }
 
+static gchar *exif_build_formatted_DateTimeDigitized(ExifData *exif)
+{
+	gchar *text = exif_get_data_as_text(exif, "Exif.Photo.DateTimeDigitized");
+	gchar *subsec = NULL;
+	gchar buf[128];
+	gchar *tmp;
+	gint buflen;
+	struct tm tm;
+	GError *error = NULL;
+
+	if (text)
+		{
+		subsec = exif_get_data_as_text(exif, "Exif.Photo.SubSecTimeDigitized");
+		}
+	else
+		{
+		text = exif_get_data_as_text(exif, "Exif.Image.DateTime");
+		if (text) subsec = exif_get_data_as_text(exif, "Exif.Photo.SubSecTime");
+		}
+
+	/* Convert the stuff into a tm struct */
+	memset(&tm, 0, sizeof(tm)); /* Uh, strptime could let garbage in tm! */
+	if (text && strptime(text, "%Y:%m:%d %H:%M:%S", &tm))
+		{
+		buflen = strftime(buf, sizeof(buf), "%x %X", &tm);
+		if (buflen > 0)
+			{
+			tmp = g_locale_to_utf8(buf, buflen, NULL, NULL, &error);
+			if (error)
+				{
+				log_printf("Error converting locale strftime to UTF-8: %s\n", error->message);
+				g_error_free(error);
+				}
+			else
+				{
+				g_free(text);
+				text = g_strdup(tmp);
+				}
+			}
+		}
+
+	if (subsec)
+		{
+		tmp = text;
+		text = g_strconcat(tmp, ".", subsec, NULL);
+		g_free(tmp);
+		g_free(subsec);
+		}
+	return text;
+}
+
 static gchar *exif_build_formatted_ShutterSpeed(ExifData *exif)
 {
 	ExifRational *r;
@@ -320,6 +386,8 @@ static gchar *exif_build_formatted_ISOSpeedRating(ExifData *exif)
 	gchar *text;
 
 	text = exif_get_data_as_text(exif, "Exif.Photo.ISOSpeedRatings");
+	/* old canon may set this instead */
+	if (!text) text = exif_get_data_as_text(exif, "Exif.CanonSi.ISOSpeed");
 	/* kodak may set this instead */
 	if (!text) text = exif_get_data_as_text(exif, "Exif.Photo.ExposureIndex");
 	return text;
@@ -544,6 +612,305 @@ static gchar *exif_build_formatted_GPSAltitude(ExifData *exif)
 	return g_strdup_printf("%0.f m %s", alt, (ref==0)?_("Above Sea Level"):_("Below Sea Level"));
 }
 
+/**
+ * @brief Extracts timezone data from a ZoneDetect search structure
+ * @param[in] results ZoneDetect search structure
+ * @param[out] timezone in the form "Europe/London"
+ * @param[out] countryname in the form "United Kingdom"
+ * @param[out] countryalpha2 in the form "GB"
+ * 
+ * Refer to https://github.com/BertoldVdb/ZoneDetect
+ * for structure details
+ */
+static void zd_tz(ZoneDetectResult *results, gchar **timezone, gchar **countryname, gchar **countryalpha2)
+{
+	gchar *timezone_pre = NULL;
+	gchar *timezone_id = NULL;
+	unsigned int index = 0;
+
+	while(results[index].lookupResult != ZD_LOOKUP_END)
+		{
+		if(results[index].data)
+			{
+			for(unsigned int i=0; i<results[index].numFields; i++)
+				{
+				if (g_strstr_len(results[index].fieldNames[i], -1, "TimezoneIdPrefix"))
+					{
+					timezone_pre = g_strdup(results[index].data[i]);
+					}
+				if (g_strstr_len(results[index].fieldNames[i], -1, "TimezoneId"))
+					{
+					timezone_id = g_strdup(results[index].data[i]);
+					}
+				if (g_strstr_len(results[index].fieldNames[i], -1, "CountryName"))
+					{
+					*countryname = g_strdup(results[index].data[i]);
+					}
+				if (g_strstr_len(results[index].fieldNames[i], -1, "CountryAlpha2"))
+					{
+					*countryalpha2 = g_strdup(results[index].data[i]);
+					}
+				}
+			}
+		index++;
+		}
+
+	*timezone = g_strconcat(timezone_pre, timezone_id, NULL);
+	g_free(timezone_pre);
+	g_free(timezone_id);
+}
+
+/**
+ * @brief Gets timezone data from an exif structure
+ * @param[in] exif
+ * @returns TRUE if timezone data found AND GPS date and time found
+ * @param[out] exif_date_time exif date/time in the form 2018:11:30:17:05:04
+ * @param[out] timezone in the form "Europe/London"
+ * @param[out] countryname in the form "United Kingdom"
+ * @param[out] countryalpha2 in the form "GB"
+ *
+ *
+ */
+static gboolean exif_build_tz_data(ExifData *exif, gchar **exif_date_time, gchar **timezone, gchar **countryname, gchar **countryalpha2)
+{
+	gfloat latitude;
+	gfloat longitude;
+	gchar *text_latitude;
+	gchar *text_longitude;
+	gchar *text_latitude_ref;
+	gchar *text_longitude_ref;
+	gchar *text_date;
+	gchar *text_time;
+	gchar *lat_deg;
+	gchar *lat_min;
+	gchar *lon_deg;
+	gchar *lon_min;
+	gchar *timezone_path;
+	ZoneDetect *cd;
+	ZoneDetectResult *results;
+	gboolean ret = FALSE;
+	gchar *basename;
+	gchar *path;
+
+	text_latitude = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSLatitude");
+	text_longitude = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSLongitude");
+	text_latitude_ref = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSLatitudeRef");
+	text_longitude_ref = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSLongitudeRef");
+	text_date = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSDateStamp");
+	text_time = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSTimeStamp");
+
+	if (text_latitude && text_longitude && text_latitude_ref && text_longitude_ref)
+		{
+		lat_deg = strtok(text_latitude, "deg'");
+		lat_min = strtok(NULL, "deg'");
+		if (!lat_deg || !lat_min)
+			{
+			return FALSE;
+			}
+		latitude = atof(lat_deg) + atof(lat_min) / 60;
+		if (!g_strcmp0(text_latitude_ref, "South"))
+			{
+			latitude = -latitude;
+			}
+		lon_deg = strtok(text_longitude, "deg'");
+		lon_min = strtok(NULL, "deg'");
+		if (!lon_deg || !lon_min)
+			{
+			return FALSE;
+			}
+		longitude = atof(lon_deg) + atof(lon_min) / 60;
+		if (!g_strcmp0(text_longitude_ref, "West"))
+			{
+			longitude = -longitude;
+			}
+
+		path = path_from_utf8(TIMEZONE_DATABASE);
+		basename = g_path_get_basename(path);
+		timezone_path = g_build_filename(get_rc_dir(), basename, NULL);
+		if (g_file_test(timezone_path, G_FILE_TEST_EXISTS))
+			{
+			cd = ZDOpenDatabase(timezone_path);
+			if (cd)
+				{
+				results = ZDLookup(cd, latitude, longitude, NULL);
+				if (results)
+					{
+					zd_tz(results, timezone, countryname, countryalpha2);
+					ret = TRUE;
+					}
+				}
+			else
+				{
+				log_printf("Error: Init of timezone database %s failed\n", timezone_path);
+				}
+			ZDCloseDatabase(cd);
+			}
+		g_free(path);
+		g_free(timezone_path);
+		g_free(basename);
+		}
+
+	if (ret && text_date && text_time)
+		{
+		*exif_date_time = g_strconcat(text_date, ":", text_time, NULL);
+		}
+	else
+		{
+		ret = FALSE;
+		}
+	return ret;
+}
+
+/**
+ * @brief Creates local time from GPS lat/long
+ * @param[in] exif
+ * @returns Localised time and date
+ *
+ * GPS lat/long is translated to timezone using ZoneDetect.
+ * GPS UTC is converted to Unix time stamp (seconds since 1970).
+ * The TZ environment variable is set to the relevant timezone
+ * and the Unix timestamp converted to local time using locale.
+ * If the conversion fails, unformatted UTC is returned.
+ */
+static gchar *exif_build_formatted_localtime(ExifData *exif)
+{
+	gchar buf[128];
+	gchar *tmp;
+	gint buflen;
+	GError *error = NULL;
+	gchar *time_zone_image;
+	gchar *time_zone_org;
+	struct tm *tm_local;
+	struct tm tm_utc;
+	time_t stamp;
+	gchar *exif_date_time = NULL;
+	gchar *timezone = NULL;
+	gchar *countryname = NULL;
+	gchar *countryalpha2 = NULL;
+
+	if (exif_build_tz_data(exif, &exif_date_time, &timezone, &countryname, &countryalpha2))
+		{
+		time_zone_image = g_strconcat("TZ=", timezone, NULL);
+		time_zone_org = g_strconcat("TZ=", getenv("TZ"), NULL);
+		putenv("TZ=UTC");
+
+		memset(&tm_utc, 0, sizeof(tm_utc));
+		if (exif_date_time && strptime(exif_date_time, "%Y:%m:%d:%H:%M:%S", &tm_utc))
+			{
+			stamp = mktime(&tm_utc);	// Convert the struct to a Unix timestamp
+			putenv(time_zone_image);	// Switch to destination time zone
+
+			tm_local = localtime(&stamp);
+
+			/* Convert to localtime using locale */
+			buflen = strftime(buf, sizeof(buf), "%x %X", tm_local);
+			if (buflen > 0)
+				{
+				tmp = g_locale_to_utf8(buf, buflen, NULL, NULL, &error);
+				if (error)
+					{
+					log_printf("Error converting locale strftime to UTF-8: %s\n", error->message);
+					g_error_free(error);
+					}
+				else
+					{
+					g_free(exif_date_time);
+					exif_date_time = g_strdup(tmp);
+					}
+				}
+				g_free(tmp);
+			}
+		putenv(time_zone_org);
+
+		g_free(time_zone_image);
+		g_free(time_zone_org);
+		}
+
+	g_free(timezone);
+	g_free(countryname);
+	g_free(countryalpha2);
+
+	return exif_date_time;
+}
+
+/**
+ * @brief Gets timezone from GPS lat/long
+ * @param[in] exif
+ * @returns Timezone string in the form "Europe/London"
+ *
+ *
+ */
+static gchar *exif_build_formatted_timezone(ExifData *exif)
+{
+	gchar *time_zone = NULL;
+	gchar *exif_date_time = NULL;
+	gchar *timezone = NULL;
+	gchar *countryname = NULL;
+	gchar *countryalpha2 = NULL;
+
+	exif_build_tz_data(exif, &exif_date_time, &timezone, &countryname, &countryalpha2);
+
+	g_free(exif_date_time);
+	g_free(countryname);
+	g_free(countryalpha2);
+
+	return timezone;
+}
+
+/**
+ * @brief Gets countryname from GPS lat/long
+ * @param[in] exif
+ * @returns Countryname string
+ *
+ *
+ */
+static gchar *exif_build_formatted_countryname(ExifData *exif)
+{
+	gchar *exif_date_time = NULL;
+	gchar *timezone = NULL;
+	gchar *countryname = NULL;
+	gchar *countryalpha2 = NULL;
+
+	exif_build_tz_data(exif, &exif_date_time, &timezone, &countryname, &countryalpha2);
+
+	g_free(exif_date_time);
+	g_free(timezone);
+	g_free(countryalpha2);
+
+	return countryname;
+}
+
+/**
+ * @brief Gets two-letter country code from GPS lat/long
+ * @param[in] exif
+ * @returns Countryalpha2 string
+ *
+ *
+ */
+static gchar *exif_build_formatted_countrycode(ExifData *exif)
+{
+	gchar *exif_date_time = NULL;
+	gchar *timezone = NULL;
+	gchar *countryname = NULL;
+	gchar *countryalpha2 = NULL;
+
+	exif_build_tz_data(exif, &exif_date_time, &timezone, &countryname, &countryalpha2);
+
+	g_free(exif_date_time);
+	g_free(timezone);
+	g_free(countryname);
+
+	return countryalpha2;
+}
+
+static gchar *exif_build_formatted_star_rating(ExifData *exif)
+{
+	gint n = 0;
+
+	exif_get_integer(exif, "Xmp.xmp.Rating", &n);
+
+	return convert_rating_to_stars(n);
+}
 
 /* List of custom formatted pseudo-exif tags */
 #define EXIF_FORMATTED_TAG(name, label) { EXIF_FORMATTED()#name, label, exif_build_formatted##_##name }
@@ -551,6 +918,7 @@ static gchar *exif_build_formatted_GPSAltitude(ExifData *exif)
 ExifFormattedText ExifFormattedList[] = {
 	EXIF_FORMATTED_TAG(Camera,		N_("Camera")),
 	EXIF_FORMATTED_TAG(DateTime,		N_("Date")),
+	EXIF_FORMATTED_TAG(DateTimeDigitized,	N_("DateDigitized")),
 	EXIF_FORMATTED_TAG(ShutterSpeed,	N_("Shutter speed")),
 	EXIF_FORMATTED_TAG(Aperture,		N_("Aperture")),
 	EXIF_FORMATTED_TAG(ExposureBias,	N_("Exposure bias")),
@@ -563,9 +931,19 @@ ExifFormattedText ExifFormattedList[] = {
 	EXIF_FORMATTED_TAG(ColorProfile,	N_("Color profile")),
 	EXIF_FORMATTED_TAG(GPSPosition,		N_("GPS position")),
 	EXIF_FORMATTED_TAG(GPSAltitude,		N_("GPS altitude")),
+	EXIF_FORMATTED_TAG(localtime,		N_("Local time")),
+	EXIF_FORMATTED_TAG(timezone,		N_("Time zone")),
+	EXIF_FORMATTED_TAG(countryname,		N_("Country name")),
+	EXIF_FORMATTED_TAG(countrycode,		N_("Country code")),
+	EXIF_FORMATTED_TAG(star_rating,		N_("Star rating")),
 	{"file.size",				N_("File size"), 	NULL},
 	{"file.date",				N_("File date"), 	NULL},
 	{"file.mode",				N_("File mode"), 	NULL},
+	{"file.ctime",				N_("File ctime"), 	NULL},
+	{"file.owner",				N_("File owner"), 	NULL},
+	{"file.group",				N_("File group"), 	NULL},
+	{"file.link",				N_("File link"), 	NULL},
+	{"file.class",				N_("File class"), 	NULL},
 	{ NULL, NULL, NULL }
 };
 
@@ -826,8 +1204,51 @@ gchar *metadata_file_info(FileData *fd, const gchar *key, MetadataFormat format)
 		{
 		return mode_number(fd->mode);
 		}
+	if (strcmp(key, "file.ctime") == 0)
+		{
+		return g_strdup(text_from_time(fd->cdate));
+		}
+	if (strcmp(key, "file.class") == 0)
+		{
+		return g_strdup(format_class_list[fd->format_class]);
+		}
+	if (strcmp(key, "file.owner") == 0)
+		{
+		return g_strdup(fd->owner);
+		}
+	if (strcmp(key, "file.group") == 0)
+		{
+		return g_strdup(fd->group);
+		}
+	if (strcmp(key, "file.link") == 0)
+		{
+		return g_strdup(fd->sym_link);
+		}
 	return g_strdup("");
 }
 
+#ifdef HAVE_LUA
+gchar *metadata_lua_info(FileData *fd, const gchar *key, MetadataFormat format)
+{
+	gchar *script_name;
+	gchar *script_name_utf8;
+	gchar *data;
+	gchar *raw_data;
+	gchar *valid_data;
 
+	script_name_utf8 = g_strdup(key + 4);
+	script_name = path_from_utf8(script_name_utf8);
+
+	raw_data = lua_callvalue(fd, script_name, NULL);
+	valid_data = g_utf8_make_valid(raw_data, -1);
+	data = g_utf8_substring(valid_data, 0, 150);
+
+	g_free(script_name);
+	g_free(script_name_utf8);
+	g_free(raw_data);
+	g_free(valid_data);
+
+	return data;
+}
+#endif
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
