@@ -86,6 +86,7 @@ GList *filelist_filter(GList *list, gboolean is_dir_list);
 
 GList *filelist_sort_path(GList *list);
 GList *filelist_recursive(FileData *dir_fd);
+GList *filelist_recursive_full(FileData *dir_fd, SortType method, gboolean ascend);
 
 typedef gboolean (* FileDataGetMarkFunc)(FileData *fd, gint n, gpointer data);
 typedef gboolean (* FileDataSetMarkFunc)(FileData *fd, gint n, gboolean value, gpointer data);
@@ -99,6 +100,9 @@ void file_data_set_mark(FileData *fd, gint n, gboolean value);
 gboolean file_data_filter_marks(FileData *fd, guint filter);
 GList *file_data_filter_marks_list(GList *list, guint filter);
 
+gboolean file_data_filter_file_filter(FileData *fd, GRegex *filter);
+GList *file_data_filter_file_filter_list(GList *list, GRegex *filter);
+
 gint file_data_get_user_orientation(FileData *fd);
 void file_data_set_user_orientation(FileData *fd, gint value);
 
@@ -161,5 +165,12 @@ void file_data_send_notification(FileData *fd, NotifyType type);
 gboolean file_data_register_real_time_monitor(FileData *fd);
 gboolean file_data_unregister_real_time_monitor(FileData *fd);
 
+void read_exif_time_data(FileData *file);
+void read_exif_time_digitized_data(FileData *file);
+
+gboolean marks_list_save(gchar *path, gboolean clear);
+gboolean marks_list_load(const gchar *path);
+void marks_clear_all();
+void read_rating_data(FileData *file);
 #endif
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
