@@ -99,7 +99,7 @@ static void fullscreen_hide_mouse_reset(FullScreenData *fs)
 	fs->hide_mouse_id = g_timeout_add(FULL_SCREEN_HIDE_MOUSE_DELAY, fullscreen_hide_mouse_cb, fs);
 }
 
-static gboolean fullscreen_mouse_moved(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gboolean fullscreen_mouse_moved(GtkWidget *widget, GdkEventMotion *event, gpointer data)
 {
 	FullScreenData *fs = data;
 
@@ -240,6 +240,7 @@ FullScreenData *fullscreen_start(GtkWidget *window, ImageWindow *imd,
 				      &screen, &fs->same_region);
 
 	fs->window = window_new(GTK_WINDOW_TOPLEVEL, "fullscreen", NULL, NULL, _("Full screen"));
+	DEBUG_NAME(fs->window);
 
 	g_signal_connect(G_OBJECT(fs->window), "delete_event",
 			 G_CALLBACK(fullscreen_delete_cb), fs);
@@ -248,14 +249,6 @@ FullScreenData *fullscreen_start(GtkWidget *window, ImageWindow *imd,
 	gtk_window_set_decorated(GTK_WINDOW(fs->window), FALSE);
 	gtk_container_set_border_width(GTK_CONTAINER(fs->window), 0);
 
-	/* make window fullscreen -- let Gtk do it's job, don't screw it in any way */
-	gtk_window_fullscreen(GTK_WINDOW(fs->window));
-
-	/* move it to requested screen */
-	if (options->fullscreen.screen >= 0) {
-		gtk_window_set_screen(GTK_WINDOW(fs->window), screen);
-	}
-
 	/* keep window above others, if requested */
 	if (options->fullscreen.above) {
 		gtk_window_set_keep_above(GTK_WINDOW(fs->window), TRUE);
@@ -281,6 +274,23 @@ FullScreenData *fullscreen_start(GtkWidget *window, ImageWindow *imd,
 			GDK_HINT_WIN_GRAVITY | GDK_HINT_USER_POS | GDK_HINT_USER_SIZE);
 
 	gtk_widget_realize(fs->window);
+#if GTK_CHECK_VERSION(3,8,0)
+	if ((options->fullscreen.screen % 100) == 0)
+		{
+		GdkWindow *gdkwin;
+		gdkwin = gtk_widget_get_window(fs->window);
+		if (gdkwin != NULL)
+			gdk_window_set_fullscreen_mode(gdkwin, GDK_FULLSCREEN_ON_ALL_MONITORS);
+		}
+#endif
+	/* make window fullscreen -- let Gtk do it's job, don't screw it in any way */
+	gtk_window_fullscreen(GTK_WINDOW(fs->window));
+
+	/* move it to requested screen */
+	if (options->fullscreen.screen >= 0)
+		{
+		gtk_window_set_screen(GTK_WINDOW(fs->window), screen);
+		}
 
 	fs->imd = image_new(FALSE);
 
@@ -300,12 +310,12 @@ FullScreenData *fullscreen_start(GtkWidget *window, ImageWindow *imd,
 
 	if (fs->same_region)
 		{
-		DEBUG_0("Original window is not visible, enabling std. fullscreen mode");
+		DEBUG_2("Original window is not visible, enabling std. fullscreen mode");
 		image_move_from_image(fs->imd, fs->normal_imd);
 		}
 	else
 		{
-		DEBUG_0("Original window is still visible, enabling presentation fullscreen mode");
+		DEBUG_2("Original window is still visible, enabling presentation fullscreen mode");
 		image_copy_from_image(fs->imd, fs->normal_imd);
 		}
 
@@ -364,6 +374,8 @@ void fullscreen_stop(FullScreenData *fs)
 
 	gtk_widget_destroy(fs->window);
 
+	gtk_window_present(GTK_WINDOW(fs->normal_window));
+
 	g_free(fs);
 }
 
@@ -644,6 +656,7 @@ GtkWidget *fullscreen_prefs_selection_new(const gchar *text, gint *screen_value,
 	if (!screen_value) return NULL;
 
 	vbox = gtk_vbox_new(FALSE, PREF_PAD_GAP);
+	DEBUG_NAME(vbox);
 	hbox = pref_box_new(vbox, FALSE, GTK_ORIENTATION_HORIZONTAL, PREF_PAD_SPACE);
 	if (text) pref_label_new(hbox, text);
 
