@@ -24,6 +24,10 @@
 #include "image_load_gdk.h"
 #include "image_load_jpeg.h"
 #include "image_load_tiff.h"
+#include "image_load_dds.h"
+#include "image_load_pdf.h"
+#include "image_load_ffmpegthumbnailer.h"
+#include "image_load_collection.h"
 
 #include "exif.h"
 #include "filedata.h"
@@ -539,9 +543,13 @@ static void image_loader_size_cb(gpointer loader,
 		}
 	g_mutex_unlock(il->data_mutex);
 
+#ifdef HAVE_FFMPEGTHUMBNAILER
+	if (il->fd->format_class == FORMAT_CLASS_VIDEO)
+		scale = TRUE;
+#endif
 	mime_types = il->backend.get_format_mime_types(loader);
 	n = 0;
-	while (mime_types[n])
+	while (mime_types[n] && !scale)
 		{
 		if (strstr(mime_types[n], "jpeg")) scale = TRUE;
 		n++;
@@ -603,6 +611,22 @@ static void image_loader_stop_loader(ImageLoader *il)
 static void image_loader_setup_loader(ImageLoader *il)
 {
 	g_mutex_lock(il->data_mutex);
+#ifdef HAVE_FFMPEGTHUMBNAILER
+	if (il->fd->format_class == FORMAT_CLASS_VIDEO)
+		{
+		DEBUG_1("Using custom ffmpegthumbnailer loader");
+		image_loader_backend_set_ft(&il->backend);
+		}
+	else
+#endif
+#ifdef HAVE_PDF
+	if (il->fd->format_class == FORMAT_CLASS_PDF)
+		{
+		DEBUG_1("Using custom pdf loader");
+		image_loader_backend_set_pdf(&il->backend);
+		}
+	else
+#endif
 #ifdef HAVE_JPEG
 	if (il->bytes_total >= 2 && il->mapped_file[0] == 0xff && il->mapped_file[1] == 0xd8)
 		{
@@ -614,6 +638,8 @@ static void image_loader_setup_loader(ImageLoader *il)
 #ifdef HAVE_TIFF
 	if (il->bytes_total >= 10 &&
 	    (memcmp(il->mapped_file, "MM\0*", 4) == 0 ||
+	     memcmp(il->mapped_file, "MM\0+\0\x08\0\0", 8) == 0 ||
+	     memcmp(il->mapped_file, "II+\0\x08\0\0\0", 8) == 0 ||
 	     memcmp(il->mapped_file, "II*\0", 4) == 0))
 	     	{
 		DEBUG_1("Using custom tiff loader");
@@ -621,6 +647,18 @@ static void image_loader_setup_loader(ImageLoader *il)
 		}
 	else
 #endif
+	if (il->bytes_total >= 3 && il->mapped_file[0] == 0x44 && il->mapped_file[1] == 0x44 && il->mapped_file[2] == 0x53)
+		{
+		DEBUG_1("Using dds loader");
+		image_loader_backend_set_dds(&il->backend);
+		}
+	else
+	if (il->fd->format_class == FORMAT_CLASS_COLLECTION)
+		{
+		DEBUG_1("Using custom collection loader");
+		image_loader_backend_set_collection(&il->backend);
+		}
+	else
 		image_loader_backend_set_default(&il->backend);
 
 	il->loader = il->backend.loader_new(image_loader_area_updated_cb, image_loader_size_cb, image_loader_area_prepared_cb, il);
@@ -1076,23 +1114,6 @@ GdkPixbuf *image_loader_get_pixbuf(ImageLoader *il)
 	return ret;
 }
 
-gchar *image_loader_get_format(ImageLoader *il)
-{
-	gchar **mimev;
-	gchar *mime;
-
-	if (!il || !il->loader) return NULL;
-
-	mimev = il->backend.get_format_mime_types(il->loader);
-	if (!mimev) return NULL;
-
-	/* return first member of mimev, as GdkPixbufLoader has no way to tell us which exact one ? */
-	mime = g_strdup(mimev[0]);
-	g_strfreev(mimev);
-
-	return mime;
-}
-
 void image_loader_set_requested_size(ImageLoader *il, gint width, gint height)
 {
 	if (!il) return;
