@@ -24,6 +24,7 @@
 
 
 #include "collect.h"
+#include "collect-table.h"
 #include "color-man.h"
 #include "exif.h"
 #include "metadata.h"
@@ -47,6 +48,14 @@ static void image_update_title(ImageWindow *imd);
 static void image_read_ahead_start(ImageWindow *imd);
 static void image_cache_set(ImageWindow *imd, FileData *fd);
 
+// For draw rectangle function
+static gint pixbuf_start_x;
+static gint pixbuf_start_y;
+static gint image_start_x;
+static gint image_start_y;
+static gint rect_x1, rect_x2, rect_y1, rect_y2;
+static gint rect_id = 0;
+
 /*
  *-------------------------------------------------------------------
  * 'signals'
@@ -56,6 +65,10 @@ static void image_cache_set(ImageWindow *imd, FileData *fd);
 static void image_click_cb(PixbufRenderer *pr, GdkEventButton *event, gpointer data)
 {
 	ImageWindow *imd = data;
+	if (!options->image_lm_click_nav && event->button == MOUSE_BUTTON_MIDDLE)
+		{
+		imd->mouse_wheel_mode = !imd->mouse_wheel_mode;
+		}
 
 	if (imd->func_button)
 		{
@@ -63,10 +76,175 @@ static void image_click_cb(PixbufRenderer *pr, GdkEventButton *event, gpointer d
 		}
 }
 
-static void image_drag_cb(PixbufRenderer *pr, GdkEventButton *event, gpointer data)
+static void switch_coords_orientation(ImageWindow *imd, gint x, gint y, gint width, gint height)
+{
+	switch (imd->orientation)
+		{
+		case EXIF_ORIENTATION_TOP_LEFT:
+			/* normal -- nothing to do */
+			rect_x1 = image_start_x;
+			rect_y1 = image_start_y;
+			rect_x2 = x;
+			rect_y2 = y;
+			break;
+		case EXIF_ORIENTATION_TOP_RIGHT:
+			/* mirrored */
+			rect_x1 = width - x;
+			rect_y1 = image_start_y;
+			rect_x2 = width - image_start_x;
+			rect_y2 = y;
+			break;
+		case EXIF_ORIENTATION_BOTTOM_RIGHT:
+			/* upside down */
+			rect_x1 = width - x;
+			rect_y1 = height - y;
+			rect_x2 = width - image_start_x;
+			rect_y2 = height - image_start_y;
+			break;
+		case EXIF_ORIENTATION_BOTTOM_LEFT:
+			/* flipped */
+			rect_x1 = image_start_x;
+			rect_y1 = height - y;
+			rect_x2 = x;
+			rect_y2 = height - image_start_y;
+			break;
+		case EXIF_ORIENTATION_LEFT_TOP:
+			/* left mirrored */
+			rect_x1 = image_start_y;
+			rect_y1 = image_start_x;
+			rect_x2 = y;
+			rect_y2 = x;
+			break;
+		case EXIF_ORIENTATION_RIGHT_TOP:
+			/* rotated -90 (270) */
+			rect_x1 = image_start_y;
+			rect_y1 = width - x;
+			rect_x2 = y;
+			rect_y2 = width - image_start_x;
+			break;
+		case EXIF_ORIENTATION_RIGHT_BOTTOM:
+			/* right mirrored */
+			rect_x1 = height - y;
+			rect_y1 = width - x;
+			rect_x2 = height - image_start_y;
+			rect_y2 = width - image_start_x;
+			break;
+		case EXIF_ORIENTATION_LEFT_BOTTOM:
+			/* rotated 90 */
+			rect_x1 = height - y;
+			rect_y1 = image_start_x;
+			rect_x2 = height - image_start_y;
+			rect_y2 = x;
+			break;
+		default:
+			/* The other values are out of range */
+			break;
+		}
+}
+
+static void image_press_cb(PixbufRenderer *pr, GdkEventButton *event, gpointer data)
+{
+	ImageWindow *imd = data;
+	LayoutWindow *lw;
+	gint x_pixel, y_pixel;
+
+	if(options->draw_rectangle)
+		{
+		pixbuf_renderer_get_mouse_position(pr, &x_pixel, &y_pixel);
+
+		pixbuf_start_x = event->x;
+		pixbuf_start_y = event->y;
+
+		if (x_pixel == -1)
+			{
+			image_start_x = 0;
+			}
+		else
+			{
+			image_start_x = x_pixel;
+			}
+
+		if (y_pixel == -1)
+			{
+			image_start_y = 0;
+			}
+		else
+			{
+			image_start_y = y_pixel;
+			}
+		}
+
+	if (rect_id)
+		{
+		pixbuf_renderer_overlay_remove((PixbufRenderer *)imd->pr, rect_id);
+		}
+
+	lw = layout_find_by_image(imd);
+	if (lw && event->button == MOUSE_BUTTON_LEFT && event->type == GDK_2BUTTON_PRESS
+												&& !options->image_lm_click_nav)
+		{
+		if (lw->full_screen)
+			layout_image_full_screen_stop(lw);
+		}
+}
+
+static void image_drag_cb(PixbufRenderer *pr, GdkEventMotion *event, gpointer data)
 {
 	ImageWindow *imd = data;
 	gint width, height;
+	gint rect_width;
+	gint rect_height;
+	GdkPixbuf *rect_pixbuf;
+	gint x_pixel, y_pixel;
+	gint image_x_pixel, image_y_pixel;
+
+	if (options->draw_rectangle)
+		{
+		pixbuf_renderer_get_image_size(pr, &width, &height);
+		pixbuf_renderer_get_mouse_position(pr, &x_pixel, &y_pixel);
+
+		if (x_pixel == -1)
+			{
+			image_x_pixel = width;
+			}
+		else
+			{
+			image_x_pixel = x_pixel;
+			}
+
+		if (y_pixel == -1)
+			{
+			image_y_pixel = height;
+			}
+		else
+			{
+			image_y_pixel = y_pixel;
+			}
+
+		switch_coords_orientation(imd, image_x_pixel, image_y_pixel, width, height);
+		if (rect_id)
+			{
+			pixbuf_renderer_overlay_remove((PixbufRenderer *)imd->pr, rect_id);
+			}
+
+		rect_width = pr->drag_last_x - pixbuf_start_x;
+		if (rect_width <= 0)
+			{
+			rect_width = 1;
+			}
+		rect_height = pr->drag_last_y - pixbuf_start_y;
+		if (rect_height <= 0)
+			{
+			rect_height = 1;
+			}
+
+		rect_pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, rect_width, rect_height);
+		pixbuf_set_rect_fill(rect_pixbuf, 0, 0, rect_width, rect_height, 255, 255, 255, 0);
+		pixbuf_set_rect(rect_pixbuf, 1, 1, rect_width-2, rect_height - 2, 0, 0, 0, 255, 1, 1, 1, 1);
+		pixbuf_set_rect(rect_pixbuf, 2, 2, rect_width-4, rect_height - 4, 255, 255, 255, 255, 1, 1, 1, 1);
+
+		rect_id = pixbuf_renderer_overlay_add((PixbufRenderer *)imd->pr, rect_pixbuf, pixbuf_start_x, pixbuf_start_y, OVL_NORMAL);
+		}
 
 	pixbuf_renderer_get_scaled_size(pr, &width, &height);
 
@@ -159,6 +337,8 @@ static void image_update_title(ImageWindow *imd)
 	gchar *title = NULL;
 	gchar *zoom = NULL;
 	gchar *collection = NULL;
+	LayoutWindow *lw;
+	gchar *lw_ident = NULL;
 
 	if (!imd->top_window) return;
 
@@ -176,13 +356,25 @@ static void image_update_title(ImageWindow *imd)
 		g_free(buf);
 		}
 
-	title = g_strdup_printf("%s%s%s%s%s%s",
+	lw = layout_find_by_image(imd);
+	if (lw)
+		{
+		lw_ident = g_strconcat(" (", lw->options.id, ")", NULL);
+		}
+
+	title = g_strdup_printf("%s%s%s%s%s%s%s",
 		imd->title ? imd->title : "",
 		imd->image_fd ? imd->image_fd->name : "",
 		zoom ? zoom : "",
 		collection ? collection : "",
 		imd->image_fd ? " - " : "",
-		imd->title_right ? imd->title_right : "");
+		imd->title_right ? imd->title_right : "",
+		options->show_window_ids ? (lw_ident ? lw_ident : "") : ""
+		);
+	if (lw_ident)
+		{
+		g_free(lw_ident);
+		}
 
 	gtk_window_set_title(GTK_WINDOW(imd->top_window), title);
 
@@ -375,7 +567,7 @@ static void image_post_process_tile_color_cb(PixbufRenderer *pr, GdkPixbuf **pix
 
 }
 
-void image_alter_orientation(ImageWindow *imd, AlterType type)
+void image_alter_orientation(ImageWindow *imd, FileData *fd_n, AlterType type)
 {
 	static const gint rotate_90[]    = {1,   6, 7, 8, 5, 2, 3, 4, 1};
 	static const gint rotate_90_cc[] = {1,   8, 5, 6, 7, 4, 1, 2, 3};
@@ -383,37 +575,49 @@ void image_alter_orientation(ImageWindow *imd, AlterType type)
 	static const gint mirror[]       = {1,   2, 1, 4, 3, 6, 5, 8, 7};
 	static const gint flip[]         = {1,   4, 3, 2, 1, 8, 7, 6, 5};
 
+	gint orientation;
 
-	if (!imd || !imd->pr || !imd->image_fd) return;
+	if (!imd || !imd->pr || !imd->image_fd || !fd_n) return;
 
-	if (imd->orientation < 1 || imd->orientation > 8) imd->orientation = 1;
+	orientation = EXIF_ORIENTATION_TOP_LEFT;
+	{
+	if (fd_n->user_orientation)
+		{
+		orientation = fd_n->user_orientation;
+		}
+	else
+		if (options->metadata.write_orientation)
+			{
+			orientation = metadata_read_int(fd_n, ORIENTATION_KEY, EXIF_ORIENTATION_TOP_LEFT);
+			}
+	}
 
 	switch (type)
 		{
 		case ALTER_ROTATE_90:
-			imd->orientation = rotate_90[imd->orientation];
+			orientation = rotate_90[orientation];
 			break;
 		case ALTER_ROTATE_90_CC:
-			imd->orientation = rotate_90_cc[imd->orientation];
+			orientation = rotate_90_cc[orientation];
 			break;
 		case ALTER_ROTATE_180:
-			imd->orientation = rotate_180[imd->orientation];
+			orientation = rotate_180[orientation];
 			break;
 		case ALTER_MIRROR:
-			imd->orientation = mirror[imd->orientation];
+			orientation = mirror[orientation];
 			break;
 		case ALTER_FLIP:
-			imd->orientation = flip[imd->orientation];
+			orientation = flip[orientation];
 			break;
 		case ALTER_NONE:
-			imd->orientation = imd->image_fd->exif_orientation ? imd->image_fd->exif_orientation : 1;
+			orientation = fd_n->exif_orientation ? fd_n->exif_orientation : 1;
 			break;
 		default:
 			return;
 			break;
 		}
 
-	if (imd->orientation != imd->image_fd->exif_orientation ? imd->image_fd->exif_orientation : 1)
+	if (orientation != (fd_n->exif_orientation ? fd_n->exif_orientation : 1))
 		{
 		if (!options->metadata.write_orientation)
 			{
@@ -422,29 +626,33 @@ void image_alter_orientation(ImageWindow *imd, AlterType type)
 			   we must however handle switching metadata.write_orientation on and off, therefore
 			   we just disable referencing new fd's, not unreferencing the old ones
 			*/
-			if (imd->image_fd->user_orientation == 0) file_data_ref(imd->image_fd);
-			imd->image_fd->user_orientation = imd->orientation;
+			if (fd_n->user_orientation == 0) file_data_ref(fd_n);
+			fd_n->user_orientation = orientation;
 			}
 		}
 	else
 		{
-		if (imd->image_fd->user_orientation != 0) file_data_unref(imd->image_fd);
-		imd->image_fd->user_orientation = 0;
+		if (fd_n->user_orientation != 0) file_data_unref(fd_n);
+		fd_n->user_orientation = 0;
 		}
 
 	if (options->metadata.write_orientation)
 		{
 		if (type == ALTER_NONE)
 			{
-			metadata_write_revert(imd->image_fd, ORIENTATION_KEY);
+			metadata_write_revert(fd_n, ORIENTATION_KEY);
 			}
 		else
 			{
-			metadata_write_int(imd->image_fd, ORIENTATION_KEY, imd->orientation);
+			metadata_write_int(fd_n, ORIENTATION_KEY, orientation);
 			}
 		}
 
-	pixbuf_renderer_set_orientation((PixbufRenderer *)imd->pr, imd->orientation);
+	if (imd->image_fd == fd_n && !(options->metadata.write_orientation && !options->image.exif_rotate_enable))
+		{
+		imd->orientation = orientation;
+		pixbuf_renderer_set_orientation((PixbufRenderer *)imd->pr, orientation);
+		}
 }
 
 void image_set_desaturate(ImageWindow *imd, gboolean desaturate)
@@ -638,7 +846,26 @@ static void image_load_done_cb(ImageLoader *il, gpointer data)
 		{
 		GdkPixbuf *pixbuf;
 
-		pixbuf = pixbuf_inline(PIXBUF_INLINE_BROKEN);
+		switch (imd->image_fd->format_class)
+			{
+			case FORMAT_CLASS_UNKNOWN:
+				pixbuf = pixbuf_inline(PIXBUF_INLINE_UNKNOWN);
+				break;
+			case FORMAT_CLASS_META:
+				pixbuf = pixbuf_inline(PIXBUF_INLINE_METADATA);
+				break;
+			case FORMAT_CLASS_VIDEO:
+				pixbuf = pixbuf_inline(PIXBUF_INLINE_VIDEO);
+			case FORMAT_CLASS_COLLECTION:
+				pixbuf = pixbuf_inline(PIXBUF_INLINE_COLLECTION);
+				break;
+			case FORMAT_CLASS_PDF:
+				pixbuf = pixbuf_inline(PIXBUF_INLINE_ICON_PDF);
+				break;
+			default:
+				pixbuf = pixbuf_inline(PIXBUF_INLINE_BROKEN);
+			}
+
 		image_change_pixbuf(imd, pixbuf, image_zoom_get(imd), FALSE);
 		g_object_unref(pixbuf);
 
@@ -929,6 +1156,8 @@ static gboolean image_scroll_cb(GtkWidget *widget, GdkEventScroll *event, gpoint
 void image_attach_window(ImageWindow *imd, GtkWidget *window,
 			 const gchar *title, const gchar *title_right, gboolean show_zoom)
 {
+	LayoutWindow *lw;
+
 	imd->top_window = window;
 	g_free(imd->title);
 	imd->title = g_strdup(title);
@@ -936,7 +1165,9 @@ void image_attach_window(ImageWindow *imd, GtkWidget *window,
 	imd->title_right = g_strdup(title_right);
 	imd->title_show_zoom = show_zoom;
 
-	if (!options->image.fit_window_to_image) window = NULL;
+	lw = layout_find_by_image(imd);
+
+	if (!(options->image.fit_window_to_image && lw && lw->options.tools_float)) window = NULL;
 
 	pixbuf_renderer_set_parent((PixbufRenderer *)imd->pr, (GtkWindow *)window);
 
@@ -977,7 +1208,7 @@ void image_set_button_func(ImageWindow *imd,
 }
 
 void image_set_drag_func(ImageWindow *imd,
-			   void (*func)(ImageWindow *, GdkEventButton *event, gdouble dx, gdouble dy, gpointer),
+			   void (*func)(ImageWindow *, GdkEventMotion *event, gdouble dx, gdouble dy, gpointer),
 			   gpointer data)
 {
 	imd->func_drag = func;
@@ -1070,6 +1301,7 @@ void image_change_pixbuf(ImageWindow *imd, GdkPixbuf *pixbuf, gdouble zoom, gboo
 	   here before it is taken over by the renderer. */
 	if (pixbuf) g_object_ref(pixbuf);
 
+	imd->orientation = EXIF_ORIENTATION_TOP_LEFT;
 	if (imd->image_fd)
 		{
 		if (imd->image_fd->user_orientation)
@@ -1125,9 +1357,18 @@ void image_change_pixbuf(ImageWindow *imd, GdkPixbuf *pixbuf, gdouble zoom, gboo
 
 void image_change_from_collection(ImageWindow *imd, CollectionData *cd, CollectInfo *info, gdouble zoom)
 {
+	CollectWindow *cw;
+
 	if (!cd || !info || !g_list_find(cd->list, info)) return;
 
 	image_change_real(imd, info->fd, cd, info, zoom);
+	cw = collection_window_find(cd);
+	if (cw)
+		{
+		collection_table_set_focus(cw->table, info);
+		collection_table_unselect_all(cw->table);
+		collection_table_select(cw->table,info);
+		}
 }
 
 CollectionData *image_get_collection(ImageWindow *imd, CollectInfo **info)
@@ -1722,6 +1963,7 @@ static void image_options_set(ImageWindow *imd)
 					"window_limit_value", options->image.max_window_size,
 					"autofit_limit", options->image.limit_autofit_size,
 					"autofit_limit_value", options->image.max_autofit_size,
+					"enlargement_limit_value", options->image.max_enlargement_size,
 
 					NULL);
 
@@ -1856,6 +2098,7 @@ void image_set_frame(ImageWindow *imd, gboolean frame)
 	if (frame)
 		{
 		imd->frame = gtk_frame_new(NULL);
+		DEBUG_NAME(imd->frame);
         	g_object_ref(imd->pr);
 		if (imd->has_frame != -1) gtk_container_remove(GTK_CONTAINER(imd->widget), imd->pr);
 		gtk_container_add(GTK_CONTAINER(imd->frame), imd->pr);
@@ -1910,10 +2153,12 @@ ImageWindow *image_new(gboolean frame)
 	imd->orientation = 1;
 
 	imd->pr = GTK_WIDGET(pixbuf_renderer_new());
+	DEBUG_NAME(imd->pr);
 
 	image_options_set(imd);
 
 	imd->widget = gtk_vbox_new(0, 0);
+	DEBUG_NAME(imd->widget);
 
 	image_set_frame(imd, frame);
 
@@ -1921,6 +2166,8 @@ ImageWindow *image_new(gboolean frame)
 
 	g_signal_connect(G_OBJECT(imd->pr), "clicked",
 			 G_CALLBACK(image_click_cb), imd);
+	g_signal_connect(G_OBJECT(imd->pr), "button_press_event",
+			 G_CALLBACK(image_press_cb), imd);
 	g_signal_connect(G_OBJECT(imd->pr), "scroll_notify",
 			 G_CALLBACK(image_scroll_notify_cb), imd);
 
@@ -1943,4 +2190,13 @@ ImageWindow *image_new(gboolean frame)
 
 	return imd;
 }
+
+void image_get_rectangle(gint *x1, gint *y1, gint *x2, gint *y2)
+{
+	*x1 = rect_x1;
+	*y1 = rect_y1;
+	*x2 = rect_x2;
+	*y2 = rect_y2;
+}
+
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
