@@ -29,9 +29,11 @@
 #include "filedata.h"
 #include "fullscreen.h"
 #include "image.h"
+#include "image-load.h"
 #include "image-overlay.h"
 #include "layout.h"
 #include "layout_image.h"
+#include "layout_util.h"
 #include "menu.h"
 #include "misc.h"
 #include "pixbuf_util.h"
@@ -405,6 +407,7 @@ static gboolean view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event,
 				file_util_rename(image_get_fd(imd), NULL, imd->widget);
 				break;
 			case 'D': case 'd':
+				options->file_ops.safe_delete_enable = TRUE;
 				file_util_delete(image_get_fd(imd), NULL, imd->widget);
 				break;
 			case 'W': case 'w':
@@ -421,13 +424,13 @@ static gboolean view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event,
 		switch (event->keyval)
 			{
 			case 'R': case 'r':
-				image_alter_orientation(imd, ALTER_ROTATE_180);
+				image_alter_orientation(imd, imd->image_fd, ALTER_ROTATE_180);
 				break;
 			case 'M': case 'm':
-				image_alter_orientation(imd, ALTER_MIRROR);
+				image_alter_orientation(imd, imd->image_fd, ALTER_MIRROR);
 				break;
 			case 'F': case 'f':
-				image_alter_orientation(imd, ALTER_FLIP);
+				image_alter_orientation(imd, imd->image_fd, ALTER_FLIP);
 				break;
 			case 'G': case 'g':
 				image_set_desaturate(imd, !image_get_desaturate(imd));
@@ -444,6 +447,13 @@ static gboolean view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event,
 						 filelist_copy(vw->list), vw->window);
 				}
 				break;
+			case GDK_KEY_Delete: case GDK_KEY_KP_Delete:
+				if (options->file_ops.enable_delete_key)
+					{
+					options->file_ops.safe_delete_enable = FALSE;
+					file_util_delete(image_get_fd(imd), NULL, imd->widget);
+					}
+				break;
 			default:
 				stop_signal = FALSE;
 				break;
@@ -531,14 +541,15 @@ static gboolean view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event,
 				view_overlay_toggle(vw);
 				break;
 			case ']':
-				image_alter_orientation(imd, ALTER_ROTATE_90);
+				image_alter_orientation(imd, imd->image_fd, ALTER_ROTATE_90);
 				break;
 			case '[':
-				image_alter_orientation(imd, ALTER_ROTATE_90_CC);
+				image_alter_orientation(imd, imd->image_fd, ALTER_ROTATE_90_CC);
 				break;
 			case GDK_KEY_Delete: case GDK_KEY_KP_Delete:
 				if (options->file_ops.enable_delete_key)
 					{
+					options->file_ops.safe_delete_enable = TRUE;
 					file_util_delete(image_get_fd(imd), NULL, imd->widget);
 					}
 				break;
@@ -563,6 +574,11 @@ static gboolean view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event,
 				break;
 			}
 		}
+	if (!stop_signal && is_help_key(event))
+		{
+		help_window_show("GuideOtherWindowsImageWindow.html");
+		stop_signal = TRUE;
+		}
 
 	return stop_signal;
 }
@@ -580,10 +596,16 @@ static void button_cb(ImageWindow *imd, GdkEventButton *event, gpointer data)
 	switch (event->button)
 		{
 		case MOUSE_BUTTON_LEFT:
-			view_step_next(vw);
+			if (options->image_l_click_video && options->image_l_click_video_editor && imd->image_fd->format_class == FORMAT_CLASS_VIDEO)
+				{
+				start_editor_from_file(options->image_l_click_video_editor, imd->image_fd);
+				}
+			else if (options->image_lm_click_nav)
+				view_step_next(vw);
 			break;
 		case MOUSE_BUTTON_MIDDLE:
-			view_step_prev(vw);
+			if (options->image_lm_click_nav)
+				view_step_prev(vw);
 			break;
 		case MOUSE_BUTTON_RIGHT:
 			menu = view_popup_menu(vw);
@@ -598,7 +620,8 @@ static void scroll_cb(ImageWindow *imd, GdkEventScroll *event, gpointer data)
 {
 	ViewWindow *vw = data;
 
-	if (event->state & GDK_CONTROL_MASK)
+	if ((event->state & GDK_CONTROL_MASK) ||
+				(imd->mouse_wheel_mode && !options->image_lm_click_nav))
 		{
 		switch (event->direction)
 			{
@@ -811,6 +834,7 @@ static ViewWindow *real_view_window_new(FileData *fd, GList *list, CollectionDat
 	vw = g_new0(ViewWindow, 1);
 
 	vw->window = window_new(GTK_WINDOW_TOPLEVEL, "view", PIXBUF_INLINE_ICON_VIEW, NULL, NULL);
+	DEBUG_NAME(vw->window);
 
 	geometry.min_width = DEFAULT_MINIMAL_WINDOW_SIZE;
 	geometry.min_height = DEFAULT_MINIMAL_WINDOW_SIZE;
@@ -848,6 +872,9 @@ static ViewWindow *real_view_window_new(FileData *fd, GList *list, CollectionDat
 	if (cd && info)
 		{
 		image_change_from_collection(vw->imd, cd, info, image_zoom_get_default(NULL));
+		/* Grab the fd so we can correctly size the window in
+		   the call to image_load_dimensions() below. */
+		fd = info->fd;
 		if (options->image.enable_read_ahead)
 			{
 			CollectInfo * r_info = collection_next_by_info(cd, info);
@@ -860,6 +887,8 @@ static ViewWindow *real_view_window_new(FileData *fd, GList *list, CollectionDat
 		view_window_set_list(vw, list);
 		vw->list_pointer = vw->list;
 		image_change_fd(vw->imd, (FileData *)vw->list->data, image_zoom_get_default(NULL));
+		/* Set fd to first in list */
+		fd = vw->list->data;
 
 		if (options->image.enable_read_ahead)
 			{
@@ -873,21 +902,8 @@ static ViewWindow *real_view_window_new(FileData *fd, GList *list, CollectionDat
 		}
 
 	/* Wait until image is loaded otherwise size is not defined */
-	int count;
-	for (count = 10; count && !w && !h; count++)
-		{
-		image_get_image_size(vw->imd, &w, &h);
-		usleep(100000);
-		}
+	image_load_dimensions(fd, &w, &h);
 
-	if (image_zoom_get(vw->imd) == 0.0)
-		{
-		image_get_image_size(vw->imd, &w, &h);
-		}
-	else
-		{
-		pixbuf_renderer_get_scaled_size(PIXBUF_RENDERER(vw->imd->pr), &w, &h);
-		}
 	if (options->image.limit_window_size)
 		{
 		gint mw = gdk_screen_width() * options->image.max_window_size / 100;
@@ -903,8 +919,11 @@ static ViewWindow *real_view_window_new(FileData *fd, GList *list, CollectionDat
 	req_size.height = h;
 	gtk_widget_size_allocate(GTK_WIDGET(vw->window), &req_size);
 
+#if !GTK_CHECK_VERSION(3,0,0)
 	gtk_widget_set_size_request(vw->imd->pr, w, h);
+#endif
 
+	gtk_window_set_focus_on_map(GTK_WINDOW(vw->window), FALSE);
 	gtk_widget_show(vw->window);
 
 	view_window_list = g_list_append(view_window_list, vw);
@@ -925,40 +944,43 @@ void view_window_new(FileData *fd)
 {
 	GList *list;
 
-	if (file_extension_match(fd->path, GQ_COLLECTION_EXT))
+	if (fd)
 		{
-		ViewWindow *vw;
-		CollectionData *cd;
-		CollectInfo *info;
-
-		cd = collection_new(fd->path);
-		if (collection_load(cd, fd->path, COLLECTION_LOAD_NONE))
+		if (file_extension_match(fd->path, GQ_COLLECTION_EXT))
 			{
-			info = collection_get_first(cd);
+			ViewWindow *vw;
+			CollectionData *cd;
+			CollectInfo *info;
+
+			cd = collection_new(fd->path);
+			if (collection_load(cd, fd->path, COLLECTION_LOAD_NONE))
+				{
+				info = collection_get_first(cd);
+				}
+			else
+				{
+				collection_unref(cd);
+				cd = NULL;
+				info = NULL;
+				}
+			vw = real_view_window_new(NULL, NULL, cd, info);
+			if (vw && cd)
+				{
+				g_signal_connect(G_OBJECT(vw->window), "destroy",
+						 G_CALLBACK(view_window_collection_unref_cb), cd);
+				}
+			}
+		else if (isdir(fd->path) && filelist_read(fd, &list, NULL))
+			{
+			list = filelist_sort_path(list);
+			list = filelist_filter(list, FALSE);
+			real_view_window_new(NULL, list, NULL, NULL);
+			filelist_free(list);
 			}
 		else
 			{
-			collection_unref(cd);
-			cd = NULL;
-			info = NULL;
+			real_view_window_new(fd, NULL, NULL, NULL);
 			}
-		vw = real_view_window_new(NULL, NULL, cd, info);
-		if (vw && cd)
-			{
-			g_signal_connect(G_OBJECT(vw->window), "destroy",
-					 G_CALLBACK(view_window_collection_unref_cb), cd);
-			}
-		}
-	else if (isdir(fd->path) && filelist_read(fd, &list, NULL))
-		{
-		list = filelist_sort_path(list);
-		list = filelist_filter(list, FALSE);
-		real_view_window_new(NULL, list, NULL, NULL);
-		filelist_free(list);
-		}
-	else
-		{
-		real_view_window_new(fd, NULL, NULL, NULL);
 		}
 }
 
@@ -1079,7 +1101,7 @@ static void view_alter_cb(GtkWidget *widget, gpointer data)
 	type = GPOINTER_TO_INT(data);
 
 	if (!vw) return;
-	image_alter_orientation(vw->imd, type);
+	image_alter_orientation(vw->imd, vw->imd->image_fd, type);
 }
 
 static void view_wallpaper_cb(GtkWidget *widget, gpointer data)
@@ -1152,6 +1174,17 @@ static void view_delete_cb(GtkWidget *widget, gpointer data)
 	ImageWindow *imd;
 
 	imd = view_window_active_image(vw);
+	options->file_ops.safe_delete_enable = FALSE;
+	file_util_delete(image_get_fd(imd), NULL, imd->widget);
+}
+
+static void view_move_to_trash_cb(GtkWidget *widget, gpointer data)
+{
+	ViewWindow *vw = data;
+	ImageWindow *imd;
+
+	imd = view_window_active_image(vw);
+	options->file_ops.safe_delete_enable = TRUE;
 	file_util_delete(image_get_fd(imd), NULL, imd->widget);
 }
 
@@ -1161,7 +1194,16 @@ static void view_copy_path_cb(GtkWidget *widget, gpointer data)
 	ImageWindow *imd;
 
 	imd = view_window_active_image(vw);
-	file_util_copy_path_to_clipboard(image_get_fd(imd));
+	file_util_copy_path_to_clipboard(image_get_fd(imd), TRUE);
+}
+
+static void view_copy_path_unquoted_cb(GtkWidget *widget, gpointer data)
+{
+	ViewWindow *vw = data;
+	ImageWindow *imd;
+
+	imd = view_window_active_image(vw);
+	file_util_copy_path_to_clipboard(image_get_fd(imd), FALSE);
 }
 
 static void view_fullscreen_cb(GtkWidget *widget, gpointer data)
@@ -1249,9 +1291,33 @@ static GList *view_window_get_fd_list(ViewWindow *vw)
 	return list;
 }
 
+/**
+ * @brief Add file selection list to a collection
+ * @param[in] widget 
+ * @param[in] data Index to the collection list menu item selected, or -1 for new collection
+ * 
+ * 
+ */
+static void image_pop_menu_collections_cb(GtkWidget *widget, gpointer data)
+{
+	ViewWindow *vw;
+	ImageWindow *imd;
+	FileData *fd;
+	GList *selection_list = NULL;
+
+	vw = submenu_item_get_data(widget);
+	imd = view_window_active_image(vw);
+	fd = image_get_fd(imd);
+	selection_list = g_list_append(selection_list, fd);
+	pop_menu_collections(selection_list, data);
+
+	filelist_free(selection_list);
+}
+
 static GtkWidget *view_popup_menu(ViewWindow *vw)
 {
 	GtkWidget *menu;
+	GtkWidget *submenu;
 	GtkWidget *item;
 	GList *editmenu_fd_list;
 
@@ -1269,8 +1335,9 @@ static GtkWidget *view_popup_menu(ViewWindow *vw)
 			 G_CALLBACK(view_popup_menu_destroy_cb), editmenu_fd_list);
 	item = submenu_add_edit(menu, NULL, G_CALLBACK(view_edit_cb), vw, editmenu_fd_list);
 	menu_item_add_divider(item);
+#if !GTK_CHECK_VERSION(3,0,0)
 	menu_item_add(item, _("Set as _wallpaper"), G_CALLBACK(view_wallpaper_cb), vw);
-
+#endif
 	submenu_add_alter(menu, G_CALLBACK(view_alter_cb), vw);
 
 	menu_item_add_stock(menu, _("View in _new window"), GTK_STOCK_NEW, G_CALLBACK(view_new_window_cb), vw);
@@ -1280,9 +1347,24 @@ static GtkWidget *view_popup_menu(ViewWindow *vw)
 	menu_item_add_stock(menu, _("_Copy..."), GTK_STOCK_COPY, G_CALLBACK(view_copy_cb), vw);
 	menu_item_add(menu, _("_Move..."), G_CALLBACK(view_move_cb), vw);
 	menu_item_add(menu, _("_Rename..."), G_CALLBACK(view_rename_cb), vw);
-	menu_item_add_stock(menu, _("_Delete..."), GTK_STOCK_DELETE, G_CALLBACK(view_delete_cb), vw);
 	menu_item_add(menu, _("_Copy path"), G_CALLBACK(view_copy_path_cb), vw);
+	menu_item_add(menu, _("_Copy path unquoted"), G_CALLBACK(view_copy_path_unquoted_cb), vw);
 
+	menu_item_add_divider(menu);
+	menu_item_add_stock(menu,
+				options->file_ops.confirm_move_to_trash ? _("Move to Trash...") :
+					_("Move to Trash"), PIXBUF_INLINE_ICON_TRASH,
+				G_CALLBACK(view_move_to_trash_cb), vw);
+	menu_item_add_stock(menu,
+				options->file_ops.confirm_delete ? _("_Delete...") :
+					_("_Delete"), GTK_STOCK_DELETE,
+				G_CALLBACK(view_delete_cb), vw);
+
+	menu_item_add_divider(menu);
+
+	submenu_add_collections(menu, &item,
+				G_CALLBACK(image_pop_menu_collections_cb), vw);
+	gtk_widget_set_sensitive(item, TRUE);
 	menu_item_add_divider(menu);
 
 	if (vw->ss)
