@@ -25,11 +25,13 @@
 #include "color-man.h"
 #include "filedata.h"
 #include "histogram.h"
+#include "history_list.h"
 #include "image.h"
 #include "image-overlay.h"
 #include "layout_config.h"
 #include "layout_image.h"
 #include "layout_util.h"
+#include "logwindow.h"
 #include "menu.h"
 #include "pixbuf-renderer.h"
 #include "pixbuf_util.h"
@@ -46,7 +48,7 @@
 #include "bar.h"
 #include "bar_sort.h"
 #include "preferences.h"
-
+#include "shortcuts.h"
 #ifdef HAVE_LIRC
 #include "lirc.h"
 #endif
@@ -61,7 +63,7 @@
 #define TOOLWINDOW_DEF_HEIGHT 450
 
 #define PROGRESS_WIDTH 150
-#define ZOOM_LABEL_WIDTH 64
+#define ZOOM_LABEL_WIDTH 120
 
 #define PANE_DIVIDER_SIZE 10
 
@@ -183,6 +185,21 @@ static gboolean layout_set_current_cb(GtkWidget *widget, GdkEventFocus *event, g
 	return FALSE;
 }
 
+static void layout_box_folders_changed_cb(GtkWidget *widget, gpointer data)
+{
+	LayoutWindow *lw;
+	GList *work;
+
+/* FIXME: this is probably not the correct way to implement this */
+	work = layout_window_list;
+	while (work)
+		{
+		lw = work->data;
+		lw->options.folder_window.vdivider_pos = gtk_paned_get_position(GTK_PANED(widget));
+		work = work->next;
+		}
+}
+
 /*
  *-----------------------------------------------------------------------------
  * menu, toolbar, and dir view
@@ -217,6 +234,7 @@ static void layout_path_entry_tab_cb(const gchar *path, gpointer data)
 		{
 		if ((!lw->dir_fd || strcmp(lw->dir_fd->path, buf) != 0) && layout_set_path(lw, buf))
 			{
+			gtk_widget_grab_focus(GTK_WIDGET(lw->path_entry));
 			gint pos = -1;
 			/* put the G_DIR_SEPARATOR back, if we are in tab completion for a dir and result was path change */
 			gtk_editable_insert_text(GTK_EDITABLE(lw->path_entry), G_DIR_SEPARATOR_S, -1, &pos);
@@ -269,41 +287,81 @@ static void layout_path_entry_tab_append_cb(const gchar *path, gpointer data, gi
 	gtk_widget_set_sensitive(lw->back_button, (n > 1));
 }
 
+static gboolean path_entry_tooltip_cb(GtkWidget *widget, gpointer data)
+{
+	LayoutWindow *lw = data;
+	GList *box_child_list;
+	GtkComboBox *path_entry;
+	gchar *current_path;
+
+	box_child_list = gtk_container_get_children(GTK_CONTAINER(widget));
+	path_entry = box_child_list->data;
+	current_path = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(path_entry));
+	gtk_widget_set_tooltip_text(GTK_WIDGET(widget), current_path);
+
+	g_free(current_path);
+	g_list_free(box_child_list);
+
+	return FALSE;
+}
+
 static GtkWidget *layout_tool_setup(LayoutWindow *lw)
 {
 	GtkWidget *box;
-	GtkWidget *menu_bar;
+	GtkWidget *box_folders;
+	GtkWidget *scd;
+	GtkWidget *menu_tool_bar;
 	GtkWidget *tabcomp;
-	GtkWidget *toolbar;
 
 	box = gtk_vbox_new(FALSE, 0);
 
-	menu_bar = layout_actions_menu_bar(lw);
-	gtk_box_pack_start(GTK_BOX(box), menu_bar, FALSE, FALSE, 0);
-
-	toolbar = layout_actions_toolbar(lw, TOOLBAR_MAIN);
-	gtk_box_pack_start(GTK_BOX(box), toolbar, FALSE, FALSE, 0);
-	if (lw->options.toolbar_hidden) gtk_widget_hide(toolbar);
+	menu_tool_bar = layout_actions_menu_tool_bar(lw);
+	DEBUG_NAME(menu_tool_bar);
+	gtk_widget_show(menu_tool_bar);
+	gtk_box_pack_start(GTK_BOX(lw->main_box), lw->menu_tool_bar, FALSE, FALSE, 0);
+	if (lw->options.toolbar_hidden) gtk_widget_hide(lw->toolbar[TOOLBAR_MAIN]);
 
 	tabcomp = tab_completion_new_with_history(&lw->path_entry, NULL, "path_list", -1,
 						  layout_path_entry_cb, lw);
+	DEBUG_NAME(tabcomp);
 	tab_completion_add_tab_func(lw->path_entry, layout_path_entry_tab_cb, lw);
 	tab_completion_add_append_func(lw->path_entry, layout_path_entry_tab_append_cb, lw);
 	gtk_box_pack_start(GTK_BOX(box), tabcomp, FALSE, FALSE, 0);
 	gtk_widget_show(tabcomp);
+	gtk_widget_set_has_tooltip(GTK_WIDGET(tabcomp), TRUE);
+	g_signal_connect(G_OBJECT(tabcomp), "query_tooltip", G_CALLBACK(path_entry_tooltip_cb), lw);
 
+#if GTK_CHECK_VERSION(3,20,0)
+	g_signal_connect(G_OBJECT(gtk_widget_get_parent(gtk_widget_get_parent(lw->path_entry))), "changed",
+			 G_CALLBACK(layout_path_entry_changed_cb), lw);
+#else
 	g_signal_connect(G_OBJECT(gtk_widget_get_parent(lw->path_entry)), "changed",
 			 G_CALLBACK(layout_path_entry_changed_cb), lw);
+#endif
+
+	box_folders = GTK_WIDGET(gtk_hpaned_new());
+	DEBUG_NAME(box_folders);
+	gtk_box_pack_start(GTK_BOX(box), box_folders, TRUE, TRUE, 0);
 
 	lw->vd = vd_new(lw->options.dir_view_type, lw->dir_fd);
 	vd_set_layout(lw->vd, lw);
 	vd_set_select_func(lw->vd, layout_vd_select_cb, lw);
 
 	lw->dir_view = lw->vd->widget;
-
-	gtk_box_pack_start(GTK_BOX(box), lw->dir_view, TRUE, TRUE, 0);
+	DEBUG_NAME(lw->dir_view);
+	gtk_paned_add2(GTK_PANED(box_folders), lw->dir_view);
 	gtk_widget_show(lw->dir_view);
 
+	scd = shortcuts_new_default(lw);
+	DEBUG_NAME(scd);
+	gtk_paned_add1(GTK_PANED(box_folders), scd);
+	gtk_paned_set_position(GTK_PANED(box_folders), lw->options.folder_window.vdivider_pos);
+
+	gtk_widget_show(box_folders);
+
+	g_signal_connect(G_OBJECT(box_folders), "notify::position",
+			 G_CALLBACK(layout_box_folders_changed_cb), lw);
+
 	gtk_widget_show(box);
 
 	return box;
@@ -327,6 +385,10 @@ static void layout_sort_menu_cb(GtkWidget *widget, gpointer data)
 
 	type = (SortType)GPOINTER_TO_INT(data);
 
+	if (type == SORT_EXIFTIME || type == SORT_EXIFTIMEDIGITIZED || type == SORT_RATING)
+		{
+		vf_read_metadata_in_idle(lw->vf);
+		}
 	layout_sort_set(lw, type, lw->sort_ascend);
 }
 
@@ -388,6 +450,7 @@ static GtkWidget *layout_sort_button(LayoutWindow *lw)
 	GtkWidget *button;
 
 	button = gtk_button_new_with_label(sort_type_get_text(lw->sort_method));
+	DEBUG_NAME(button);
 	g_signal_connect(G_OBJECT(button), "clicked",
 			 G_CALLBACK(layout_sort_button_press_cb), lw);
 	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
@@ -395,6 +458,114 @@ static GtkWidget *layout_sort_button(LayoutWindow *lw)
 	return button;
 }
 
+static void layout_zoom_menu_cb(GtkWidget *widget, gpointer data)
+{
+	ZoomMode mode;
+
+	if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget))) return;
+
+	mode = (ZoomMode)GPOINTER_TO_INT(data);
+	options->image.zoom_mode = mode;
+}
+
+static void layout_scroll_menu_cb(GtkWidget *widget, gpointer data)
+{
+	guint scroll_type;
+
+	if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget))) return;
+
+	scroll_type = GPOINTER_TO_UINT(data);
+	options->image.scroll_reset_method = scroll_type;
+	image_options_sync();
+}
+
+static void layout_zoom_menu_hide_cb(GtkWidget *widget, gpointer data)
+{
+	/* destroy the menu */
+	g_object_unref(widget);
+}
+
+static void layout_zoom_button_press_cb(GtkWidget *widget, gpointer data)
+{
+	LayoutWindow *lw = data;
+	GtkWidget *menu;
+	GdkEvent *event;
+	guint32 etime;
+
+	menu = submenu_add_zoom(NULL, G_CALLBACK(layout_zoom_menu_cb),
+			lw, FALSE, FALSE, TRUE, options->image.zoom_mode);
+
+	/* take ownership of menu */
+#ifdef GTK_OBJECT_FLOATING
+	/* GTK+ < 2.10 */
+	g_object_ref(G_OBJECT(menu));
+	gtk_object_sink(GTK_OBJECT(menu));
+#else
+	/* GTK+ >= 2.10 */
+	g_object_ref_sink(G_OBJECT(menu));
+#endif
+
+	menu_item_add_divider(menu);
+
+	menu_item_add_radio(menu, _("Scroll to top left corner"),
+			GUINT_TO_POINTER(SCROLL_RESET_TOPLEFT),
+			options->image.scroll_reset_method == SCROLL_RESET_TOPLEFT,
+			G_CALLBACK(layout_scroll_menu_cb),
+			GUINT_TO_POINTER(SCROLL_RESET_TOPLEFT));
+	menu_item_add_radio(menu, _("Scroll to image center"),
+			GUINT_TO_POINTER(SCROLL_RESET_CENTER),
+			options->image.scroll_reset_method == SCROLL_RESET_CENTER,
+			G_CALLBACK(layout_scroll_menu_cb),
+			GUINT_TO_POINTER(SCROLL_RESET_CENTER));
+	menu_item_add_radio(menu, _("Keep the region from previous image"),
+			GUINT_TO_POINTER(SCROLL_RESET_NOCHANGE),
+			options->image.scroll_reset_method == SCROLL_RESET_NOCHANGE,
+			G_CALLBACK(layout_scroll_menu_cb),
+			GUINT_TO_POINTER(SCROLL_RESET_NOCHANGE));
+
+	g_signal_connect(G_OBJECT(menu), "selection_done",
+			 G_CALLBACK(layout_zoom_menu_hide_cb), NULL);
+
+	event = gtk_get_current_event();
+	if (event)
+		{
+		etime = gdk_event_get_time(event);
+		gdk_event_free(event);
+		}
+	else
+		{
+		etime = 0;
+		}
+
+	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, etime);
+}
+
+static GtkWidget *layout_zoom_button(LayoutWindow *lw, GtkWidget *box, gint size, gboolean expand)
+{
+	GtkWidget *button;
+	GtkWidget *frame;
+
+
+	frame = gtk_frame_new(NULL);
+	DEBUG_NAME(frame);
+	if (size) gtk_widget_set_size_request(frame, size, -1);
+	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
+
+	gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
+
+	gtk_widget_show(frame);
+
+	button = gtk_button_new_with_label("1:1");
+	g_signal_connect(G_OBJECT(button), "clicked",
+			 G_CALLBACK(layout_zoom_button_press_cb), lw);
+	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+
+	gtk_container_add(GTK_CONTAINER(frame), button);
+	gtk_widget_show(button);
+
+	return button;
+}
+
 /*
  *-----------------------------------------------------------------------------
  * status bar
@@ -404,11 +575,30 @@ static GtkWidget *layout_sort_button(LayoutWindow *lw)
 
 void layout_status_update_progress(LayoutWindow *lw, gdouble val, const gchar *text)
 {
+	static gdouble thumb = 0;
+	static gdouble meta = 0;
+
 	if (!layout_valid(&lw)) return;
 	if (!lw->info_progress_bar) return;
 
+	/* Give priority to the loading meta data message
+	 */
+	if(!g_strcmp0(text, "Loading thumbs..."))
+		{
+		thumb = val;
+		if (meta)
+			{
+			return;
+			}
+		}
+	else
+		{
+		meta = val;
+		}
+
 	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(lw->info_progress_bar), val);
-	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(lw->info_progress_bar), (text) ? text : " ");
+	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(lw->info_progress_bar),
+									val ? ((text) ? text : " ") : " ");
 }
 
 void layout_status_update_info(LayoutWindow *lw, const gchar *text)
@@ -494,7 +684,7 @@ void layout_status_update_image(LayoutWindow *lw)
 
 	if (!n)
 		{
-		gtk_label_set_text(GTK_LABEL(lw->info_zoom), "");
+		gtk_button_set_label(GTK_BUTTON(lw->info_zoom), "");
 		gtk_label_set_text(GTK_LABEL(lw->info_details), "");
 		}
 	else
@@ -503,7 +693,7 @@ void layout_status_update_image(LayoutWindow *lw)
 		gchar *b;
 
 		text = image_zoom_get_as_text(lw->image);
-		gtk_label_set_text(GTK_LABEL(lw->info_zoom), text);
+		gtk_button_set_label(GTK_BUTTON(lw->info_zoom), text);
 		g_free(text);
 
 		b = image_get_fd(lw->image) ? text_from_size(image_get_fd(lw->image)->size) : g_strdup("0");
@@ -552,6 +742,7 @@ static GtkWidget *layout_status_label(gchar *text, GtkWidget *box, gboolean star
 	GtkWidget *frame;
 
 	frame = gtk_frame_new(NULL);
+	DEBUG_NAME(frame);
 	if (size) gtk_widget_set_size_request(frame, size, -1);
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
 	if (start)
@@ -582,10 +773,12 @@ static void layout_status_setup(LayoutWindow *lw, GtkWidget *box, gboolean small
 	if (small_format)
 		{
 		lw->info_box = gtk_vbox_new(FALSE, 0);
+		DEBUG_NAME(lw->info_box);
 		}
 	else
 		{
 		lw->info_box = gtk_hbox_new(FALSE, 0);
+		DEBUG_NAME(lw->info_box);
 		}
 	gtk_box_pack_end(GTK_BOX(box), lw->info_box, FALSE, FALSE, 0);
 	gtk_widget_show(lw->info_box);
@@ -593,6 +786,7 @@ static void layout_status_setup(LayoutWindow *lw, GtkWidget *box, gboolean small
 	if (small_format)
 		{
 		hbox = gtk_hbox_new(FALSE, 0);
+		DEBUG_NAME(hbox);
 		gtk_box_pack_start(GTK_BOX(lw->info_box), hbox, FALSE, FALSE, 0);
 		gtk_widget_show(hbox);
 		}
@@ -601,39 +795,57 @@ static void layout_status_setup(LayoutWindow *lw, GtkWidget *box, gboolean small
 		hbox = lw->info_box;
 		}
 	lw->info_progress_bar = gtk_progress_bar_new();
+	DEBUG_NAME(lw->info_progress_bar);
 	gtk_widget_set_size_request(lw->info_progress_bar, PROGRESS_WIDTH, -1);
+#if GTK_CHECK_VERSION(3,0,0)
+	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(lw->info_progress_bar), "");
+	gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(lw->info_progress_bar), TRUE);
+#endif
 	gtk_box_pack_start(GTK_BOX(hbox), lw->info_progress_bar, FALSE, FALSE, 0);
 	gtk_widget_show(lw->info_progress_bar);
 
 	lw->info_sort = layout_sort_button(lw);
+	gtk_widget_set_tooltip_text(GTK_WIDGET(lw->info_sort), _("Select sort order"));
 	gtk_box_pack_start(GTK_BOX(hbox), lw->info_sort, FALSE, FALSE, 0);
 	gtk_widget_show(lw->info_sort);
 
 	lw->info_status = layout_status_label(NULL, lw->info_box, TRUE, 0, (!small_format));
+	DEBUG_NAME(lw->info_status);
+	gtk_widget_set_tooltip_text(GTK_WIDGET(lw->info_status), _("Folder contents (files selected)"));
 
 	if (small_format)
 		{
 		hbox = gtk_hbox_new(FALSE, 0);
+		DEBUG_NAME(hbox);
 		gtk_box_pack_start(GTK_BOX(lw->info_box), hbox, FALSE, FALSE, 0);
 		gtk_widget_show(hbox);
 		}
 	lw->info_details = layout_status_label(NULL, hbox, TRUE, 0, TRUE);
+	DEBUG_NAME(lw->info_details);
+	gtk_widget_set_tooltip_text(GTK_WIDGET(lw->info_details), _("(Image dimensions) Image size"));
 	toolbar = layout_actions_toolbar(lw, TOOLBAR_STATUS);
 
 	toolbar_frame = gtk_frame_new(NULL);
+	DEBUG_NAME(toolbar_frame);
 	gtk_frame_set_shadow_type(GTK_FRAME(toolbar_frame), GTK_SHADOW_IN);
 	gtk_container_add(GTK_CONTAINER(toolbar_frame), toolbar);
 	gtk_widget_show(toolbar_frame);
 	gtk_widget_show(toolbar);
 	gtk_box_pack_end(GTK_BOX(hbox), toolbar_frame, FALSE, FALSE, 0);
-	lw->info_zoom = layout_status_label(NULL, hbox, FALSE, ZOOM_LABEL_WIDTH, FALSE);
+	lw->info_zoom = layout_zoom_button(lw, hbox, ZOOM_LABEL_WIDTH, TRUE);
+	gtk_widget_set_tooltip_text(GTK_WIDGET(lw->info_zoom), _("Select zoom and scroll mode"));
+	gtk_widget_show(lw->info_zoom);
+
 	if (small_format)
 		{
 		hbox = gtk_hbox_new(FALSE, 0);
+		DEBUG_NAME(hbox);
 		gtk_box_pack_start(GTK_BOX(lw->info_box), hbox, FALSE, FALSE, 0);
 		gtk_widget_show(hbox);
 		}
 	lw->info_pixel = layout_status_label(NULL, hbox, FALSE, 0, small_format); /* expand only in small format */
+	DEBUG_NAME(lw->info_pixel);
+	gtk_widget_set_tooltip_text(GTK_WIDGET(lw->info_pixel), _("[Pixel x,y coord]: (Pixel R,G,B value)"));
 	if (!lw->options.show_info_pixel) gtk_widget_hide(gtk_widget_get_parent(lw->info_pixel));
 }
 
@@ -668,6 +880,11 @@ static void layout_list_sync_thumb(LayoutWindow *lw)
 	if (lw->vf) vf_thumb_set(lw->vf, lw->options.show_thumbnails);
 }
 
+static void layout_list_sync_file_filter(LayoutWindow *lw)
+{
+	if (lw->vf) vf_file_filter_set(lw->vf, lw->options.show_file_filter);
+}
+
 static GtkWidget *layout_list_new(LayoutWindow *lw)
 {
 	lw->vf = vf_new(lw->options.file_view_type, NULL);
@@ -679,6 +896,7 @@ static GtkWidget *layout_list_new(LayoutWindow *lw)
 	vf_marks_set(lw->vf, lw->options.show_marks);
 
 	layout_list_sync_thumb(lw);
+	layout_list_sync_file_filter(lw);
 
 	return lw->vf->widget;
 }
@@ -912,7 +1130,11 @@ gboolean layout_set_fd(LayoutWindow *lw, FileData *fd)
 		if (isfile(fd->path)) have_file = TRUE;
 		}
 
-	if (lw->path_entry) tab_completion_append_to_history(lw->path_entry, lw->dir_fd->path);
+	if (lw->path_entry)
+		{
+		history_chain_append_end(lw->dir_fd->path);
+		tab_completion_append_to_history(lw->path_entry, lw->dir_fd->path);
+		}
 	layout_sync_path(lw);
 	layout_list_sync_sort(lw);
 
@@ -938,6 +1160,11 @@ gboolean layout_set_fd(LayoutWindow *lw, FileData *fd)
 	if (options->metadata.confirm_on_dir_change && dir_changed)
 		metadata_write_queue_confirm(FALSE, NULL, NULL);
 
+	if (lw->vf && (options->read_metadata_in_idle || (lw->sort_method == SORT_EXIFTIME || lw->sort_method == SORT_EXIFTIMEDIGITIZED || lw->sort_method == SORT_RATING)))
+		{
+		vf_read_metadata_in_idle(lw->vf);
+		}
+
 	return TRUE;
 }
 
@@ -975,6 +1202,18 @@ void layout_thumb_set(LayoutWindow *lw, gboolean enable)
 	layout_list_sync_thumb(lw);
 }
 
+void layout_file_filter_set(LayoutWindow *lw, gboolean enable)
+{
+	if (!layout_valid(&lw)) return;
+
+	if (lw->options.show_file_filter == enable) return;
+
+	lw->options.show_file_filter = enable;
+
+	layout_util_sync_file_filter(lw);
+	layout_list_sync_file_filter(lw);
+}
+
 void layout_marks_set(LayoutWindow *lw, gboolean enable)
 {
 	if (!layout_valid(&lw)) return;
@@ -983,7 +1222,7 @@ void layout_marks_set(LayoutWindow *lw, gboolean enable)
 
 	lw->options.show_marks = enable;
 
-//	layout_util_sync_marks(lw);
+	layout_util_sync_marks(lw);
 	layout_list_sync_marks(lw);
 }
 
@@ -1174,13 +1413,13 @@ gboolean layout_geometry_get_tools(LayoutWindow *lw, gint *x, gint *y, gint *w,
 		return FALSE;
 		}
 
-	window = gtk_widget_get_window(lw->window);
+	window = gtk_widget_get_window(lw->tools);
 	gdk_window_get_root_origin(window, x, y);
 	*w = gdk_window_get_width(window);
 	*h = gdk_window_get_height(window);
 	gtk_widget_get_allocation(gtk_paned_get_child1(GTK_PANED(lw->tools_pane)), &allocation);
 
-	if (GTK_IS_VPANED(lw->tools_pane))
+	if (gtk_orientable_get_orientation(GTK_ORIENTABLE(lw->tools_pane)) == GTK_ORIENTATION_VERTICAL)
 		{
 		*divider_pos = allocation.height;
 		}
@@ -1192,9 +1431,29 @@ gboolean layout_geometry_get_tools(LayoutWindow *lw, gint *x, gint *y, gint *w,
 	return TRUE;
 }
 
+gboolean layout_geometry_get_log_window(LayoutWindow *lw, gint *x, gint *y,
+														gint *w, gint *h)
+{
+	GdkWindow *window;
+
+	if (!layout_valid(&lw)) return FALSE;
+
+	if (!lw->log_window)
+		{
+		return FALSE;
+		}
+
+	window = gtk_widget_get_window(lw->log_window);
+	gdk_window_get_root_origin(window, x, y);
+	*w = gdk_window_get_width(window);
+	*h = gdk_window_get_height(window);
+
+	return TRUE;
+}
+
 static void layout_tools_geometry_sync(LayoutWindow *lw)
 {
-	layout_geometry_get_tools(lw, &lw->options.float_window.x, &lw->options.float_window.x,
+	layout_geometry_get_tools(lw, &lw->options.float_window.x, &lw->options.float_window.y,
 				  &lw->options.float_window.w, &lw->options.float_window.h, &lw->options.float_window.vdivider_pos);
 }
 
@@ -1250,6 +1509,7 @@ static void layout_tools_setup(LayoutWindow *lw, GtkWidget *tools, GtkWidget *fi
 		GdkWindowHints hints;
 
 		lw->tools = window_new(GTK_WINDOW_TOPLEVEL, "tools", PIXBUF_INLINE_ICON_TOOLS, NULL, _("Tools"));
+		DEBUG_NAME(lw->tools);
 		g_signal_connect(G_OBJECT(lw->tools), "delete_event",
 				 G_CALLBACK(layout_tools_delete_cb), lw);
 		layout_keyboard_init(lw, lw->tools);
@@ -1273,6 +1533,7 @@ static void layout_tools_setup(LayoutWindow *lw, GtkWidget *tools, GtkWidget *fi
 
 		gtk_window_set_resizable(GTK_WINDOW(lw->tools), TRUE);
 		gtk_container_set_border_width(GTK_CONTAINER(lw->tools), 0);
+		gtk_container_remove(GTK_CONTAINER(lw->main_box), lw->menu_tool_bar);
 
 		new_window = TRUE;
 		}
@@ -1286,7 +1547,9 @@ static void layout_tools_setup(LayoutWindow *lw, GtkWidget *tools, GtkWidget *fi
 	layout_actions_add_window(lw, lw->tools);
 
 	vbox = gtk_vbox_new(FALSE, 0);
+	DEBUG_NAME(vbox);
 	gtk_container_add(GTK_CONTAINER(lw->tools), vbox);
+	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(lw->menu_tool_bar), FALSE, FALSE, 0);
 	gtk_widget_show(vbox);
 
 	layout_status_setup(lw, vbox, TRUE);
@@ -1294,10 +1557,12 @@ static void layout_tools_setup(LayoutWindow *lw, GtkWidget *tools, GtkWidget *fi
 	if (vertical)
 		{
 		lw->tools_pane = gtk_vpaned_new();
+		DEBUG_NAME(lw->tools_pane);
 		}
 	else
 		{
 		lw->tools_pane = gtk_hpaned_new();
+		DEBUG_NAME(lw->tools_pane);
 		}
 	gtk_box_pack_start(GTK_BOX(vbox), lw->tools_pane, TRUE, TRUE, 0);
 	gtk_widget_show(lw->tools_pane);
@@ -1436,7 +1701,8 @@ static void layout_grid_setup(LayoutWindow *lw)
 	layout_actions_setup(lw);
 
 	lw->group_box = gtk_vbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(lw->main_box), lw->group_box, TRUE, TRUE, 0);
+	DEBUG_NAME(lw->group_box);
+	gtk_box_pack_end(GTK_BOX(lw->main_box), lw->group_box, TRUE, TRUE, 0);
 	gtk_widget_show(lw->group_box);
 
 	priority_location = layout_grid_compass(lw);
@@ -1445,16 +1711,20 @@ static void layout_grid_setup(LayoutWindow *lw)
 		{
 		layout_split_change(lw, lw->split_mode); /* this re-creates image frame for the new configuration */
 		image_sb = lw->utility_box;
+		DEBUG_NAME(image_sb);
 		}
 	else
 		{
 		GtkWidget *image; /* image or split images together */
 		image = layout_image_setup_split(lw, lw->split_mode);
 		image_sb = layout_bars_prepare(lw, image);
+		DEBUG_NAME(image_sb);
 		}
 
 	tools = layout_tools_new(lw);
+	DEBUG_NAME(tools);
 	files = layout_list_new(lw);
+	DEBUG_NAME(files);
 
 
 	if (lw->options.tools_float || lw->options.tools_hidden)
@@ -1481,8 +1751,10 @@ static void layout_grid_setup(LayoutWindow *lw)
 	layout_grid_compute(lw, image_sb, tools, files, &w1, &w2, &w3);
 
 	v = lw->v_pane = gtk_vpaned_new();
+	DEBUG_NAME(v);
 
 	h = lw->h_pane = gtk_hpaned_new();
+	DEBUG_NAME(h);
 
 	if (!layout_location_vertical(priority_location))
 		{
@@ -1570,9 +1842,10 @@ void layout_style_set(LayoutWindow *lw, gint style, const gchar *order)
 	/* preserve utility_box (image + sidebars), menu_bar and toolbars to be reused later in layout_grid_setup */
 	/* lw->image is preserved together with lw->utility_box */
 	if (lw->utility_box) gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(lw->utility_box)), lw->utility_box);
-	if (lw->menu_bar) gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(lw->menu_bar)), lw->menu_bar);
-	for (i = 0; i < TOOLBAR_COUNT; i++)
-		if (lw->toolbar[i]) gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(lw->toolbar[i])), lw->toolbar[i]);
+
+	if (lw->toolbar[TOOLBAR_STATUS]) gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(lw->toolbar[TOOLBAR_STATUS])), lw->toolbar[TOOLBAR_STATUS]);
+
+	if (lw->menu_tool_bar) gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(lw->menu_tool_bar)), lw->menu_tool_bar);
 
 	/* clear it all */
 
@@ -1728,7 +2001,7 @@ gboolean layout_tools_float_get(LayoutWindow *lw, gboolean *popped, gboolean *hi
 void layout_toolbar_toggle(LayoutWindow *lw)
 {
 	if (!layout_valid(&lw)) return;
-	if (!lw->toolbar) return;
+	if (!lw->toolbar[TOOLBAR_MAIN]) return;
 
 	lw->options.toolbar_hidden = !lw->options.toolbar_hidden;
 
@@ -1814,6 +2087,11 @@ static void layout_config_apply_cb(GtkWidget *widget, gpointer data)
 	layout_apply_options(lc->lw, &lc->options);
 }
 
+static void layout_config_help_cb(GtkWidget *widget, gpointer data)
+{
+	help_window_show("GuideOptionsLayout.html");
+}
+
 static void layout_config_ok_cb(GtkWidget *widget, gpointer data)
 {
 	LayoutConfig *lc = data;
@@ -1830,18 +2108,30 @@ static void home_path_set_current_cb(GtkWidget *widget, gpointer data)
 static void startup_path_set_current_cb(GtkWidget *widget, gpointer data)
 {
 	LayoutConfig *lc = data;
+	if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
+		{
+		return;
+		}
 	lc->options.startup_path = STARTUP_PATH_CURRENT;
 }
 
 static void startup_path_set_last_cb(GtkWidget *widget, gpointer data)
 {
 	LayoutConfig *lc = data;
+	if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
+		{
+		return;
+		}
 	lc->options.startup_path = STARTUP_PATH_LAST;
 }
 
 static void startup_path_set_home_cb(GtkWidget *widget, gpointer data)
 {
 	LayoutConfig *lc = data;
+	if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
+		{
+		return;
+		}
 	lc->options.startup_path = STARTUP_PATH_HOME;
 }
 
@@ -1872,6 +2162,7 @@ void layout_show_config_window(LayoutWindow *lw)
 	copy_layout_options(&lc->options, &lw->options);
 
 	lc->configwindow = window_new(GTK_WINDOW_TOPLEVEL, "Layout", PIXBUF_INLINE_ICON_CONFIG, NULL, _("Window options and layout"));
+	DEBUG_NAME(lc->configwindow);
 	gtk_window_set_type_hint(GTK_WINDOW(lc->configwindow), GDK_WINDOW_TYPE_HINT_DIALOG);
 
 	g_signal_connect(G_OBJECT(lc->configwindow), "delete_event",
@@ -1882,6 +2173,7 @@ void layout_show_config_window(LayoutWindow *lw)
 	gtk_container_set_border_width(GTK_CONTAINER(lc->configwindow), PREF_PAD_BORDER);
 
 	win_vbox = gtk_vbox_new(FALSE, PREF_PAD_SPACE);
+	DEBUG_NAME(win_vbox);
 	gtk_container_add(GTK_CONTAINER(lc->configwindow), win_vbox);
 	gtk_widget_show(win_vbox);
 
@@ -1906,6 +2198,12 @@ void layout_show_config_window(LayoutWindow *lw)
 	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 	gtk_widget_show(button);
 */
+	button = pref_button_new(NULL, GTK_STOCK_HELP, NULL, FALSE,
+				 G_CALLBACK(layout_config_help_cb), lc);
+	gtk_container_add(GTK_CONTAINER(hbox), button);
+	gtk_widget_set_can_default(button, TRUE);
+	gtk_widget_show(button);
+
 	button = pref_button_new(NULL, GTK_STOCK_APPLY, NULL, FALSE,
 				 G_CALLBACK(layout_config_apply_cb), lc);
 	gtk_container_add(GTK_CONTAINER(hbox), button);
@@ -1924,8 +2222,10 @@ void layout_show_config_window(LayoutWindow *lw)
 		}
 
 	frame = pref_frame_new(win_vbox, TRUE, NULL, GTK_ORIENTATION_VERTICAL, PREF_PAD_GAP);
+	DEBUG_NAME(frame);
 
 	vbox = gtk_vbox_new(FALSE, PREF_PAD_SPACE);
+	DEBUG_NAME(vbox);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 	gtk_widget_show(vbox);
 
@@ -1935,7 +2235,7 @@ void layout_show_config_window(LayoutWindow *lw)
 	pref_label_new(group, _("Home path (empty to use your home directory)"));
 	hbox = pref_box_new(group, FALSE, GTK_ORIENTATION_HORIZONTAL, PREF_PAD_SPACE);
 
-	tabcomp = tab_completion_new(&lc->home_path_entry, lc->options.home_path, NULL, NULL);
+	tabcomp = tab_completion_new(&lc->home_path_entry, lc->options.home_path, NULL, NULL, NULL, NULL);
 	tab_completion_add_select_button(lc->home_path_entry, NULL, TRUE);
 	gtk_box_pack_start(GTK_BOX(hbox), tabcomp, TRUE, TRUE, 0);
 	gtk_widget_show(tabcomp);
@@ -1964,6 +2264,7 @@ void layout_show_config_window(LayoutWindow *lw)
 	group = pref_group_new(vbox, FALSE, _("Layout"), GTK_ORIENTATION_VERTICAL);
 
 	lc->layout_widget = layout_config_new();
+	DEBUG_NAME(lc->layout_widget);
 	layout_config_set(lc->layout_widget, lw->options.style, lw->options.order);
 	gtk_box_pack_start(GTK_BOX(group), lc->layout_widget, TRUE, TRUE, 0);
 
@@ -2004,6 +2305,10 @@ void layout_sync_options_with_current_state(LayoutWindow *lw)
 
 	g_free(lw->options.last_path);
 	lw->options.last_path = g_strdup(layout_get_path(lw));
+
+	layout_geometry_get_log_window(lw, &lw->options.log_window.x, &lw->options.log_window.y,
+	                                 &lw->options.log_window.w, &lw->options.log_window.h);
+
 }
 
 void layout_apply_options(LayoutWindow *lw, LayoutOptions *lop)
@@ -2114,8 +2419,8 @@ LayoutWindow *layout_new_with_geometry(FileData *dir_fd, LayoutOptions *lop,
 
 	layout_config_parse(lw->options.style, lw->options.order,
 			    &lw->dir_location,  &lw->file_location, &lw->image_location);
-	if (lw->options.dir_view_type >= VIEW_DIR_TYPES_COUNT) lw->options.dir_view_type = 0;
-	if (lw->options.file_view_type >= VIEW_FILE_TYPES_COUNT) lw->options.file_view_type = 0;
+	if (lw->options.dir_view_type > DIRVIEW_LAST) lw->options.dir_view_type = 0;
+	if (lw->options.file_view_type > FILEVIEW_LAST) lw->options.file_view_type = 0;
 
 	/* divider positions */
 
@@ -2129,6 +2434,7 @@ LayoutWindow *layout_new_with_geometry(FileData *dir_fd, LayoutOptions *lop,
 	/* window */
 
 	lw->window = window_new(GTK_WINDOW_TOPLEVEL, GQ_APPNAME_LC, NULL, NULL, NULL);
+	DEBUG_NAME(lw->window);
 	gtk_window_set_resizable(GTK_WINDOW(lw->window), TRUE);
 	gtk_container_set_border_width(GTK_CONTAINER(lw->window), 0);
 
@@ -2175,6 +2481,7 @@ LayoutWindow *layout_new_with_geometry(FileData *dir_fd, LayoutOptions *lop,
 #endif
 
 	lw->main_box = gtk_vbox_new(FALSE, 0);
+	DEBUG_NAME(lw->main_box);
 	gtk_container_add(GTK_CONTAINER(lw->window), lw->main_box);
 	gtk_widget_show(lw->main_box);
 
@@ -2235,6 +2542,7 @@ void layout_write_attributes(LayoutOptions *layout, GString *outstr, gint indent
 	WRITE_NL(); WRITE_UINT(*layout, dir_view_type);
 	WRITE_NL(); WRITE_UINT(*layout, file_view_type);
 	WRITE_NL(); WRITE_BOOL(*layout, show_marks);
+	WRITE_NL(); WRITE_BOOL(*layout, show_file_filter);
 	WRITE_NL(); WRITE_BOOL(*layout, show_thumbnails);
 	WRITE_NL(); WRITE_BOOL(*layout, show_directory_date);
 	WRITE_NL(); WRITE_CHAR(*layout, home_path);
@@ -2252,6 +2560,9 @@ void layout_write_attributes(LayoutOptions *layout, GString *outstr, gint indent
 	WRITE_NL(); WRITE_INT(*layout, main_window.vdivider_pos);
 	WRITE_SEPARATOR();
 
+	WRITE_NL(); WRITE_INT(*layout, folder_window.vdivider_pos);
+	WRITE_SEPARATOR();
+
 	WRITE_NL(); WRITE_INT(*layout, float_window.x);
 	WRITE_NL(); WRITE_INT(*layout, float_window.y);
 	WRITE_NL(); WRITE_INT(*layout, float_window.w);
@@ -2269,10 +2580,26 @@ void layout_write_attributes(LayoutOptions *layout, GString *outstr, gint indent
 
 	WRITE_NL(); WRITE_BOOL(*layout, toolbar_hidden);
 	WRITE_NL(); WRITE_BOOL(*layout, show_info_pixel);
+	WRITE_SEPARATOR();
+
+	WRITE_NL(); WRITE_BOOL(*layout, bars_state.info);
+	WRITE_NL(); WRITE_BOOL(*layout, bars_state.sort);
+	WRITE_NL(); WRITE_BOOL(*layout, bars_state.tools_float);
+	WRITE_NL(); WRITE_BOOL(*layout, bars_state.tools_hidden);
+	WRITE_NL(); WRITE_BOOL(*layout, bars_state.hidden);
+	WRITE_SEPARATOR();
 
 	WRITE_NL(); WRITE_UINT(*layout, image_overlay.state);
 	WRITE_NL(); WRITE_INT(*layout, image_overlay.histogram_channel);
 	WRITE_NL(); WRITE_INT(*layout, image_overlay.histogram_mode);
+
+	WRITE_NL(); WRITE_INT(*layout, log_window.x);
+	WRITE_NL(); WRITE_INT(*layout, log_window.y);
+	WRITE_NL(); WRITE_INT(*layout, log_window.w);
+	WRITE_NL(); WRITE_INT(*layout, log_window.h);
+	WRITE_SEPARATOR();
+
+	WRITE_NL(); WRITE_BOOL(*layout, animate);
 }
 
 
@@ -2286,6 +2613,12 @@ void layout_write_config(LayoutWindow *lw, GString *outstr, gint indent)
 	bar_sort_write_config(lw->bar_sort, outstr, indent + 1);
 	bar_write_config(lw->bar, outstr, indent + 1);
 
+	WRITE_SEPARATOR();
+	generic_dialog_windows_write_config(outstr, indent + 1);
+
+	WRITE_SEPARATOR();
+	layout_toolbar_write_config(lw, TOOLBAR_MAIN, outstr, indent + 1);
+
 	WRITE_NL(); WRITE_STRING("</layout>");
 }
 
@@ -2307,6 +2640,7 @@ void layout_load_attributes(LayoutOptions *layout, const gchar **attribute_names
 		if (READ_UINT(*layout, dir_view_type)) continue;
 		if (READ_UINT(*layout, file_view_type)) continue;
 		if (READ_BOOL(*layout, show_marks)) continue;
+		if (READ_BOOL(*layout, show_file_filter)) continue;
 		if (READ_BOOL(*layout, show_thumbnails)) continue;
 		if (READ_BOOL(*layout, show_directory_date)) continue;
 		if (READ_CHAR(*layout, home_path)) continue;
@@ -2324,6 +2658,8 @@ void layout_load_attributes(LayoutOptions *layout, const gchar **attribute_names
 		if (READ_INT(*layout, main_window.hdivider_pos)) continue;
 		if (READ_INT(*layout, main_window.vdivider_pos)) continue;
 
+		if (READ_INT_CLAMP(*layout, folder_window.vdivider_pos, 1, 1000)) continue;
+
 		if (READ_INT(*layout, float_window.x)) continue;
 		if (READ_INT(*layout, float_window.y)) continue;
 		if (READ_INT(*layout, float_window.w)) continue;
@@ -2338,10 +2674,23 @@ void layout_load_attributes(LayoutOptions *layout, const gchar **attribute_names
 		if (READ_BOOL(*layout, toolbar_hidden)) continue;
 		if (READ_BOOL(*layout, show_info_pixel)) continue;
 
+		if (READ_BOOL(*layout, bars_state.info)) continue;
+		if (READ_BOOL(*layout, bars_state.sort)) continue;
+		if (READ_BOOL(*layout, bars_state.tools_float)) continue;
+		if (READ_BOOL(*layout, bars_state.tools_hidden)) continue;
+		if (READ_BOOL(*layout, bars_state.hidden)) continue;
+
 		if (READ_UINT(*layout, image_overlay.state)) continue;
 		if (READ_INT(*layout, image_overlay.histogram_channel)) continue;
 		if (READ_INT(*layout, image_overlay.histogram_mode)) continue;
 
+		if (READ_INT(*layout, log_window.x)) continue;
+		if (READ_INT(*layout, log_window.y)) continue;
+		if (READ_INT(*layout, log_window.w)) continue;
+		if (READ_INT(*layout, log_window.h)) continue;
+
+		if (READ_BOOL(*layout, animate)) continue;
+
 		log_printf("unknown attribute %s = %s\n", option, value);
 		}
 	if (id && strcmp(id, LAYOUT_ID_CURRENT) != 0)
@@ -2424,7 +2773,7 @@ LayoutWindow *layout_new_from_config(const gchar **attribute_names, const gchar
 
 	if (use_commandline && command_line->startup_full_screen) layout_image_full_screen_start(lw);
 	if (use_commandline && command_line->startup_in_slideshow) layout_image_slideshow_start(lw);
-
+	if (use_commandline && command_line->log_window_show) log_window_new(lw);
 
 	g_free(path);
 	free_layout_options_content(&lop);
