@@ -26,14 +26,17 @@
 #include "color-man.h"
 #include "dnd.h"
 #include "editors.h"
+#include "exif.h"
 #include "filedata.h"
 #include "fullscreen.h"
+#include "history_list.h"
 #include "image.h"
 #include "image-overlay.h"
 #include "img-view.h"
 #include "layout.h"
 #include "layout_util.h"
 #include "menu.h"
+#include "metadata.h"
 #include "misc.h"
 #include "pixbuf_util.h"
 #include "pixbuf-renderer.h"
@@ -46,9 +49,12 @@
 
 #include <gdk/gdkkeysyms.h> /* for keyboard values */
 
+#define FILE_COLUMN_POINTER 0
 
 static GtkWidget *layout_image_pop_menu(LayoutWindow *lw);
 static void layout_image_set_buttons(LayoutWindow *lw);
+static gboolean layout_image_animate_new_file(LayoutWindow *lw);
+static void layout_image_animate_update_image(LayoutWindow *lw);
 
 /*
  *----------------------------------------------------------------------------
@@ -100,6 +106,7 @@ void layout_image_full_screen_start(LayoutWindow *lw)
 	layout_actions_add_window(lw, lw->full_screen->window);
 
 	image_osd_copy_status(lw->full_screen->normal_imd, lw->image);
+	layout_image_animate_update_image(lw);
 }
 
 void layout_image_full_screen_stop(LayoutWindow *lw)
@@ -111,6 +118,8 @@ void layout_image_full_screen_stop(LayoutWindow *lw)
 		image_osd_copy_status(lw->image, lw->full_screen->normal_imd);
 
 	fullscreen_stop(lw->full_screen);
+
+	layout_image_animate_update_image(lw);
 }
 
 void layout_image_full_screen_toggle(LayoutWindow *lw)
@@ -263,6 +272,205 @@ static gboolean layout_image_slideshow_continue_check(LayoutWindow *lw)
 	return TRUE;
 }
 
+/*
+ *----------------------------------------------------------------------------
+ * Animation
+ *----------------------------------------------------------------------------
+ */
+
+static void image_animation_data_free(AnimationData *fd)
+{
+	if(!fd) return;
+	if(fd->iter) g_object_unref(fd->iter);
+	if(fd->gpa) g_object_unref(fd->gpa);
+	if(fd->cancellable) g_object_unref(fd->cancellable);
+	g_free(fd);
+}
+
+static gboolean animation_should_continue(AnimationData *fd)
+{
+	if (!fd->valid)
+		return FALSE;
+
+	return TRUE;
+}
+
+static gboolean show_next_frame(gpointer data)
+{
+	AnimationData *fd = (AnimationData*)data;
+	int delay;
+	PixbufRenderer *pr;
+
+	if(animation_should_continue(fd)==FALSE)
+		{
+		image_animation_data_free(fd);
+		return FALSE;
+		}
+
+	pr = (PixbufRenderer*)fd->iw->pr;
+
+	if (gdk_pixbuf_animation_iter_advance(fd->iter,NULL)==FALSE)
+		{
+		/* This indicates the animation is complete.
+		   Return FALSE here to disable looping. */
+		}
+
+	fd->gpb = gdk_pixbuf_animation_iter_get_pixbuf(fd->iter);
+	image_change_pixbuf(fd->iw,fd->gpb,pr->zoom,FALSE);
+
+	if (fd->iw->func_update)
+		fd->iw->func_update(fd->iw, fd->iw->data_update);
+
+	delay = gdk_pixbuf_animation_iter_get_delay_time(fd->iter);
+	if (delay!=fd->delay)
+		{
+		if (delay>0) /* Current frame not static. */
+			{
+			fd->delay=delay;
+			g_timeout_add(delay,show_next_frame,fd);
+			}
+		else
+			{
+			image_animation_data_free(fd);
+			}
+		return FALSE;
+		}
+
+	return TRUE;
+}
+
+static gboolean layout_image_animate_check(LayoutWindow *lw)
+{
+	if (!layout_valid(&lw)) return FALSE;
+
+	if(!lw->options.animate || lw->image->image_fd == NULL || lw->image->image_fd->extension == NULL || g_ascii_strcasecmp(lw->image->image_fd->extension,".GIF")!=0)
+		{
+		if(lw->animation)
+			{
+			lw->animation->valid = FALSE;
+			if (lw->animation->cancellable)
+				{
+				g_cancellable_cancel(lw->animation->cancellable);
+				}
+			lw->animation = NULL;
+			}
+		return FALSE;
+		}
+
+	return TRUE;
+}
+
+static void layout_image_animate_update_image(LayoutWindow *lw)
+{
+	if (!layout_valid(&lw)) return;
+
+	if(lw->options.animate && lw->animation)
+		{
+		if (lw->full_screen && lw->image != lw->full_screen->imd)
+			lw->animation->iw = lw->full_screen->imd;
+		else
+			lw->animation->iw = lw->image;
+		}
+}
+
+
+static void animation_async_ready_cb(GObject *source_object, GAsyncResult *res, gpointer data)
+{
+	GError *error = NULL;
+	AnimationData *animation = data;
+
+	if (animation)
+		{
+		if (g_cancellable_is_cancelled(animation->cancellable))
+			{
+			gdk_pixbuf_animation_new_from_stream_finish(res, NULL);
+			g_object_unref(animation->in_file);
+			g_object_unref(animation->gfstream);
+			image_animation_data_free(animation);
+			return;
+			}
+
+		animation->gpa = gdk_pixbuf_animation_new_from_stream_finish(res, &error);
+		if (animation->gpa)
+			{
+			if (!gdk_pixbuf_animation_is_static_image(animation->gpa))
+				{
+				if (animation->iter = gdk_pixbuf_animation_get_iter(animation->gpa,NULL))
+					{
+					animation->iter = gdk_pixbuf_animation_get_iter(animation->gpa, NULL);
+					// FIXME: the reference count is being incremented here. Reason unknown.
+					g_object_unref(animation->gpa);
+					animation->data_adr = animation->lw->image->image_fd;
+					animation->delay = gdk_pixbuf_animation_iter_get_delay_time(animation->iter);
+					animation->valid = TRUE;
+
+					layout_image_animate_update_image(animation->lw);
+
+					g_timeout_add(animation->delay, show_next_frame, animation);
+					}
+				}
+			}
+		else
+			{
+			log_printf("Error reading GIF file: %s\n", error->message);
+			}
+
+		g_object_unref(animation->in_file);
+		g_object_unref(animation->gfstream);
+		}
+}
+
+static gboolean layout_image_animate_new_file(LayoutWindow *lw)
+{
+	GFileInputStream *gfstream;
+	GError *error = NULL;
+	AnimationData *animation;
+	GFile *in_file;
+
+	if(!layout_image_animate_check(lw)) return FALSE;
+
+	if(lw->animation) lw->animation->valid = FALSE;
+
+	if (lw->animation)
+		{
+		g_cancellable_cancel(lw->animation->cancellable);
+		}
+
+	animation = g_new0(AnimationData, 1);
+	lw->animation = animation;
+	animation->lw = lw;
+	animation->cancellable = g_cancellable_new();
+
+	in_file = g_file_new_for_path(lw->image->image_fd->path);
+	animation->in_file = in_file;
+	gfstream = g_file_read(in_file, NULL, &error);
+	if (gfstream)
+		{
+		animation->gfstream = gfstream;
+		gdk_pixbuf_animation_new_from_stream_async((GInputStream*)gfstream, animation->cancellable, animation_async_ready_cb, animation);
+		}
+	else
+		{
+		log_printf("Error reading GIF file: %s\nError: %s\n", lw->image->image_fd->path, error->message);
+		}
+
+	return TRUE;
+}
+
+void layout_image_animate_toggle(LayoutWindow *lw)
+{
+	GtkAction *action;
+
+	if (!lw) return;
+
+	lw->options.animate = !lw->options.animate;
+
+	action = gtk_action_group_get_action(lw->action_group, "Animate");
+	gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), lw->options.animate);
+
+	layout_image_animate_new_file(lw);
+}
+
 /*
  *----------------------------------------------------------------------------
  * pop-up menus
@@ -325,7 +533,7 @@ static void li_pop_menu_alter_cb(GtkWidget *widget, gpointer data)
 	lw = submenu_item_get_data(widget);
 	type = (AlterType)GPOINTER_TO_INT(data);
 
-	image_alter_orientation(lw->image, type);
+	image_alter_orientation(lw->image, lw->image->image_fd, type);
 }
 
 static void li_pop_menu_new_cb(GtkWidget *widget, gpointer data)
@@ -365,7 +573,14 @@ static void li_pop_menu_copy_path_cb(GtkWidget *widget, gpointer data)
 {
 	LayoutWindow *lw = data;
 
-	file_util_copy_path_to_clipboard(layout_image_get_fd(lw));
+	file_util_copy_path_to_clipboard(layout_image_get_fd(lw), TRUE);
+}
+
+static void li_pop_menu_copy_path_unquoted_cb(GtkWidget *widget, gpointer data)
+{
+	LayoutWindow *lw = data;
+
+	file_util_copy_path_to_clipboard(layout_image_get_fd(lw), FALSE);
 }
 
 static void li_pop_menu_move_cb(GtkWidget *widget, gpointer data)
@@ -388,6 +603,16 @@ static void li_pop_menu_delete_cb(GtkWidget *widget, gpointer data)
 {
 	LayoutWindow *lw = data;
 
+	options->file_ops.safe_delete_enable = FALSE;
+	file_util_delete(layout_image_get_fd(lw), NULL,
+			 li_pop_menu_click_parent(widget, lw));
+}
+
+static void li_pop_menu_move_to_trash_cb(GtkWidget *widget, gpointer data)
+{
+	LayoutWindow *lw = data;
+
+	options->file_ops.safe_delete_enable = TRUE;
 	file_util_delete(layout_image_get_fd(lw), NULL,
 			 li_pop_menu_click_parent(widget, lw));
 }
@@ -420,6 +645,13 @@ static void li_pop_menu_full_screen_cb(GtkWidget *widget, gpointer data)
 	layout_image_full_screen_toggle(lw);
 }
 
+static void li_pop_menu_animate_cb(GtkWidget *widget, gpointer data)
+{
+	LayoutWindow *lw = data;
+
+	layout_image_animate_toggle(lw);
+}
+
 static void li_pop_menu_hide_cb(GtkWidget *widget, gpointer data)
 {
 	LayoutWindow *lw = data;
@@ -475,6 +707,25 @@ static GList *layout_image_get_fd_list(LayoutWindow *lw)
 	return list;
 }
 
+/**
+ * @brief Add file selection list to a collection
+ * @param[in] widget 
+ * @param[in] data Index to the collection list menu item selected, or -1 for new collection
+ * 
+ * 
+ */
+static void layout_pop_menu_collections_cb(GtkWidget *widget, gpointer data)
+{
+	LayoutWindow *lw;
+	GList *selection_list = NULL;
+
+	lw = submenu_item_get_data(widget);
+	selection_list = g_list_append(selection_list, layout_image_get_fd(lw));
+	pop_menu_collections(selection_list, data);
+
+	filelist_free(selection_list);
+}
+
 static GtkWidget *layout_image_pop_menu(LayoutWindow *lw)
 {
 	GtkWidget *menu;
@@ -501,8 +752,9 @@ static GtkWidget *layout_image_pop_menu(LayoutWindow *lw)
 	submenu = submenu_add_edit(menu, &item, G_CALLBACK(li_pop_menu_edit_cb), lw, editmenu_fd_list);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
 	menu_item_add_divider(submenu);
+#if !GTK_CHECK_VERSION(3,0,0)
 	menu_item_add(submenu, _("Set as _wallpaper"), G_CALLBACK(li_pop_menu_wallpaper_cb), lw);
-
+#endif
 	item = submenu_add_alter(menu, G_CALLBACK(li_pop_menu_alter_cb), lw);
 
 	item = menu_item_add_stock(menu, _("View in _new window"), GTK_STOCK_NEW, G_CALLBACK(li_pop_menu_new_cb), lw);
@@ -519,12 +771,26 @@ static GtkWidget *layout_image_pop_menu(LayoutWindow *lw)
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
 	item = menu_item_add(menu, _("_Rename..."), G_CALLBACK(li_pop_menu_rename_cb), lw);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
-	item = menu_item_add_stock(menu, _("_Delete..."), GTK_STOCK_DELETE, G_CALLBACK(li_pop_menu_delete_cb), lw);
-	if (!path) gtk_widget_set_sensitive(item, FALSE);
-
 	item = menu_item_add(menu, _("_Copy path"), G_CALLBACK(li_pop_menu_copy_path_cb), lw);
+	item = menu_item_add(menu, _("_Copy path unquoted"), G_CALLBACK(li_pop_menu_copy_path_unquoted_cb), lw);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
+	menu_item_add_divider(menu);
 
+	item = menu_item_add_stock(menu,
+				options->file_ops.confirm_move_to_trash ? _("Move to Trash...") :
+					_("Move to Trash"), PIXBUF_INLINE_ICON_TRASH,
+								G_CALLBACK(li_pop_menu_move_to_trash_cb), lw);
+	if (!path) gtk_widget_set_sensitive(item, FALSE);
+	item = menu_item_add_stock(menu,
+				options->file_ops.confirm_delete ? _("_Delete...") :
+					_("_Delete"), GTK_STOCK_DELETE,
+								G_CALLBACK(li_pop_menu_delete_cb), lw);
+	if (!path) gtk_widget_set_sensitive(item, FALSE);
+	menu_item_add_divider(menu);
+
+	submenu = submenu_add_collections(menu, &item,
+				G_CALLBACK(layout_pop_menu_collections_cb), lw);
+	gtk_widget_set_sensitive(item, TRUE);
 	menu_item_add_divider(menu);
 
 	if (layout_image_slideshow_active(lw))
@@ -557,6 +823,8 @@ static GtkWidget *layout_image_pop_menu(LayoutWindow *lw)
 		menu_item_add(menu, _("Exit _full screen"), G_CALLBACK(li_pop_menu_full_screen_cb), lw);
 		}
 
+	menu_item_add_check(menu, _("_Animate"), lw->options.animate, G_CALLBACK(li_pop_menu_animate_cb), lw);
+
 	menu_item_add_divider(menu);
 
 	item = menu_item_add_check(menu, _("Hide file _list"), lw->options.tools_hidden,
@@ -838,12 +1106,12 @@ void layout_image_zoom_adjust_at_point(LayoutWindow *lw, gdouble increment, gint
 		{
 		image_zoom_adjust_at_point(lw->full_screen->imd, increment, x, y);
 		}
-
-	if (!connect_zoom) return;
+	if (!connect_zoom && !lw->split_mode) return;
 
 	for (i = 0; i < MAX_SPLIT_IMAGES; i++)
 		{
-		if (lw->split_images[i] && lw->split_images[i] != lw->image)
+		if (lw->split_images[i] && lw->split_images[i] != lw->image &&
+						lw->split_images[i]->mouse_wheel_mode)
 			image_zoom_adjust_at_point(lw->split_images[i], increment, x, y);
 		}
 }
@@ -894,7 +1162,118 @@ void layout_image_alter_orientation(LayoutWindow *lw, AlterType type)
 {
 	if (!layout_valid(&lw)) return;
 
-	image_alter_orientation(lw->image, type);
+	GtkTreeModel *store;
+	GList *work;
+	GtkTreeSelection *selection;
+	GtkTreePath *tpath;
+	FileData *fd_n;
+	GtkTreeIter iter;
+
+	if (!lw || !lw->vf) return;
+
+	if (lw->vf->type == FILEVIEW_ICON)
+		{
+		if (!VFICON(lw->vf)->selection) return;
+		work = VFICON(lw->vf)->selection;
+		}
+	else
+		{
+		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(lw->vf->listview));
+		work = gtk_tree_selection_get_selected_rows(selection, &store);
+		}
+
+	while (work)
+		{
+		if (lw->vf->type == FILEVIEW_ICON)
+			{
+			fd_n = work->data;
+			work = work->next;
+			}
+		else
+			{
+			tpath = work->data;
+			gtk_tree_model_get_iter(store, &iter, tpath);
+			gtk_tree_model_get(store, &iter, FILE_COLUMN_POINTER, &fd_n, -1);
+			work = work->next;
+			}
+
+		image_alter_orientation(lw->image, fd_n, type);
+		}
+}
+
+static void image_alter_rating(FileData *fd_n, const gchar *rating)
+{
+	metadata_write_string(fd_n, RATING_KEY, rating);
+	read_rating_data(fd_n);
+}
+
+void layout_image_rating(LayoutWindow *lw, const gchar *rating)
+{
+	if (!layout_valid(&lw)) return;
+
+	GtkTreeModel *store;
+	GList *work;
+	GtkTreeSelection *selection;
+	GtkTreePath *tpath;
+	FileData *fd_n;
+	GtkTreeIter iter;
+
+	if (!lw || !lw->vf) return;
+
+	if (lw->vf->type == FILEVIEW_ICON)
+		{
+		if (!VFICON(lw->vf)->selection) return;
+		work = VFICON(lw->vf)->selection;
+		}
+	else
+		{
+		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(lw->vf->listview));
+		work = gtk_tree_selection_get_selected_rows(selection, &store);
+		}
+
+	while (work)
+		{
+		if (lw->vf->type == FILEVIEW_ICON)
+			{
+			fd_n = work->data;
+			work = work->next;
+			}
+		else
+			{
+			tpath = work->data;
+			gtk_tree_model_get_iter(store, &iter, tpath);
+			gtk_tree_model_get(store, &iter, FILE_COLUMN_POINTER, &fd_n, -1);
+			work = work->next;
+			}
+
+		image_alter_rating(fd_n, rating);
+		}
+}
+
+void layout_image_reset_orientation(LayoutWindow *lw)
+{
+	ImageWindow *imd= lw->image;
+
+	if (!layout_valid(&lw)) return;
+	if (!imd || !imd->pr || !imd->image_fd) return;
+
+	if (imd->orientation < 1 || imd->orientation > 8) imd->orientation = 1;
+
+	if (options->image.exif_rotate_enable)
+		{
+		imd->orientation = metadata_read_int(imd->image_fd, ORIENTATION_KEY, EXIF_ORIENTATION_TOP_LEFT);
+		}
+	else
+		{
+		imd->orientation = 1;
+		}
+
+	if (imd->image_fd->user_orientation != 0)
+		{
+		 imd->orientation = imd->image_fd->user_orientation;
+		}
+
+	pixbuf_renderer_set_orientation((PixbufRenderer *)imd->pr, imd->orientation);
 }
 
 void layout_image_set_desaturate(LayoutWindow *lw, gboolean desaturate)
@@ -1002,6 +1381,7 @@ void layout_image_set_fd(LayoutWindow *lw, FileData *fd)
 	layout_list_sync_fd(lw, fd);
 	layout_image_slideshow_continue_check(lw);
 	layout_bars_new_image(lw);
+	layout_image_animate_new_file(lw);
 }
 
 void layout_image_set_with_ahead(LayoutWindow *lw, FileData *fd, FileData *read_ahead_fd)
@@ -1401,15 +1781,20 @@ static void layout_image_button_cb(ImageWindow *imd, GdkEventButton *event, gpoi
 {
 	LayoutWindow *lw = data;
 	GtkWidget *menu;
+	FileData *dir_fd;
 
 	switch (event->button)
 		{
 		case MOUSE_BUTTON_LEFT:
-			if (lw->split_mode == SPLIT_NONE)
+			if (options->image_l_click_video && options->image_l_click_video_editor && imd->image_fd && imd->image_fd->format_class == FORMAT_CLASS_VIDEO)
+				{
+				start_editor_from_file(options->image_l_click_video_editor, imd->image_fd);
+				}
+			else if (options->image_lm_click_nav && lw->split_mode == SPLIT_NONE)
 				layout_image_next(lw);
 			break;
 		case MOUSE_BUTTON_MIDDLE:
-			if (lw->split_mode == SPLIT_NONE)
+			if (options->image_lm_click_nav && lw->split_mode == SPLIT_NONE)
 				layout_image_prev(lw);
 			break;
 		case MOUSE_BUTTON_RIGHT:
@@ -1420,6 +1805,16 @@ static void layout_image_button_cb(ImageWindow *imd, GdkEventButton *event, gpoi
 				}
 			gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 3, event->time);
 			break;
+		case MOUSE_BUTTON_BACK:
+			dir_fd = file_data_new_dir(history_chain_back());
+			layout_set_fd(lw, dir_fd);
+			file_data_unref(dir_fd);
+			break;
+		case MOUSE_BUTTON_FORWARD:
+			dir_fd = file_data_new_dir(history_chain_forward());
+			layout_set_fd(lw, dir_fd);
+			file_data_unref(dir_fd);
+			break;
 		default:
 			break;
 		}
@@ -1438,7 +1833,8 @@ static void layout_image_scroll_cb(ImageWindow *imd, GdkEventScroll *event, gpoi
 		}
 
 
-	if (event->state & GDK_CONTROL_MASK)
+	if ((event->state & GDK_CONTROL_MASK) ||
+				(imd->mouse_wheel_mode && !options->image_lm_click_nav))
 		{
 		switch (event->direction)
 			{
@@ -1488,7 +1884,7 @@ static void layout_image_scroll_cb(ImageWindow *imd, GdkEventScroll *event, gpoi
 		}
 }
 
-static void layout_image_drag_cb(ImageWindow *imd, GdkEventButton *event, gdouble dx, gdouble dy, gpointer data)
+static void layout_image_drag_cb(ImageWindow *imd, GdkEventMotion *event, gdouble dx, gdouble dy, gpointer data)
 {
 	gint i;
 	LayoutWindow *lw = data;
@@ -1559,7 +1955,7 @@ static void layout_image_button_inactive_cb(ImageWindow *imd, GdkEventButton *ev
 
 }
 
-static void layout_image_drag_inactive_cb(ImageWindow *imd, GdkEventButton *event, gdouble dx, gdouble dy, gpointer data)
+static void layout_image_drag_inactive_cb(ImageWindow *imd, GdkEventMotion *event, gdouble dx, gdouble dy, gpointer data)
 {
 	LayoutWindow *lw = data;
 	gint i = image_idx(lw, imd);
@@ -1765,16 +2161,19 @@ static void layout_image_setup_split_common(LayoutWindow *lw, gint n)
 				GList *work = g_list_last(layout_selection_list(lw));
 				gint j = 0;
 
-				if (work) work = work->prev;
-
 				while (work && j < i)
 					{
 					FileData *fd = work->data;
 					work = work->prev;
 
-					j++;
-					if (!fd || !*fd->path) continue;
+					if (!fd || !*fd->path || fd->parent ||
+										fd == lw->split_images[0]->image_fd)
+						{
+						continue;
+						}
 					img_fd = fd;
+
+					j++;
 					}
 				}
 
@@ -1842,9 +2241,15 @@ GtkWidget *layout_image_setup_split_hv(LayoutWindow *lw, gboolean horizontal)
 
 	/* horizontal split means vpaned and vice versa */
 	if (horizontal)
+		{
 		paned = gtk_vpaned_new();
+		DEBUG_NAME(paned);
+		}
 	else
+		{
 		paned = gtk_hpaned_new();
+		DEBUG_NAME(paned);
+		}
 
 	gtk_paned_pack1(GTK_PANED(paned), lw->split_images[0]->widget, TRUE, TRUE);
 	gtk_paned_pack2(GTK_PANED(paned), lw->split_images[1]->widget, TRUE, TRUE);
@@ -1870,8 +2275,11 @@ GtkWidget *layout_image_setup_split_quad(LayoutWindow *lw)
 	layout_image_setup_split_common(lw, 4);
 
 	hpaned = gtk_hpaned_new();
+	DEBUG_NAME(hpaned);
 	vpaned1 = gtk_vpaned_new();
+	DEBUG_NAME(vpaned1);
 	vpaned2 = gtk_vpaned_new();
+	DEBUG_NAME(vpaned2);
 
 	gtk_paned_pack1(GTK_PANED(vpaned1), lw->split_images[0]->widget, TRUE, TRUE);
 	gtk_paned_pack2(GTK_PANED(vpaned1), lw->split_images[2]->widget, TRUE, TRUE);
