@@ -41,6 +41,25 @@
 static lua_State *L; /** The LUA object needed for all operations (NOTE: That is
 		       * a upper-case variable to match the documentation!) */
 
+/* Taking that definition from lua 5.1 source */
+#if defined(LUA_VERSION_NUM) && LUA_VERSION_NUM >= 502
+int luaL_typerror(lua_State *L, int narg, const char *tname)
+{
+	const char *msg = lua_pushfstring(L, "%s expected, got %s", tname, luaL_typename(L, narg));
+	return luaL_argerror(L, narg, msg);
+}
+
+# define LUA_register_meta(L, meta) luaL_setfuncs(L, meta, 0);
+# define LUA_register_global(L, string, func) \
+	lua_newtable(L); \
+	luaL_setfuncs(L, func, 0); \
+	lua_pushvalue(L, -1); \
+	lua_setglobal(L, string)
+#else
+# define LUA_register_meta(L, meta) luaL_register(L, NULL, meta)
+# define LUA_register_global(L, string, func) luaL_register(L, string, func)
+#endif
+
 static FileData *lua_check_image(lua_State *L, int index)
 {
 	FileData **fd;
@@ -113,6 +132,15 @@ static int lua_image_get_size(lua_State *L)
 	return 1;
 }
 
+static int lua_image_get_marks(lua_State *L)
+{
+	FileData *fd;
+
+	fd = lua_check_image(L, 1);
+	lua_pushnumber(L, fd->marks);
+	return 1;
+}
+
 static ExifData *lua_check_exif(lua_State *L, int index)
 {
 	ExifData **exif;
@@ -158,7 +186,22 @@ static int lua_exif_get_datum(lua_State *L)
 			lua_pushnil(L);
 			return 1;
 			}
-		} // if (strcmp(key, "Exif.Photo.Da...
+		}
+	else if (strcmp(key, "Exif.Photo.DateTimeDigitized") == 0)
+		{
+		memset(&tm, 0, sizeof(tm));
+		if (value && strptime(value, "%Y:%m:%d %H:%M:%S", &tm))
+			{
+			datetime = mktime(&tm);
+			lua_pushnumber(L, datetime);
+			return 1;
+			}
+		else
+			{
+			lua_pushnil(L);
+			return 1;
+			}
+		}
 	lua_pushstring(L, value);
 	return 1;
 }
@@ -184,11 +227,12 @@ void lua_init(void)
 			{"get_date", lua_image_get_date},
 			{"get_size", lua_image_get_size},
 			{"get_exif", lua_image_get_exif},
+			{"get_marks", lua_image_get_marks},
 			{NULL, NULL}
 	};
-	luaL_register(L, "Image", image_methods);
+	LUA_register_global(L, "Image", image_methods);
 	luaL_newmetatable(L, "Image");
-	luaL_register(L, NULL, meta_methods);
+	LUA_register_meta(L, meta_methods);
 	lua_pushliteral(L, "__index");
 	lua_pushvalue(L, -3);
 	lua_settable(L, -3);
@@ -203,9 +247,9 @@ void lua_init(void)
 			{"get_datum", lua_exif_get_datum},
 			{NULL, NULL}
 	};
-	luaL_register(L, "Exif", exif_methods);
+	LUA_register_global(L, "Exif", exif_methods);
 	luaL_newmetatable(L, "Exif");
-	luaL_register(L, NULL, meta_methods);
+	LUA_register_meta(L, meta_methods);
 	lua_pushliteral(L, "__index");
 	lua_pushvalue(L, -3);
 	lua_settable(L, -3);
