@@ -106,8 +106,8 @@ void keyboard_scroll_calc(gint *x, gint *y, GdkEventKey *event)
 		delta = 8;
 		}
 
-	*x = *x * delta;
-	*y = *y * delta;
+	*x = *x * delta * options->keyboard_scroll_step;
+	*y = *y * delta * options->keyboard_scroll_step;
 }
 
 
@@ -221,11 +221,15 @@ static void parse_command_line(gint argc, gchar *argv[])
 	GList *remote_errors = NULL;
 	gboolean remote_do = FALSE;
 	gchar *first_dir = NULL;
+	gchar *app_lock;
+	gchar *pwd;
+	gchar *current_dir;
 
 	command_line = g_new0(CommandLine, 1);
 
 	command_line->argc = argc;
 	command_line->argv = argv;
+	command_line->regexp = NULL;
 
 	if (argc > 1)
 		{
@@ -255,6 +259,15 @@ static void parse_command_line(gint argc, gchar *argv[])
 				parse_command_line_process_file(cmd_all, &command_line->path, &command_line->file,
 								&list, &command_line->collection_list, &first_dir);
 				}
+			else if (is_collection(cmd_line))
+				{
+				gchar *path = NULL;
+
+				path = collection_path(cmd_line);
+				parse_command_line_process_file(path, &command_line->path, &command_line->file,
+								&list, &command_line->collection_list, &first_dir);
+				g_free(path);
+				}
 			else if (strncmp(cmd_line, "--debug", 7) == 0 && (cmd_line[7] == '\0' || cmd_line[7] == '='))
 				{
 				/* do nothing but do not produce warnings */
@@ -301,6 +314,27 @@ static void parse_command_line(gint argc, gchar *argv[])
 					remote_list = remote_build_list(remote_list, argc - i, &argv[i], &remote_errors);
 					}
 				}
+			else if ((strcmp(cmd_line, "+w") == 0) ||
+						strcmp(cmd_line, "--show-log-window") == 0)
+				{
+				command_line->log_window_show = TRUE;
+				}
+			else if (strncmp(cmd_line, "-o:", 3) == 0)
+				{
+				command_line->log_file = g_strdup(cmd_line + 3);
+				}
+			else if (strncmp(cmd_line, "--log-file:", 11) == 0)
+				{
+				command_line->log_file = g_strdup(cmd_line + 11);
+				}
+			else if (strncmp(cmd_line, "-g:", 3) == 0)
+				{
+				set_regexp(g_strdup(cmd_line+3));
+				}
+			else if (strncmp(cmd_line, "-grep:", 6) == 0)
+				{
+				set_regexp(g_strdup(cmd_line+3));
+				}
 			else if (strcmp(cmd_line, "-rh") == 0 ||
 				 strcmp(cmd_line, "--remote-help") == 0)
 				{
@@ -314,7 +348,7 @@ static void parse_command_line(gint argc, gchar *argv[])
 			else if (strcmp(cmd_line, "-v") == 0 ||
 				 strcmp(cmd_line, "--version") == 0)
 				{
-				printf_term("%s %s\n", GQ_APPNAME, VERSION);
+				printf_term(FALSE, "%s %s\n", GQ_APPNAME, VERSION);
 				exit(0);
 				}
 			else if (strcmp(cmd_line, "--alternate") == 0)
@@ -326,28 +360,31 @@ static void parse_command_line(gint argc, gchar *argv[])
 			else if (strcmp(cmd_line, "-h") == 0 ||
 				 strcmp(cmd_line, "--help") == 0)
 				{
-				printf_term("%s %s\n", GQ_APPNAME, VERSION);
-				printf_term(_("Usage: %s [options] [path]\n\n"), GQ_APPNAME_LC);
-				print_term(_("valid options are:\n"));
-				print_term(_("  +t, --with-tools                 force show of tools\n"));
-				print_term(_("  -t, --without-tools              force hide of tools\n"));
-				print_term(_("  -f, --fullscreen                 start in full screen mode\n"));
-				print_term(_("  -s, --slideshow                  start in slideshow mode\n"));
-				print_term(_("  -l, --list [files] [collections] open collection window for command line\n"));
-				print_term(_("      --blank                      start with blank file list\n"));
-				print_term(_("      --geometry=XxY+XOFF+YOFF     set main window location\n"));
-				print_term(_("  -r, --remote                     send following commands to open window\n"));
-				print_term(_("  -rh,--remote-help                print remote command list\n"));
+				printf_term(FALSE, "%s %s\n", GQ_APPNAME, VERSION);
+				printf_term(FALSE, _("Usage: %s [options] [path]\n\n"), GQ_APPNAME_LC);
+				print_term(FALSE, _("valid options are:\n"));
+				print_term(FALSE, _("  +t, --with-tools                 force show of tools\n"));
+				print_term(FALSE, _("  -t, --without-tools              force hide of tools\n"));
+				print_term(FALSE, _("  -f, --fullscreen                 start in full screen mode\n"));
+				print_term(FALSE, _("  -s, --slideshow                  start in slideshow mode\n"));
+				print_term(FALSE, _("  -l, --list [files] [collections] open collection window for command line\n"));
+				print_term(FALSE, _("      --blank                      start with blank file list\n"));
+				print_term(FALSE, _("      --geometry=XxY+XOFF+YOFF     set main window location\n"));
+				print_term(FALSE, _("  -r, --remote                     send following commands to open window\n"));
+				print_term(FALSE, _("  -rh,--remote-help                print remote command list\n"));
 #ifdef DEBUG
-				print_term(_("      --debug[=level]              turn on debug output\n"));
+				print_term(FALSE, _("      --debug[=level]              turn on debug output\n"));
+				print_term(FALSE, _("  -g:<regexp>, --grep:<regexp>     filter debug output\n"));
 #endif
-				print_term(_("  -v, --version                    print version info\n"));
-				print_term(_("  -h, --help                       show this message\n\n"));
+				print_term(FALSE, _("  +w, --show-log-window            show log window\n"));
+				print_term(FALSE, _("  -o:<file>, --log-file:<file>     save log data to file\n"));
+				print_term(FALSE, _("  -v, --version                    print version info\n"));
+				print_term(FALSE, _("  -h, --help                       show this message\n\n"));
 
 #if 0
 				/* these options are not officially supported!
 				 * only for testing new features, no need to translate them */
-				print_term(  "  --alternate                use alternate similarity algorithm\n");
+				print_term(FALSE, "  --alternate                use alternate similarity algorithm\n");
 #endif
 
 
@@ -355,7 +392,7 @@ static void parse_command_line(gint argc, gchar *argv[])
 				}
 			else if (!remote_do)
 				{
-				printf_term(_("invalid or ignored: %s\nUse --help for options\n"), cmd_line);
+				printf_term(TRUE, _("invalid or ignored: %s\nUse --help for options\n"), cmd_line);
 				}
 
 			g_free(cmd_all);
@@ -378,25 +415,45 @@ static void parse_command_line(gint argc, gchar *argv[])
 		}
 	g_free(first_dir);
 
+	/* If Geeqie is already running, prevent a second instance
+	 * from being started. Open a new window instead.
+	 */
+	app_lock = g_build_filename(get_rc_dir(), ".command", NULL);
+	if (remote_server_exists(app_lock) && !remote_do)
+		{
+		remote_do = TRUE;
+		remote_list = g_list_append(remote_list, "--new-window");
+	}
+	g_free(app_lock);
+
 	if (remote_do)
 		{
 		if (remote_errors)
 			{
 			GList *work = remote_errors;
 
-			printf_term(_("Invalid or ignored remote options: "));
+			printf_term(TRUE,_("Invalid or ignored remote options: "));
 			while (work)
 				{
 				gchar *opt = work->data;
 
-				printf_term("%s%s", (work == remote_errors) ? "" : ", ", opt);
+				printf_term(TRUE, "%s%s", (work == remote_errors) ? "" : ", ", opt);
 				work = work->next;
 				}
 
-			printf_term(_("\nUse --remote-help for valid remote options.\n"));
+			printf_term(TRUE, _("\nUse --remote-help for valid remote options.\n"));
 			}
 
+		/* prepend the current dir the remote command was made from,
+		 * for use by any remote command that needs it
+		 */
+		current_dir = g_get_current_dir();
+		pwd = g_strconcat("--PWD:", current_dir, NULL);
+		remote_list = g_list_prepend(remote_list, pwd);
+
 		remote_control(argv[0], remote_list, command_line->path, list, command_line->collection_list);
+		g_free(pwd);
+		g_free(current_dir);
 		}
 	g_list_free(remote_list);
 
@@ -464,6 +521,7 @@ static void parse_command_line_for_debug_option(gint argc, gchar *argv[])
  */
 
 #define RC_HISTORY_NAME "history"
+#define RC_MARKS_NAME "marks"
 
 static void setup_env_path(void)
 {
@@ -491,6 +549,24 @@ static void keys_save(void)
 	g_free(path);
 }
 
+static void marks_load(void)
+{
+	gchar *path;
+
+	path = g_build_filename(get_rc_dir(), RC_MARKS_NAME, NULL);
+	marks_list_load(path);
+	g_free(path);
+}
+
+static void marks_save(gboolean save)
+{
+	gchar *path;
+
+	path = g_build_filename(get_rc_dir(), RC_MARKS_NAME, NULL);
+	marks_list_save(path, save);
+	g_free(path);
+}
+
 static void mkdir_if_not_exists(const gchar *path)
 {
 	if (isdir(path)) return;
@@ -639,6 +715,8 @@ static void exit_program_final(void)
 		layout_free(lw);
 		}
 
+	secure_close(command_line->ssi);
+
 	gtk_main_quit();
 }
 
@@ -685,7 +763,7 @@ static gint exit_confirm_dlg(void)
 	g_free(msg);
 	msg = g_strdup_printf(_("Quit %s"), GQ_APPNAME);
 	generic_dialog_add_message(exit_dialog, GTK_STOCK_DIALOG_QUESTION,
-				   msg, _("Collections have been modified. Quit anyway?"));
+				   msg, _("Collections have been modified. Quit anyway?"), TRUE);
 	g_free(msg);
 	generic_dialog_add_button(exit_dialog, GTK_STOCK_QUIT, NULL, exit_confirm_exit_cb, TRUE);
 
@@ -705,6 +783,8 @@ void exit_program(void)
 
 	if (metadata_write_queue_confirm(FALSE, exit_program_write_metadata_cb, NULL)) return;
 
+	options->marks_save ? marks_save(TRUE) : marks_save(FALSE);
+
 	if (exit_confirm_dlg()) return;
 
 	exit_program_final();
@@ -746,6 +826,7 @@ gint main(gint argc, gchar *argv[])
 	CollectionData *first_collection = NULL;
 	gchar *buf;
 	CollectionData *cd = NULL;
+	gchar *app_lock;
 
 #ifdef HAVE_GTHREAD
 #if !GLIB_CHECK_VERSION(2,32,0)
@@ -871,6 +952,15 @@ gint main(gint argc, gchar *argv[])
 			}
 		}
 
+	if (command_line->log_file)
+		{
+		gchar *pathl;
+		gchar *path = g_strdup(command_line->log_file);
+
+		pathl = path_from_utf8(path);
+		command_line->ssi = secure_open(pathl);
+		}
+
 	if (command_line->cmd_list ||
 	    (command_line->startup_command_line_collection && command_line->collection_list))
 		{
@@ -930,6 +1020,8 @@ gint main(gint argc, gchar *argv[])
 	remote_connection = remote_server_init(buf, cd);
 	g_free(buf);
 
+	marks_load();
+
 	DEBUG_1("%s main: gtk_main", get_exec_time());
 	gtk_main();
 #ifdef HAVE_GTHREAD
