@@ -77,8 +77,8 @@
 
 #define GQ_APPNAME "Geeqie"
 #define GQ_APPNAME_LC "geeqie"
-#define GQ_WEBSITE "www.geeqie.org"
-#define GQ_EMAIL_ADDRESS "geeqie-devel@lists.sourceforge.net"
+#define GQ_WEBSITE "http://www.geeqie.org/"
+#define GQ_EMAIL_ADDRESS "geeqie@freelists.org"
 
 #define GQ_RC_DIR		"." GQ_APPNAME_LC
 #define GQ_COLLECTIONS_DIR	"collections"
@@ -115,7 +115,8 @@
 				"(%number%/%total%) [%zoom%] <b>%name%</b>\n" \
 				"%res%|%date%|%size%\n" \
 				"%formatted.Aperture%|%formatted.ShutterSpeed%|%formatted.ISOSpeedRating:ISO *%|%formatted.FocalLength%|%formatted.ExposureBias:* Ev%\n" \
-				"%formatted.Camera:40%|%formatted.Flash%"
+				"%formatted.Camera:40%|%formatted.Flash%\n"            \
+				"%formatted.star_rating%"
 
 #define GQ_LINK_STR "↗"
 #include "typedefs.h"
@@ -123,6 +124,15 @@
 #include "options.h"
 
 #define DESKTOP_FILE_TEMPLATE GQ_APP_DIR "/template.desktop"
+
+#define TIMEZONE_DATABASE GQ_WEBSITE"downloads/timezone21.bin"
+
+#define HELP_SEARCH_ENGINE "https://duckduckgo.com/?q=site:geeqie.org/help "
+
+#define STAR_RATING_NOT_READ -12345
+#define STAR_RATING_REJECTED 0x274C //Unicode Character 'Cross Mark'
+#define STAR_RATING_STAR 0x2738 //Unicode Character 'Heavy Eight Pointed Rectilinear Black Star'
+
 /*
  *----------------------------------------------------------------------------
  * main.c
