@@ -26,6 +26,10 @@ gchar *utf8_validate_or_convert(const gchar *text);
 gint utf8_compare(const gchar *s1, const gchar *s2, gboolean case_sensitive);
 gchar *expand_tilde(const gchar *filename);
 int runcmd(gchar *cmd);
-
+gchar *decode_geo_parameters(const gchar *input_text);
+gint date_get_first_day_of_week();
+gchar *date_get_abbreviated_day_name(gint day);
+gchar *convert_rating_to_stars(gint rating);
+gchar *get_symbolic_link(const gchar *path_utf8);
 #endif /* MISC_H */
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
