@@ -105,7 +105,8 @@ enum {
 	PROP_WINDOW_LIMIT,
 	PROP_WINDOW_LIMIT_VALUE,
 	PROP_AUTOFIT_LIMIT,
-	PROP_AUTOFIT_LIMIT_VALUE
+	PROP_AUTOFIT_LIMIT_VALUE,
+	PROP_ENLARGEMENT_LIMIT_VALUE
 };
 
 typedef enum {
@@ -329,6 +330,16 @@ static void pixbuf_renderer_class_init(PixbufRendererClass *class)
 							  100,
 							  G_PARAM_READABLE | G_PARAM_WRITABLE));
 
+	g_object_class_install_property(gobject_class,
+					PROP_ENLARGEMENT_LIMIT_VALUE,
+					g_param_spec_uint("enlargement_limit_value",
+							  "Size increase limit of image when autofitting",
+							  NULL,
+							  100,
+							  999,
+							  500,
+							  G_PARAM_READABLE | G_PARAM_WRITABLE));
+
 
 	signals[SIGNAL_ZOOM] =
 		g_signal_new("zoom",
@@ -395,7 +406,7 @@ static RendererFuncs *pr_backend_renderer_new(PixbufRenderer *pr)
 #ifdef HAVE_CLUTTER
 		return renderer_clutter_new(pr);
 #else
-		DEBUG_0("Geeqie is built without clutter renderer support");
+		log_printf("Warning: Geeqie is built without clutter renderer support");
 #endif
 		}
 	return renderer_tiles_new(pr);
@@ -435,6 +446,10 @@ static void pixbuf_renderer_init(PixbufRenderer *pr)
 
 	pr->stereo_mode = PR_STEREO_NONE;
 
+	pr->color.red =0;
+	pr->color.green =0;
+	pr->color.blue =0;
+
 	pr->renderer = pr_backend_renderer_new(pr);
 
 	pr->renderer2 = NULL;
@@ -526,6 +541,9 @@ static void pixbuf_renderer_set_property(GObject *object, guint prop_id,
 		case PROP_AUTOFIT_LIMIT_VALUE:
 			pr->autofit_limit_size = g_value_get_uint(value);
 			break;
+		case PROP_ENLARGEMENT_LIMIT_VALUE:
+			pr->enlargement_limit_size = g_value_get_uint(value);
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 			break;
@@ -589,6 +607,9 @@ static void pixbuf_renderer_get_property(GObject *object, guint prop_id,
 		case PROP_AUTOFIT_LIMIT_VALUE:
 			g_value_set_uint(value, pr->autofit_limit_size);
 			break;
+		case PROP_ENLARGEMENT_LIMIT_VALUE:
+			g_value_set_uint(value, pr->enlargement_limit_size);
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 			break;
@@ -899,28 +920,20 @@ static void pr_scroller_stop(PixbufRenderer *pr)
 
 void pixbuf_renderer_set_color(PixbufRenderer *pr, GdkColor *color)
 {
-	GtkStyle *style;
-	GtkWidget *widget;
-
 	g_return_if_fail(IS_PIXBUF_RENDERER(pr));
 
-	widget = GTK_WIDGET(pr);
-
-	if (color) {
-		GdkColor *slot;
-
-		style = gtk_style_copy(gtk_widget_get_style(widget));
-		slot = &style->bg[GTK_STATE_NORMAL];
-
-		slot->red = color->red;
-		slot->green = color->green;
-		slot->blue = color->blue;
+	if (color)
+		{
+		pr->color.red = color->red;
+		pr->color.green = color->green;
+		pr->color.blue = color->blue;
+		}
+	else
+		{
+		pr->color.red = 0;
+		pr->color.green = 0;
+		pr->color.blue = 0;
 		}
-	else {
-		style = gtk_style_copy(gtk_widget_get_default_style());
-	}
-
-	gtk_widget_set_style(widget, style);
 
 	pr->renderer->update_viewport(pr->renderer);
 	if (pr->renderer2) pr->renderer2->update_viewport(pr->renderer2);
@@ -1238,24 +1251,38 @@ static void pr_zoom_adjust_real(PixbufRenderer *pr, gdouble increment,
 
 	if (increment < 0.0)
 		{
-		if (zoom >= 1.0 && zoom + increment < 1.0)
+		if (zoom >= 1.0)
 			{
-			zoom = zoom + increment - 2.0;
+			if (zoom / -(increment - 1.0) < 1.0)
+				{
+				zoom = 1.0 / (zoom / (increment - 1.0));
+				}
+			else
+				{
+				zoom = zoom / -(increment - 1.0) ;
+				}
 			}
 		else
 			{
-			zoom = zoom + increment;
+			zoom = zoom * -(increment - 1.0);
 			}
 		}
 	else
 		{
-		if (zoom <= -1.0 && zoom + increment > -1.0)
+		if (zoom <= -1.0 )
 			{
-			zoom = zoom + increment + 2.0;
+			if (zoom / (increment + 1.0) > -1.0)
+				{
+				zoom = -(1.0 / (zoom / (increment + 1.0)));
+				}
+			else
+				{
+				zoom = zoom / (increment + 1.0) ;
+				}
 			}
 		else
 			{
-			zoom = zoom + increment;
+			zoom = zoom * (increment + 1.0);
 			}
 		}
 
@@ -1304,9 +1331,9 @@ void pr_render_complete_signal(PixbufRenderer *pr)
 		}
 }
 
-static void pr_drag_signal(PixbufRenderer *pr, GdkEventButton *bevent)
+static void pr_drag_signal(PixbufRenderer *pr, GdkEventMotion *event)
 {
-	g_signal_emit(pr, signals[SIGNAL_DRAG], 0, bevent);
+	g_signal_emit(pr, signals[SIGNAL_DRAG], 0, event);
 }
 
 static void pr_update_pixel_signal(PixbufRenderer *pr)
@@ -1666,6 +1693,17 @@ static gboolean pr_zoom_clamp(PixbufRenderer *pr, gdouble zoom,
 				scale = scale * factor;
 				}
 
+			if (pr->zoom_expand)
+				{
+				gdouble factor = (gdouble)pr->enlargement_limit_size / 100;
+				if (scale > factor)
+					{
+					w = w * factor / scale;
+					h = h * factor / scale;
+					scale = factor;
+					}
+				}
+
 			if (w < 1) w = 1;
 			if (h < 1) h = 1;
 			}
@@ -1724,10 +1762,10 @@ static void pr_zoom_sync(PixbufRenderer *pr, gdouble zoom,
 		}
 
 	if (force) clamp_flags |= PR_ZOOM_INVALIDATE;
+	(void) pr_parent_window_resize(pr, pr->width, pr->height);
 	if (!pr_zoom_clamp(pr, zoom, clamp_flags)) return;
 
 	(void) pr_size_clamp(pr);
-	(void) pr_parent_window_resize(pr, pr->width, pr->height);
 
 	if (force && new)
 		{
@@ -1955,28 +1993,38 @@ void pixbuf_renderer_set_scroll_center(PixbufRenderer *pr, gdouble x, gdouble y)
  *-------------------------------------------------------------------
  */
 
-static gboolean pr_mouse_motion_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gboolean pr_mouse_motion_cb(GtkWidget *widget, GdkEventMotion *event, gpointer data)
 {
 	PixbufRenderer *pr;
 	gint accel;
+#if GTK_CHECK_VERSION(3,0,0)
+	GdkDeviceManager *device_manager;
+	GdkDevice *device;
+#endif
 
 	/* This is a hack, but work far the best, at least for single pointer systems.
 	 * See http://bugzilla.gnome.org/show_bug.cgi?id=587714 for more. */
 	gint x, y;
-	gdk_window_get_pointer (bevent->window, &x, &y, NULL);
-	bevent->x = x;
-	bevent->y = y;
+#if GTK_CHECK_VERSION(3,0,0)
+	device_manager = gdk_display_get_device_manager(gdk_window_get_display(event->window));
+	device = gdk_device_manager_get_client_pointer(device_manager);
+	gdk_window_get_device_position(event->window, device, &x, &y, NULL);
+#else
+	gdk_window_get_pointer (event->window, &x, &y, NULL);
+#endif
+	event->x = x;
+	event->y = y;
 
 	pr = PIXBUF_RENDERER(widget);
 
 	if (pr->scroller_id)
 		{
-		pr->scroller_xpos = bevent->x;
-		pr->scroller_ypos = bevent->y;
+		pr->scroller_xpos = event->x;
+		pr->scroller_ypos = event->y;
 		}
 
-	pr->x_mouse = bevent->x;
-	pr->y_mouse = bevent->y;
+	pr->x_mouse = event->x;
+	pr->y_mouse = event->y;
 	pr_update_pixel_signal(pr);
 
 	if (!pr->in_drag || !gdk_pointer_is_grabbed()) return FALSE;
@@ -1990,7 +2038,7 @@ static gboolean pr_mouse_motion_cb(GtkWidget *widget, GdkEventButton *bevent, gp
 		widget_set_cursor(widget, GDK_FLEUR);
 		}
 
-	if (bevent->state & GDK_CONTROL_MASK)
+	if (event->state & GDK_CONTROL_MASK)
 		{
 		accel = PR_PAN_SHIFT_MULTIPLIER;
 		}
@@ -1999,20 +2047,22 @@ static gboolean pr_mouse_motion_cb(GtkWidget *widget, GdkEventButton *bevent, gp
 		accel = 1;
 		}
 
-	/* do the scroll */
-	pixbuf_renderer_scroll(pr, (pr->drag_last_x - bevent->x) * accel,
-			       (pr->drag_last_y - bevent->y) * accel);
+	/* do the scroll - not when drawing rectangle*/
+	if (!options->draw_rectangle)
+		{
+		pixbuf_renderer_scroll(pr, (pr->drag_last_x - event->x) * accel,
+					(pr->drag_last_y - event->y) * accel);
+		}
+	pr_drag_signal(pr, event);
 
-	pr_drag_signal(pr, bevent);
-
-	pr->drag_last_x = bevent->x;
-	pr->drag_last_y = bevent->y;
+	pr->drag_last_x = event->x;
+	pr->drag_last_y = event->y;
 
 	/* This is recommended by the GTK+ documentation, but does not work properly.
 	 * Use deprecated way until GTK+ gets a solution for correct motion hint handling:
 	 * http://bugzilla.gnome.org/show_bug.cgi?id=587714
 	 */
-	/* gdk_event_request_motions (bevent); */
+	/* gdk_event_request_motions (event); */
 	return FALSE;
 }
 
@@ -2093,7 +2143,7 @@ static gboolean pr_mouse_release_cb(GtkWidget *widget, GdkEventButton *bevent, g
 			{
 			pr_scroller_start(pr, bevent->x, bevent->y);
 			}
-		else if (bevent->button == MOUSE_BUTTON_LEFT || bevent->button == MOUSE_BUTTON_MIDDLE)
+		else if (bevent->button == MOUSE_BUTTON_LEFT || bevent->button == MOUSE_BUTTON_MIDDLE || bevent->button == MOUSE_BUTTON_BACK || bevent->button == MOUSE_BUTTON_FORWARD)
 			{
 			pr_clicked_signal(pr, bevent);
 			}
@@ -2261,15 +2311,15 @@ static void pr_create_anaglyph_dubois(GdkPixbuf *pixbuf, GdkPixbuf *right, gint
 	guchar *spi, *dpi;
 	gint i, j, k;
 	double pr_dubois_matrix[3][6];
-	const static double pr_dubois_matrix_RC[3][6] = {
+	static const double pr_dubois_matrix_RC[3][6] = {
 		{ 0.456,  0.500,  0.176, -0.043, -0.088, -0.002},
 		{-0.040, -0.038, -0.016,  0.378,  0.734, -0.018},
 		{-0.015, -0.021, -0.005, -0.072, -0.113,  1.226}};
-	const static double pr_dubois_matrix_GM[3][6] = {
+	static const double pr_dubois_matrix_GM[3][6] = {
 		{-0.062, -0.158, -0.039,  0.529,  0.705,  0.024},
 		{ 0.284,  0.668,  0.143, -0.016, -0.015, -0.065},
 		{-0.015, -0.027,  0.021,  0.009,  0.075,  0.937}};
-	const static double pr_dubois_matrix_YB[3][6] = {
+	static const double pr_dubois_matrix_YB[3][6] = {
 		{ 1.000, -0.193,  0.282, -0.015, -0.116, -0.016},
 		{-0.024,  0.855,  0.064,  0.006,  0.058, -0.016},
 		{-0.036, -0.163,  0.021,  0.089,  0.174,  0.858}};
@@ -2769,6 +2819,7 @@ gboolean pixbuf_renderer_get_pixel_colors(PixbufRenderer *pr, gint x_pixel, gint
 	gint p_alpha, prs;
 	guchar *p_pix, *pp;
 	gint map_x, map_y, map_w, map_h;
+	size_t xoff, yoff;
 
 	g_return_val_if_fail(IS_PIXBUF_RENDERER(pr), FALSE);
 	g_return_val_if_fail(r_mouse != NULL && g_mouse != NULL && b_mouse != NULL, FALSE);
@@ -2797,7 +2848,9 @@ gboolean pixbuf_renderer_get_pixel_colors(PixbufRenderer *pr, gint x_pixel, gint
 	prs = gdk_pixbuf_get_rowstride(pb);
 	p_pix = gdk_pixbuf_get_pixels(pb);
 
-	pp = p_pix + map_y * prs + (map_x * (p_alpha ? 4 : 3));
+	xoff = (size_t)map_x * (p_alpha ? 4 : 3);
+	yoff = (size_t)map_y * prs;
+	pp = p_pix + yoff + xoff;
 	*r_mouse = *pp;
 	pp++;
 	*g_mouse = *pp;
@@ -2826,10 +2879,15 @@ gboolean pixbuf_renderer_get_mouse_position(PixbufRenderer *pr, gint *x_pixel_re
 	x_pixel_clamped = CLAMP(x_pixel, 0, pr->image_width - 1);
 	y_pixel_clamped = CLAMP(y_pixel, 0, pr->image_height - 1);
 
-	if(x_pixel != x_pixel_clamped || y_pixel != y_pixel_clamped)
+	if (x_pixel != x_pixel_clamped)
 		{
 		/* mouse is not on pr */
-		x_pixel = y_pixel = -1;
+		x_pixel = -1;
+		}
+	if (y_pixel != y_pixel_clamped)
+		{
+		/* mouse is not on pr */
+		y_pixel = -1;
 		}
 
 	*x_pixel_return = x_pixel;
