@@ -108,6 +108,10 @@ static PixbufInline inline_pixbuf_data[] = {
 	{ PIXBUF_INLINE_FOLDER_UP,	folder_up },
 	{ PIXBUF_INLINE_SCROLLER,	icon_scroller },
 	{ PIXBUF_INLINE_BROKEN,		icon_broken },
+	{ PIXBUF_INLINE_METADATA,	icon_metadata },
+	{ PIXBUF_INLINE_UNKNOWN,	icon_unknown },
+	{ PIXBUF_INLINE_VIDEO,		icon_video },
+	{ PIXBUF_INLINE_COLLECTION,	icon_collection },
 	{ PIXBUF_INLINE_ICON,		gqview_icon },
 	{ PIXBUF_INLINE_LOGO,		geeqie_logo },
 	{ PIXBUF_INLINE_ICON_FLOAT,	icon_float },
@@ -116,6 +120,32 @@ static PixbufInline inline_pixbuf_data[] = {
 	{ PIXBUF_INLINE_ICON_CONFIG,	icon_config },
 	{ PIXBUF_INLINE_ICON_TOOLS,	icon_tools },
 	{ PIXBUF_INLINE_ICON_VIEW,	icon_view },
+	{ PIXBUF_INLINE_ICON_GUIDELINES,	icon_guidelines },
+	{ PIXBUF_INLINE_ICON_PANORAMA,	icon_panorama },
+	{ PIXBUF_INLINE_ICON_MAINTENANCE,	icon_maintenance },
+	{ PIXBUF_INLINE_ICON_ZOOMFILLHOR,	icon_zoomfillhor },
+	{ PIXBUF_INLINE_ICON_ZOOMFILLVERT,	icon_zoomfillvert },
+	{ PIXBUF_INLINE_ICON_HIDETOOLS,	icon_hidetools },
+	{ PIXBUF_INLINE_ICON_EXIF,	icon_exif },
+	{ PIXBUF_INLINE_ICON_MARKS,	icon_marks },
+	{ PIXBUF_INLINE_ICON_INFO,	icon_info },
+	{ PIXBUF_INLINE_ICON_SORT,	icon_sort },
+	{ PIXBUF_INLINE_ICON_PDF,	icon_pdf },
+	{ PIXBUF_INLINE_ICON_DRAW_RECTANGLE,	icon_draw_rectangle },
+	{ PIXBUF_INLINE_ICON_MOVE,	icon_move },
+	{ PIXBUF_INLINE_ICON_RENAME,	icon_rename },
+	{ PIXBUF_INLINE_ICON_SELECT_ALL,	icon_select_all },
+	{ PIXBUF_INLINE_ICON_SELECT_NONE,	icon_select_none },
+	{ PIXBUF_INLINE_ICON_SELECT_INVERT,	icon_select_invert },
+	{ PIXBUF_INLINE_ICON_SELECT_RECTANGLE,	icon_select_rectangle },
+	{ PIXBUF_INLINE_ICON_FILE_FILTER,	icon_file_filter },
+	{ PIXBUF_INLINE_ICON_CW,	icon_rotate_clockwise },
+	{ PIXBUF_INLINE_ICON_CCW,	icon_rotate_counter_clockwise },
+	{ PIXBUF_INLINE_ICON_180,	icon_rotate_180 },
+	{ PIXBUF_INLINE_ICON_MIRROR,	icon_mirror },
+	{ PIXBUF_INLINE_ICON_FLIP,	icon_flip },
+	{ PIXBUF_INLINE_ICON_ORIGINAL,	icon_original },
+	{ PIXBUF_INLINE_ICON_TRASH,	icon_trash },
 	{ NULL, NULL }
 };
 
@@ -205,6 +235,15 @@ gboolean register_theme_icon_as_stock(const gchar *key, const gchar *icon)
 				{
 				DEBUG_1("Couldn't load icon %s: %s", icon2, error->message);
 				g_error_free(error);
+				error = NULL;
+
+				/* try as an absolute path */
+				pixbuf = gdk_pixbuf_new_from_file(icon, &error);
+				if (error)
+					{
+					DEBUG_1("Couldn't load icon as absolute path %s: %s", icon, error->message);
+					g_error_free(error);
+					}
 				}
 			g_free(icon2);
 			}
@@ -238,7 +277,28 @@ gboolean pixbuf_scale_aspect(gint req_w, gint req_h,
 
 GdkPixbuf *pixbuf_fallback(FileData *fd, gint requested_width, gint requested_height)
 {
-	GdkPixbuf *pixbuf = pixbuf_inline(PIXBUF_INLINE_BROKEN); /* FIXME use different images according to FORMAT_CLASS */
+	GdkPixbuf *pixbuf;
+
+	switch (fd->format_class)
+		{
+		case FORMAT_CLASS_UNKNOWN:
+			pixbuf = pixbuf_inline(PIXBUF_INLINE_UNKNOWN);
+			break;
+		case FORMAT_CLASS_META:
+			pixbuf = pixbuf_inline(PIXBUF_INLINE_METADATA);
+			break;
+		case FORMAT_CLASS_VIDEO:
+			pixbuf = pixbuf_inline(PIXBUF_INLINE_VIDEO);
+			break;
+		case FORMAT_CLASS_COLLECTION:
+			pixbuf = pixbuf_inline(PIXBUF_INLINE_COLLECTION);
+			break;
+		case FORMAT_CLASS_PDF:
+			pixbuf = pixbuf_inline(PIXBUF_INLINE_ICON_PDF);
+			break;
+		default:
+			pixbuf = pixbuf_inline(PIXBUF_INLINE_BROKEN);
+		}
 
 	if (requested_width && requested_height)
 		{
