@@ -22,12 +22,18 @@
 #include "main.h"
 #include "remote.h"
 
+#include "cache_maint.h"
 #include "collect.h"
+#include "collect-io.h"
 #include "filedata.h"
+#include "filefilter.h"
+#include "image.h"
 #include "img-view.h"
 #include "layout.h"
 #include "layout_image.h"
+#include "layout_util.h"
 #include "misc.h"
+#include "pixbuf-renderer.h"
 #include "slideshow.h"
 #include "ui_fileops.h"
 #include "rcfile.h"
@@ -37,6 +43,7 @@
 #include <signal.h>
 #include <errno.h>
 
+#include "glua.h"
 
 #define SERVER_MAX_CLIENTS 8
 
@@ -50,7 +57,9 @@
 
 static RemoteConnection *remote_client_open(const gchar *path);
 static gint remote_client_send(RemoteConnection *rc, const gchar *text);
+static void gr_raise(const gchar *text, GIOChannel *channel, gpointer data);
 
+static LayoutWindow *lw_id = NULL; /* points to the window set by the --id option */
 
 typedef struct _RemoteClient RemoteClient;
 struct _RemoteClient {
@@ -64,6 +73,38 @@ struct _RemoteData {
 	CollectionData *command_collection;
 };
 
+/* Remote commands from main.c are prepended with the current dir the remote
+ * command was made from. Some remote commands require this. The
+ * value is stored here
+ */
+static gchar *pwd = NULL;
+
+/**
+ * @brief Ensures file path is absolute.
+ * @param[in] filename Filepath, absolute or relative to calling directory
+ * @returns absolute path
+ * 
+ * If first character of input filepath is not the directory
+ * separator, assume it as a relative path and prepend
+ * the directory the remote command was initiated from
+ * 
+ * Return value must be freed with g_free()
+ */
+static gchar *set_pwd(gchar *filename)
+{
+	gchar *temp;
+
+	if (strncmp(filename, G_DIR_SEPARATOR_S, 1) != 0)
+		{
+		temp = g_build_filename(pwd, filename, NULL);
+		}
+	else
+		{
+		temp = g_strdup(filename);
+		}
+
+	return temp;
+}
 
 static gboolean remote_server_client_cb(GIOChannel *source, GIOCondition condition, gpointer data)
 {
@@ -71,6 +112,7 @@ static gboolean remote_server_client_cb(GIOChannel *source, GIOCondition conditi
 	RemoteConnection *rc;
 	GIOStatus status = G_IO_STATUS_NORMAL;
 
+	lw_id = NULL;
 	rc = client->rc;
 
 	if (condition & G_IO_IN)
@@ -176,7 +218,7 @@ static gboolean remote_server_read_cb(GIOChannel *source, GIOCondition condition
 	return TRUE;
 }
 
-static gboolean remote_server_exists(const gchar *path)
+gboolean remote_server_exists(const gchar *path)
 {
 	RemoteConnection *rc;
 
@@ -275,11 +317,11 @@ static RemoteConnection *remote_client_open(const gchar *path)
 	return rc;
 }
 
-static sig_atomic_t sigpipe_occured = FALSE;
+static sig_atomic_t sigpipe_occurred = FALSE;
 
 static void sighandler_sigpipe(gint sig)
 {
-	sigpipe_occured = TRUE;
+	sigpipe_occurred = TRUE;
 }
 
 static gboolean remote_client_send(RemoteConnection *rc, const gchar *text)
@@ -292,7 +334,7 @@ static gboolean remote_client_send(RemoteConnection *rc, const gchar *text)
 	if (!rc || rc->server) return FALSE;
 	if (!text) return TRUE;
 
-	sigpipe_occured = FALSE;
+	sigpipe_occurred = FALSE;
 
 	new_action.sa_handler = sighandler_sigpipe;
 	sigemptyset(&new_action.sa_mask);
@@ -383,77 +425,187 @@ void remote_close(RemoteConnection *rc)
 
 static void gr_image_next(const gchar *text, GIOChannel *channel, gpointer data)
 {
-	layout_image_next(NULL);
+	layout_image_next(lw_id);
+}
+
+static void gr_new_window(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	LayoutWindow *lw = NULL;
+
+	if (!layout_valid(&lw)) return;
+
+	lw_id = layout_menu_new_window(NULL, lw);
+}
+
+static gboolean gr_close_window_cb()
+{
+	if (!layout_valid(&lw_id)) return FALSE;
+
+	layout_menu_close_cb(NULL, lw_id);
+
+	return FALSE;
+}
+
+static void gr_close_window(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	g_idle_add(gr_close_window_cb, NULL);
 }
 
 static void gr_image_prev(const gchar *text, GIOChannel *channel, gpointer data)
 {
-	layout_image_prev(NULL);
+	layout_image_prev(lw_id);
 }
 
 static void gr_image_first(const gchar *text, GIOChannel *channel, gpointer data)
 {
-	layout_image_first(NULL);
+	layout_image_first(lw_id);
 }
 
 static void gr_image_last(const gchar *text, GIOChannel *channel, gpointer data)
 {
-	layout_image_last(NULL);
+	layout_image_last(lw_id);
 }
 
 static void gr_fullscreen_toggle(const gchar *text, GIOChannel *channel, gpointer data)
 {
-	layout_image_full_screen_toggle(NULL);
+	layout_image_full_screen_toggle(lw_id);
 }
 
 static void gr_fullscreen_start(const gchar *text, GIOChannel *channel, gpointer data)
 {
-	layout_image_full_screen_start(NULL);
+	layout_image_full_screen_start(lw_id);
 }
 
 static void gr_fullscreen_stop(const gchar *text, GIOChannel *channel, gpointer data)
 {
-	layout_image_full_screen_stop(NULL);
+	layout_image_full_screen_stop(lw_id);
+}
+
+static void gr_lw_id(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	lw_id = layout_find_by_layout_id(text);
+	if (!lw_id)
+		{
+		log_printf("remote sent window ID that does not exist:\"%s\"\n",text);
+		}
+	layout_valid(&lw_id);
 }
 
 static void gr_slideshow_start_rec(const gchar *text, GIOChannel *channel, gpointer data)
 {
 	GList *list;
 	FileData *dir_fd = file_data_new_dir(text);
-	list = filelist_recursive(dir_fd);
+
+	layout_valid(&lw_id);
+	list = filelist_recursive_full(dir_fd, lw_id->sort_method, lw_id->sort_ascend);
 	file_data_unref(dir_fd);
 	if (!list) return;
 //printf("length: %d\n", g_list_length(list));
-	layout_image_slideshow_stop(NULL);
-	layout_image_slideshow_start_from_list(NULL, list);
+	layout_image_slideshow_stop(lw_id);
+	layout_image_slideshow_start_from_list(lw_id, list);
+}
+
+static void gr_cache_thumb(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	if (!g_strcmp0(text, "clear"))
+		cache_maintain_home_remote(FALSE, TRUE);
+	else if (!g_strcmp0(text, "clean"))
+		cache_maintain_home_remote(FALSE, FALSE);
+}
+
+static void gr_cache_shared(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	if (!g_strcmp0(text, "clear"))
+		cache_manager_standard_process_remote(TRUE);
+	else if (!g_strcmp0(text, "clean"))
+		cache_manager_standard_process_remote(FALSE);
+}
+
+static void gr_cache_metadata(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	cache_maintain_home_remote(TRUE, FALSE);
+}
+
+static void gr_cache_render(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	cache_manager_render_remote(text, FALSE, FALSE);
+}
+
+static void gr_cache_render_recurse(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	cache_manager_render_remote(text, TRUE, FALSE);
+}
+
+static void gr_cache_render_standard(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	if(options->thumbnails.spec_standard)
+		cache_manager_render_remote(text, FALSE, TRUE);
+}
+
+static void gr_cache_render_standard_recurse(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	if(options->thumbnails.spec_standard)
+		cache_manager_render_remote(text, TRUE, TRUE);
 }
 
 static void gr_slideshow_toggle(const gchar *text, GIOChannel *channel, gpointer data)
 {
-	layout_image_slideshow_toggle(NULL);
+	layout_image_slideshow_toggle(lw_id);
 }
 
 static void gr_slideshow_start(const gchar *text, GIOChannel *channel, gpointer data)
 {
-	layout_image_slideshow_start(NULL);
+	layout_image_slideshow_start(lw_id);
 }
 
 static void gr_slideshow_stop(const gchar *text, GIOChannel *channel, gpointer data)
 {
-	layout_image_slideshow_stop(NULL);
+	layout_image_slideshow_stop(lw_id);
 }
 
 static void gr_slideshow_delay(const gchar *text, GIOChannel *channel, gpointer data)
 {
-	gdouble n;
+	gdouble t1, t2, t3, n;
+	gint res;
 
-	n = g_ascii_strtod(text, NULL);
-	if (n < SLIDESHOW_MIN_SECONDS || n > SLIDESHOW_MAX_SECONDS)
+	res = sscanf(text, "%lf:%lf:%lf", &t1, &t2, &t3);
+	if (res == 3)
 		{
-		printf_term("Remote slideshow delay out of range (%.1f to %.1f)\n",
-			    SLIDESHOW_MIN_SECONDS, SLIDESHOW_MAX_SECONDS);
-		return;
+		n = (t1 * 3600) + (t2 * 60) + t3;
+		if (n < SLIDESHOW_MIN_SECONDS || n > SLIDESHOW_MAX_SECONDS ||
+				t1 >= 24 || t2 >= 60 || t3 >= 60)
+			{
+			printf_term(TRUE, "Remote slideshow delay out of range (%.1f to %.1f)\n",
+								SLIDESHOW_MIN_SECONDS, SLIDESHOW_MAX_SECONDS);
+			return;
+			}
 		}
+	else if (res == 2)
+		{
+		n = t1 * 60 + t2;
+		if (n < SLIDESHOW_MIN_SECONDS || n > SLIDESHOW_MAX_SECONDS ||
+				t1 >= 60 || t2 >= 60)
+			{
+			printf_term(TRUE, "Remote slideshow delay out of range (%.1f to %.1f)\n",
+								SLIDESHOW_MIN_SECONDS, SLIDESHOW_MAX_SECONDS);
+			return;
+			}
+		}
+	else if (res == 1)
+		{
+		n = t1;
+		if (n < SLIDESHOW_MIN_SECONDS || n > SLIDESHOW_MAX_SECONDS)
+			{
+			printf_term(TRUE, "Remote slideshow delay out of range (%.1f to %.1f)\n",
+								SLIDESHOW_MIN_SECONDS, SLIDESHOW_MAX_SECONDS);
+			return;
+			}
+		}
+	else
+		{
+		n = 0;
+		}
+
 	options->slideshow.delay = (gint)(n * 10.0 + 0.01);
 }
 
@@ -462,9 +614,9 @@ static void gr_tools_show(const gchar *text, GIOChannel *channel, gpointer data)
 	gboolean popped;
 	gboolean hidden;
 
-	if (layout_tools_float_get(NULL, &popped, &hidden) && hidden)
+	if (layout_tools_float_get(lw_id, &popped, &hidden) && hidden)
 		{
-		layout_tools_float_set(NULL, popped, FALSE);
+		layout_tools_float_set(lw_id, popped, FALSE);
 		}
 }
 
@@ -473,9 +625,9 @@ static void gr_tools_hide(const gchar *text, GIOChannel *channel, gpointer data)
 	gboolean popped;
 	gboolean hidden;
 
-	if (layout_tools_float_get(NULL, &popped, &hidden) && !hidden)
+	if (layout_tools_float_get(lw_id, &popped, &hidden) && !hidden)
 		{
-		layout_tools_float_set(NULL, popped, TRUE);
+		layout_tools_float_set(lw_id, popped, TRUE);
 		}
 }
 
@@ -494,9 +646,12 @@ static void gr_quit(const gchar *text, GIOChannel *channel, gpointer data)
 	g_idle_add(gr_quit_idle_cb, NULL);
 }
 
-static void gr_file_load(const gchar *text, GIOChannel *channel, gpointer data)
+static void gr_file_load_no_raise(const gchar *text, GIOChannel *channel, gpointer data)
 {
-	gchar *filename = expand_tilde(text);
+	gchar *filename;
+	gchar *tilde_filename = expand_tilde(text);
+
+	filename = set_pwd(tilde_filename);
 
 	if (isfile(filename))
 		{
@@ -506,19 +661,300 @@ static void gr_file_load(const gchar *text, GIOChannel *channel, gpointer data)
 			}
 		else
 			{
-			layout_set_path(NULL, filename);
+			layout_set_path(lw_id, filename);
 			}
 		}
 	else if (isdir(filename))
 		{
-		layout_set_path(NULL, filename);
+		layout_set_path(lw_id, filename);
 		}
 	else
 		{
 		log_printf("remote sent filename that does not exist:\"%s\"\n", filename);
+		layout_set_path(lw_id, homedir());
 		}
 
 	g_free(filename);
+	g_free(tilde_filename);
+}
+
+static void gr_file_load(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	gr_file_load_no_raise(text, channel, data);
+
+	gr_raise(text, channel, data);
+}
+
+static void gr_pixel_info(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	gchar *pixel_info;
+	gint x_pixel, y_pixel;
+	gint width, height;
+	gint r_mouse, g_mouse, b_mouse;
+	PixbufRenderer *pr;
+	LayoutWindow *lw = NULL;
+
+	if (!layout_valid(&lw_id)) return;
+
+	pr = (PixbufRenderer*)lw_id->image->pr;
+
+	if (pr)
+		{
+		pixbuf_renderer_get_image_size(pr, &width, &height);
+		if (width < 1 || height < 1) return;
+
+		pixbuf_renderer_get_mouse_position(pr, &x_pixel, &y_pixel);
+
+		if (x_pixel >= 0 && y_pixel >= 0)
+			{
+			pixbuf_renderer_get_pixel_colors(pr, x_pixel, y_pixel,
+							 &r_mouse, &g_mouse, &b_mouse);
+
+			pixel_info = g_strdup_printf(_("[%d,%d]: RGB(%3d,%3d,%3d)"),
+						 x_pixel, y_pixel,
+						 r_mouse, g_mouse, b_mouse);
+
+			g_io_channel_write_chars(channel, pixel_info, -1, NULL, NULL);
+			g_io_channel_write_chars(channel, "\n", -1, NULL, NULL);
+
+			g_free(pixel_info);
+			}
+		else
+			{
+			return;
+			}
+		}
+	else
+		{
+		return;
+		}
+}
+
+static void gr_rectangle(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	gchar *rectangle_info;
+	PixbufRenderer *pr;
+	LayoutWindow *lw = NULL;
+	gint x1, y1, x2, y2;
+
+	if (!options->draw_rectangle) return;
+	if (!layout_valid(&lw_id)) return;
+
+	pr = (PixbufRenderer*)lw_id->image->pr;
+
+	if (pr)
+		{
+		image_get_rectangle(&x1, &y1, &x2, &y2);
+		rectangle_info = g_strdup_printf(_("%dx%d+%d+%d"),
+					(x2 > x1) ? x2 - x1 : x1 - x2,
+					(y2 > y1) ? y2 - y1 : y1 - y2,
+					(x2 > x1) ? x1 : x2,
+					(y2 > y1) ? y1 : y2);
+
+		g_io_channel_write_chars(channel, rectangle_info, -1, NULL, NULL);
+		g_io_channel_write_chars(channel, "\n", -1, NULL, NULL);
+
+		g_free(rectangle_info);
+		}
+}
+
+static void gr_render_intent(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	gchar *render_intent;
+
+	switch (options->color_profile.render_intent)
+		{
+		case 0:
+			render_intent = g_strdup("Perceptual");
+			break;
+		case 1:
+			render_intent = g_strdup("Relative Colorimetric");
+			break;
+		case 2:
+			render_intent = g_strdup("Saturation");
+			break;
+		case 3:
+			render_intent = g_strdup("Absolute Colorimetric");
+			break;
+		default:
+			render_intent = g_strdup("none");
+			break;
+		}
+
+	g_io_channel_write_chars(channel, render_intent, -1, NULL, NULL);
+	g_io_channel_write_chars(channel, "\n", -1, NULL, NULL);
+
+	g_free(render_intent);
+}
+
+static void get_filelist(const gchar *text, GIOChannel *channel, gboolean recurse)
+{
+	GList *list = NULL;
+	FileFormatClass class;
+	FileData *dir_fd;
+	FileData *fd;
+	GString *out_string = g_string_new(NULL);
+	GList *work;
+
+	if (strcmp(text, "") == 0)
+		{
+		if (layout_valid(&lw_id))
+			{
+			dir_fd = file_data_new_dir(lw_id->dir_fd->path);
+			}
+		else
+			{
+			return;
+			}
+		}
+	else
+		{
+		if (isdir(text))
+			{
+			dir_fd = file_data_new_dir(text);
+			}
+		else
+			{
+			return;
+			}
+		}
+
+	if (recurse)
+		{
+		list = filelist_recursive(dir_fd);
+		}
+	else
+		{
+		filelist_read(dir_fd, &list, NULL);
+		}
+
+	work = list;
+	while (work)
+		{
+		fd = work->data;
+		g_string_append_printf(out_string, "%s", fd->path);
+		class = filter_file_get_class(fd->path);
+
+		switch (class)
+			{
+			case FORMAT_CLASS_IMAGE:
+				out_string = g_string_append(out_string, "    Class: Image");
+				break;
+			case FORMAT_CLASS_RAWIMAGE:
+				out_string = g_string_append(out_string, "    Class: RAW image");
+				break;
+			case FORMAT_CLASS_META:
+				out_string = g_string_append(out_string, "    Class: Metadata");
+				break;
+			case FORMAT_CLASS_VIDEO:
+				out_string = g_string_append(out_string, "    Class: Video");
+				break;
+			case FORMAT_CLASS_COLLECTION:
+				out_string = g_string_append(out_string, "    Class: Collection");
+				break;
+			case FORMAT_CLASS_PDF:
+				out_string = g_string_append(out_string, "    Class: PDF");
+				break;
+			case FORMAT_CLASS_UNKNOWN:
+				out_string = g_string_append(out_string, "    Class: Unknown");
+				break;
+			default:
+				out_string = g_string_append(out_string, "    Class: Unknown");
+				break;
+			}
+		out_string = g_string_append(out_string, "\n");
+		work = work->next;
+		}
+
+	g_io_channel_write_chars(channel, out_string->str, -1, NULL, NULL);
+	g_io_channel_write_chars(channel, "\n", -1, NULL, NULL);
+
+	g_string_free(out_string, TRUE);
+	filelist_free(list);
+	file_data_unref(dir_fd);
+}
+
+static void gr_collection(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	GString *contents = g_string_new(NULL);
+
+	if (is_collection(text))
+		{
+		collection_contents(text, &contents);
+		}
+	else
+		{
+		return;
+		}
+
+	g_io_channel_write_chars(channel, contents->str, -1, NULL, NULL);
+	g_io_channel_write_chars(channel, "\n", -1, NULL, NULL);
+
+	g_string_free(contents, TRUE);
+}
+
+static void gr_collection_list(const gchar *text, GIOChannel *channel, gpointer data)
+{
+
+	GList *collection_list = NULL;
+	GList *work;
+	GString *out_string = g_string_new(NULL);
+
+	collect_manager_list(&collection_list, NULL, NULL);
+
+	work = collection_list;
+	while (work)
+		{
+		const gchar *collection_name = work->data;
+		out_string = g_string_append(out_string, g_strdup(collection_name));
+		out_string = g_string_append(out_string, "\n");
+
+		work = work->next;
+		}
+
+	g_io_channel_write_chars(channel, out_string->str, -1, NULL, NULL);
+	g_io_channel_write_chars(channel, "\n", -1, NULL, NULL);
+
+	string_list_free(collection_list);
+	g_string_free(out_string, TRUE);
+}
+
+
+static void gr_filelist(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	get_filelist(text, channel, FALSE);
+}
+
+static void gr_filelist_recurse(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	get_filelist(text, channel, TRUE);
+}
+
+static void gr_file_tell(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	gchar *out_string;
+	gchar *collection_name = NULL;
+
+	if (!layout_valid(&lw_id)) return;
+
+	if (image_get_path(lw_id->image))
+		{
+		if (lw_id->image->collection && lw_id->image->collection->name)
+			{
+			collection_name = remove_extension_from_path(lw_id->image->collection->name);
+			out_string = g_strconcat(image_get_path(lw_id->image), "    Collection: ", collection_name, NULL);
+			}
+		else
+			{
+			out_string = g_strconcat(image_get_path(lw_id->image), NULL);
+			}
+
+		g_io_channel_write_chars(channel, out_string, -1, NULL, NULL);
+		g_io_channel_write_chars(channel, "\n", -1, NULL, NULL);
+
+		g_free(collection_name);
+		g_free(out_string);
+		}
 }
 
 static void gr_config_load(const gchar *text, GIOChannel *channel, gpointer data)
@@ -532,6 +968,7 @@ static void gr_config_load(const gchar *text, GIOChannel *channel, gpointer data
 	else
 		{
 		log_printf("remote sent filename that does not exist:\"%s\"\n", filename);
+		layout_set_path(NULL, homedir());
 		}
 
 	g_free(filename);
@@ -575,10 +1012,14 @@ static void gr_get_destination(const gchar *text, GIOChannel *channel, gpointer
 
 static void gr_file_view(const gchar *text, GIOChannel *channel, gpointer data)
 {
-	gchar *filename = expand_tilde(text);
+	gchar *filename;
+	gchar *tilde_filename = expand_tilde(text);
+
+	filename = set_pwd(tilde_filename);
 
 	view_window_new(file_data_new_group(filename));
 	g_free(filename);
+	g_free(tilde_filename);
 }
 
 static void gr_list_clear(const gchar *text, GIOChannel *channel, gpointer data)
@@ -626,12 +1067,53 @@ static void gr_raise(const gchar *text, GIOChannel *channel, gpointer data)
 {
 	LayoutWindow *lw = NULL;
 
-	if (layout_valid(&lw))
+	if (layout_valid(&lw_id))
 		{
-		gtk_window_present(GTK_WINDOW(lw->window));
+		gtk_window_present(GTK_WINDOW(lw_id->window));
 		}
 }
 
+static void gr_pwd(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	LayoutWindow *lw = NULL;
+
+	g_free(pwd);
+	pwd = g_strdup(text);
+}
+
+#ifdef HAVE_LUA
+static void gr_lua(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	gchar *result = NULL;
+	gchar **lua_command;
+
+	lua_command = g_strsplit(text, ",", 2);
+
+	if (lua_command[0] && lua_command[1])
+		{
+		FileData *fd = file_data_new_group(lua_command[0]);
+		result = g_strdup(lua_callvalue(fd, lua_command[1], NULL));
+		if (result)
+			{
+			g_io_channel_write_chars(channel, result, -1, NULL, NULL);
+			}
+		else
+			{
+			g_io_channel_write_chars(channel, N_("lua error: no data"), -1, NULL, NULL);
+			}
+		}
+	else
+		{
+		g_io_channel_write_chars(channel, N_("lua error: no data"), -1, NULL, NULL);
+		}
+
+	g_io_channel_write_chars(channel, "\n", -1, NULL, NULL);
+
+	g_strfreev(lua_command);
+	g_free(result);
+}
+#endif
+
 typedef struct _RemoteCommandEntry RemoteCommandEntry;
 struct _RemoteCommandEntry {
 	gchar *opt_s;
@@ -656,19 +1138,46 @@ static RemoteCommandEntry remote_commands[] = {
 	{ "-ss","--slideshow-start",    gr_slideshow_start,     FALSE, FALSE, NULL, N_("start slide show") },
 	{ "-sS","--slideshow-stop",     gr_slideshow_stop,      FALSE, FALSE, NULL, N_("stop slide show") },
 	{ NULL, "--slideshow-recurse:", gr_slideshow_start_rec, TRUE,  FALSE, N_("<FOLDER>"), N_("start recursive slide show in FOLDER") },
-	{ "-d", "--delay=",             gr_slideshow_delay,     TRUE,  FALSE, N_("<[N][.M]>"), N_("set slide show delay to N.M seconds") },
+	{ "-d", "--delay=",             gr_slideshow_delay,     TRUE,  FALSE, N_("<[H:][M:][N][.M]>"), N_("set slide show delay to Hrs Mins N.M seconds") },
 	{ "+t", "--tools-show",         gr_tools_show,          FALSE, TRUE,  NULL, N_("show tools") },
 	{ "-t", "--tools-hide",	        gr_tools_hide,          FALSE, TRUE,  NULL, N_("hide tools") },
 	{ "-q", "--quit",               gr_quit,                FALSE, FALSE, NULL, N_("quit") },
 	{ NULL, "--config-load:",       gr_config_load,         TRUE,  FALSE, N_("<FILE>"), N_("load configuration from FILE") },
 	{ NULL, "--get-sidecars:",      gr_get_sidecars,        TRUE,  FALSE, N_("<FILE>"), N_("get list of sidecars of FILE") },
 	{ NULL, "--get-destination:",  	gr_get_destination,     TRUE,  FALSE, N_("<FILE>"), N_("get destination path of FILE") },
-	{ NULL, "file:",                gr_file_load,           TRUE,  FALSE, N_("<FILE>"), N_("open FILE") },
+	{ NULL, "file:",                gr_file_load,           TRUE,  FALSE, N_("<FILE>"), N_("open FILE, bring Geeqie window to the top") },
+	{ NULL, "--file:",              gr_file_load,           TRUE,  FALSE, N_("<FILE>"), N_("open FILE, bring Geeqie window to the top") },
+	{ NULL, "File:",                gr_file_load_no_raise,  TRUE,  FALSE, N_("<FILE>"), N_("open FILE, do not bring Geeqie window to the top") },
+	{ NULL, "--File:",              gr_file_load_no_raise,  TRUE,  FALSE, N_("<FILE>"), N_("open FILE, do not bring Geeqie window to the top") },
+	{ NULL, "--tell",               gr_file_tell,           FALSE, FALSE, NULL, N_("print filename [and Collection] of current image") },
+	{ NULL, "--pixel-info",         gr_pixel_info,          FALSE, FALSE, NULL, N_("print pixel info of mouse pointer on current image") },
+	{ NULL, "--get-rectangle",      gr_rectangle,           FALSE, FALSE, NULL, N_("get rectangle co-ordinates") },
+	{ NULL, "--get-render-intent",  gr_render_intent,       FALSE, FALSE, NULL, N_("get render intent") },
+	{ NULL, "--get-filelist:",      gr_filelist,            TRUE,  FALSE, N_("[<FOLDER>]"), N_("get list of files and class") },
+	{ NULL, "--get-filelist-recurse:", gr_filelist_recurse, TRUE,  FALSE, N_("[<FOLDER>]"), N_("get list of files and class recursive") },
+	{ NULL, "--get-collection:",    gr_collection,          TRUE,  FALSE, N_("<COLLECTION>"), N_("get collection content") },
+	{ NULL, "--get-collection-list", gr_collection_list,    FALSE, FALSE, NULL, N_("get collection list") },
 	{ NULL, "view:",                gr_file_view,           TRUE,  FALSE, N_("<FILE>"), N_("open FILE in new window") },
+	{ NULL, "--view:",              gr_file_view,           TRUE,  FALSE, N_("<FILE>"), N_("open FILE in new window") },
 	{ NULL, "--list-clear",         gr_list_clear,          FALSE, FALSE, NULL, N_("clear command line collection list") },
 	{ NULL, "--list-add:",          gr_list_add,            TRUE,  FALSE, N_("<FILE>"), N_("add FILE to command line collection list") },
 	{ NULL, "raise",                gr_raise,               FALSE, FALSE, NULL, N_("bring the Geeqie window to the top") },
-	{ NULL, NULL, NULL, FALSE, FALSE, NULL }
+	{ NULL, "--raise",              gr_raise,               FALSE, FALSE, NULL, N_("bring the Geeqie window to the top") },
+	{ NULL, "--id:",                gr_lw_id,               TRUE, FALSE, N_("<ID>"), N_("window id for following commands") },
+	{ NULL, "--new-window",         gr_new_window,          FALSE, FALSE, NULL, N_("new window") },
+	{ NULL, "--close-window",       gr_close_window,        FALSE, FALSE, NULL, N_("close window") },
+	{ "-ct:", "--cache-thumbs:",    gr_cache_thumb,         TRUE, FALSE, N_("clear|clean"), N_("clear or clean thumbnail cache") },
+	{ "-cs:", "--cache-shared:",    gr_cache_shared,        TRUE, FALSE, N_("clear|clean"), N_("clear or clean shared thumbnail cache") },
+	{ "-cm","--cache-metadata",      gr_cache_metadata,               FALSE, FALSE, NULL, N_("    clean the metadata cache") },
+	{ "-cr:", "--cache-render:",    gr_cache_render,        TRUE, FALSE, N_("<folder>  "), N_(" render thumbnails") },
+	{ "-crr:", "--cache-render-recurse:", gr_cache_render_recurse, TRUE, FALSE, N_("<folder> "), N_("render thumbnails recursively") },
+	{ "-crs:", "--cache-render-shared:", gr_cache_render_standard, TRUE, FALSE, N_("<folder> "), N_(" render thumbnails (see Help)") },
+	{ "-crsr:", "--cache-render-shared-recurse:", gr_cache_render_standard_recurse, TRUE, FALSE, N_("<folder>"), N_(" render thumbnails recursively (see Help)") },
+#ifdef HAVE_LUA
+	{ NULL, "--lua:",               gr_lua,                 TRUE, FALSE, N_("<FILE>,<lua script>"), N_("run lua script on FILE") },
+#endif
+	{ NULL, "--PWD:",               gr_pwd,                 TRUE, FALSE, N_("<PWD>"), N_("use PWD as working directory for following commands") },
+	{ NULL, NULL, NULL, FALSE, FALSE, NULL, NULL }
 };
 
 static RemoteCommandEntry *remote_command_find(const gchar *text, const gchar **offset)
@@ -732,7 +1241,7 @@ void remote_help(void)
 	gchar *s_opt_param;
 	gchar *l_opt_param;
 
-	print_term(_("Remote command list:\n"));
+	print_term(FALSE, _("Remote command list:\n"));
 
 	i = 0;
 	while (remote_commands[i].func != NULL)
@@ -741,7 +1250,7 @@ void remote_help(void)
 			{
 			s_opt_param = g_strconcat(remote_commands[i].opt_s, remote_commands[i].parameter, NULL);
 			l_opt_param = g_strconcat(remote_commands[i].opt_l, remote_commands[i].parameter, NULL);
-			printf_term("  %-11s%-1s %-30s%-s\n",
+			printf_term(FALSE, "  %-11s%-1s %-30s%-s\n",
 				    (remote_commands[i].opt_s) ? s_opt_param : "",
 				    (remote_commands[i].opt_s && remote_commands[i].opt_l) ? "," : " ",
 				    (remote_commands[i].opt_l) ? l_opt_param : "",
@@ -751,7 +1260,7 @@ void remote_help(void)
 			}
 		i++;
 		}
-	printf_term(N_("\n  All other command line parameters are used as plain files if they exists.\n"));
+	printf_term(FALSE, N_("\n  All other command line parameters are used as plain files if they exists.\n"));
 }
 
 GList *remote_build_list(GList *list, gint argc, gchar *argv[], GList **errors)
@@ -801,7 +1310,7 @@ void remote_control(const gchar *arg_exec, GList *remote_list, const gchar *path
 		gint retry_count = 12;
 		gboolean blank = FALSE;
 
-		printf_term(_("Remote %s not running, starting..."), GQ_APPNAME);
+		printf_term(FALSE, _("Remote %s not running, starting..."), GQ_APPNAME);
 
 		command = g_string_new(arg_exec);
 
@@ -841,11 +1350,11 @@ void remote_control(const gchar *arg_exec, GList *remote_list, const gchar *path
 			{
 			usleep((retry_count > 10) ? 500000 : 1000000);
 			rc = remote_client_open(buf);
-			if (!rc) print_term(".");
+			if (!rc) print_term(FALSE, ".");
 			retry_count--;
 			}
 
-		print_term("\n");
+		print_term(FALSE, "\n");
 
 		started = TRUE;
 		}
@@ -937,7 +1446,7 @@ void remote_control(const gchar *arg_exec, GList *remote_list, const gchar *path
 		}
 	else
 		{
-		print_term(_("Remote not available\n"));
+		print_term(TRUE, _("Remote not available\n"));
 		}
 
 	_exit(0);
