@@ -32,9 +32,12 @@
 #include "image-load.h"
 #include "img-view.h"
 #include "layout.h"
+#include "layout_util.h"
+#include "math.h"
 #include "menu.h"
 #include "metadata.h"
 #include "misc.h"
+#include "pixbuf_util.h"
 #include "print.h"
 #include "thumb.h"
 #include "ui_bookmark.h"
@@ -53,14 +56,13 @@
 
 
 #define DEF_SEARCH_WIDTH  700
-#define DEF_SEARCH_HEIGHT 450
+#define DEF_SEARCH_HEIGHT 650
 
 #define SEARCH_BUFFER_MATCH_LOAD 20
 #define SEARCH_BUFFER_MATCH_HIT  5
 #define SEARCH_BUFFER_MATCH_MISS 1
 #define SEARCH_BUFFER_FLUSH_SIZE 99
 
-
 typedef enum {
 	SEARCH_MATCH_NONE,
 	SEARCH_MATCH_EQUAL,
@@ -118,6 +120,7 @@ struct _SearchData
 	GtkWidget *menu_date;
 	GtkWidget *date_sel;
 	GtkWidget *date_sel_end;
+	GtkWidget *date_type;
 
 	GtkWidget *check_dimensions;
 	GtkWidget *menu_dimensions;
@@ -138,9 +141,21 @@ struct _SearchData
 	GtkWidget *menu_comment;
 	GtkWidget *entry_comment;
 
+	GtkWidget *check_rating;
+	GtkWidget *menu_rating;
+	GtkWidget *spin_rating;
+	GtkWidget *spin_rating_end;
+
+	GtkWidget *check_class;
+	GtkWidget *menu_class;
+	GtkWidget *class_type;
+	GtkWidget *marks_type;
+	GtkWidget *menu_marks;
+
 	FileData *search_dir_fd;
 	gboolean   search_path_recurse;
 	gchar *search_name;
+	GRegex *search_name_regex;
 	gboolean   search_name_match_case;
 	gint64 search_size;
 	gint64 search_size_end;
@@ -159,6 +174,9 @@ struct _SearchData
 	CacheData *search_similarity_cd;
 	GList *search_keyword_list;
 	gchar *search_comment;
+	GRegex *search_comment_regex;
+	gint   search_rating;
+	gint   search_rating_end;
 	gboolean   search_comment_match_case;
 
 	MatchType search_type;
@@ -169,6 +187,10 @@ struct _SearchData
 	MatchType match_dimensions;
 	MatchType match_keywords;
 	MatchType match_comment;
+	MatchType match_rating;
+	MatchType match_gps;
+	MatchType match_class;
+	MatchType match_marks;
 
 	gboolean match_name_enable;
 	gboolean match_size_enable;
@@ -177,6 +199,9 @@ struct _SearchData
 	gboolean match_similarity_enable;
 	gboolean match_keywords_enable;
 	gboolean match_comment_enable;
+	gboolean match_rating_enable;
+	gboolean match_class_enable;
+	gboolean match_marks_enable;
 
 	GList *search_folder_list;
 	GList *search_done_list;
@@ -198,6 +223,18 @@ struct _SearchData
 	ThumbLoader *thumb_loader;
 	gboolean thumb_enable;
 	FileData *thumb_fd;
+
+	/* Used for lat/long coordinate search
+	*/
+	gint search_gps;
+	gdouble search_lat, search_lon;
+	GtkWidget *entry_gps_coord;
+	GtkWidget *check_gps;
+	GtkWidget *spin_gps;
+	GtkWidget *units_gps;
+	GtkWidget *menu_gps;
+	gboolean match_gps_enable;
+
 };
 
 typedef struct _MatchFileData MatchFileData;
@@ -252,6 +289,30 @@ static const MatchList text_search_menu_comment[] = {
 	{ N_("miss"),		SEARCH_MATCH_NONE }
 };
 
+
+static const MatchList text_search_menu_rating[] = {
+	{ N_("equal to"),	SEARCH_MATCH_EQUAL },
+	{ N_("less than"),	SEARCH_MATCH_UNDER },
+	{ N_("greater than"),	SEARCH_MATCH_OVER },
+	{ N_("between"),	SEARCH_MATCH_BETWEEN }
+};
+
+static const MatchList text_search_menu_gps[] = {
+	{ N_("not geocoded"),	SEARCH_MATCH_NONE },
+	{ N_("less than"),	SEARCH_MATCH_UNDER },
+	{ N_("greater than"),	SEARCH_MATCH_OVER }
+};
+
+static const MatchList text_search_menu_class[] = {
+	{ N_("is"),	SEARCH_MATCH_EQUAL },
+	{ N_("is not"),	SEARCH_MATCH_NONE }
+};
+
+static const MatchList text_search_menu_marks[] = {
+	{ N_("is"),	SEARCH_MATCH_EQUAL },
+	{ N_("is not"),	SEARCH_MATCH_NONE }
+};
+
 static GList *search_window_list = NULL;
 
 
@@ -922,13 +983,6 @@ static void sr_menu_edit_cb(GtkWidget *widget, gpointer data)
 	search_result_edit_selected(sd, key);
 }
 
-static void sr_menu_collection_cb(GtkWidget *widget, gpointer data)
-{
-	SearchData *sd = data;
-
-	search_result_collection_from_selection(sd);
-}
-
 static void sr_menu_print_cb(GtkWidget *widget, gpointer data)
 {
 	SearchData *sd = data;
@@ -962,6 +1016,15 @@ static void sr_menu_delete_cb(GtkWidget *widget, gpointer data)
 {
 	SearchData *sd = data;
 
+	options->file_ops.safe_delete_enable = FALSE;
+	file_util_delete(NULL, search_result_selection_list(sd), sd->window);
+}
+
+static void sr_menu_move_to_trash_cb(GtkWidget *widget, gpointer data)
+{
+	SearchData *sd = data;
+
+	options->file_ops.safe_delete_enable = TRUE;
 	file_util_delete(NULL, search_result_selection_list(sd), sd->window);
 }
 
@@ -969,7 +1032,14 @@ static void sr_menu_copy_path_cb(GtkWidget *widget, gpointer data)
 {
 	SearchData *sd = data;
 
-	file_util_copy_path_list_to_clipboard(search_result_selection_list(sd));
+	file_util_copy_path_list_to_clipboard(search_result_selection_list(sd), TRUE);
+}
+
+static void sr_menu_copy_path_unquoted_cb(GtkWidget *widget, gpointer data)
+{
+	SearchData *sd = data;
+
+	file_util_copy_path_list_to_clipboard(search_result_selection_list(sd), FALSE);
 }
 
 static void sr_menu_remove_cb(GtkWidget *widget, gpointer data)
@@ -986,6 +1056,13 @@ static void sr_menu_clear_cb(GtkWidget *widget, gpointer data)
 	search_result_clear(sd);
 }
 
+static void sr_menu_play_cb(GtkWidget *widget, gpointer data)
+{
+	SearchData *sd = data;
+
+	start_editor_from_file(options->image_l_click_video_editor, sd->click_fd);
+}
+
 static void search_result_menu_destroy_cb(GtkWidget *widget, gpointer data)
 {
 	GList *editmenu_fd_list = data;
@@ -993,14 +1070,40 @@ static void search_result_menu_destroy_cb(GtkWidget *widget, gpointer data)
 	filelist_free(editmenu_fd_list);
 }
 
+/**
+ * @brief Add file selection list to a collection
+ * @param[in] widget 
+ * @param[in] data Index to the collection list menu item selected, or -1 for new collection
+ * 
+ * 
+ */
+static void search_pop_menu_collections_cb(GtkWidget *widget, gpointer data)
+{
+	SearchData *sd;
+	GList *selection_list;
+
+	sd = submenu_item_get_data(widget);
+	selection_list = search_result_selection_list(sd);
+	pop_menu_collections(selection_list, data);
+
+	filelist_free(selection_list);
+}
+
 static GtkWidget *search_result_menu(SearchData *sd, gboolean on_row, gboolean empty)
 {
 	GtkWidget *menu;
 	GtkWidget *item;
 	GList *editmenu_fd_list;
+	GtkWidget *submenu;
+	gboolean video;
 
 	menu = popup_menu_short_lived();
 
+	video = (on_row && sd->click_fd && sd->click_fd->format_class == FORMAT_CLASS_VIDEO);
+	menu_item_add_stock_sensitive(menu, _("_Play"), GTK_STOCK_MEDIA_PLAY, video,
+			    G_CALLBACK(sr_menu_play_cb), sd);
+	menu_item_add_divider(menu);
+
 	menu_item_add_sensitive(menu, _("_View"), on_row,
 				G_CALLBACK(sr_menu_view_cb), sd);
 	menu_item_add_stock_sensitive(menu, _("View in _new window"), GTK_STOCK_NEW, on_row,
@@ -1017,8 +1120,11 @@ static GtkWidget *search_result_menu(SearchData *sd, gboolean on_row, gboolean e
 			 G_CALLBACK(search_result_menu_destroy_cb), editmenu_fd_list);
 	submenu_add_edit(menu, &item, G_CALLBACK(sr_menu_edit_cb), sd, editmenu_fd_list);
 	if (!on_row) gtk_widget_set_sensitive(item, FALSE);
-	menu_item_add_stock_sensitive(menu, _("Add to new collection"), GTK_STOCK_INDEX, on_row,
-				      G_CALLBACK(sr_menu_collection_cb), sd);
+
+	submenu_add_collections(menu, &item,
+				G_CALLBACK(search_pop_menu_collections_cb), sd);
+	gtk_widget_set_sensitive(item, on_row);
+
 	menu_item_add_stock_sensitive(menu, _("Print..."), GTK_STOCK_PRINT, on_row,
 				      G_CALLBACK(sr_menu_print_cb), sd);
 	menu_item_add_divider(menu);
@@ -1028,15 +1134,20 @@ static GtkWidget *search_result_menu(SearchData *sd, gboolean on_row, gboolean e
 				G_CALLBACK(sr_menu_move_cb), sd);
 	menu_item_add_sensitive(menu, _("_Rename..."), on_row,
 				G_CALLBACK(sr_menu_rename_cb), sd);
-	menu_item_add_stock_sensitive(menu, _("_Delete..."), GTK_STOCK_DELETE, on_row,
-				      G_CALLBACK(sr_menu_delete_cb), sd);
 	menu_item_add_sensitive(menu, _("_Copy path"), on_row,
 				G_CALLBACK(sr_menu_copy_path_cb), sd);
+	menu_item_add_sensitive(menu, _("_Copy path unquoted"), on_row,
+				G_CALLBACK(sr_menu_copy_path_unquoted_cb), sd);
+
 	menu_item_add_divider(menu);
-	menu_item_add_stock_sensitive(menu, _("Rem_ove"), GTK_STOCK_REMOVE, on_row,
-				      G_CALLBACK(sr_menu_remove_cb), sd);
-	menu_item_add_stock_sensitive(menu, _("C_lear"), GTK_STOCK_CLEAR, !empty,
-				      G_CALLBACK(sr_menu_clear_cb), sd);
+	menu_item_add_stock_sensitive(menu,
+				options->file_ops.confirm_move_to_trash ? _("Move to Trash...") :
+					_("Move to Trash"), PIXBUF_INLINE_ICON_TRASH, on_row,
+				G_CALLBACK(sr_menu_move_to_trash_cb), sd);
+	menu_item_add_stock_sensitive(menu,
+				options->file_ops.confirm_delete ? _("_Delete...") :
+					_("_Delete"), GTK_STOCK_DELETE, on_row,
+				G_CALLBACK(sr_menu_delete_cb), sd);
 
 	return menu;
 }
@@ -1247,6 +1358,7 @@ static gboolean search_result_keypress_cb(GtkWidget *widget, GdkEventKey *event,
 				file_util_rename(NULL, search_result_selection_list(sd), widget);
 				break;
 			case 'D': case 'd':
+				options->file_ops.safe_delete_enable = TRUE;
 				file_util_delete(NULL, search_result_selection_list(sd), widget);
 				break;
 			case 'A': case 'a':
@@ -1295,7 +1407,11 @@ static gboolean search_result_keypress_cb(GtkWidget *widget, GdkEventKey *event,
 				{
 				GtkWidget *menu;
 
-				sd->click_fd = mfd->fd;
+				if (mfd)
+					sd->click_fd = mfd->fd;
+				else
+					sd->click_fd = NULL;
+
 				menu = search_result_menu(sd, (mfd != NULL), (search_result_count(sd, NULL) > 0));
 				gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
 					       search_result_menu_pos_cb, sd, 0, GDK_CURRENT_TIME);
@@ -1332,6 +1448,11 @@ static gboolean search_window_keypress_cb(GtkWidget *widget, GdkEventKey *event,
 				break;
 			}
 		}
+	if (!stop_signal && is_help_key(event))
+		{
+		help_window_show("GuideImageSearchSearch.html");
+		stop_signal = TRUE;
+		}
 
 	return stop_signal;
 }
@@ -1348,6 +1469,12 @@ static GtkTargetEntry result_drag_types[] = {
 };
 static gint n_result_drag_types = 2;
 
+static GtkTargetEntry result_drop_types[] = {
+	{ "text/uri-list", 0, TARGET_URI_LIST },
+	{ "text/plain", 0, TARGET_TEXT_PLAIN }
+};
+static gint n_result_drop_types = 2;
+
 static void search_dnd_data_set(GtkWidget *widget, GdkDragContext *context,
 				GtkSelectionData *selection_data, guint info,
 				guint time, gpointer data)
@@ -1394,6 +1521,103 @@ static void search_dnd_begin(GtkWidget *widget, GdkDragContext *context, gpointe
 		}
 }
 
+static void search_gps_dnd_received_cb(GtkWidget *pane, GdkDragContext *context,
+										gint x, gint y,
+										GtkSelectionData *selection_data, guint info,
+										guint time, gpointer data)
+{
+	SearchData *sd = data;
+	GList *list;
+	gdouble latitude, longitude;
+	FileData *fd;
+
+	if (info == TARGET_URI_LIST)
+		{
+		list = uri_filelist_from_gtk_selection_data(selection_data);
+
+		/* If more than one file, use only the first file in a list.
+		*/
+		if (list != NULL)
+			{
+			fd = list->data;
+			latitude = metadata_read_GPS_coord(fd, "Xmp.exif.GPSLatitude", 1000);
+			longitude = metadata_read_GPS_coord(fd, "Xmp.exif.GPSLongitude", 1000);
+			if (latitude != 1000 && longitude != 1000)
+				{
+				gtk_entry_set_text(GTK_ENTRY(sd->entry_gps_coord),
+							g_strdup_printf("%lf %lf", latitude, longitude));
+				}
+			else
+				{
+				gtk_entry_set_text(GTK_ENTRY(sd->entry_gps_coord), "Image is not geocoded");
+				}
+			}
+		}
+
+	if (info == TARGET_TEXT_PLAIN)
+		{
+		gtk_entry_set_text(GTK_ENTRY(sd->entry_gps_coord),"");
+		}
+}
+
+static void search_path_entry_dnd_received_cb(GtkWidget *pane, GdkDragContext *context,
+										gint x, gint y,
+										GtkSelectionData *selection_data, guint info,
+										guint time, gpointer data)
+{
+	SearchData *sd = data;
+	GList *list;
+	FileData *fd;
+
+	if (info == TARGET_URI_LIST)
+		{
+		list = uri_filelist_from_gtk_selection_data(selection_data);
+		/* If more than one file, use only the first file in a list.
+		*/
+		if (list != NULL)
+			{
+			fd = list->data;
+			gtk_entry_set_text(GTK_ENTRY(sd->path_entry),
+						g_strdup_printf("%s", fd->path));
+			gtk_widget_set_tooltip_text(GTK_WIDGET(sd->path_entry),g_strdup_printf("%s", fd->path));
+			}
+		}
+
+	if (info == TARGET_TEXT_PLAIN)
+		{
+		gtk_entry_set_text(GTK_ENTRY(sd->path_entry),"");
+		}
+}
+
+static void search_image_content_dnd_received_cb(GtkWidget *pane, GdkDragContext *context,
+										gint x, gint y,
+										GtkSelectionData *selection_data, guint info,
+										guint time, gpointer data)
+{
+	SearchData *sd = data;
+	GList *list;
+	FileData *fd;
+
+	if (info == TARGET_URI_LIST)
+		{
+		list = uri_filelist_from_gtk_selection_data(selection_data);
+		/* If more than one file, use only the first file in a list.
+		*/
+		if (list != NULL)
+			{
+			fd = list->data;
+			gtk_entry_set_text(GTK_ENTRY(sd->entry_similarity),
+						g_strdup_printf("%s", fd->path));
+			gtk_widget_set_tooltip_text(GTK_WIDGET(sd->entry_similarity),g_strdup_printf("%s", fd->path));
+			}
+		}
+
+	if (info == TARGET_TEXT_PLAIN)
+		{
+		gtk_entry_set_text(GTK_ENTRY(sd->entry_similarity),"");
+		}
+}
+
 static void search_dnd_init(SearchData *sd)
 {
 	gtk_drag_source_set(sd->result_view, GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
@@ -1403,6 +1627,30 @@ static void search_dnd_init(SearchData *sd)
 			 G_CALLBACK(search_dnd_data_set), sd);
 	g_signal_connect(G_OBJECT(sd->result_view), "drag_begin",
 			 G_CALLBACK(search_dnd_begin), sd);
+
+	gtk_drag_dest_set(GTK_WIDGET(sd->entry_gps_coord),
+					 GTK_DEST_DEFAULT_ALL,
+					  result_drop_types, n_result_drop_types,
+					 GDK_ACTION_COPY);
+
+	g_signal_connect(G_OBJECT(sd->entry_gps_coord), "drag_data_received",
+					G_CALLBACK(search_gps_dnd_received_cb), sd);
+
+	gtk_drag_dest_set(GTK_WIDGET(sd->path_entry),
+					GTK_DEST_DEFAULT_ALL,
+					result_drop_types, n_result_drop_types,
+					GDK_ACTION_COPY);
+
+	g_signal_connect(G_OBJECT(sd->path_entry), "drag_data_received",
+					G_CALLBACK(search_path_entry_dnd_received_cb), sd);
+
+	gtk_drag_dest_set(GTK_WIDGET(sd->entry_similarity),
+					GTK_DEST_DEFAULT_ALL,
+					result_drop_types, n_result_drop_types,
+					GDK_ACTION_COPY);
+
+	g_signal_connect(G_OBJECT(sd->entry_similarity), "drag_data_received",
+					G_CALLBACK(search_image_content_dnd_received_cb), sd);
 }
 
 /*
@@ -1646,6 +1894,7 @@ static gboolean search_file_next(SearchData *sd)
 	gint width = 0;
 	gint height = 0;
 	gint sim = 0;
+	time_t file_date;
 
 	if (!sd->search_file_list) return FALSE;
 
@@ -1682,13 +1931,13 @@ static gboolean search_file_next(SearchData *sd)
 			{
 			if (sd->search_name_match_case)
 				{
-				match = (strstr(fd->name, sd->search_name) != NULL);
+				match = g_regex_match(sd->search_name_regex, fd->name, 0, NULL);
 				}
 			else
 				{
 				/* sd->search_name is converted in search_start() */
 				gchar *haystack = g_utf8_strdown(fd->name, -1);
-				match = (strstr(haystack, sd->search_name) != NULL);
+				match = g_regex_match(sd->search_name_regex, haystack, 0, NULL);
 				g_free(haystack);
 				}
 			}
@@ -1722,11 +1971,33 @@ static gboolean search_file_next(SearchData *sd)
 		tested = TRUE;
 		match = FALSE;
 
+		if (g_strcmp0(gtk_combo_box_text_get_active_text(
+						GTK_COMBO_BOX_TEXT(sd->date_type)), _("Changed")) == 0)
+			{
+			file_date = fd->cdate;
+			}
+		else if (g_strcmp0(gtk_combo_box_text_get_active_text(
+						GTK_COMBO_BOX_TEXT(sd->date_type)), _("Original")) == 0)
+			{
+			read_exif_time_data(fd);
+			file_date = fd->exifdate;
+			}
+		else if (g_strcmp0(gtk_combo_box_text_get_active_text(
+						GTK_COMBO_BOX_TEXT(sd->date_type)), _("Digitized")) == 0)
+			{
+			read_exif_time_digitized_data(fd);
+			file_date = fd->exifdate_digitized;
+			}
+		else
+			{
+			file_date = fd->date;
+			}
+
 		if (sd->match_date == SEARCH_MATCH_EQUAL)
 			{
 			struct tm *lt;
 
-			lt = localtime(&fd->date);
+			lt = localtime(&file_date);
 			match = (lt &&
 				 lt->tm_year == sd->search_date_y - 1900 &&
 				 lt->tm_mon == sd->search_date_m - 1 &&
@@ -1734,11 +2005,11 @@ static gboolean search_file_next(SearchData *sd)
 			}
 		else if (sd->match_date == SEARCH_MATCH_UNDER)
 			{
-			match = (fd->date < convert_dmy_to_time(sd->search_date_d, sd->search_date_m, sd->search_date_y));
+			match = (file_date < convert_dmy_to_time(sd->search_date_d, sd->search_date_m, sd->search_date_y));
 			}
 		else if (sd->match_date == SEARCH_MATCH_OVER)
 			{
-			match = (fd->date > convert_dmy_to_time(sd->search_date_d, sd->search_date_m, sd->search_date_y) + 60 * 60 * 24 - 1);
+			match = (file_date > convert_dmy_to_time(sd->search_date_d, sd->search_date_m, sd->search_date_y) + 60 * 60 * 24 - 1);
 			}
 		else if (sd->match_date == SEARCH_MATCH_BETWEEN)
 			{
@@ -1753,7 +2024,7 @@ static gboolean search_file_next(SearchData *sd)
 				{
 				a += 60 * 60 * 24 - 1;
 				}
-			match = MATCH_IS_BETWEEN(fd->date, a, b);
+			match = MATCH_IS_BETWEEN(file_date, a, b);
 			}
 		}
 
@@ -1857,11 +2128,11 @@ static gboolean search_file_next(SearchData *sd)
 
 			if (sd->match_comment == SEARCH_MATCH_CONTAINS)
 				{
-				match = (strstr(comment, sd->search_comment) != NULL);
+				match = g_regex_match(sd->search_comment_regex, comment, 0, NULL);
 				}
 			else if (sd->match_comment == SEARCH_MATCH_NONE)
 				{
-				match = (strstr(comment, sd->search_comment) == NULL);
+				match = !g_regex_match(sd->search_comment_regex, comment, 0, NULL);
 				}
 			g_free(comment);
 			}
@@ -1871,8 +2142,183 @@ static gboolean search_file_next(SearchData *sd)
 			}
 		}
 
-	if ((match || extra_only) &&
-	    (sd->match_dimensions_enable || sd->match_similarity_enable))
+	if (match && sd->match_rating_enable)
+		{
+		tested = TRUE;
+		match = FALSE;
+		gint rating;
+
+		rating = metadata_read_int(fd, RATING_KEY, 0);
+		if (sd->match_rating == SEARCH_MATCH_EQUAL)
+			{
+			match = (rating == sd->search_rating);
+			}
+		else if (sd->match_rating == SEARCH_MATCH_UNDER)
+			{
+			match = (rating < sd->search_rating);
+			}
+		else if (sd->match_rating == SEARCH_MATCH_OVER)
+			{
+			match = (rating > sd->search_rating);
+			}
+		else if (sd->match_rating == SEARCH_MATCH_BETWEEN)
+			{
+			match = MATCH_IS_BETWEEN(rating, sd->search_rating, sd->search_rating_end);
+			}
+		}
+
+	if (match && sd->match_class_enable)
+		{
+		tested = TRUE;
+		match = FALSE;
+		gint class;
+		FileFormatClass search_class;
+
+		if (g_strcmp0(gtk_combo_box_text_get_active_text(
+						GTK_COMBO_BOX_TEXT(sd->class_type)), _("Image")) == 0)
+			{
+			search_class = FORMAT_CLASS_IMAGE;
+			}
+		else if (g_strcmp0(gtk_combo_box_text_get_active_text(
+						GTK_COMBO_BOX_TEXT(sd->class_type)), _("Raw Image")) == 0)
+			{
+			search_class = FORMAT_CLASS_RAWIMAGE;
+			}
+		else if (g_strcmp0(gtk_combo_box_text_get_active_text(
+						GTK_COMBO_BOX_TEXT(sd->class_type)), _("Video")) == 0)
+			{
+			search_class = FORMAT_CLASS_VIDEO;
+			}
+		else if (g_strcmp0(gtk_combo_box_text_get_active_text(
+						GTK_COMBO_BOX_TEXT(sd->class_type)), _("Metadata")) == 0)
+			{
+			search_class = FORMAT_CLASS_META;
+			}
+		else
+			{
+			search_class = FORMAT_CLASS_UNKNOWN;
+			}
+
+		class = fd->format_class;
+		if (sd->match_class == SEARCH_MATCH_EQUAL)
+			{
+			match = (class == search_class);
+			}
+		else if (sd->match_class == SEARCH_MATCH_NONE)
+			{
+			match = (class != search_class);
+			}
+		}
+
+	if (match && sd->match_marks_enable)
+		{
+		tested = TRUE;
+		match = FALSE;
+		gint search_marks;
+		gint i;
+		gchar *marks_string;
+
+		if (g_strcmp0(gtk_combo_box_text_get_active_text(
+						GTK_COMBO_BOX_TEXT(sd->marks_type)), _("Any mark")) == 0)
+			{
+			search_marks = -1;
+			}
+		else
+			{
+			for (i = 0; i < FILEDATA_MARKS_SIZE; i++)
+				{
+				marks_string = g_strdup_printf("%s%d", _("Mark "), i + 1);
+				if (g_strcmp0(marks_string, options->marks_tooltips[i]) != 0)
+					{
+					g_free(marks_string);
+					marks_string = g_strdup_printf("%s%d %s", _("Mark "), i + 1,
+													options->marks_tooltips[i]);
+					}
+
+				if (g_strcmp0(gtk_combo_box_text_get_active_text(
+								GTK_COMBO_BOX_TEXT(sd->marks_type)),
+								marks_string) == 0)
+					{
+					search_marks = 1 << i;
+					}
+				g_free(marks_string);
+				}
+			}
+
+		if (sd->match_marks == SEARCH_MATCH_EQUAL)
+			{
+			match = (fd->marks & search_marks);
+			}
+		else
+			{
+			if (search_marks == -1)
+				{
+				match = fd->marks ? FALSE : TRUE;
+				}
+			else
+				{
+				match = (fd->marks & search_marks) ? FALSE : TRUE;
+				}
+			}
+		}
+
+	if (match && sd->match_gps_enable)
+		{
+		/* Calculate the distance the image is from the specified origin.
+		* This is a standard algorithm. A simplified one may be faster.
+		*/
+		#define RADIANS  0.0174532925
+		#define KM_EARTH_RADIUS 6371
+		#define MILES_EARTH_RADIUS 3959
+		#define NAUTICAL_MILES_EARTH_RADIUS 3440
+
+		gdouble latitude, longitude, range, conversion;
+
+		if (g_strcmp0(gtk_combo_box_text_get_active_text(
+						GTK_COMBO_BOX_TEXT(sd->units_gps)), _("km")) == 0)
+			{
+			conversion = KM_EARTH_RADIUS;
+			}
+		else if (g_strcmp0(gtk_combo_box_text_get_active_text(
+						GTK_COMBO_BOX_TEXT(sd->units_gps)), _("miles")) == 0)
+			{
+			conversion = MILES_EARTH_RADIUS;
+			}
+		else
+			{
+			conversion = NAUTICAL_MILES_EARTH_RADIUS;
+			}
+
+		tested = TRUE;
+		match = FALSE;
+
+		latitude = metadata_read_GPS_coord(fd, "Xmp.exif.GPSLatitude", 1000);
+		longitude = metadata_read_GPS_coord(fd, "Xmp.exif.GPSLongitude", 1000);
+		if (latitude != 1000 && longitude != 1000)
+			{
+			range = conversion * acos(sin(latitude * RADIANS) *
+						sin(sd->search_lat * RADIANS) + cos(latitude * RADIANS) *
+						cos(sd->search_lat * RADIANS) * cos((sd->search_lon -
+						longitude) * RADIANS));
+			if (sd->match_gps == SEARCH_MATCH_UNDER)
+				{
+				if (sd->search_gps >= range)
+					match = TRUE;
+				}
+			else if (sd->match_gps == SEARCH_MATCH_OVER)
+				{
+				if (sd->search_gps < range)
+					match = TRUE;
+				}
+			}
+		else if (sd->match_gps == SEARCH_MATCH_NONE)
+			{
+			match = TRUE;
+			}
+		}
+
+	if ((match || extra_only) && (sd->match_dimensions_enable ||
+								sd ->match_similarity_enable))
 		{
 		tested = TRUE;
 
@@ -2029,6 +2475,8 @@ static void search_similarity_load_done_cb(ImageLoader *il, gpointer data)
 
 static void search_start(SearchData *sd)
 {
+	GError *error = NULL;
+
 	search_stop(sd);
 	search_result_clear(sd);
 
@@ -2045,6 +2493,20 @@ static void search_start(SearchData *sd)
 		sd->search_name = tmp;
 		}
 
+	if(sd->search_name_regex)
+		{
+		g_regex_unref(sd->search_name_regex);
+		}
+
+	sd->search_name_regex = g_regex_new(sd->search_name, 0, 0, &error);
+	if (error)
+		{
+		log_printf("Error: could not compile regular expression %s\n%s\n", sd->search_name, error->message);
+		g_error_free(error);
+		error = NULL;
+		sd->search_name_regex = g_regex_new("", 0, 0, NULL);
+		}
+
 	if (!sd->search_comment_match_case)
 		{
 		/* convert to lowercase here, so that this is only done once per search */
@@ -2053,6 +2515,20 @@ static void search_start(SearchData *sd)
 		sd->search_comment = tmp;
 		}
 
+	if(sd->search_comment_regex)
+		{
+		g_regex_unref(sd->search_comment_regex);
+		}
+
+	sd->search_comment_regex = g_regex_new(sd->search_comment, 0, 0, &error);
+	if (error)
+		{
+		log_printf("Error: could not compile regular expression %s\n%s\n", sd->search_comment, error->message);
+		g_error_free(error);
+		error = NULL;
+		sd->search_comment_regex = g_regex_new("", 0, 0, NULL);
+		}
+
 	sd->search_count = 0;
 	sd->search_total = 0;
 
@@ -2103,6 +2579,7 @@ static void search_start_cb(GtkWidget *widget, gpointer data)
 	SearchData *sd = data;
 	GtkTreeViewColumn *column;
 	gchar *path;
+	gchar *entry_text;
 
 	if (sd->search_folder_list)
 		{
@@ -2133,6 +2610,32 @@ static void search_start_cb(GtkWidget *widget, gpointer data)
 		tab_completion_append_to_history(sd->entry_similarity, sd->search_similarity_path);
 		}
 
+	/* Check the coordinate entry.
+	* If the result is not sensible, it should get blocked.
+	*/
+	if (sd->match_gps_enable)
+		{
+		if (sd->match_gps != SEARCH_MATCH_NONE)
+			{
+			entry_text = decode_geo_parameters(gtk_entry_get_text(
+										GTK_ENTRY(sd->entry_gps_coord)));
+
+			sd->search_lat = 1000;
+			sd->search_lon = 1000;
+			sscanf(entry_text," %lf  %lf ", &sd->search_lat, &sd->search_lon );
+			if (!(entry_text != NULL && !g_strstr_len(entry_text, -1, "Error") &&
+						sd->search_lat >= -90 && sd->search_lat <= 90 &&
+						sd->search_lon >= -180 && sd->search_lon <= 180))
+				{
+				file_util_warning_dialog(_(
+						"Entry does not contain a valid lat/long value"),
+							entry_text, GTK_STOCK_DIALOG_WARNING, sd->window);
+				return;
+				}
+			g_free(entry_text);
+			}
+		}
+
 	string_list_free(sd->search_keyword_list);
 	sd->search_keyword_list = keyword_list_pull(sd->entry_keywords);
 
@@ -2365,6 +2868,30 @@ static void menu_choice_size_cb(GtkWidget *combo, gpointer data)
 				(sd->match_size == SEARCH_MATCH_BETWEEN));
 }
 
+static void menu_choice_rating_cb(GtkWidget *combo, gpointer data)
+{
+	SearchData *sd = data;
+
+	if (!menu_choice_get_match_type(combo, &sd->match_rating)) return;
+
+	menu_choice_set_visible(gtk_widget_get_parent(sd->spin_rating_end),
+				(sd->match_rating == SEARCH_MATCH_BETWEEN));
+}
+
+static void menu_choice_class_cb(GtkWidget *combo, gpointer data)
+{
+	SearchData *sd = data;
+
+	if (!menu_choice_get_match_type(combo, &sd->match_class)) return;
+}
+
+static void menu_choice_marks_cb(GtkWidget *combo, gpointer data)
+{
+	SearchData *sd = data;
+
+	if (!menu_choice_get_match_type(combo, &sd->match_marks)) return;
+}
+
 static void menu_choice_date_cb(GtkWidget *combo, gpointer data)
 {
 	SearchData *sd = data;
@@ -2406,6 +2933,16 @@ static void menu_choice_spin_cb(GtkAdjustment *adjustment, gpointer data)
 	*value = (gint)gtk_adjustment_get_value(adjustment);
 }
 
+static void menu_choice_gps_cb(GtkWidget *combo, gpointer data)
+{
+	SearchData *sd = data;
+
+	if (!menu_choice_get_match_type(combo, &sd->match_gps)) return;
+
+	menu_choice_set_visible(gtk_widget_get_parent(sd->spin_gps),
+					(sd->match_gps != SEARCH_MATCH_NONE));
+}
+
 static GtkWidget *menu_spin(GtkWidget *box, gdouble min, gdouble max, gint value,
 			    GCallback func, gpointer data)
 {
@@ -2541,7 +3078,15 @@ static void search_window_destroy_cb(GtkWidget *widget, gpointer data)
 	file_data_unref(sd->search_dir_fd);
 
 	g_free(sd->search_name);
+	if(sd->search_name_regex)
+		{
+		g_regex_unref(sd->search_name_regex);
+		}
 	g_free(sd->search_comment);
+	if(sd->search_comment_regex)
+		{
+		g_regex_unref(sd->search_comment_regex);
+		}
 	g_free(sd->search_similarity_path);
 	string_list_free(sd->search_keyword_list);
 
@@ -2564,6 +3109,8 @@ void search_new(FileData *dir_fd, FileData *example_file)
 	GtkTreeSelection *selection;
 	GtkWidget *combo;
 	GdkGeometry geometry;
+	gint i;
+	gchar *marks_string;
 
 	sd = g_new0(SearchData, 1);
 
@@ -2583,17 +3130,24 @@ void search_new(FileData *dir_fd, FileData *example_file)
 	sd->match_dimensions = SEARCH_MATCH_EQUAL;
 	sd->match_keywords = SEARCH_MATCH_ALL;
 	sd->match_comment = SEARCH_MATCH_CONTAINS;
+	sd->match_rating = SEARCH_MATCH_EQUAL;
+	sd->match_class = SEARCH_MATCH_EQUAL;
+	sd->match_marks = SEARCH_MATCH_EQUAL;
 
 	sd->match_name_enable = TRUE;
 
 	sd->search_similarity = 95;
 
+	sd->search_gps = 1;
+	sd->match_gps = SEARCH_MATCH_NONE;
+
 	if (example_file)
 		{
 		sd->search_similarity_path = g_strdup(example_file->path);
 		}
 
 	sd->window = window_new(GTK_WINDOW_TOPLEVEL, "search", NULL, NULL, _("Image search"));
+	DEBUG_NAME(sd->window);
 
 	gtk_window_set_resizable(GTK_WINDOW(sd->window), TRUE);
 
@@ -2650,6 +3204,7 @@ void search_new(FileData *dir_fd, FileData *example_file)
 	gtk_widget_show(combo);
 	pref_checkbox_new_int(hbox, _("Match case"),
 			      sd->search_name_match_case, &sd->search_name_match_case);
+	gtk_widget_set_tooltip_text(GTK_WIDGET(combo), "When set to \"contains\", this field uses Perl Compatible Regular Expressions.\ne.g. use \n.*\\.jpg\n and not \n*.jpg\n\nSee the Help file.");
 
 	/* Search for file size */
 	hbox = menu_choice(sd->box_search, &sd->check_size, &sd->menu_size,
@@ -2669,6 +3224,7 @@ void search_new(FileData *dir_fd, FileData *example_file)
 			   _("File date is"), &sd->match_date_enable,
 			   text_search_menu_date, sizeof(text_search_menu_date) / sizeof(MatchList),
 			   G_CALLBACK(menu_choice_date_cb), sd);
+
 	sd->date_sel = date_selection_new();
 	date_selection_time_set(sd->date_sel, time(NULL));
 	gtk_box_pack_start(GTK_BOX(hbox), sd->date_sel, FALSE, FALSE, 0);
@@ -2682,6 +3238,16 @@ void search_new(FileData *dir_fd, FileData *example_file)
 	gtk_box_pack_start(GTK_BOX(hbox2), sd->date_sel_end, FALSE, FALSE, 0);
 	gtk_widget_show(sd->date_sel_end);
 
+	sd->date_type = gtk_combo_box_text_new();
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(sd->date_type), _("Modified"));
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(sd->date_type), _("Status Changed"));
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(sd->date_type), _("Original"));
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(sd->date_type), _("Digitized"));
+	gtk_box_pack_start(GTK_BOX(hbox), sd->date_type, FALSE, FALSE, 0);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(sd->date_type), 0);
+	gtk_widget_set_tooltip_text(sd->date_type, "Modified (mtime)\nStatus Changed (ctime)\nOriginal (Exif.Photo.DateTimeOriginal)\nDigitized (Exif.Photo.DateTimeDigitized)");
+	gtk_widget_show(sd->date_type);
+
 	/* Search for image dimensions */
 	hbox = menu_choice(sd->box_search, &sd->check_dimensions, &sd->menu_dimensions,
 			   _("Image dimensions are"), &sd->match_dimensions_enable,
@@ -2719,6 +3285,8 @@ void search_new(FileData *dir_fd, FileData *example_file)
 	tab_completion_add_select_button(sd->entry_similarity, NULL, FALSE);
 	gtk_box_pack_start(GTK_BOX(hbox), combo, TRUE, TRUE, 0);
 	gtk_widget_show(combo);
+	pref_checkbox_new_int(hbox, _("Ignore rotation"),
+				options->rot_invariant_sim, &options->rot_invariant_sim);
 
 	/* Search for image keywords */
 	hbox = menu_choice(sd->box_search, &sd->check_keywords, &sd->menu_keywords,
@@ -2745,6 +3313,96 @@ void search_new(FileData *dir_fd, FileData *example_file)
 	gtk_widget_show(sd->entry_comment);
 	pref_checkbox_new_int(hbox, _("Match case"),
 			      sd->search_comment_match_case, &sd->search_comment_match_case);
+	gtk_widget_set_tooltip_text(GTK_WIDGET(sd->entry_comment), "This field uses Perl Compatible Regular Expressions.\ne.g. use \nabc.*ghk\n and not \nabc*ghk\n\nSee the Help file.");
+
+	/* Search for image rating */
+	hbox = menu_choice(sd->box_search, &sd->check_rating, &sd->menu_rating,
+			   _("Image rating is"), &sd->match_rating_enable,
+			   text_search_menu_rating, sizeof(text_search_menu_rating) / sizeof(MatchList),
+			   G_CALLBACK(menu_choice_rating_cb), sd);
+	sd->spin_size = menu_spin(hbox, -1, 5, sd->search_rating,
+				  G_CALLBACK(menu_choice_spin_cb), &sd->search_rating);
+	hbox2 = gtk_hbox_new(FALSE, PREF_PAD_SPACE);
+	gtk_box_pack_start(GTK_BOX(hbox), hbox2, FALSE, FALSE, 0);
+	pref_label_new(hbox2, _("and"));
+	sd->spin_rating_end = menu_spin(hbox2, -1, 5, sd->search_rating_end,
+				      G_CALLBACK(menu_choice_spin_cb), &sd->search_rating_end);
+
+	/* Search for images within a specified range of a lat/long coordinate
+	*/
+	hbox = menu_choice(sd->box_search, &sd->check_gps, &sd->menu_gps,
+			   _("Image is"), &sd->match_gps_enable,
+			   text_search_menu_gps, sizeof(text_search_menu_gps) / sizeof(MatchList),
+			   G_CALLBACK(menu_choice_gps_cb), sd);
+
+	hbox2 = gtk_hbox_new(FALSE, PREF_PAD_SPACE);
+	gtk_box_pack_start(GTK_BOX(hbox), hbox2, FALSE, FALSE, 0);
+	sd->spin_gps = menu_spin(hbox2, 1, 9999, sd->search_gps,
+								   G_CALLBACK(menu_choice_spin_cb), &sd->search_gps);
+
+	sd->units_gps = gtk_combo_box_text_new();
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(sd->units_gps), _("km"));
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(sd->units_gps), _("miles"));
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(sd->units_gps), _("n.m."));
+	gtk_box_pack_start(GTK_BOX(hbox2), sd->units_gps, FALSE, FALSE, 0);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(sd->units_gps), 0);
+	gtk_widget_set_tooltip_text(sd->units_gps, "kilometres, miles or nautical miles");
+	gtk_widget_show(sd->units_gps);
+
+	pref_label_new(hbox2, _("from"));
+
+	sd->entry_gps_coord = gtk_entry_new();
+	gtk_editable_set_editable(GTK_EDITABLE(sd->entry_gps_coord), TRUE);
+	gtk_widget_set_has_tooltip(sd->entry_gps_coord, TRUE);
+	gtk_widget_set_tooltip_text(sd->entry_gps_coord, _("Enter a coordinate in the form:\n89.123 179.456\nor drag-and-drop a geo-coded image\nor left-click on the map and paste\nor cut-and-paste or drag-and-drop\nan internet search URL\nSee the Help file"));
+	gtk_box_pack_start(GTK_BOX(hbox2), sd->entry_gps_coord, TRUE, TRUE, 0);
+	gtk_widget_set_sensitive(sd->entry_gps_coord, TRUE);
+
+	gtk_widget_show(sd->entry_gps_coord);
+
+	/* Search for image class */
+	hbox = menu_choice(sd->box_search, &sd->check_class, &sd->menu_class,
+			   _("Image class"), &sd->match_class_enable,
+			   text_search_menu_class, sizeof(text_search_menu_class) / sizeof(MatchList),
+			   G_CALLBACK(menu_choice_class_cb), sd);
+
+	sd->class_type = gtk_combo_box_text_new();
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(sd->class_type), _("Image"));
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(sd->class_type), _("Raw Image"));
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(sd->class_type), _("Video"));
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(sd->class_type), _("Metadata"));
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(sd->class_type), _("Unknown"));
+	gtk_box_pack_start(GTK_BOX(hbox), sd->class_type, FALSE, FALSE, 0);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(sd->class_type), 0);
+	gtk_widget_show(sd->class_type);
+
+	/* Search for image marks */
+	hbox = menu_choice(sd->box_search, &sd->check_class, &sd->menu_marks,
+			   _("Marks"), &sd->match_marks_enable,
+			   text_search_menu_marks, sizeof(text_search_menu_marks) / sizeof(MatchList),
+			   G_CALLBACK(menu_choice_marks_cb), sd);
+
+	sd->marks_type = gtk_combo_box_text_new();
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(sd->marks_type), _("Any mark"));
+	for (i = 0; i < FILEDATA_MARKS_SIZE; i++)
+		{
+		marks_string = g_strdup_printf("%s%d", _("Mark "), i + 1);
+		if (g_strcmp0(marks_string, options->marks_tooltips[i]) != 0)
+			{
+			g_free(marks_string);
+			marks_string = g_strdup_printf("%s%d %s", _("Mark "), i + 1,
+											options->marks_tooltips[i]);
+			gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(sd->marks_type), marks_string);
+			}
+		else
+			{
+			gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(sd->marks_type), marks_string);
+			}
+		g_free(marks_string);
+		}
+	gtk_box_pack_start(GTK_BOX(hbox), sd->marks_type, FALSE, FALSE, 0);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(sd->marks_type), 0);
+	gtk_widget_show(sd->marks_type);
 
 	/* Done the types of searches */
 
@@ -2816,6 +3474,7 @@ void search_new(FileData *dir_fd, FileData *example_file)
 					      G_CALLBACK(search_thumb_toggle_cb), sd);
 
 	frame = gtk_frame_new(NULL);
+	DEBUG_NAME(frame);
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
 	gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, PREF_PAD_SPACE);
 	gtk_widget_show(frame);
@@ -2827,6 +3486,10 @@ void search_new(FileData *dir_fd, FileData *example_file)
 
 	sd->label_progress = gtk_progress_bar_new();
 	gtk_widget_set_size_request(sd->label_progress, 50, -1);
+#if GTK_CHECK_VERSION(3,0,0)
+	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(sd->label_progress), "");
+	gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(sd->label_progress), TRUE);
+#endif
 	gtk_box_pack_start(GTK_BOX(hbox), sd->label_progress, TRUE, TRUE, 0);
 	gtk_widget_show(sd->label_progress);
 
