@@ -22,6 +22,7 @@
 #include "main.h"
 #include "thumb_standard.h"
 
+#include "cache.h"
 #include "image-load.h"
 #include "md5-util.h"
 #include "pixbuf_util.h"
@@ -156,7 +157,8 @@ static gchar *thumb_std_cache_path(const gchar *path, const gchar *uri, gboolean
 		}
 	else
 		{
-		result = g_build_filename(homedir(), THUMB_FOLDER_GLOBAL, cache_subfolder, name, NULL);
+		result = g_build_filename(get_thumbnails_standard_cache_dir(),
+													cache_subfolder, name, NULL);
 		}
 
 	g_free(name);
@@ -346,8 +348,7 @@ static void thumb_loader_std_save(ThumbLoaderStd *tl, GdkPixbuf *pixbuf)
 		mark_uri = (tl->cache_local) ? tl->local_uri :tl->thumb_uri;
 
 		mark_app = g_strdup_printf("%s %s", GQ_APPNAME, VERSION);
-		mark_mtime = g_strdup_printf("%lu", tl->source_mtime);
-
+		mark_mtime = g_strdup_printf("%llu", (unsigned long long)tl->source_mtime);
 		pathl = path_from_utf8(tmp_path);
 		success = gdk_pixbuf_save(pixbuf, pathl, "png", NULL,
 					  THUMB_MARKER_URI, mark_uri,
@@ -666,7 +667,7 @@ gboolean thumb_loader_std_start(ThumbLoaderStd *tl, FileData *fd)
 
 
 	tl->fd = file_data_ref(fd);
-	if (!stat_utf8(fd->path, &st))
+	if (!stat_utf8(fd->path, &st) || (tl->fd->format_class != FORMAT_CLASS_IMAGE && tl->fd->format_class != FORMAT_CLASS_RAWIMAGE && tl->fd->format_class != FORMAT_CLASS_VIDEO && tl->fd->format_class != FORMAT_CLASS_COLLECTION && tl->fd->format_class != FORMAT_CLASS_PDF && !options->file_filter.disable))
 		{
 		thumb_loader_std_set_fallback(tl);
 		return FALSE;
@@ -675,7 +676,11 @@ gboolean thumb_loader_std_start(ThumbLoaderStd *tl, FileData *fd)
 	tl->source_size = st.st_size;
 	tl->source_mode = st.st_mode;
 
-	if (!thumb_cache) thumb_cache = g_build_filename(homedir(), THUMB_FOLDER_GLOBAL, NULL);
+	if (!thumb_cache)
+		{
+		thumb_cache = g_strdup(get_thumbnails_standard_cache_dir());
+		}
+
 	if (strncmp(tl->fd->path, thumb_cache, strlen(thumb_cache)) != 0)
 		{
 		gchar *pathl;
