@@ -28,22 +28,35 @@ typedef enum {
 	ZOOM_RESET_NONE		= 2
 } ZoomMode;
 
+typedef enum {
+	PRIMARY	= 0,
+	CLIPBOARD = 1,
+} ClipboardSelection;
+
 typedef enum {
 	MOUSE_BUTTON_LEFT	= 1,
 	MOUSE_BUTTON_MIDDLE	= 2,
 	MOUSE_BUTTON_RIGHT	= 3,
 	MOUSE_BUTTON_WHEEL_UP	= 4,
-	MOUSE_BUTTON_WHEEL_DOWN	= 5
+	MOUSE_BUTTON_WHEEL_DOWN	= 5,
+	MOUSE_BUTTON_BACK	= 8,
+	MOUSE_BUTTON_FORWARD	= 9
 } MouseButton;
 
 typedef enum {
 	DIRVIEW_LIST,
-	DIRVIEW_TREE
+	DIRVIEW_TREE,
+
+	// Keep this up to date!
+	DIRVIEW_LAST = DIRVIEW_TREE
 } DirViewType;
 
 typedef enum {
 	FILEVIEW_LIST,
-	FILEVIEW_ICON
+	FILEVIEW_ICON,
+
+	// Keep this up to date!
+	FILEVIEW_LAST = FILEVIEW_ICON
 } FileViewType;
 
 #define	CMD_COPY     "geeqie-copy-command.desktop"
@@ -60,7 +73,10 @@ typedef enum {
 	SORT_CTIME,
 	SORT_PATH,
 	SORT_NUMBER,
-	SORT_EXIFTIME
+	SORT_EXIFTIME,
+	SORT_EXIFTIMEDIGITIZED,
+	SORT_RATING,
+	SORT_CLASS
 } SortType;
 
 typedef enum {
@@ -126,9 +142,14 @@ typedef enum {
 	FORMAT_CLASS_IMAGE,
 	FORMAT_CLASS_RAWIMAGE,
 	FORMAT_CLASS_META,
+	FORMAT_CLASS_VIDEO,
+	FORMAT_CLASS_COLLECTION,
+	FORMAT_CLASS_PDF,
 	FILE_FORMAT_CLASSES
 } FileFormatClass;
 
+extern gchar *format_class_list[];
+
 typedef enum {
 	SS_ERR_NONE = 0,
 	SS_ERR_DISABLED, /**< secsave is disabled. */
@@ -245,9 +266,18 @@ typedef enum {
 
 #define MAX_SPLIT_IMAGES 4
 
+typedef enum {
+	SELECTION_NONE		= 0,
+	SELECTION_SELECTED	= 1 << 0,
+	SELECTION_PRELIGHT	= 1 << 1,
+	SELECTION_FOCUS		= 1 << 2
+} SelectionType;
+
 typedef struct _ImageLoader ImageLoader;
 typedef struct _ThumbLoader ThumbLoader;
 
+typedef struct _AnimationData AnimationData;
+
 typedef struct _CollectInfo CollectInfo;
 typedef struct _CollectionData CollectionData;
 typedef struct _CollectTable CollectTable;
@@ -329,6 +359,21 @@ struct _ThumbLoader
 	guint idle_done_id; /* event source id */
 };
 
+struct _AnimationData
+{
+	ImageWindow *iw;
+	LayoutWindow *lw;
+	GdkPixbufAnimation *gpa;
+	GdkPixbufAnimationIter *iter;
+	GdkPixbuf *gpb;
+	FileData *data_adr;
+	guint delay;
+	gboolean valid;
+	GCancellable *cancellable;
+	GFile *in_file;
+	GFileInputStream *gfstream;
+};
+
 struct _CollectInfo
 {
 	FileData *fd;
@@ -400,6 +445,7 @@ struct _CollectTable
 	guint drop_idle_id; /* event source id */
 
 	gboolean show_text;
+	gboolean show_stars;
 
 	/* file list for edit menu */
 	GList *editmenu_fd_list;
@@ -410,7 +456,6 @@ struct _CollectWindow
 	GtkWidget *window;
 	CollectTable *table;
 	GtkWidget *status_box;
-	GList *list;
 
 	GtkWidget *close_dialog;
 
@@ -460,7 +505,7 @@ struct _ImageWindow
 
 	/* button, scroll functions */
 	void (*func_button)(ImageWindow *, GdkEventButton *event, gpointer);
-	void (*func_drag)(ImageWindow *, GdkEventButton *event, gdouble dx, gdouble dy, gpointer);
+	void (*func_drag)(ImageWindow *, GdkEventMotion *event, gdouble dx, gdouble dy, gpointer);
 	void (*func_scroll)(ImageWindow *, GdkEventScroll *event, gpointer);
 	void (*func_focus_in)(ImageWindow *, gpointer);
 
@@ -498,9 +543,11 @@ struct _ImageWindow
 	gint orientation;
 	gboolean desaturate;
 	gint user_stereo;
+
+	gboolean mouse_wheel_mode;
 };
 
-#define FILEDATA_MARKS_SIZE 6
+#define FILEDATA_MARKS_SIZE 10
 
 struct _FileDataChangeInfo {
 	FileDataChangeType type;
@@ -517,6 +564,8 @@ struct _FileData {
 	gchar *path;
 	const gchar *name;
 	const gchar *extension;
+	gchar *extended_extension;
+	FileFormatClass format_class;
 	gchar *collate_key_name;
 	gchar *collate_key_name_nocase;
 	gint64 size;
@@ -549,8 +598,17 @@ struct _FileData {
 
 	ExifData *exif;
 	time_t exifdate;
+	time_t exifdate_digitized;
 	GHashTable *modified_xmp; // hash table which contains unwritten xmp metadata in format: key->list of string values
 	GList *cached_metadata;
+	gint rating;
+	gboolean metadata_in_idle_loaded;
+
+	gchar *owner;
+	gchar *group;
+	gchar *sym_link;
+
+	SelectionType selected;  // Used by view_file_icon.
 };
 
 struct _LayoutOptions
@@ -565,6 +623,7 @@ struct _LayoutOptions
 
 	gboolean show_thumbnails;
 	gboolean show_marks;
+	gboolean show_file_filter;
 	gboolean show_directory_date;
 	gboolean show_info_pixel;
 
@@ -586,6 +645,10 @@ struct _LayoutOptions
 		gint vdivider_pos;
 	} float_window;
 
+	struct {
+		gint vdivider_pos;
+	} folder_window;
+
 	struct {
 		gint w;
 		gint h;
@@ -597,16 +660,33 @@ struct _LayoutOptions
 		gint histogram_mode;
 	} image_overlay;
 
+	struct {
+		gint w;
+		gint h;
+		gint x;
+		gint y;
+	} log_window;
+
 	gboolean tools_float;
 	gboolean tools_hidden;
 	gboolean toolbar_hidden;
 
+	struct {
+		gboolean info;
+		gboolean sort;
+		gboolean tools_float;
+		gboolean tools_hidden;
+		gboolean hidden;
+	} bars_state;
+
 	gchar *home_path;
 	gchar *last_path;
 
 	StartUpPath startup_path;
 
 	gboolean exit_on_close;
+
+	gboolean animate;
 };
 
 struct _LayoutWindow
@@ -631,6 +711,7 @@ struct _LayoutWindow
 	GtkActionGroup *action_group_editors;
 	guint ui_editors_id;
 	GtkUIManager *ui_manager;
+	guint toolbar_merge_id[TOOLBAR_COUNT];
 	GList *toolbar_actions[TOOLBAR_COUNT];
 
 	GtkWidget *path_entry;
@@ -655,7 +736,7 @@ struct _LayoutWindow
 
 //	gint tools_float;
 //	gint tools_hidden;
-
+	GtkWidget *menu_tool_bar; /*Combined menu and toolbar box */
 	GtkWidget *menu_bar; /* referenced by lw, exist during whole lw lifetime */
 	/* toolbar */
 
@@ -720,12 +801,16 @@ struct _LayoutWindow
 	GtkWidget *bar_sort;
 	GtkWidget *bar;
 
-//	gint bar_sort_enabled;
+	gboolean bar_sort_enabled; /* Set during start-up, and checked when the editors have loaded */
 //	gint bar_enabled;
 
 //	gint bar_width;
 
 	GtkWidget *exif_window;
+
+	AnimationData *animation;
+
+	GtkWidget *log_window;
 };
 
 struct _ViewDir
@@ -773,6 +858,7 @@ struct _ViewDirInfoTree
 struct _ViewFile
 {
 	FileViewType type;
+	// TODO(xsdg): Turn this into a union (see VFLIST and VFICON from view_file.h).
 	gpointer info;
 
 	GtkWidget *widget;
@@ -781,6 +867,13 @@ struct _ViewFile
 	GtkWidget *filter;
 	GtkWidget *filter_check[FILEDATA_MARKS_SIZE];
 
+	struct {
+		GtkWidget *combo;
+		GtkWidget *frame;
+		gint count;
+		gint last_selected;
+	} file_filter;
+
 	FileData *dir_fd;
 	GList *list;
 
@@ -814,6 +907,8 @@ struct _ViewFile
 
 	/* file list for edit menu */
 	GList *editmenu_fd_list;
+
+	guint read_metadata_in_idle_id;
 };
 
 struct _ViewFileInfoList
@@ -826,8 +921,6 @@ struct _ViewFileInfoList
 	guint select_idle_id; /* event source id */
 };
 
-struct _IconData;
-
 struct _ViewFileInfoIcon
 {
 	/* table stuff */
@@ -835,15 +928,15 @@ struct _ViewFileInfoIcon
 	gint rows;
 
 	GList *selection;
-	struct _IconData *prev_selection;
+	FileData *prev_selection;
 
 	GtkWidget *tip_window;
 	guint tip_delay_id; /* event source id */
-	struct _IconData *tip_id;
+	FileData *tip_fd;
 
-	struct _IconData *click_id;
+	FileData *click_fd;
 
-	struct _IconData *focus_id;
+	FileData *focus_fd;
 	gint focus_row;
 	gint focus_column;
 
@@ -903,6 +996,7 @@ struct _PixmapFolders
 	GdkPixbuf *open;
 	GdkPixbuf *deny;
 	GdkPixbuf *parent;
+	GdkPixbuf *link;
 };
 
 struct _SecureSaveInfo {
@@ -926,11 +1020,15 @@ struct _CommandLine
 	gboolean startup_command_line_collection;
 	gboolean tools_hide;
 	gboolean tools_show;
+	gboolean log_window_show;
 	gchar *path;
 	gchar *file;
 	GList *cmd_list;
 	GList *collection_list;
 	gchar *geometry;
+	gchar *regexp;
+	gchar *log_file;
+	SecureSaveInfo *ssi;
 };
 
 #endif
