@@ -41,6 +41,9 @@
 #include "ui_utildlg.h"	/* for locale warning dialog */
 #include "md5-util.h"
 
+#include "filefilter.h"
+#include "secure_save.h"
+
 /*
  *-----------------------------------------------------------------------------
  * generic file information and manipulation routines (public)
@@ -49,12 +52,21 @@
 
 
 
-void print_term(const gchar *text_utf8)
+void print_term(gboolean err, const gchar *text_utf8)
 {
 	gchar *text_l;
 
 	text_l = g_locale_from_utf8(text_utf8, -1, NULL, NULL, NULL);
-	fputs((text_l) ? text_l : text_utf8, stderr);
+	if (err)
+		{
+		fputs((text_l) ? text_l : text_utf8, stderr);
+		}
+	else
+		{
+		fputs((text_l) ? text_l : text_utf8, stdout);
+		}
+	if(command_line && command_line->ssi)
+		secure_fputs(command_line->ssi, (text_l) ? text_l : text_utf8);
 	g_free(text_l);
 }
 
@@ -103,7 +115,7 @@ static void encoding_dialog(const gchar *path)
 	generic_dialog_add_button(gd, GTK_STOCK_CLOSE, NULL, NULL, TRUE);
 
 	generic_dialog_add_message(gd, GTK_STOCK_DIALOG_WARNING,
-				   _("Filename encoding locale mismatch"), string->str);
+				   _("Filename encoding locale mismatch"), string->str, TRUE);
 
 	gtk_widget_show(gd->dialog);
 
@@ -182,6 +194,8 @@ const gchar *homedir(void)
 	if (!home)
 		home = path_to_utf8(g_get_home_dir());
 
+	DEBUG_1("Home directory: %s", home);
+
 	return home;
 }
 
@@ -194,6 +208,8 @@ static gchar *xdg_dir_get(const gchar *key, const gchar *fallback)
     		return g_build_filename(homedir(), fallback, NULL);
     		}
 
+	DEBUG_1("Got xdg %s: %s", key, dir);
+
 	return path_to_utf8(dir);
 }
 
@@ -478,7 +494,12 @@ gboolean copy_file_attributes(const gchar *s, const gchar *t, gint perms, gint m
 			/* Ignores chown errors, while still doing chown
 			   (so root still can copy files preserving ownership) */
 			ret = TRUE;
-			if (chmod(tl, st.st_mode) < 0) ret = FALSE;
+			if (chmod(tl, st.st_mode) < 0) {
+                            struct stat st2;
+                            if (stat(tl, &st2) != 0 || st2.st_mode != st.st_mode) {
+                                ret = FALSE;
+                            }
+                        }
 			}
 
 		tb.actime = st.st_atime;
@@ -525,6 +546,68 @@ gboolean copy_file(const gchar *s, const gchar *t)
 		goto end;
 		}
 
+	/* Do not dereference absolute symlinks, but copy them "as is".
+	* For a relative symlink, we don't know how to properly change it when
+	* copied/moved to another dir to keep pointing it to same target as
+	* a relative symlink, so we turn it into absolute symlink using
+	* realpath() instead. */
+	struct stat st;
+	if (lstat_utf8(sl, &st) && S_ISLNK(st.st_mode))
+		{
+		gchar *link_target;
+		ssize_t i;
+
+		link_target = g_malloc(st.st_size + 1);
+		i = readlink(sl, link_target, st.st_size);
+		if (i<0)
+			{
+			g_free(link_target);
+			goto orig_copy;  // try a "normal" copy
+			}
+		link_target[st.st_size] = '\0';
+
+		if (link_target[0] != G_DIR_SEPARATOR) // if it is a relative symlink
+			{
+			gchar *absolute;
+
+			gchar *lastslash = strrchr(sl, G_DIR_SEPARATOR);
+			gint len = lastslash - sl + 1;
+
+			absolute = g_malloc(len + st.st_size + 1);
+			strncpy(absolute, sl, len);
+			strcpy(absolute + len, link_target);
+			g_free(link_target);
+			link_target = absolute;
+
+			gchar *realPath;
+			realPath = realpath(link_target, NULL);
+
+			if (realPath != NULL) // successfully resolved into an absolute path
+				{
+				g_free(link_target);
+				link_target = g_strdup(realPath);
+				g_free(realPath);
+				}
+			else                 // could not get absolute path, got some error instead
+				{
+				g_free(link_target);
+				goto orig_copy;  // so try a "normal" copy
+				}
+			}
+
+		if (stat_utf8(tl, &st)) unlink(tl); // first try to remove directory entry in destination directory if such entry exists
+
+		gint success = (symlink(link_target, tl) == 0);
+		g_free(link_target);
+
+		if (success)
+			{
+			ret = TRUE;
+			goto end;
+			}
+		} // if symlink did not succeed, continue on to try a copy procedure
+	orig_copy:
+
 	fi = fopen(sl, "rb");
 	if (!fi) goto end;
 
@@ -697,7 +780,7 @@ gchar *unique_filename_simple(const gchar *path)
 	name = filename_from_path(path);
 	if (!name) return NULL;
 
-	ext = extension_from_path(name);
+	ext = registered_extension_from_path(name);
 
 	if (!ext)
 		{
@@ -729,24 +812,16 @@ const gchar *filename_from_path(const gchar *path)
 
 gchar *remove_level_from_path(const gchar *path)
 {
-	gint p = 0, n = -1;
+	const gchar *base;
 
-	if (!path) return NULL;
+	if (!path) return g_strdup("");
 
-	while (path[p])
-		{
-		if (path[p] == G_DIR_SEPARATOR) n = p;
-		p++;
-		}
-	if (n <= 0) n++;
+	base = strrchr(path, G_DIR_SEPARATOR);
+	/* Take account of a file being in the root ( / ) folder - ensure the returned value
+	 * is at least one character long */
+	if (base) return g_strndup(path, (strlen(path)-strlen(base)) == 0 ? 1 : (strlen(path)-strlen(base)));
 
-	return g_strndup(path, (gsize) n);
-}
-
-const gchar *extension_from_path(const gchar *path)
-{
-	if (!path) return NULL;
-	return strrchr(path, '.');
+	return g_strdup("");
 }
 
 gboolean file_extension_match(const gchar *path, const gchar *ext)
@@ -766,18 +841,13 @@ gboolean file_extension_match(const gchar *path, const gchar *ext)
 
 gchar *remove_extension_from_path(const gchar *path)
 {
-	gint p = 0, n = -1;
+	const gchar *reg_ext;
 
 	if (!path) return NULL;
 
-	while (path[p])
-		{
-		if (path[p] == '.') n = p;
-		p++;
-		}
-	if (n < 0) n = p;
+	reg_ext = registered_extension_from_path(path);
 
-	return g_strndup(path, (gsize) n);
+	return g_strndup(path, strlen(path) - (reg_ext == NULL ? 0 : strlen(reg_ext)));
 }
 
 void parse_out_relatives(gchar *path)
