@@ -353,7 +353,7 @@ static void dest_dnd_set_data(GtkWidget *view,
 
 	list = g_list_append(list, path);
 
-	gchar **uris = uris_from_filelist(list);
+	gchar **uris = uris_from_pathlist(list);
 	gboolean ret = gtk_selection_data_set_uris(selection_data, uris);
 	if (!ret)
 		{
@@ -542,7 +542,7 @@ static void dest_view_delete(Dest_Data *dd, GtkTreeView *view)
 
 	text = g_strdup_printf(_("About to delete the file:\n %s"), path);
 	generic_dialog_add_message(dd->gd, GTK_STOCK_DIALOG_QUESTION,
-				   _("Delete file"), text);
+				   _("Delete file"), text, TRUE);
 	g_free(text);
 
 	gtk_widget_show(dd->gd->dialog);
@@ -698,6 +698,7 @@ static gboolean dest_keypress_cb(GtkWidget *view, GdkEventKey *event, gpointer d
 		{
 		case GDK_KEY_F10:
 			if (!(event->state & GDK_CONTROL_MASK)) return FALSE;
+			/* fall through */
 		case GDK_KEY_Menu:
 			dest_view_store_selection(dd, GTK_TREE_VIEW(view));
 			dest_popup_menu(dd, GTK_TREE_VIEW(view), 0, event->time, TRUE);
@@ -786,6 +787,7 @@ static void dest_new_dir_cb(GtkWidget *widget, gpointer data)
 
 		tree_edit_by_path(GTK_TREE_VIEW(dd->d_view), dd->right_click_path, 0, text,
 				  dest_view_rename_cb, dd);
+		gtk_entry_set_text(GTK_ENTRY(dd->entry), path);
 		}
 
 	g_free(path);
@@ -1066,6 +1068,7 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 	if (filter)
 		{
 		paned = gtk_hpaned_new();
+		DEBUG_NAME(paned);
 		gtk_table_attach(GTK_TABLE(table), paned, 0, 3, 1, 2,
 				 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 		gtk_widget_show(paned);
@@ -1145,6 +1148,9 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 		store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
 
 		dd->filter_combo = gtk_combo_box_new_with_model_and_entry(GTK_TREE_MODEL(store));
+		gtk_combo_box_set_entry_text_column(GTK_COMBO_BOX(dd->filter_combo),
+														FILTER_COLUMN_FILTER);
+
 		g_object_unref(store);
 		gtk_cell_layout_clear(GTK_CELL_LAYOUT(dd->filter_combo));
 		renderer = gtk_cell_renderer_text_new();
