@@ -52,42 +52,43 @@ static GdkPixbuf *file_util_get_error_icon(FileData *fd, GList *list, GtkWidget
  *--------------------------------------------------------------------------
  */
 
-#define DIALOG_DEF_IMAGE_DIM_X 200
-#define DIALOG_DEF_IMAGE_DIM_Y 150
+#define DIALOG_DEF_IMAGE_DIM_X 150
+#define DIALOG_DEF_IMAGE_DIM_Y 100
 
 static void generic_dialog_add_image(GenericDialog *gd, GtkWidget *box,
 				     FileData *fd1, const gchar *header1,
+				     gboolean second_image,
 				     FileData *fd2, const gchar *header2,
 				     gboolean show_filename)
 {
 	ImageWindow *imd;
-	GtkWidget *hbox = NULL;
+	GtkWidget *preview_box = NULL;
 	GtkWidget *vbox;
 	GtkWidget *label = NULL;
 
 	if (!box) box = gd->vbox;
 
-	if (fd2)
+	if (second_image)
 		{
-		hbox = pref_box_new(box, TRUE, GTK_ORIENTATION_HORIZONTAL, PREF_PAD_SPACE);
+		preview_box = pref_box_new(box, FALSE, GTK_ORIENTATION_VERTICAL, PREF_PAD_SPACE);
 		}
 
 	/* image 1 */
 
 	vbox = gtk_vbox_new(FALSE, PREF_PAD_GAP);
-	if (hbox)
+	if (preview_box)
 		{
 		GtkWidget *sep;
 
-		gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
+		gtk_box_pack_start(GTK_BOX(preview_box), vbox, FALSE, TRUE, 0);
 
-		sep = gtk_vseparator_new();
-		gtk_box_pack_start(GTK_BOX(hbox), sep, FALSE, FALSE, 0);
+		sep = gtk_hseparator_new();
+		gtk_box_pack_start(GTK_BOX(preview_box), sep, FALSE, FALSE, 0);
 		gtk_widget_show(sep);
 		}
 	else
 		{
-		gtk_box_pack_start(GTK_BOX(box), vbox, TRUE, TRUE, PREF_PAD_GAP);
+		gtk_box_pack_start(GTK_BOX(box), vbox, FALSE, TRUE, PREF_PAD_GAP);
 		}
 	gtk_widget_show(vbox);
 
@@ -119,9 +120,9 @@ static void generic_dialog_add_image(GenericDialog *gd, GtkWidget *box,
 
 	/* image 2 */
 
-	if (hbox && fd2)
+	if (preview_box)
 		{
-		vbox = pref_box_new(hbox, TRUE, GTK_ORIENTATION_VERTICAL, PREF_PAD_GAP);
+		vbox = pref_box_new(preview_box, TRUE, GTK_ORIENTATION_VERTICAL, PREF_PAD_GAP);
 
 		if (header2)
 			{
@@ -136,27 +137,18 @@ static void generic_dialog_add_image(GenericDialog *gd, GtkWidget *box,
 		g_object_set(G_OBJECT(imd->pr), "zoom_expand", FALSE, NULL);
 		gtk_widget_set_size_request(imd->widget, DIALOG_DEF_IMAGE_DIM_X, DIALOG_DEF_IMAGE_DIM_Y);
 		gtk_box_pack_start(GTK_BOX(vbox), imd->widget, TRUE, TRUE, 0);
-		image_change_fd(imd, fd2, 0.0);
+		if (fd2) image_change_fd(imd, fd2, 0.0);
 		gtk_widget_show(imd->widget);
 
-		pref_label_new(vbox, fd2->name);
+		if (show_filename)
+			{
+			label = pref_label_new(vbox, (fd2 == NULL) ? "" : fd2->name);
+			}
+		g_object_set_data(G_OBJECT(gd->dialog), "img_image2", imd);
+		g_object_set_data(G_OBJECT(gd->dialog), "img_label2", label);
 		}
 }
 
-static void generic_dialog_image_set(GenericDialog *gd, FileData *fd)
-{
-	ImageWindow *imd;
-	GtkWidget *label;
-
-	imd = g_object_get_data(G_OBJECT(gd->dialog), "img_image");
-	label = g_object_get_data(G_OBJECT(gd->dialog), "img_label");
-
-	if (!imd) return;
-
-	image_change_fd(imd, fd, 0.0);
-	if (label) gtk_label_set_text(GTK_LABEL(label), fd->name);
-}
-
 /*
  *--------------------------------------------------------------------------
  * Wrappers to aid in setting additional dialog properties (unde mouse, etc.)
@@ -210,7 +202,7 @@ GenericDialog *file_util_warning_dialog(const gchar *heading, const gchar *messa
 	GenericDialog *gd;
 
 	gd = file_util_gen_dlg(heading, "warning", parent, TRUE, NULL, NULL);
-	generic_dialog_add_message(gd, icon_stock_id, heading, message);
+	generic_dialog_add_message(gd, icon_stock_id, heading, message, TRUE);
 	generic_dialog_add_button(gd, GTK_STOCK_OK, NULL, file_util_warning_dialog_ok_cb, TRUE);
 	if (options->place_dialogs_under_mouse)
 		{
@@ -233,7 +225,7 @@ static gint filename_base_length(const gchar *name)
 		{
 		const gchar *ext;
 
-		ext = extension_from_path(name);
+		ext = registered_extension_from_path(name);
 		if (ext) n -= strlen(ext);
 		}
 
@@ -259,6 +251,7 @@ typedef enum {
 
 typedef enum {
 	UTILITY_PHASE_START = 0,
+	UTILITY_PHASE_INTERMEDIATE,
 	UTILITY_PHASE_ENTERING,
 	UTILITY_PHASE_CHECKED,
 	UTILITY_PHASE_DONE,
@@ -358,6 +351,48 @@ struct _UtilityDelayData {
 	guint idle_id; /* event source id */
 	};
 
+static void generic_dialog_image_set(UtilityData *ud, FileData *fd)
+{
+	ImageWindow *imd;
+	GtkWidget *label;
+	FileData *fd2 = NULL;
+	gchar *buf;
+
+	imd = g_object_get_data(G_OBJECT(ud->gd->dialog), "img_image");
+	label = g_object_get_data(G_OBJECT(ud->gd->dialog), "img_label");
+
+	if (!imd) return;
+
+	image_change_fd(imd, fd, 0.0);
+	buf = g_strjoin("\n", text_from_time(fd->date), text_from_size(fd->size), NULL);
+	if (label) gtk_label_set_text(GTK_LABEL(label), buf);
+	g_free(buf);
+
+	if (ud->type == UTILITY_TYPE_RENAME || ud->type == UTILITY_TYPE_COPY || ud->type == UTILITY_TYPE_MOVE)
+		{
+		imd = g_object_get_data(G_OBJECT(ud->gd->dialog), "img_image2");
+		label = g_object_get_data(G_OBJECT(ud->gd->dialog), "img_label2");
+
+		if (imd)
+			{
+			if (isfile(fd->change->dest))
+				{
+				fd2 = file_data_new_group(fd->change->dest);
+				image_change_fd(imd, fd2, 0.0);
+				buf = g_strjoin("\n", text_from_time(fd2->date), text_from_size(fd2->size), NULL);
+				if (label && fd->change->dest) gtk_label_set_text(GTK_LABEL(label), buf);
+				file_data_unref(fd2);
+				g_free(buf);
+				}
+			else
+				{
+				image_change_fd(imd, NULL, 0.0);
+				if (label) gtk_label_set_text(GTK_LABEL(label), "");
+				}
+			}
+		}
+}
+
 static gboolean file_util_write_metadata_first(UtilityType type, UtilityPhase phase, GList *flist, const gchar *dest_path, const gchar *editor_key, GtkWidget *parent);
 
 #define UTILITY_LIST_MIN_WIDTH  250
@@ -557,7 +592,7 @@ static gint file_util_perform_ci_cb(gpointer resume_data, EditorFlags flags, GLi
 					      NULL, TRUE,
 					      file_util_abort_cb, ud);
 
-			generic_dialog_add_message(d, GTK_STOCK_DIALOG_WARNING, NULL, msg->str);
+			generic_dialog_add_message(d, GTK_STOCK_DIALOG_WARNING, NULL, msg->str, TRUE);
 
 			generic_dialog_add_button(d, GTK_STOCK_GO_FORWARD, _("Co_ntinue"),
 						  file_util_resume_cb, TRUE);
@@ -883,6 +918,10 @@ void file_util_perform_ci(UtilityData *ud)
 			gchar *text = g_strdup_printf(_("%s\nUnable to start external command.\n"), editor_get_error_str(flags));
 			file_util_warning_dialog(ud->messages.fail, text, GTK_STOCK_DIALOG_ERROR, NULL);
 			g_free(text);
+
+			ud->gd = NULL;
+			ud->phase = UTILITY_PHASE_CANCEL;
+			file_util_dialog_run(ud);
 			}
 		}
 	else
@@ -990,7 +1029,7 @@ void file_util_check_ci(UtilityData *ud)
 					ud->parent, TRUE,
 					file_util_check_abort_cb, ud);
 
-		generic_dialog_add_message(d, GTK_STOCK_DIALOG_WARNING, _("Really continue?"), desc);
+		generic_dialog_add_message(d, GTK_STOCK_DIALOG_WARNING, _("Really continue?"), desc, TRUE);
 
 		generic_dialog_add_button(d, GTK_STOCK_GO_FORWARD, _("Co_ntinue"),
 					  file_util_check_resume_cb, TRUE);
@@ -1004,7 +1043,7 @@ void file_util_check_ci(UtilityData *ud)
 		d = file_util_gen_dlg(ud->messages.title, "dlg_confirm",
 					ud->parent, TRUE,
 					file_util_check_abort_cb, ud);
-		generic_dialog_add_message(d, GTK_STOCK_DIALOG_WARNING, _("This operation can't continue:"), desc);
+		generic_dialog_add_message(d, GTK_STOCK_DIALOG_WARNING, _("This operation can't continue:"), desc, TRUE);
 
 		gtk_widget_show(d->dialog);
 		}
@@ -1071,6 +1110,7 @@ static void file_util_dest_folder_update_path(UtilityData *ud)
 		{
 		case UTILITY_TYPE_COPY:
 			file_data_sc_update_ci_copy_list(ud->flist, ud->dest_path);
+			break;
 		case UTILITY_TYPE_MOVE:
 			file_data_sc_update_ci_move_list(ud->flist, ud->dest_path);
 			break;
@@ -1092,6 +1132,42 @@ static void file_util_dest_folder_update_path(UtilityData *ud)
 		}
 }
 
+static void file_util_fdlg_rename_cb(FileDialog *fdlg, gpointer data)
+{
+	UtilityData *ud = data;
+	gchar *desc = NULL;
+	GenericDialog *d = NULL;
+
+	file_util_dest_folder_update_path(ud);
+	if (isdir(ud->dest_path))
+		{
+		file_dialog_sync_history(fdlg, TRUE);
+		file_dialog_close(fdlg);
+		ud->fdlg = NULL;
+		file_util_dialog_run(ud);
+		}
+	else
+		{
+		/* During copy/move operations it is necessary to ensure that the
+		 * target directory exists before continuing with the next step.
+		 * If not revert to the select directory dialog
+		 */
+		desc = g_strdup_printf(_("%s is not a directory"), ud->dest_path);
+
+		d = file_util_gen_dlg(ud->messages.title, "dlg_confirm",
+					ud->parent, TRUE,
+					file_util_check_abort_cb, ud);
+		generic_dialog_add_message(d, GTK_STOCK_DIALOG_WARNING, _("This operation can't continue:"), desc, TRUE);
+
+		gtk_widget_show(d->dialog);
+		ud->phase = UTILITY_PHASE_START;
+
+		file_dialog_close(fdlg);
+		ud->fdlg = NULL;
+		g_free(desc);
+		}
+}
+
 static void file_util_fdlg_ok_cb(FileDialog *fdlg, gpointer data)
 {
 	UtilityData *ud = data;
@@ -1101,10 +1177,12 @@ static void file_util_fdlg_ok_cb(FileDialog *fdlg, gpointer data)
 	file_dialog_close(fdlg);
 
 	ud->fdlg = NULL;
+	ud->phase = UTILITY_PHASE_ENTERING;
 
 	file_util_dialog_run(ud);
-}
 
+	return;
+}
 
 static void file_util_dest_folder_entry_cb(GtkWidget *entry, gpointer data)
 {
@@ -1147,7 +1225,7 @@ static gchar *file_util_rename_multiple_auto_format_name(const gchar *format, co
 		parsed = tmp;
 		}
 
-	ext = extension_from_path(name);
+	ext = registered_extension_from_path(name);
 
 	middle = strchr(parsed, '*');
 	if (middle)
@@ -1176,7 +1254,8 @@ static void file_util_rename_preview_update(UtilityData *ud)
 {
 	GtkTreeModel *store;
 	GtkTreeSelection *selection;
-	GtkTreeIter iter;
+	GtkTreeIter iter, iter_selected;
+	GtkTreePath *path_iter, *path_selected;
 	const gchar *front;
 	const gchar *end;
 	const gchar *format;
@@ -1185,6 +1264,8 @@ static void file_util_rename_preview_update(UtilityData *ud)
 	gint padding;
 	gint n;
 	gint mode;
+	gchar *dirname;
+	gchar *destname;
 
 	mode = gtk_notebook_get_current_page(GTK_NOTEBOOK(ud->notebook));
 
@@ -1198,7 +1279,23 @@ static void file_util_rename_preview_update(UtilityData *ud)
 
 			gtk_tree_model_get(store, &iter, UTILITY_COLUMN_FD, &fd, -1);
 			g_assert(ud->with_sidecars); /* sidecars must be renamed too, it would break the pairing otherwise */
-			file_data_sc_update_ci_rename(fd, dest);
+
+			dirname = g_path_get_dirname(fd->change->dest);
+			destname = g_build_filename(dirname, dest, NULL);
+			switch (ud->type)
+				{
+				case UTILITY_TYPE_RENAME:
+					file_data_sc_update_ci_rename(fd, dest);
+					break;
+				case UTILITY_TYPE_COPY:
+					file_data_sc_update_ci_copy(fd, destname);
+					break;
+				case UTILITY_TYPE_MOVE:
+					file_data_sc_update_ci_move(fd, destname);
+					break;
+				default:;
+				}
+			generic_dialog_image_set(ud, fd);
 
 			gtk_list_store_set(GTK_LIST_STORE(store), &iter,
 				   UTILITY_COLUMN_DEST_PATH, fd->change->dest,
@@ -1214,13 +1311,19 @@ static void file_util_rename_preview_update(UtilityData *ud)
 
 		format = gtk_entry_get_text(GTK_ENTRY(ud->format_entry));
 
+		g_free(options->cp_mv_rn.auto_end);
+		options->cp_mv_rn.auto_end = g_strdup(end);
+		options->cp_mv_rn.auto_padding = padding;
+
 		if (mode == UTILITY_RENAME_FORMATTED)
 			{
 			start_n = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ud->format_spin));
+			options->cp_mv_rn.formatted_start = start_n;
 			}
 		else
 			{
 			start_n = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ud->auto_spin_start));
+			options->cp_mv_rn.auto_start = start_n;
 			}
 
 		store = gtk_tree_view_get_model(GTK_TREE_VIEW(ud->listview));
@@ -1242,13 +1345,43 @@ static void file_util_rename_preview_update(UtilityData *ud)
 				}
 
 			g_assert(ud->with_sidecars); /* sidecars must be renamed too, it would break the pairing otherwise */
-			file_data_sc_update_ci_rename(fd, dest);
+
+			dirname = g_path_get_dirname(fd->change->dest);
+			destname = g_build_filename(dirname, dest, NULL);
+
+			switch (ud->type)
+				{
+				case UTILITY_TYPE_RENAME:
+					file_data_sc_update_ci_rename(fd, dest);
+					break;
+				case UTILITY_TYPE_COPY:
+					file_data_sc_update_ci_copy(fd, destname);
+					break;
+				case UTILITY_TYPE_MOVE:
+					file_data_sc_update_ci_move(fd, destname);
+					break;
+				default:;
+				}
+
+			g_free(dirname);
+			g_free(destname);
+			g_free(dest);
+
+			selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(ud->listview));
+			gtk_tree_selection_get_selected(selection, &store, &iter_selected);
+			path_iter=gtk_tree_model_get_path(store,&iter);
+			path_selected=gtk_tree_model_get_path(store,&iter_selected);
+			if (!gtk_tree_path_compare(path_iter,path_selected))
+				{
+				generic_dialog_image_set(ud, fd);
+				}
+			gtk_tree_path_free(path_iter);
+			gtk_tree_path_free(path_selected);
+
 			gtk_list_store_set(GTK_LIST_STORE(store), &iter,
 					   UTILITY_COLUMN_DEST_PATH, fd->change->dest,
 					   UTILITY_COLUMN_DEST_NAME, filename_from_path(fd->change->dest),
 					   -1);
-			g_free(dest);
-
 			n++;
 			valid = gtk_tree_model_iter_next(store, &iter);
 			}
@@ -1317,11 +1450,11 @@ static gboolean file_util_preview_cb(GtkTreeSelection *selection, GtkTreeModel *
 	    !gtk_tree_model_get_iter(store, &iter, tpath)) return TRUE;
 
 	gtk_tree_model_get(store, &iter, UTILITY_COLUMN_FD, &fd, -1);
-	generic_dialog_image_set(ud->gd, fd);
+	generic_dialog_image_set(ud, fd);
 
 	ud->sel_fd = fd;
 
-	if (ud->type == UTILITY_TYPE_RENAME)
+	if (ud->type == UTILITY_TYPE_RENAME || ud->type == UTILITY_TYPE_COPY || ud->type == UTILITY_TYPE_MOVE)
 		{
 		const gchar *name = filename_from_path(fd->change->dest);
 
@@ -1398,7 +1531,7 @@ static void file_util_dialog_init_simple_list(UtilityData *ud)
 
 	box = generic_dialog_add_message(ud->gd, GTK_STOCK_DIALOG_QUESTION,
 					 ud->messages.question,
-					 dir_msg);
+					 dir_msg, TRUE);
 
 	g_free(dir_msg);
 
@@ -1413,7 +1546,7 @@ static void file_util_dialog_init_simple_list(UtilityData *ud)
 	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
 	gtk_tree_selection_set_select_function(selection, file_util_preview_cb, ud, NULL);
 
-	generic_dialog_add_image(ud->gd, box, NULL, NULL, NULL, NULL, FALSE);
+	generic_dialog_add_image(ud->gd, box, NULL, NULL, FALSE, NULL, NULL, FALSE);
 
 	if (ud->type == UTILITY_TYPE_DELETE ||
 	    ud->type == UTILITY_TYPE_DELETE_LINK ||
@@ -1445,13 +1578,22 @@ static void file_util_dialog_init_dest_folder(UtilityData *ud)
 
 	ud->fdlg = fdlg;
 
-	generic_dialog_add_message(GENERIC_DIALOG(fdlg), NULL, ud->messages.question, NULL);
+	generic_dialog_add_message(GENERIC_DIALOG(fdlg), NULL, ud->messages.question, NULL, FALSE);
 
 	label = pref_label_new(GENERIC_DIALOG(fdlg)->vbox, _("Choose the destination folder."));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	pref_spacer(GENERIC_DIALOG(fdlg)->vbox, 0);
 
-	file_dialog_add_button(fdlg, stock_id, ud->messages.title, file_util_fdlg_ok_cb, TRUE);
+	if (options->with_rename)
+		{
+		file_dialog_add_button(fdlg, stock_id, ud->messages.title, file_util_fdlg_ok_cb, TRUE);
+		file_dialog_add_button(fdlg, GTK_STOCK_EDIT, "With Rename", file_util_fdlg_rename_cb, TRUE);
+		}
+	else
+		{
+		file_dialog_add_button(fdlg, GTK_STOCK_EDIT, "With Rename", file_util_fdlg_rename_cb, TRUE);
+		file_dialog_add_button(fdlg, stock_id, ud->messages.title, file_util_fdlg_ok_cb, TRUE);
+		}
 
 	file_dialog_add_path_widgets(fdlg, NULL, ud->dest_path, "move_copy", NULL, NULL);
 
@@ -1479,7 +1621,7 @@ static GtkWidget *furm_simple_vlabel(GtkWidget *box, const gchar *text, gboolean
 }
 
 
-static void file_util_dialog_init_source_dest(UtilityData *ud)
+static void file_util_dialog_init_source_dest(UtilityData *ud, gboolean second_image)
 {
 	GtkTreeModel *store;
 	GtkTreeSelection *selection;
@@ -1489,18 +1631,29 @@ static void file_util_dialog_init_source_dest(UtilityData *ud)
 	GtkWidget *table;
 	GtkWidget *combo;
 	GtkWidget *page;
+	gchar *destination_message;
 
 	ud->gd = file_util_gen_dlg(ud->messages.title, "dlg_confirm",
 				   ud->parent, FALSE,  file_util_cancel_cb, ud);
 
-	box = generic_dialog_add_message(ud->gd, NULL, ud->messages.question, NULL);
+	box = generic_dialog_add_message(ud->gd, NULL, ud->messages.question, NULL, TRUE);
 
 	if (ud->discard_func) generic_dialog_add_button(ud->gd, GTK_STOCK_REVERT_TO_SAVED, _("Discard changes"), file_util_discard_cb, FALSE);
 	if (ud->details_func) generic_dialog_add_button(ud->gd, GTK_STOCK_INFO, _("File details"), file_util_details_cb, FALSE);
 
 	generic_dialog_add_button(ud->gd, GTK_STOCK_OK, ud->messages.title, file_util_ok_cb, TRUE);
 
-	box = pref_group_new(box, TRUE, ud->messages.desc_flist, GTK_ORIENTATION_HORIZONTAL);
+	if (ud->type == UTILITY_TYPE_COPY || ud->type == UTILITY_TYPE_MOVE)
+		{
+		destination_message = g_strconcat(ud->messages.desc_flist," to: ", ud->dest_path, NULL);
+		}
+	else
+		{
+		destination_message = g_strdup(ud->messages.desc_flist);
+		}
+
+	box = pref_group_new(box, TRUE, destination_message, GTK_ORIENTATION_HORIZONTAL);
+	g_free(destination_message);
 
 	ud->listview = file_util_dialog_add_list(box, ud->flist, FALSE, ud->with_sidecars);
 	file_util_dialog_add_list_column(ud->listview, _("Sidecars"), FALSE, UTILITY_COLUMN_SIDECARS);
@@ -1522,7 +1675,14 @@ static void file_util_dialog_init_source_dest(UtilityData *ud)
 			 G_CALLBACK(file_util_rename_preview_order_cb), ud);
 	gtk_widget_set_size_request(ud->listview, 300, 150);
 
-	generic_dialog_add_image(ud->gd, box, NULL, NULL, NULL, NULL, FALSE);
+	if (second_image)
+		{
+		generic_dialog_add_image(ud->gd, box, NULL, "Source", TRUE, NULL, "Destination", TRUE);
+		}
+	else
+		{
+		generic_dialog_add_image(ud->gd, box, NULL, NULL, FALSE, NULL, NULL, FALSE);
+		}
 
 //	gtk_container_add(GTK_CONTAINER(scrolled), view);
 	gtk_widget_show(ud->gd->dialog);
@@ -1573,19 +1733,19 @@ static void file_util_dialog_init_source_dest(UtilityData *ud)
 	box2 = furm_simple_vlabel(hbox, _("Start #"), FALSE);
 
 	ud->auto_spin_start = pref_spin_new(box2, NULL, NULL,
-					    0.0, 1000000.0, 1.0, 0, 1.0,
+					    0.0, 1000000.0, 1.0, 0, options->cp_mv_rn.auto_start,
 					    G_CALLBACK(file_util_rename_preview_adj_cb), ud);
 
 	box2 = furm_simple_vlabel(hbox, _("End text"), TRUE);
 
-	combo = history_combo_new(&ud->auto_entry_end, "", "numerical_rename_suffix", -1);
+	combo = history_combo_new(&ud->auto_entry_end, options->cp_mv_rn.auto_end, "numerical_rename_suffix", -1);
 	g_signal_connect(G_OBJECT(ud->auto_entry_end), "changed",
 			 G_CALLBACK(file_util_rename_preview_entry_cb), ud);
 	gtk_box_pack_start(GTK_BOX(box2), combo, TRUE, TRUE, 0);
 	gtk_widget_show(combo);
 
 	ud->auto_spin_pad = pref_spin_new(page, _("Padding:"), NULL,
-					  1.0, 8.0, 1.0, 0, 1.0,
+					  1.0, 8.0, 1.0, 0, options->cp_mv_rn.auto_padding,
 					  G_CALLBACK(file_util_rename_preview_adj_cb), ud);
 
 	page = gtk_vbox_new(FALSE, PREF_PAD_GAP);
@@ -1605,7 +1765,7 @@ static void file_util_dialog_init_source_dest(UtilityData *ud)
 	box2 = furm_simple_vlabel(hbox, _("Start #"), FALSE);
 
 	ud->format_spin = pref_spin_new(box2, NULL, NULL,
-					0.0, 1000000.0, 1.0, 0, 1.0,
+					0.0, 1000000.0, 1.0, 0, options->cp_mv_rn.formatted_start,
 					G_CALLBACK(file_util_rename_preview_adj_cb), ud);
 
 //	gtk_combo_box_set_active(GTK_COMBO_BOX(ud->combo_type), 0); /* callback will take care of the rest */
@@ -1678,28 +1838,42 @@ void file_util_dialog_run(UtilityData *ud)
 				case UTILITY_TYPE_EDITOR:
 				case UTILITY_TYPE_WRITE_METADATA:
 					file_util_dialog_init_simple_list(ud);
+					ud->phase = UTILITY_PHASE_ENTERING;
 					break;
 				case UTILITY_TYPE_RENAME:
-					file_util_dialog_init_source_dest(ud);
+					file_util_dialog_init_source_dest(ud, TRUE);
+					ud->phase = UTILITY_PHASE_ENTERING;
 					break;
 				case UTILITY_TYPE_COPY:
 				case UTILITY_TYPE_MOVE:
+					file_util_dialog_init_dest_folder(ud);
+					ud->phase = UTILITY_PHASE_INTERMEDIATE;
+					break;
 				case UTILITY_TYPE_FILTER:
 				case UTILITY_TYPE_CREATE_FOLDER:
 					file_util_dialog_init_dest_folder(ud);
+					ud->phase = UTILITY_PHASE_ENTERING;
 					break;
 				case UTILITY_TYPE_RENAME_FOLDER:
 					ud->phase = UTILITY_PHASE_CANCEL; /* FIXME - not handled for now */
 					file_util_dialog_run(ud);
 					return;
 				}
+			break;
+		case UTILITY_PHASE_INTERMEDIATE:
+			switch (ud->type)
+				{
+				case UTILITY_TYPE_COPY:
+				case UTILITY_TYPE_MOVE:
+					file_util_dialog_init_source_dest(ud, TRUE);
+					break;
+				default:;
+				}
 			ud->phase = UTILITY_PHASE_ENTERING;
 			break;
 		case UTILITY_PHASE_ENTERING:
 			file_util_check_ci(ud);
 			break;
-
-			ud->phase = UTILITY_PHASE_CHECKED;
 		case UTILITY_PHASE_CHECKED:
 			file_util_perform_ci(ud);
 			break;
@@ -1845,9 +2019,9 @@ static void file_util_details_dialog(UtilityData *ud, FileData *fd)
 
 	message = file_util_details_get_message(ud, fd, &stock_id);
 
-	box = generic_dialog_add_message(gd, stock_id, _("File details"), message);
+	box = generic_dialog_add_message(gd, stock_id, _("File details"), message, TRUE);
 
-	generic_dialog_add_image(gd, box, fd, NULL, NULL, NULL, FALSE);
+	generic_dialog_add_image(gd, box, fd, NULL, FALSE, NULL, NULL, FALSE);
 
 	gtk_widget_show(gd->dialog);
 
@@ -1901,7 +2075,7 @@ static void file_util_write_metadata_details_dialog(UtilityData *ud, FileData *f
 		message2 = g_strdup_printf(_("The following metadata tags will be written to the image file itself."));
 		}
 
-	box = generic_dialog_add_message(gd, stock_id, _("Overview of changed metadata"), message1);
+	box = generic_dialog_add_message(gd, stock_id, _("Overview of changed metadata"), message1, TRUE);
 
 	box = pref_group_new(box, TRUE, message2, GTK_ORIENTATION_HORIZONTAL);
 
@@ -1944,7 +2118,7 @@ static void file_util_write_metadata_details_dialog(UtilityData *ud, FileData *f
 		i++;
 		}
 
-	generic_dialog_add_image(gd, box, fd, NULL, NULL, NULL, FALSE);
+	generic_dialog_add_image(gd, box, fd, NULL, FALSE, NULL, NULL, FALSE);
 
 	gtk_widget_set_size_request(gd->dialog, DIALOG_WIDTH, -1);
 	gtk_widget_show(gd->dialog);
@@ -1970,6 +2144,7 @@ static void file_util_delete_full(FileData *source_fd, GList *flist, GtkWidget *
 {
 	UtilityData *ud;
 	GList *ungrouped = NULL;
+	gchar *message;
 
 	if (source_fd)
 		flist = g_list_append(flist, file_data_ref(source_fd));
@@ -2002,10 +2177,17 @@ static void file_util_delete_full(FileData *source_fd, GList *flist, GtkWidget *
 	ud->parent = parent;
 
 	ud->details_func = file_util_details_dialog;
-
+	if(options->file_ops.safe_delete_enable)
+		{
+		message = _("This will move the following files to the Trash bin");
+		}
+	else
+		{
+		message = _("This will permanently delete the following files");
+		}
 	ud->messages.title = _("Delete");
 	ud->messages.question = _("Delete files?");
-	ud->messages.desc_flist = _("This will delete the following files");
+	ud->messages.desc_flist = message;
 	ud->messages.desc_source_fd = "";
 	ud->messages.fail = _("File deletion failed");
 
@@ -2090,6 +2272,7 @@ static void file_util_move_full(FileData *source_fd, GList *flist, const gchar *
 	ud->flist = flist;
 	ud->content_list = NULL;
 	ud->parent = parent;
+	ud->details_func = file_util_details_dialog;
 
 	if (dest_path) ud->dest_path = g_strdup(dest_path);
 
@@ -2139,6 +2322,7 @@ static void file_util_copy_full(FileData *source_fd, GList *flist, const gchar *
 	ud->flist = flist;
 	ud->content_list = NULL;
 	ud->parent = parent;
+	ud->details_func = file_util_details_dialog;
 
 	if (dest_path) ud->dest_path = g_strdup(dest_path);
 
@@ -2499,7 +2683,7 @@ static void file_util_delete_dir_full(FileData *fd, GtkWidget *parent, UtilityPh
 					fd->path);
 		box = generic_dialog_add_message(gd, GTK_STOCK_DIALOG_WARNING,
 						 _("Folder contains subfolders"),
-						 text);
+						 text, TRUE);
 		g_free(text);
 
 		box = pref_group_new(box, TRUE, _("Subfolders:"), GTK_ORIENTATION_VERTICAL);
@@ -2880,23 +3064,44 @@ void file_util_rename_dir(FileData *source_fd, const gchar *new_path, GtkWidget
 }
 
 
-void file_util_copy_path_to_clipboard(FileData *fd)
+void file_util_copy_path_to_clipboard(FileData *fd, gboolean quoted)
 {
 	GtkClipboard *clipboard;
 
 	if (!fd || !*fd->path) return;
 
-	clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
-	gtk_clipboard_set_text(clipboard, g_shell_quote(fd->path), -1);
+	if (options->clipboard_selection == PRIMARY)
+		{
+		clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+		}
+	else
+		{
+		clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+		}
+	if (quoted)
+		{
+		gtk_clipboard_set_text(clipboard, g_shell_quote(fd->path), -1);
+		}
+	else
+		{
+		gtk_clipboard_set_text(clipboard, fd->path, -1);
+		}
 }
 
-void file_util_copy_path_list_to_clipboard(GList *list)
+void file_util_copy_path_list_to_clipboard(GList *list, gboolean quoted)
 {
 	GtkClipboard *clipboard;
 	GList *work;
 	GString *new;
 
-	clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+	if (options->clipboard_selection == PRIMARY)
+		{
+		clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+		}
+	else
+		{
+		clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+		}
 
 	new = g_string_new("");
 	work = list;
@@ -2906,7 +3111,14 @@ void file_util_copy_path_list_to_clipboard(GList *list)
 
 		if (!fd || !*fd->path) continue;
 
-		g_string_append(new, g_shell_quote(fd->path));
+		if (quoted)
+			{
+			g_string_append(new, g_shell_quote(fd->path));
+			}
+		else
+			{
+			g_string_append(new, fd->path);
+			}
 		if (work) g_string_append_c(new, ' ');
 		}
 
