@@ -49,6 +49,8 @@ static PixmapFolders *folder_icons_new(GtkWidget *widget)
 	pf->open   = gtk_widget_render_icon(widget, GTK_STOCK_OPEN, size, NULL);
 	pf->deny   = gtk_widget_render_icon(widget, GTK_STOCK_STOP, size, NULL);
 	pf->parent = gtk_widget_render_icon(widget, GTK_STOCK_GO_UP, size, NULL);
+	/* FIXME: this is not a suitable icon */
+	pf->link = gtk_widget_render_icon(widget, GTK_STOCK_REDO, size, NULL);
 #else
 	/* GQView legacy icons */
 	pf->close  = pixbuf_inline(PIXBUF_INLINE_FOLDER_CLOSED);
@@ -67,6 +69,7 @@ static void folder_icons_free(PixmapFolders *pf)
 	g_object_unref(pf->open);
 	g_object_unref(pf->deny);
 	g_object_unref(pf->parent);
+	g_object_unref(pf->link);
 
 	g_free(pf);
 }
@@ -473,7 +476,7 @@ static void vd_pop_menu_slide_rec_cb(GtkWidget *widget, gpointer data)
 	if (!vd->layout) return;
 	if (!vd->click_fd) return;
 
-	list = filelist_recursive(vd->click_fd);
+	list = filelist_recursive_full(vd->click_fd, vd->layout->sort_method, vd->layout->sort_ascend);
 
 	layout_image_slideshow_stop(vd->layout);
 	layout_image_slideshow_start_from_list(vd->layout, list);
@@ -496,7 +499,7 @@ static void vd_pop_menu_dupe(ViewDir *vd, gint recursive)
 		list = filelist_filter(list, FALSE);
 		}
 
-	dw = dupe_window_new(DUPE_MATCH_NAME);
+	dw = dupe_window_new();
 	dupe_window_add_files(dw, list, recursive);
 
 	filelist_free(list);
@@ -528,7 +531,16 @@ static void vd_pop_menu_copy_path_cb(GtkWidget *widget, gpointer data)
 
 	if (!vd->click_fd) return;
 
-	file_util_copy_path_to_clipboard(vd->click_fd);
+	file_util_copy_path_to_clipboard(vd->click_fd, TRUE);
+}
+
+static void vd_pop_menu_copy_path_unquoted_cb(GtkWidget *widget, gpointer data)
+{
+	ViewDir *vd = data;
+
+	if (!vd->click_fd) return;
+
+	file_util_copy_path_to_clipboard(vd->click_fd, FALSE);
 }
 
 static void vd_pop_submenu_dir_view_as_cb(GtkWidget *widget, gpointer data)
@@ -673,12 +685,15 @@ GtkWidget *vd_pop_menu(ViewDir *vd, FileData *fd)
 
 	menu_item_add_sensitive(menu, _("_Rename..."), rename_delete_active,
 				G_CALLBACK(vd_pop_menu_rename_cb), vd);
-	menu_item_add_stock_sensitive(menu, _("_Delete..."), GTK_STOCK_DELETE, rename_delete_active,
-				      G_CALLBACK(vd_pop_menu_delete_cb), vd);
 
 	menu_item_add(menu, _("_Copy path"),
 		      G_CALLBACK(vd_pop_menu_copy_path_cb), vd);
 
+	menu_item_add(menu, _("_Copy path unquoted"),
+		      G_CALLBACK(vd_pop_menu_copy_path_unquoted_cb), vd);
+
+	menu_item_add_stock_sensitive(menu, _("_Delete..."), GTK_STOCK_DELETE, rename_delete_active,
+				      G_CALLBACK(vd_pop_menu_delete_cb), vd);
 	menu_item_add_divider(menu);
 
 
@@ -865,6 +880,10 @@ void vd_dnd_drop_scroll_cancel(ViewDir *vd)
 static gboolean vd_auto_scroll_idle_cb(gpointer data)
 {
 	ViewDir *vd = data;
+#if GTK_CHECK_VERSION(3,0,0)
+	GdkDeviceManager *device_manager;
+	GdkDevice *device;
+#endif
 
 	if (vd->drop_fd)
 		{
@@ -873,7 +892,13 @@ static gboolean vd_auto_scroll_idle_cb(gpointer data)
 		gint w, h;
 
 		window = gtk_widget_get_window(vd->view);
+#if GTK_CHECK_VERSION(3,0,0)
+		device_manager = gdk_display_get_device_manager(gdk_window_get_display(window));
+		device = gdk_device_manager_get_client_pointer(device_manager);
+		gdk_window_get_device_position(window, device, &x, &y, NULL);
+#else
 		gdk_window_get_pointer(window, &x, &y, NULL);
+#endif
 		w = gdk_window_get_width(window);
 		h = gdk_window_get_height(window);
 		if (x >= 0 && x < w && y >= 0 && y < h)
@@ -1025,6 +1050,9 @@ gboolean vd_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 	GtkTreePath *tpath;
 	FileData *fd = NULL;
 
+	if (vd->type == DIRVIEW_LIST && !options->view_dir_list_single_click_enter)
+		return FALSE;
+
 	if (!vd->click_fd) return FALSE;
 	vd_color_set(vd, vd->click_fd, FALSE);
 
