@@ -27,11 +27,10 @@ enum {
 	DIR_COLUMN_NAME,
 	DIR_COLUMN_COLOR,
 	DIR_COLUMN_DATE,
+	DIR_COLUMN_LINK,
 	DIR_COLUMN_COUNT
 };
 
-#define VIEW_DIR_TYPES_COUNT 2
-
 ViewDir *vd_new(DirViewType type, FileData *dir_fd);
 
 void vd_set_select_func(ViewDir *vdl, void (*func)(ViewDir *vdl, FileData *fd, gpointer data), gpointer data);
