@@ -21,8 +21,6 @@
 #ifndef VIEW_FILE_H
 #define VIEW_FILE_H
 
-#define VIEW_FILE_TYPES_COUNT 2
-
 #define VFLIST(_vf_) ((ViewFileInfoList *)(_vf_->info))
 #define VFICON(_vf_) ((ViewFileInfoIcon *)(_vf_->info))
 
@@ -41,6 +39,7 @@ void vf_refresh_idle(ViewFile *vf);
 
 void vf_thumb_set(ViewFile *vf, gboolean enable);
 void vf_marks_set(ViewFile *vf, gboolean enable);
+void vf_star_rating_set(ViewFile *vf, gboolean enable);
 void vf_sort_set(ViewFile *vf, SortType type, gboolean ascend);
 
 guint vf_marks_get_filter(ViewFile *vf);
@@ -55,7 +54,6 @@ gint vf_index_by_fd(ViewFile *vf, FileData *in_fd);
 guint vf_count(ViewFile *vf, gint64 *bytes);
 GList *vf_get_list(ViewFile *vf);
 
-gint vf_index_is_selected(ViewFile *vf, gint row);
 guint vf_selection_count(ViewFile *vf, gint64 *bytes);
 GList *vf_selection_get_list(ViewFile *vf);
 GList *vf_selection_get_list_by_index(ViewFile *vf);
@@ -74,6 +72,8 @@ void vf_notify_cb(FileData *fd, NotifyType type, gpointer data);
 void vf_thumb_update(ViewFile *vf);
 void vf_thumb_cleanup(ViewFile *vf);
 void vf_thumb_stop(ViewFile *vf);
-
+void vf_read_metadata_in_idle(ViewFile *vf);
+void vf_file_filter_set(ViewFile *vf, gboolean enable);
+GRegex *vf_file_filter_get_filter(ViewFile *vf);
 #endif /* VIEW_FILE_H */
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
